<?php
/* Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/requestmanager/admin/setup.php
 *		\ingroup    requestmanager
 *		\brief      Page to setup requestmanager module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/doleditor.class.php';
dol_include_once('/requestmanager/lib/requestmanager.lib.php');
dol_include_once('/requestmanager/lib/requestmanagertimeslots.lib.php');
dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/advancedictionaries/class/dictionary.class.php');

$langs->load("admin");
$langs->load("errors");
$langs->load("mails");
$langs->load("requestmanager@requestmanager");
$langs->load("opendsi@requestmanager");

if (!$user->admin) accessforbidden();

$action = GETPOST('action','alpha');

// Get request types list
$requestmanagerrequesttype = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerrequesttype');
$request_types = $requestmanagerrequesttype->fetch_lines(1, array(), array(), 0, 0, false, true);
$request_types_array = array();
foreach ($request_types as $request_type) {
    $request_types_array[$request_type->id] = $request_type->fields['label'];
}

// Get request status list
$requestmanagerstatus = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
$request_status = $requestmanagerstatus->fetch_lines(1, array(), array(), 0, 0, false, true);

/*
 *	Actions
 */

$errors = [];
$error = 0;

if ($action == 'set_notification_assigned_options') {
    $email_address = $email = GETPOST('REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM', "alpha");
    if (preg_match('/<([^>]+)>/i', $email, $matches)) $email_address = $matches[1];
    if (empty($email) || isValidEmail($email_address)) {
        $res = dolibarr_set_const($db, 'REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM', $email, 'chaine', 0, '', $conf->entity);
        if (!$res > 0) {
            $errors[] = $db->lasterror();
            $error++;
        }
    } elseif (!empty($email)) {
        $errors[] = $langs->trans('BadEMail') . ': ' . dol_htmlentities($email);
        $error++;
    }

    $value = GETPOST('REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif ($action == 'set_notification_requester_options') {
    $email_address = $email = GETPOST('REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM', "alpha");
    if (preg_match('/<([^>]+)>/i', $email, $matches)) $email_address = $matches[1];
    if (empty($email) || isValidEmail($email_address)) {
        $res = dolibarr_set_const($db, 'REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM', $email, 'chaine', 0, '', $conf->entity);
        if (!$res > 0) {
            $errors[] = $db->lasterror();
            $error++;
        }
    } elseif (!empty($email)) {
        $errors[] = $langs->trans('BadEMail') . ': ' . dol_htmlentities($email);
        $error++;
    }

    $value = GETPOST('REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif ($action == 'set_notification_watchers_options') {
    $email_address = $email = GETPOST('REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM', "alpha");
    if (preg_match('/<([^>]+)>/i', $email, $matches)) $email_address = $matches[1];
    if (empty($email) || isValidEmail($email_address)) {
        $res = dolibarr_set_const($db, 'REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM', $email, 'chaine', 0, '', $conf->entity);
        if (!$res > 0) {
            $errors[] = $db->lasterror();
            $error++;
        }
    } elseif (!empty($email)) {
        $errors[] = $langs->trans('BadEMail') . ': ' . dol_htmlentities($email);
        $error++;
    }

    $value = GETPOST('REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif (preg_match('/set_(.*)/',$action,$reg)) {
    $code = $reg[1];
    $value = (GETPOST($code) ? GETPOST($code) : 1);
    if (dolibarr_set_const($db, $code, $value, 'chaine', 0, '', $conf->entity) > 0) {
        Header("Location: " . $_SERVER["PHP_SELF"]);
        exit;
    } else {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif (preg_match('/del_(.*)/',$action,$reg)) {
    $code = $reg[1];
    if (dolibarr_del_const($db, $code, $conf->entity) > 0) {
        Header("Location: " . $_SERVER["PHP_SELF"]);
        exit;
    } else {
        $errors[] = $db->lasterror();
        $error++;
    }
}

if ($action != '') {
    if (!$error) {
        setEventMessage($langs->trans("SetupSaved"));
    } else {
        setEventMessages(/*$langs->trans("Error")*/'', $errors, 'errors');
    }
}

/*
 *	View
 */

$wikihelp='EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es';
llxHeader('', $langs->trans("RequestManagerSetup"), $wikihelp);

$form = new Form($db);
$formrequestmanager = new FormRequestManager($db);

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("RequestManagerSetup"),$linkback,'title_setup');
print "<br>\n";

$head=requestmanager_admin_prepare_head();

print dol_get_fiche_head($head, 'settings_notify', $langs->trans("Module163018Name"), 0, 'opendsi@requestmanager');

/********************************************************
 *  Notification options
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerNotificationOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_notification_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_CREATE_NOTIFY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerCreateNotifyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerCreateNotifyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_CREATE_NOTIFY');
} else {
    if (empty($conf->global->REQUESTMANAGER_CREATE_NOTIFY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_CREATE_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_CREATE_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_SET_ASSIGNED_NOTIFY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerSetAssignedNotifyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerSetAssignedNotifyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_SET_ASSIGNED_NOTIFY');
} else {
    if (empty($conf->global->REQUESTMANAGER_SET_ASSIGNED_NOTIFY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_SET_ASSIGNED_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_SET_ASSIGNED_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_STATUS_MODIFY_NOTIFY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerStatusModifyNotifyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerStatusModifyNotifyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_STATUS_MODIFY_NOTIFY');
} else {
    if (empty($conf->global->REQUESTMANAGER_STATUS_MODIFY_NOTIFY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_STATUS_MODIFY_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_STATUS_MODIFY_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGERMESSAGE_CREATE_NOTIFY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerMessageCreateNotifyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerMessageCreateNotifyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGERMESSAGE_CREATE_NOTIFY');
} else {
    if (empty($conf->global->REQUESTMANAGERMESSAGE_CREATE_NOTIFY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGERMESSAGE_CREATE_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGERMESSAGE_CREATE_NOTIFY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

print '</table>';

print '</form><br>' . "\n";

/********************************************************
 *  Notification assigned options
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerNotificationAssignedOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_notification_assigned_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerNotificationAssignedByEmailName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerNotificationAssignedByEmailDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL');
} else {
    if (empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_WEBSITE
/*$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerNotificationAssignedByWebSiteName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerNotificationAssignedByWebSiteDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_WEBSITE');
} else {
    if (empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_WEBSITE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_WEBSITE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_WEBSITE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";*/

// REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerAssignedNotificationSendFromName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerAssignedNotificationSendFromDesc").'</td>'."\n";
print '<td align="right">'."\n";
$email = GETPOST('REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM', "alpha");
print '<input type="text" size="100" name="REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM" value="'.(!empty($email) ? $email : (!empty($conf->global->REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM) ? $conf->global->REQUESTMANAGER_ASSIGNED_NOTIFICATION_SEND_FROM : '')).'">';
print '</td></tr>'."\n";

// REQUESTMANAGER_SPLIT_ASSIGNED_NOTIFICATION
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerSplitAssignedNotificationName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerSplitAssignedNotificationDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_SPLIT_ASSIGNED_NOTIFICATION');
} else {
    if (empty($conf->global->REQUESTMANAGER_SPLIT_ASSIGNED_NOTIFICATION)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_SPLIT_ASSIGNED_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_SPLIT_ASSIGNED_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerAssignedNotificationSignatureName").' '.img_info($langs->trans("RequestManagerAssignedNotificationSignatureDesc")).'</td>'."\n";
print '<td align="right" colspan="2">'."\n";
$signature = GETPOST('REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE', "alpha");
$doleditor = new DolEditor('REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE', !empty($signature) ? $signature : (!empty($conf->global->REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE) ? $conf->global->REQUESTMANAGER_ASSIGNED_NOTIFICATION_SIGNATURE : ''),
    '', 200, 'dolibarr_notes', 'In', false, false, isModEnabled('fckeditor'), ROWS_5, '90%');
print $doleditor->Create(1);
print '</td></tr>'."\n";

// REQUESTMANAGER_ASSIGNED_NOTIFICATION_USER_SIGNATURE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerAssignedNotificationUserSignatureName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerAssignedNotificationUserSignatureDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_ASSIGNED_NOTIFICATION_USER_SIGNATURE');
} else {
    if (empty($conf->global->REQUESTMANAGER_ASSIGNED_NOTIFICATION_USER_SIGNATURE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_ASSIGNED_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_ASSIGNED_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>';

/********************************************************
 *  Notification requester options
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerNotificationRequesterOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_notification_requester_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerRequesterNotificationSendFromName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerRequesterNotificationSendFromDesc").'</td>'."\n";
print '<td align="right">'."\n";
$email = GETPOST('REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM', "alpha");
print '<input type="text" size="100" name="REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM" value="'.(!empty($email) ? $email : (!empty($conf->global->REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM) ? $conf->global->REQUESTMANAGER_REQUESTER_NOTIFICATION_SEND_FROM : '')).'">';
print '</td></tr>'."\n";

// REQUESTMANAGER_SPLIT_REQUESTER_NOTIFICATION
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerSplitRequesterNotificationName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerSplitRequesterNotificationDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_SPLIT_REQUESTER_NOTIFICATION');
} else {
    if (empty($conf->global->REQUESTMANAGER_SPLIT_REQUESTER_NOTIFICATION)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_SPLIT_REQUESTER_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_SPLIT_REQUESTER_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerRequesterNotificationSignatureName").' '.img_info($langs->trans("RequestManagerRequesterNotificationSignatureDesc")).'</td>'."\n";
print '<td align="right" colspan="2">'."\n";
$signature = GETPOST('REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE', "alpha");
$doleditor = new DolEditor('REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE', !empty($signature) ? $signature : (!empty($conf->global->REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE) ? $conf->global->REQUESTMANAGER_REQUESTER_NOTIFICATION_SIGNATURE : ''),
    '', 200, 'dolibarr_notes', 'In', false, false, isModEnabled('fckeditor'), ROWS_5, '90%');
print $doleditor->Create(1);
print '</td></tr>'."\n";

// REQUESTMANAGER_REQUESTER_NOTIFICATION_USER_SIGNATURE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerRequesterNotificationUserSignatureName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerRequesterNotificationUserSignatureDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_REQUESTER_NOTIFICATION_USER_SIGNATURE');
} else {
    if (empty($conf->global->REQUESTMANAGER_REQUESTER_NOTIFICATION_USER_SIGNATURE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_REQUESTER_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_REQUESTER_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>';

/********************************************************
 *  Notification watchers options
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerNotificationWatchersOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_notification_watchers_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerWatchersNotificationSendFromName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerWatchersNotificationSendFromDesc").'</td>'."\n";
print '<td align="right">'."\n";
$email = GETPOST('REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM', "alpha");
print '<input type="text" size="100" name="REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM" value="'.(!empty($email) ? $email : (!empty($conf->global->REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM) ? $conf->global->REQUESTMANAGER_WATCHERS_NOTIFICATION_SEND_FROM : '')).'">';
print '</td></tr>'."\n";

// REQUESTMANAGER_SPLIT_WATCHERS_NOTIFICATION
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerSplitWatchersNotificationName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerSplitWatchersNotificationDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_SPLIT_WATCHERS_NOTIFICATION');
} else {
    if (empty($conf->global->REQUESTMANAGER_SPLIT_WATCHERS_NOTIFICATION)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_SPLIT_WATCHERS_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_SPLIT_WATCHERS_NOTIFICATION&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>'.$langs->trans("RequestManagerWatchersNotificationSignatureName").' '.img_info($langs->trans("RequestManagerWatchersNotificationSignatureDesc")).'</td>'."\n";
print '<td align="right" colspan="2">'."\n";
$signature = GETPOST('REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE', "alpha");
$doleditor = new DolEditor('REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE', !empty($signature) ? $signature : (!empty($conf->global->REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE) ? $conf->global->REQUESTMANAGER_WATCHERS_NOTIFICATION_SIGNATURE : ''),
    '', 200, 'dolibarr_notes', 'In', false, false, isModEnabled('fckeditor'), ROWS_5, '90%');
print $doleditor->Create(1);
print '</td></tr>'."\n";

// REQUESTMANAGER_WATCHERS_NOTIFICATION_USER_SIGNATURE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerWatchersNotificationUserSignatureName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerWatchersNotificationUserSignatureDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_WATCHERS_NOTIFICATION_USER_SIGNATURE');
} else {
    if (empty($conf->global->REQUESTMANAGER_WATCHERS_NOTIFICATION_USER_SIGNATURE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_WATCHERS_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_WATCHERS_NOTIFICATION_USER_SIGNATURE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>';

print dol_get_fiche_end();

llxFooter();

$db->close();
