<?php
/* Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/requestmanager/admin/setup.php
 *		\ingroup    requestmanager
 *		\brief      Page to setup requestmanager module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT . '/core/lib/admin.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/doleditor.class.php';
dol_include_once('/requestmanager/lib/requestmanager.lib.php');
dol_include_once('/requestmanager/lib/requestmanagertimeslots.lib.php');
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/advancedictionaries/class/dictionary.class.php');

$langs->load("admin");
$langs->load("errors");
$langs->load("mails");
$langs->load("requestmanager@requestmanager");
$langs->load("opendsi@requestmanager");

if (!$user->admin) accessforbidden();

$action = GETPOST('action','alpha');

// Get request types list
$requestmanagerrequesttype = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerrequesttype');
$request_types = $requestmanagerrequesttype->fetch_lines(1, array(), array(), 0, 0, false, true);
$request_types_array = array();
foreach ($request_types as $request_type) {
    $request_types_array[$request_type->id] = $request_type->fields['label'];
}

// Get request status list
$requestmanagerstatus = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
$request_status = $requestmanagerstatus->fetch_lines(1, array(), array(), 0, 0, false, true);

/*
 *	Actions
 */

$errors = [];
$error = 0;

if ($action == 'set_timeslots_options') {
    $value = GETPOST('REQUESTMANAGER_TIMESLOTS_PERIODS', "alpha");
    $res = requestmanagertimeslots_get_periods($value);
    if (is_array($res)) {
        $res = dolibarr_set_const($db, 'REQUESTMANAGER_TIMESLOTS_PERIODS', $value, 'chaine', 0, '', $conf->entity);
        if (!$res > 0) {
            $errors[] = $db->lasterror();
            $error++;
        }
    } else {
        $errors[] = $langs->trans('RequestManagerTimeSlotsPeriodsName') . ': ' . $res;
        $error++;
    }
} elseif ($action == 'set_planning_options') {
    $value = GETPOST('REQUESTMANAGER_PLANNING_REQUEST_TYPE', "array");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_PLANNING_REQUEST_TYPE', implode(',', $value), 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }

    $request_types_planned = !empty($conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) ? explode(',', $conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) : array();
    foreach ($request_types as $request_type) {
        $value = GETPOST('REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_' . $request_type->id, "int");
        if (!($value > 0) && in_array($request_type->id, $request_types_planned)) {
            $errors[] = $langs->trans('RequestManagerErrorPlanningRequestStatusToPlanMandatory', $request_type->fields['label']);
            $error++;
        } else {
            $res = dolibarr_set_const($db, 'REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_' . $request_type->id, $value > 0 && in_array($request_type->id, $request_types_planned) ? $value : '', 'chaine', 0, '', $conf->entity);
            if (!$res > 0) {
                $errors[] = $db->lasterror();
                $error++;
            }
        }

        $value = GETPOST('REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED_' . $request_type->id, "int");
        if (!($value > 0) && in_array($request_type->id, $request_types_planned)) {
            $errors[] = $langs->trans('RequestManagerErrorPlanningRequestStatusPlannedMandatory', $request_type->fields['label']);
            $error++;
        } else {
            $res = dolibarr_set_const($db, 'REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED_' . $request_type->id, $value > 0 && in_array($request_type->id, $request_types_planned) ? $value : '', 'chaine', 0, '', $conf->entity);
            if (!$res > 0) {
                $errors[] = $db->lasterror();
                $error++;
            }
        }
    }
} elseif ($action == 'set_chronometer_options') {
    $value = GETPOST('REQUESTMANAGER_CHRONOMETER_TIME', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_CHRONOMETER_TIME', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }

    $value = GETPOST('REQUESTMANAGER_CHRONOMETER_BLINK_COLOR', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_CHRONOMETER_BLINK_COLOR', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }

    /*$value =  GETPOST('REQUESTMANAGER_CHRONOMETER_SOUND', "alpha");
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_CHRONOMETER_SOUND', $value, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }*/
} elseif ($action == 'set') {
//    // operation time default in minute
//    $operationTimeDefault = GETPOST('REQUESTMANAGER_OPERATION_TIME_DEFAULT', 'int') ? GETPOST('REQUESTMANAGER_OPERATION_TIME_DEFAULT', 'int') : 0;
//    $res = dolibarr_set_const($db, 'REQUESTMANAGER_OPERATION_TIME_DEFAULT', $operationTimeDefault, 'chaine', 0, '', $conf->entity);
//    if (!$res > 0) {
//        $errors[] = $db->lasterror();
//        $error++;
//    }
//
//    // deadline time default in minute
//    $deadlineTimeDefault = GETPOST('REQUESTMANAGER_DEADLINE_TIME_DEFAULT', 'int') ? GETPOST('REQUESTMANAGER_DEADLINE_TIME_DEFAULT', 'int') : 0;
//    $res = dolibarr_set_const($db, 'REQUESTMANAGER_DEADLINE_TIME_DEFAULT', $deadlineTimeDefault, 'chaine', 0, '', $conf->entity);
//    if (!$res > 0) {
//        $errors[] = $db->lasterror();
//        $error++;
//    }

    // root product categories
    $rootProductCategories = GETPOST('REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES', 'int') > 0 ? GETPOST('REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES', 'int') : '';
    $res = dolibarr_set_const($db, 'REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES', $rootProductCategories, 'chaine', 0, '', $conf->entity);
    if (!$res > 0) {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif (preg_match('/set_(.*)/',$action,$reg)) {
    $code = $reg[1];
    $value = (GETPOST($code) ? GETPOST($code) : 1);
    if (dolibarr_set_const($db, $code, $value, 'chaine', 0, '', $conf->entity) > 0) {
        Header("Location: " . $_SERVER["PHP_SELF"]);
        exit;
    } else {
        $errors[] = $db->lasterror();
        $error++;
    }
} elseif (preg_match('/del_(.*)/',$action,$reg)) {
    $code = $reg[1];
    if (dolibarr_del_const($db, $code, $conf->entity) > 0) {
        Header("Location: " . $_SERVER["PHP_SELF"]);
        exit;
    } else {
        $errors[] = $db->lasterror();
        $error++;
    }
}

if ($action != '') {
    if (!$error) {
        setEventMessage($langs->trans("SetupSaved"));
    } else {
        setEventMessages(/*$langs->trans("Error")*/'', $errors, 'errors');
    }
}

/*
 *	View
 */

$wikihelp='EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es';
llxHeader('', $langs->trans("RequestManagerSetup"), $wikihelp);

$form = new Form($db);

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("RequestManagerSetup"),$linkback,'title_setup');
print "<br>\n";

$head=requestmanager_admin_prepare_head();

print dol_get_fiche_head($head, 'settings_other', $langs->trans("Module163018Name"), 0, 'opendsi@requestmanager');

/********************************************************
 *  Displaying options
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerDisplayingOptions"),'','');

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayButtonCreateRequestInTopMenuName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayButtonCreateRequestInTopMenuDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU');
} else {
    if (empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayButtonCreateRequestInTopRightMenuName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayButtonCreateRequestInTopRightMenuDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
	print ajax_constantonoff('REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU');
} else {
	if (empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU)) {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
	} else {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
	}
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_DISPLAY_DISABLED_FOLLOW_LINK_TO_MY_EVENT_LIST
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayDisabledFollowLinkToMyEventListName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayDisabledFollowLinkToMyEventListDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
	print ajax_constantonoff('REQUESTMANAGER_DISPLAY_DISABLED_FOLLOW_LINK_TO_MY_EVENT_LIST');
} else {
	if (empty($conf->global->REQUESTMANAGER_DISPLAY_DISABLED_FOLLOW_LINK_TO_MY_EVENT_LIST)) {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_DISABLED_FOLLOW_LINK_TO_MY_EVENT_LIST&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
	} else {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_DISABLED_FOLLOW_LINK_TO_MY_EVENT_LIST&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
	}
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_DISPLAY_STATUS_THREAD
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayStatusThreadName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayStatusThreadDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
	print ajax_constantonoff('REQUESTMANAGER_DISPLAY_STATUS_THREAD');
} else {
	if (empty($conf->global->REQUESTMANAGER_DISPLAY_STATUS_THREAD)) {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_STATUS_THREAD&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
	} else {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_STATUS_THREAD&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
	}
}
print '</td></tr>' . "\n";

print '</table>';

/********************************************************
 *  Extrafields
 ********************************************************/
print '<div id="extrafields_options"></div>';
print load_fiche_titre($langs->trans("RequestManagerExtraFieldsOptions"),'','');

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayExtraFieldsActivateName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayExtraFieldsActivateDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE');
} else {
    if (empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerDisplayExtraFieldsOnRequestChildrenName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDisplayExtraFieldsOnRequestChildrenDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
	print ajax_constantonoff('REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN');
} else {
	if (empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN)) {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
	} else {
		print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
	}
}
print '</td></tr>' . "\n";

print '</table>';

/********************************************************
 *  Time slots
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerTimeSlotsOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_timeslots_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_TIMESLOTS_ACTIVATE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerTimeSlotsActivateName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerTimeSlotsActivateDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_TIMESLOTS_ACTIVATE');
} else {
    if (empty($conf->global->REQUESTMANAGER_TIMESLOTS_ACTIVATE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_TIMESLOTS_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_TIMESLOTS_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_TIMESLOTS_PERIODS
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>' . $langs->trans("RequestManagerTimeSlotsPeriodsName") . '</td>'."\n";
print '<td>' . $langs->trans("RequestManagerTimeSlotsPeriodsDesc") . '</td>'."\n";
print '<td align="right" width="50%">'."\n";
$periods = GETPOST('REQUESTMANAGER_TIMESLOTS_PERIODS', "alpha");
print '<textarea style="width: 100%;" rows="15" name="REQUESTMANAGER_TIMESLOTS_PERIODS">'.(!empty($periods) ? $periods : $conf->global->REQUESTMANAGER_TIMESLOTS_PERIODS).'</textarea>';
print '</td></tr>'."\n";

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>' . "\n";

/********************************************************
 *  Planning
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerPlanningOptions"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set_planning_options">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_PLANNING_ACTIVATE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerPlanningActivateName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerPlanningActivateDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_PLANNING_ACTIVATE');
} else {
    if (empty($conf->global->REQUESTMANAGER_PLANNING_ACTIVATE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_PLANNING_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_PLANNING_ACTIVATE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

$request_types_planned = !empty($conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) ? explode(',', $conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) : array();

// REQUESTMANAGER_PLANNING_REQUEST_TYPE
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>' . $langs->trans("RequestManagerPlanningRequestTypesName") . '</td>'."\n";
print '<td>' . $langs->trans("RequestManagerPlanningRequestTypesDesc") . '</td>'."\n";
print '<td align="right">'."\n";
print $form->multiselectarray("REQUESTMANAGER_PLANNING_REQUEST_TYPE", $request_types_array, $request_types_planned, 0, 0, 'minwidth300');
print '</td></tr>'."\n";

foreach ($request_types as $request_type) {
    if (!in_array($request_type->id, $request_types_planned)) continue;

    $request_status_to_plan_array = array();
    $request_status_planned_array = array();

    // Get status to plan for the request type
    foreach ($request_status as $rstatus) {
        if (!in_array($request_type->id, explode(',', $rstatus->fields['request_type']))) continue;

        if (in_array($rstatus->fields['type'], array(RequestManager::STATUS_TYPE_INITIAL, RequestManager::STATUS_TYPE_IN_PROGRESS))) {
            $request_status_to_plan_array[$rstatus->id] = $rstatus->fields['code'] . ' - ' . $rstatus->fields['label'];
            $request_status_planned_array[$rstatus->id] = !empty($rstatus->fields['next_status']) ? explode(',', $rstatus->fields['next_status']) : array();
        }
    }

    // Get status planned for each status to plan for the request type
    foreach ($request_status_planned_array as $rstatus_to_plan_id => $rstatus_to_plan) {
        $status_planned = array();
        foreach ($rstatus_to_plan as $rstatus_planned_id) {
            if (isset($request_status[$rstatus_planned_id])) {
                $status_planned[$request_status[$rstatus_planned_id]->id] = $request_status[$rstatus_planned_id]->fields['code'] . ' - ' . $request_status[$rstatus_planned_id]->fields['label'];
            }
        }
        $request_status_planned_array[$rstatus_to_plan_id] = $status_planned;
    }

    // REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN
    $var = !$var;
    print '<tr ' . $bc[$var] . '>' . "\n";
    print '<td rowspan="2">' . $langs->trans("RequestManagerPlanningRequestStatusName", $request_type->fields['label']) . '</td>' . "\n";
    print '<td>' . $langs->trans("RequestManagerPlanningRequestStatusToPlanDesc") . '</td>' . "\n";
    print '<td align="right">' . "\n";
    print $form->selectarray("REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_" . $request_type->id, $request_status_to_plan_array, $conf->global->{'REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_' . $request_type->id}, 1, 0, 0, '',0, 0, 0, '', 'minwidth300');
    print '</td></tr>' . "\n";

    // REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED
    $var = !$var;
    print '<tr ' . $bc[$var] . '>' . "\n";
    print '<td>' . $langs->trans("RequestManagerPlanningRequestStatusPlannedDesc") . '</td>' . "\n";
    print '<td align="right">' . "\n";
    $request_status_planned_selected_array = isset($request_status_planned_array[$conf->global->{'REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_' . $request_type->id}]) ? $request_status_planned_array[$conf->global->{'REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_' . $request_type->id}] : array();
    print $form->selectarray("REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED_" . $request_type->id, $request_status_planned_selected_array, $conf->global->{'REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED_' . $request_type->id}, 1, 0, 0, '',0, 0, 0, '', 'minwidth300');
    print '</td></tr>' . "\n";

    $request_status_planned_array = json_encode($request_status_planned_array);
    print <<<SCRIPT
    <script type="text/javascript">
        $(document).ready(function () {
            var rm_request_status_planned_array_{$request_type->id} = $request_status_planned_array;
            var rm_planning_request_status_to_plan_{$request_type->id} = $('#REQUESTMANAGER_PLANNING_REQUEST_STATUS_TO_PLAN_{$request_type->id}');
            var rm_planning_request_status_planned_{$request_type->id} = $('#REQUESTMANAGER_PLANNING_REQUEST_STATUS_PLANNED_{$request_type->id}');

            rm_planning_request_status_to_plan_{$request_type->id}.on('change', function() {
                var selected_id = rm_planning_request_status_to_plan_{$request_type->id}.val();
              
                rm_planning_request_status_planned_{$request_type->id}.empty();
                rm_planning_request_status_planned_{$request_type->id}.append('<option value="-1">&nbsp;</option>');
                if (selected_id in rm_request_status_planned_array_{$request_type->id}) {
                    $.map(rm_request_status_planned_array_{$request_type->id}[selected_id], function(item, idx) {
                   	    rm_planning_request_status_planned_{$request_type->id}.append('<option value="' + idx + '">' + item + '</option>');
                    })
                }
            });
        });
    </script>
SCRIPT;
}

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>' . "\n";

/********************************************************
 *  General options
 ********************************************************/
print load_fiche_titre($langs->trans("Other"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="set">';

$var=true;

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td width="20%">'.$langs->trans("Name").'</td>'."\n";
print '<td>'.$langs->trans("Description").'</td>'."\n";
print '<td align="right">'.$langs->trans("Value").'</td>'."\n";
print "</tr>\n";

// REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_PRINCIPAL_COMPANY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerAutoAddContractOfPrincipalCompanyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerAutoAddContractOfPrincipalCompanyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_PRINCIPAL_COMPANY');
} else {
    if (empty($conf->global->REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_PRINCIPAL_COMPANY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_PRINCIPAL_COMPANY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_PRINCIPAL_COMPANY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_BENEFICIAL_COMPANY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerAutoAddContractOfBeneficialCompanyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerAutoAddContractOfBeneficialCompanyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_BENEFICIAL_COMPANY');
} else {
    if (empty($conf->global->REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_BENEFICIAL_COMPANY)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_BENEFICIAL_COMPANY&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_AUTO_ADD_CONTRACT_OF_BENEFICIAL_COMPANY&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerContractSearchInParentCompanyName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerContractSearchInParentCompanyDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (empty($conf->global->REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY)) {
    print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY&token=' . newToken() . '&REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY=1">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
} else {
    print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY&token=' . newToken() . '&REQUESTMANAGER_CONTRACT_SEARCH_IN_PARENT_COMPANY=0">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
}
print '</td></tr>' . "\n";

//// REQUESTMANAGER_OPERATION_TIME_DEFAULT
//$var=!$var;
//print '<tr '.$bc[$var].'>'."\n";
//print '<td>' . $langs->trans("RequestManagerOperationTimeDefaultName") . '</td>'."\n";
//print '<td>' . $langs->trans("RequestManagerOperationTimeDefaultDesc") . '</td>'."\n";
//print '<td align="right">'."\n";
//print '<input type="number" name="REQUESTMANAGER_OPERATION_TIME_DEFAULT" min="0" value="' . intval($conf->global->REQUESTMANAGER_OPERATION_TIME_DEFAULT) . '">';
//print ' ' . $langs->trans("Minutes");
//print '</td></tr>'."\n";
//
//// REQUESTMANAGER_DEADLINE_TIME_DEFAULT
//$var=!$var;
//print '<tr '.$bc[$var].'>'."\n";
//print '<td>' . $langs->trans("RequestManagerDeadlineTimeDefaultName") . '</td>'."\n";
//print '<td>' . $langs->trans("RequestManagerDeadlineTimeDefaultDesc") . '</td>'."\n";
//print '<td align="right">'."\n";
//print '<input type="number" name="REQUESTMANAGER_DEADLINE_TIME_DEFAULT" min="0" value="' . intval($conf->global->REQUESTMANAGER_DEADLINE_TIME_DEFAULT) . '">';
//print ' ' . $langs->trans("Minutes");
//print '</td></tr>'."\n";

// REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES
$var=!$var;
print '<tr '.$bc[$var].'>'."\n";
print '<td>' . $langs->trans("RequestManagerRootProductCategoriesName") . '</td>'."\n";
print '<td>' . $langs->trans("RequestManagerRootProductCategoriesDesc") . '</td>'."\n";
print '<td align="right">'."\n";
print $form->select_all_categories('product', (!empty($conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES) ? $conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES : ''), "REQUESTMANAGER_ROOT_PRODUCT_CATEGORIES");
print '</td></tr>'."\n";

// REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerRootProductCategoryIncludeName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerRootProductCategoryIncludeDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE');
} else {
    if (empty($conf->global->REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_ROOT_PRODUCT_CATEGORY_INCLUDE&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerTitleToRafCustomerWhenCreateOtherElementName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerTitleToRafCustomerWhenCreateOtherElementDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT');
} else {
    if (empty($conf->global->REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

// REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST
$var = !$var;
print '<tr ' . $bc[$var] . '>' . "\n";
print '<td>'.$langs->trans("RequestManagerShowChildrenRequestStatusInListName").'</td>'."\n";
print '<td>'.$langs->trans("RequestManagerShowChildrenRequestStatusInListDesc").'</td>'."\n";
print '<td align="right">' . "\n";
if (!empty($conf->use_javascript_ajax)) {
    print ajax_constantonoff('REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST');
} else {
    if (empty($conf->global->REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST)) {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=set_REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST&token=' . newToken() . '">' . img_picto($langs->trans("Disabled"), 'switch_off') . '</a>';
    } else {
        print '<a href="' . $_SERVER['PHP_SELF'] . '?action=del_REQUESTMANAGER_SHOW_CHILDREN_REQUEST_STATUS_IN_LIST&token=' . newToken() . '">' . img_picto($langs->trans("Enabled"), 'switch_on') . '</a>';
    }
}
print '</td></tr>' . "\n";

print '</table>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("Modify").'">';
print '</div>';

print '</form>';

print dol_get_fiche_end();

llxFooter();

$db->close();
