<?php
/* Copyright (C) 2007-2015 Laurent Destailleur  <eldy@users.sourceforge.net>
 * Copyright (C) 2018      Open-DSI             <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	    \file       htdocs/requestmanager/admin/tools.php
 *		\ingroup    requestmanager
 *		\brief      Page about of requestmanager module
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../main.inc.php")) $res=@include '../../../main.inc.php';		// to work if your module directory is into a subdir of root htdocs directory
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/lib/admin.lib.php';
dol_include_once('/requestmanager/lib/requestmanager.lib.php');
dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
dol_include_once('/advancedictionaries/class/dictionary.class.php');

$langs->load("admin");
$langs->load("advancedictionaries@advancedictionaries");
$langs->load("requestmanager@requestmanager");
$langs->load("opendsi@requestmanager");

if (!$user->admin) accessforbidden();

$action = GETPOST('action','alpha');

/*
 *	Actions
 */

$errors = [];
$error = 0;

if ($action == 'duplicate_request_type_parameters') {
    $request_type = GETPOST('request_type', "int");
    $new_code = GETPOST('new_code', "alpha");
    $new_name = GETPOST('new_name', "alpha");
    $duplicate_status = GETPOST('duplicate_status', "int") == 1;
    $duplicate_knowledge_base = GETPOST('duplicate_knowledge_base', "int") == 1;
    $duplicate_message_template = GETPOST('duplicate_message_template', "int") == 1;
    $duplicate_notify_template = GETPOST('duplicate_notify_template', "int") == 1;
    $duplicate_extrafields = GETPOST('duplicate_extrafields', "int") == 1 && $duplicate_status;
    $duplicate = GETPOST('duplicate', "int") == 1;

    if (!($request_type > 0)) {
        $errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("RequestManagerRequestType"));
        $error++;
    }

    if (empty($new_code)) {
        $errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("Code"));
        $error++;
    }

    if (empty($new_name)) {
        $errors[] = $langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("RequestManagerNewRequestTypeName"));
        $error++;
    }

    $db->begin();

    // Create new request type
    //-------------------------------
    $new_request_type_id = 0;
    if (!$error) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerrequesttype');
        $dictionaryLine = $dictionary->getNewDictionaryLine();
        $result = $dictionaryLine->fetch($request_type);
        if ($result > 0) {
            $fields = array();
            foreach ($dictionary->fields as $k => $v) $fields[$k] = $dictionaryLine->fields[$k];
            $fields['code'] = $new_code;
            $fields['label'] = $new_name;

            $result = $dictionaryLine->insert($fields, $user);
            if ($result > 0) {
                $new_request_type_id = $dictionaryLine->id;
            } else {
                $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewRequestType");
                $errors = array_merge($errors, $dictionaryLine->errors);
                $error++;
            }
        } else {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfRequestType");
            if ($result < 0) $errors = array_merge($errors, $dictionaryLine->errors);
            else $errors[] = $langs->trans("RequestManagerErrorNoDataFound");
            $error++;
        }
    }

    // Duplicate status
    //-------------------------------
    $new_status_ids = array();
    if (!$error && $duplicate_status) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
        $lines = $dictionary->fetch_lines(-1, array('request_type' => array($request_type)), array(), 0, 0, false, true);
        if (is_array($lines)) {
            if ($duplicate) {
                $all_lines = $dictionary->fetch_lines(-1, array(), array(), 0, 0, false, true);
                if (is_array($all_lines)) {
                    $new_lines = array();
                    foreach ($lines as $line) {
                        $fields = array();
                        foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];
                        $max_num = 0;
                        foreach ($all_lines as $l) if (preg_match('/^' . preg_quote($fields['code']) . '_(\d+)$/', $l->fields['code'], $matches)) $max_num = max($max_num, $matches[1]);
                        $fields['code'] = $fields['code'] . '_' . $max_num;
                        $fields['request_type'] = $new_request_type_id;
                        $old_id = $line->id;

                        $result = $line->insert($fields, $user);
                        if ($result <= 0) {
                            $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewStatus", $line->fields['code'] . ' - ' . $line->fields['label']);
                            $errors = array_merge($errors, $line->errors);
                            $error++;
                            break;
                        }

                        $new_lines[] = $line;
                        $new_status_ids[$old_id] = $line->id;
                    }

                    // Update of the ids of all related status fields
                    foreach ($new_lines as $line) {
                        $fields = array();
                        foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];

                        $current_request_status = explode(',', $line->fields['next_status']);
                        $fields['next_status'] = array();
                        if (is_array($current_request_status)) foreach ($current_request_status as $v) if (isset($new_status_ids[$v])) $fields['next_status'][] = $new_status_ids[$v];
                        $fields['next_status'] = implode(',', $fields['next_status']);

                        $current_new_request_type = explode(',', $line->fields['new_request_type']);
                        $fields['new_request_type'] = array();
                        if (is_array($current_new_request_type)) foreach ($current_new_request_type as $v) if ($v == $request_type) $fields['new_request_type'][] = $new_request_type_id; else $fields['new_request_type'][] = $v;
                        $fields['new_request_type'] = implode(',', $fields['new_request_type']);

                        $result = $line->update($fields, $user);
                        if ($result <= 0) {
                            $errors[] = $langs->trans("RequestManagerErrorWhileUpdateNewStatus", $line->fields['code'] . ' - ' . $line->fields['label']);
                            $errors = array_merge($errors, $line->errors);
                            $error++;
                            break;
                        }
                    }
                } else {
                    $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveAllCurrentDataOfStatus");
                    $errors = array_merge($errors, $dictionary->errors);
                    $error++;
                }
            } else {
                foreach ($lines as $line) {
                    $fields = array();
                    foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];
                    $fields['request_type'] = implode(',', array_merge(explode(',', $fields['request_type']), array($new_request_type_id)));

                    $result = $line->update($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileUpdateStatus", $line->fields['code'] . ' - ' . $line->fields['label']);
                        $errors = array_merge($errors, $line->errors);
                        $error++;
                        break;
                    }

                    $new_status_ids[$line->id] = $line->id;
                }
            }
        } else {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfStatus");
            $errors = array_merge($errors, $dictionary->errors);
            $error++;
        }
    }

    // Duplicate knowledge base
    //-------------------------------
    if (!$error && $duplicate_knowledge_base) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerknowledgebase');
        $lines = $dictionary->fetch_lines(-1, array('request_type' => array($request_type)), array(), 0, 0, false, true);
        if (is_array($lines)) {
            if ($duplicate) {
                $all_lines = $dictionary->fetch_lines(-1, array(), array(), 0, 0, false, true);
                if (is_array($all_lines)) {
                    foreach ($lines as $line) {
                        $fields = array();
                        foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];
                        $max_num = 0;
                        foreach ($all_lines as $l) if (preg_match('/^' . preg_quote($fields['code']) . '_(\d+)$/', $l->fields['code'], $matches)) $max_num = max($max_num, $matches[1]);
                        $fields['code'] = $fields['code'] . '_' . $max_num;
                        $fields['request_type'] = $new_request_type_id;

                        $result = $line->insert($fields, $user);
                        if ($result <= 0) {
                            $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewKnowledgeBase", $line->fields['code'] . ' - ' . $line->fields['label']);
                            $errors = array_merge($errors, $line->errors);
                            $error++;
                            break;
                        }
                    }
                } else {
                    $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveAllCurrentDataOfKnowledgeBase");
                    $errors = array_merge($errors, $dictionary->errors);
                    $error++;
                }
            } else {
                foreach ($lines as $line) {
                    $fields = array();
                    foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];
                    $fields['request_type'] = implode(',', array_merge(explode(',', $fields['request_type']), array($new_request_type_id)));

                    $result = $line->update($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileUpdateKnowledgeBase", $line->fields['code'] . ' - ' . $line->fields['label']);
                        $errors = array_merge($errors, $line->errors);
                        $error++;
                        break;
                    }
                }
            }
        } else {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfKnowledgeBase");
            $errors = array_merge($errors, $dictionary->errors);
            $error++;
        }
    }

    // Duplicate message template
    //-------------------------------
    if (!$error && $duplicate_message_template) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagermessagetemplate');
        $result = $dictionary->fetch_lines(-1, array('request_type' => array($request_type)));
        if ($result < 0) {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfMessageTemplate");
            $errors = array_merge($errors, $dictionary->errors);
            $error++;
        } else {
            foreach ($dictionary->lines as $line) {
                $fields = array();
                foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];

                if ($duplicate) {
                    $fields['request_type'] = $new_request_type_id;
                    $result = $line->insert($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewMessageTemplate", $line->fields['label']);
                    }
                } else {
                    $fields['request_type'] = implode(',', array_merge(explode(',', $fields['request_type']), array($new_request_type_id)));
                    $result = $line->update($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileUpdateMessageTemplate", $line->fields['label']);
                    }
                }

                if ($result <= 0) {
                    $errors = array_merge($errors, $line->errors);
                    $error++;
                    break;
                }
            }
        }
    }

    // Duplicate notify template
    //-------------------------------
    if (!$error && $duplicate_notify_template) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagernotifytemplate');
        $result = $dictionary->fetch_lines(-1, array('request_type' => array($request_type)));
        if ($result < 0) {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfNotifyTemplate");
            $errors = array_merge($errors, $dictionary->errors);
            $error++;
        } else {
            foreach ($dictionary->lines as $line) {
                $fields = array();
                foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];

                if ($duplicate) {
                    $fields['request_type'] = $new_request_type_id;
                    $result = $line->insert($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewNotifyTemplate", $line->id);
                    }
                } else {
                    $fields['request_type'] = implode(',', array_merge(explode(',', $fields['request_type']), array($new_request_type_id)));
                    $result = $line->update($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileUpdateNotifyTemplate", $line->id);
                    }
                }

                if ($result <= 0) {
                    $errors = array_merge($errors, $line->errors);
                    $error++;
                    break;
                }
            }
        }
    }

    // Duplicate extrafields
    //-------------------------------
    if (!$error && $duplicate_extrafields) {
        $dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerextrafields');
        $result = $dictionary->fetch_lines(-1, array('request_type' => array($request_type)));
        if ($result < 0) {
            $errors[] = $langs->trans("RequestManagerErrorWhileRetrieveCurrentDataOfExtrafields");
            $errors = array_merge($errors, $dictionary->errors);
            $error++;
        } else {
            foreach ($dictionary->lines as $line) {
                $fields = array();
                foreach ($dictionary->fields as $k => $v) $fields[$k] = $line->fields[$k];

                $current_request_status = explode(',', $line->fields['request_status']);
                $fields['request_status'] = array();
                if (is_array($current_request_status)) foreach ($current_request_status as $v) if (isset($new_status_ids[$v])) $fields['request_status'][] = $new_status_ids[$v];
                $fields['request_status'] = implode(',', $fields['request_status']);

                if ($duplicate) {
                    $fields['request_type'] = $new_request_type_id;
                    $result = $line->insert($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileCreateNewExtrafields", $line->id);
                    }
                } else {
                    $fields['request_type'] = implode(',', array_merge(explode(',', $fields['request_type']), array($new_request_type_id)));
                    $result = $line->update($fields, $user);
                    if ($result <= 0) {
                        $errors[] = $langs->trans("RequestManagerErrorWhileUpdateExtrafields", $line->id);
                    }
                }

                if ($result <= 0) {
                    $errors = array_merge($errors, $line->errors);
                    $error++;
                    break;
                }
            }
        }
    }

    if ($error) {
        $db->rollback();
    } else {
        $db->commit();
        setEventMessage($langs->trans("RequestManagerRequestTypeDuplicated"));
        header("Location: " . $_SERVER['PHP_SELF']);
        exit;
    }
}

if ($action != '' && $error) {
    setEventMessages('', $errors, 'errors');
}

/**
 * View
 */

$wikihelp='EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es';
llxHeader('', $langs->trans("RequestManagerSetup"), $wikihelp);

$form = new Form($db);
$formrequestmanager = new FormRequestManager($db);

$linkback='<a href="'.DOL_URL_ROOT.'/admin/modules.php">'.$langs->trans("BackToModuleList").'</a>';
print load_fiche_titre($langs->trans("RequestManagerSetup"),$linkback,'title_setup');
print "<br>\n";


$head=requestmanager_admin_prepare_head();

print dol_get_fiche_head($head, 'tools', $langs->trans("Module163018Name"), 0, 'opendsi@requestmanager');

/********************************************************
 *  Duplicate Request Type parameters
 ********************************************************/
print load_fiche_titre($langs->trans("RequestManagerDuplicateRequestTypeParametersTools"),'','');

print '<form method="post" action="'.$_SERVER["PHP_SELF"].'">';
print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
print '<input type="hidden" name="action" value="duplicate_request_type_parameters">';

print '<table class="noborder" width="100%">';
print '<tr class="liste_titre">';
print '<td class="fieldrequired">'.$langs->trans("RequestManagerRequestType").' *</td>'."\n";
print '<td class="fieldrequired">'.$langs->trans("Code").' *</td>'."\n";
print '<td class="fieldrequired">'.$langs->trans("RequestManagerNewRequestTypeName").' *</td>'."\n";
print '<td>'.$langs->trans("RequestManagerDuplicateData").'</td>'."\n";
print '<td width="10%" align="center">'.$form->textwithpicto($langs->trans("RequestManagerDuplicate"), $langs->trans('RequestManagerDuplicateHelp')).'</td>'."\n";
print "</tr>\n";

print '<tr>' . "\n";
print '<td>';
print $formrequestmanager->select_type(null, GETPOST('request_type'), 'request_type', 1, 0, array(), 0, 0, 'minwidth300');
print '</td>' . "\n" . '<td>';
print '<input type="text" size="30" name="new_code" value="' . htmlspecialchars(GETPOST('new_code')) . '">';
print '</td>' . "\n" . '<td>';
print '<input type="text" size="100" name="new_name" value="' . htmlspecialchars(GETPOST('new_name')) . '">';
print '</td>' . "\n" . '<td>';
print '<input type="checkbox" id="duplicate_status" name="duplicate_status" value="1"' . (GETPOST('duplicate_status') == 1 ? ' checked="checked"' : '') . '"><label for="duplicate_status"> ' . $langs->trans("RequestManagerStatusDictionaryLabel") . '</label><br>';
print '<input type="checkbox" id="duplicate_knowledge_base" name="duplicate_knowledge_base" value="1"' . (GETPOST('duplicate_knowledge_base') == 1 ? ' checked="checked"' : '') . '><label for="duplicate_knowledge_base"> ' . $langs->trans("RequestManagerKnowledgeBaseDictionaryLabel") . '</label><br>';
print '<input type="checkbox" id="duplicate_message_template" name="duplicate_message_template" value="1"' . (GETPOST('duplicate_message_template') == 1 ? ' checked="checked"' : '') . '><label for="duplicate_message_template"> ' . $langs->trans("RequestManagerMessageTemplateDictionaryLabel") . '</label><br>';
print '<input type="checkbox" id="duplicate_notify_template" name="duplicate_notify_template" value="1"' . (GETPOST('duplicate_notify_template') == 1 ? ' checked="checked"' : '') . '><label for="duplicate_notify_template"> ' . $langs->trans("RequestManagerNotifyTemplateDictionaryLabel") . '</label><br>';
print '<input type="checkbox" id="duplicate_extrafields" name="duplicate_extrafields" value="1"' . (GETPOST('duplicate_status') == 1 && GETPOST('duplicate_extrafields') == 1 ? ' checked="checked"' : '') . '' . (GETPOST('duplicate_status') != 1 ? ' disabled="disabled"' : '') . '><label for="duplicate_extrafields"> ' . $langs->trans("RequestManagerExtraFieldsDictionaryLabel") . '</label><br>';
print '</td>' . "\n" . '<td align="center">' . "\n";
print '<input type="checkbox" name="duplicate" value="1"' . (GETPOST('duplicate') == 1 ? ' checked="checked"' : '') . '>';
print '</td></tr>' . "\n";

print '</table>';
print '<i>* ' . $langs->trans("AdvanceDictionariesFieldRequired") . '</i>';

print '<br>';
print '<div align="center">';
print '<input type="submit" class="button" value="'.$langs->trans("RequestManagerDuplicate").'">';
print '</div>';

print '<script type="text/javascript" language="javascript">
    $(document).ready(function() {
        $("#duplicate_status").on("click", function() {
            if (!$("#duplicate_status").is(":checked")) {
                $("#duplicate_extrafields").prop("checked", false).prop("disabled", true);
            } else {
                $("#duplicate_extrafields").prop("disabled", false);
            }
        });
    });
</script>' . "\n";

print '</form>' . "\n";

print dol_get_fiche_end();


llxFooter();

$db->close();
