<?php
/* Copyright (C) 2018      Open-DSI              <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file 		htdocs/requestmanager/card.php
 * \ingroup 	requestmanager
 * \brief 		Page of Request card
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../main.inc.php")) $res=@include '../main.inc.php';					// to work if your module directory is into dolibarr root htdocs directory
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../dolibarr/htdocs/main.inc.php';     // Used on dev env only
if (! $res && file_exists("../../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../../dolibarr/htdocs/main.inc.php';   // Used on dev env only
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT . '/core/class/html.formother.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/html.formfile.class.php';
require_once DOL_DOCUMENT_ROOT . '/comm/action/class/actioncomm.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/functions2.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/lib/date.lib.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
require_once DOL_DOCUMENT_ROOT . '/core/class/doleditor.class.php';
require_once DOL_DOCUMENT_ROOT . '/user/class/user.class.php';
require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';
require_once DOL_DOCUMENT_ROOT . '/contact/class/contact.class.php';

if (isModEnabled('categorie')) {
    require_once DOL_DOCUMENT_ROOT . '/categories/class/categorie.class.php';
    dol_include_once('/requestmanager/class/categorierequestmanager.class.php');
}

dol_include_once('/advancedictionaries/class/dictionary.class.php');
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/requestmanager/class/requestmanagermessage.class.php');
dol_include_once('/requestmanager/class/requestmanagernotification.class.php');
dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
dol_include_once('/requestmanager/class/html.formrequestmanagermessage.class.php');
dol_include_once('/requestmanager/lib/requestmanager.lib.php');

$langs->load('mails');
$langs->load('requestmanager@requestmanager');
$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');
$v18p = version_compare(DOL_VERSION, '18.0.0', '>=');

$error = 0;

$id = GETPOST('id', 'int');
$ref = GETPOST('ref', 'alpha');
$action = GETPOST('action', 'aZ09');
$cancel = GETPOST('cancel', 'alpha');
$confirm = GETPOST('confirm', 'alpha');
// lines
$lineid = GETPOST('lineid', 'int');
//links
$addlink = GETPOST('addlink', 'alpha');
$addlinkid = GETPOST('idtolinkto', 'int');
$dellinkid = GETPOST('dellinkid', 'int');

// Security check
$result = restrictedArea($user, 'requestmanager', $id);

$object = new RequestManager($db);
$extrafields = new ExtraFields($db);

// fetch optionals attributes and labels
$extralabels = $extrafields->fetch_name_optionals_label($object->table_element);

// Load object
if ($id > 0 || !empty($ref)) {
	$ret = $object->fetch($id, $ref);
    if ($ret > 0) {
        $object->fetch_thirdparty();
        $object->fetch_optionals();
		$object->loadRequestStatus();
		if (isset(RequestManager::$status_list[$object->statut])) {
			$requestManagerStatusDictionaryLine = RequestManager::$status_list[$object->statut];
		} else {
			$requestManagerStatusDictionaryLine = Dictionary::getDictionaryLine($db, 'requestmanager', 'requestmanagerstatus');
		}
		$authorizedButtons = !empty($requestManagerStatusDictionaryLine->fields['authorized_buttons']) ? explode(',', $requestManagerStatusDictionaryLine->fields['authorized_buttons']) : array();
		//$mysoc = $object->socid;
    } elseif ($ret < 0) {
        dol_print_error('', $object->errorsToString());
    } else {
        print $langs->trans('NoRecordFound');
        exit();
    }
}

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$hookmanager->initHooks(array('requestmanagercard','globalcard'));

$permissiondellink = $user->rights->requestmanager->creer; 	// Used by the include of actions_dellink.inc.php
$permissionedittimespent = $object->isRequestTypeOption('manage_times') && $object->isRequestTypeOption('display_set_times') &&
    $user->rights->requestmanager->lire && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) &&
	($user->rights->requestmanager->timesspent->manage || in_array($user->id, $object->assigned_user_ids));

$v18p = version_compare(DOL_VERSION, '18.0.0', '>=');


/*
 * Actions
 */

$parameters = array();
$reshook = $hookmanager->executeHooks('doActions', $parameters, $object, $action); // Note that $action and $object may have been modified by some hooks
if ($reshook < 0) setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');

if (empty($reshook)) {
	if (!empty($cancel)) $action = '';
	if (!empty($cancel) && !empty($backtopage)) {
		header('Location: ' . $backtopage);
		exit();
	}

	if ($v18p) {
		include DOL_DOCUMENT_ROOT . '/core/actions_dellink.inc.php'; // Must be include, not include_once
	} else {
		/*********************************************************
		 * Links management
		 *********************************************************/
		// Add link
		if ($action == 'addlink' && !empty($permissiondellink) && !GETPOST('cancel', 'alpha') && $object->id > 0 && $addlinkid > 0 && $result > 0) {
			$object->oldcopy = clone $object;
			$result = $object->add_object_linked($addlink, $addlinkid);
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
				$error++;
			} else {
				// Call trigger
				$object->context = array('addlink' => $addlink, 'addlinkid' => $addlinkid);
				$result = $object->call_trigger('REQUESTMANAGER_ADD_LINK', $user);
				if ($result < 0) $error++;
				// End call trigger
			}
		}
		// Delete link
		elseif ($action == 'dellink' && !empty($permissiondellink) && !GETPOST('cancel', 'alpha') && $object->id > 0 && $dellinkid > 0 && $result > 0) {
			$object->oldcopy = clone $object;
			$result = $object->deleteObjectLinked(0, '', 0, '', $dellinkid);
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
				$error++;
			} else {
				// Call trigger
				$object->context = array('dellink' => $dellinkid);
				$result = $object->call_trigger('REQUESTMANAGER_DEL_LINK', $user);
				if ($result < 0) $error++;
				// End call trigger
			}
		}
	}

	// Create request
	if ($action == 'add' && $user->rights->requestmanager->creer) {
		$origin = GETPOST('origin', 'alpha');
		$originid = GETPOST('originid', 'int');

		$object->fk_type = GETPOSTINT('type');
		$object->fk_category = GETPOSTINT('category');
		$object->socid = GETPOSTINT('socid');
		$object->requester_ids = GETPOST('requester_ids', 'array');
		$object->fk_source = GETPOSTINT('source');
		$object->fk_urgency = GETPOSTINT('urgency');
		$object->fk_impact = GETPOSTINT('impact');
		$object->fk_priority = GETPOSTINT('priority');
		$object->estimated_duration = GETPOSTINT('estimated_durationhour') * 3600 + GETPOSTINT('estimated_durationmin') * 60;    // We store duration in seconds
		$object->date_operation = dol_mktime(GETPOSTINT('operation_hour'), GETPOSTINT('operation_min'), 0, GETPOSTINT('operation_month'), GETPOSTINT('operation_day'), GETPOSTINT('operation_year'));
		$object->date_deadline = dol_mktime(GETPOSTINT('deadline_hour'), GETPOSTINT('deadline_min'), 0, GETPOSTINT('deadline_month'), GETPOSTINT('deadline_day'), GETPOSTINT('deadline_year'));
		$object->assigned_usergroup_ids = GETPOST('assigned_usergroups', 'array');
		$object->assigned_user_ids = GETPOST('assigned_users', 'array');
		$object->watcher_ids = GETPOST('contact_watcher_ids', 'array');
		$object->tag_ids = GETPOST('categories', 'array');
		$object->notify_requester_by_email = GETPOSTINT('notify_requester_by_email');
		$object->notify_assigned_by_email = GETPOSTINT('notify_assigned_by_email');
		$object->notify_watcher_by_email = GETPOSTINT('notify_watcher_by_email');
		$object->label = GETPOST('label', 'alpha');
		$object->description = GETPOST('description', 'restricthtml');
		// Fill array 'array_options' with data from add form
		$ret = $extrafields->setOptionalsFromPost($extralabels, $object);
		if ($ret < 0) $error++;

		// If creation from another object of another module (Example: origin=propal, originid=1)
		if (!$error && !empty($origin) && !empty($originid)) {
			$object->origin = $origin;
			$object->origin_id = $originid;
			$object->linkedObjectsIds[$object->origin] = $object->origin_id;
		}

		// Possibility to add external linked objects with hooks
		$other_linked_objects = GETPOST('other_linked_objects', 'array');
		if (!empty($other_linked_objects)) {
			if (!is_array($object->linkedObjectsIds)) $object->linkedObjectsIds = array();
			$object->linkedObjectsIds = array_merge($object->linkedObjectsIds, $other_linked_objects);
		}
		$children_request = GETPOST('children_request', 'array');
		if (!empty($children_request)) {
			if (!is_array($object->children_request_ids)) $object->children_request_ids = array();
			$object->children_request_ids = array_merge($object->children_request_ids, $children_request);
		}

		$db->begin();

		if (!$error) {
			$object_id = $object->create($user);
			if ($object_id < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
				$error++;
			}
		}

		if (!$error && !empty($origin) && !empty($originid)) {
			$element_prop = getElementProperties($origin);
			dol_include_once('/' . $element_prop['classpath'] . '/' . $element_prop['classfile'] . '.class.php');

			$objectsrc = new $element_prop['classname']($db);
			$result = $objectsrc->fetch($originid);
			if ($result < 0) {
				setEventMessages($objectsrc->error, $objectsrc->errors, 'errors');
				$error++;
			} elseif ($result == 0) {
				setEventMessage($langs->trans('NoRecordFound') . ' - ' . $langs->trans('Origin') .
					' (' . $langs->trans('Element') . ': ' . $origin . ', ID: ' . $originid . ')', 'errors');
				$error++;
			} else {
				if (empty($objectsrc->lines)) {
					if (method_exists($objectsrc, 'fetch_lines')) {
						$objectsrc->fetch_lines();
					} elseif (method_exists($objectsrc, 'getLinesArray')) {
						if ($objectsrc->element == 'project') {
							$objectsrc->getLinesArray($user);
						} else {
							$objectsrc->getLinesArray();
						}
					}
				}

				$lines = $objectsrc->lines;
				if (!empty($lines) && is_array($lines)) {
					$fk_parent_line = 0;
					$num = count($lines);
					for ($i = 0; $i < $num; $i++) {
						if (empty($lines[$i]->fk_product)) continue;

						$label = (!empty($lines[$i]->label) ? $lines[$i]->label : '');
						$desc = (!empty($lines[$i]->desc) ? $lines[$i]->desc : '');
						$product_type = (!empty($lines[$i]->product_type) ? $lines[$i]->product_type : 0);

						// Dates
						// TODO mutualiser
						$date_start = $lines[$i]->date_debut_prevue;
						if ($lines[$i]->date_debut_reel)
							$date_start = $lines[$i]->date_debut_reel;
						if ($lines[$i]->date_start)
							$date_start = $lines[$i]->date_start;
						$date_end = $lines[$i]->date_fin_prevue;
						if ($lines[$i]->date_fin_reel)
							$date_end = $lines[$i]->date_fin_reel;
						if ($lines[$i]->date_end)
							$date_end = $lines[$i]->date_end;

						// Reset fk_parent_line for no child products and special product
						if (($lines[$i]->product_type != 9 && empty($lines[$i]->fk_parent_line)) || $lines[$i]->product_type == 9) {
							$fk_parent_line = 0;
						}

						// Extrafields
						if (empty($conf->global->MAIN_EXTRAFIELDS_DISABLED) && method_exists($lines[$i], 'fetch_optionals')) // For avoid conflicts if trigger used
						{
							$lines[$i]->fetch_optionals($lines[$i]->rowid);
							$array_options = $lines[$i]->array_options;
						}

						$tva_tx = $lines[$i]->tva_tx;
						if (!empty($lines[$i]->vat_src_code) && !preg_match('/\(/', $tva_tx)) $tva_tx .= ' (' . $lines[$i]->vat_src_code . ')';

						$result = $object->addline(
							$desc, $lines[$i]->subprice, $lines[$i]->qty, $tva_tx, $lines[$i]->localtax1_tx, $lines[$i]->localtax2_tx, $lines[$i]->fk_product,
							$lines[$i]->remise_percent, $lines[$i]->info_bits, $lines[$i]->fk_remise_except, 'HT', 0, $date_start, $date_end, $product_type,
							$lines[$i]->rang, $lines[$i]->special_code, $fk_parent_line, $lines[$i]->fk_fournprice, $lines[$i]->pa_ht, $label, $array_options,
							$lines[$i]->fk_unit, $object->origin, $lines[$i]->rowid, $lines[$i]->multicurrency_subprice
						);

						if ($result < 0) {
							setEventMessage($langs->trans('RequestManagerErrorWhenAddLine', $label, $lines[$i]->fk_product), 'errors');
							setEventMessages($objectsrc->error, $objectsrc->errors, 'errors');
							$error++;
							break;
						}

						// Defined the new fk_parent_line
						if ($result > 0 && $lines[$i]->product_type == 9) {
							$fk_parent_line = $result;
						}
					}
				}
			}

			// Hooks
			$parameters = array('objFrom' => $objectsrc);
			// Note that $action and $object may be modified by hook
			$reshook = $hookmanager->executeHooks('createFrom', $parameters, $object, $action);
			if ($reshook < 0) {
				$error++;
			}
		}

		// End of object creation, we show it
		if (!$error && $object_id > 0) {
			$db->commit();
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object_id);
			exit();
		} else {
			$db->rollback();
			$action = 'create';
		}
	} // Create child request
	elseif ($action == 'confirm_create_request_child' && $confirm == "yes" && $user->rights->requestmanager->creer) {
		$new_request_type = GETPOST('new_request_type', 'int');
		$create_for_each_requester = GETPOST('create_for_each_requester', 'alpha');
		$ids = $object->createSubRequest($new_request_type, $user, $create_for_each_requester);
		if (is_array($ids)) {
			$object_id = count($ids) == 1 ? $ids[0] : $id;
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object_id);
			exit();
		} else {
			$langs->load("errors");
			setEventMessages($object->error, $object->errors, 'errors');
		}
	} // Delete request
	elseif ($action == 'confirm_delete' && $confirm == 'yes' && $user->rights->requestmanager->supprimer) {
		$delete_event_auto = GETPOST('delete_event_auto', 'aZ09');
		$delete_event_manual = GETPOST('delete_event_manual', 'aZ09');
		$result = $object->delete($user, false, !empty($delete_event_auto), !empty($delete_event_manual));
		if ($result > 0) {
			header('Location: ' . dol_buildpath('/requestmanager/list.php', 2));
			exit();
		} else {
			$langs->load("errors");
			setEventMessages($object->error, $object->errors, 'errors');
		}
	} // Set Ref
	elseif ($action == 'set_ref' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL) {
		$new_ref = GETPOST('ref', 'alpha');
		$result = $object->set_ref($user, $new_ref);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_category';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
//    } // Set Type
//    elseif ($action == 'set_type' && $user->rights->requestmanager->creer &&
//        $object->statut_type == RequestManager::STATUS_TYPE_INITIAL
//    ) {
//        $object->oldcopy = clone $object;
//        $object->fk_type = GETPOST('type', 'int');
//        $object->fk_category = 0;
//
//        $db->begin();
//
//        if (!$error) {
//            $result = $object->update($user);
//            if ($result < 0) {
//                setEventMessages($object->error, $object->errors, 'errors');
//                $error++;
//            }
//        }
//
//        if (!$error) {
//            $result = $object->set_status(0, RequestManager::STATUS_TYPE_INITIAL, $user, 0, 1);
//            if ($result < 0) {
//                setEventMessages($object->error, $object->errors, 'errors');
//                $error++;
//            }
//        }
//
//        if (!$error) {
//            $db->commit();
//            header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
//            exit();
//        } else {
//            $db->rollback();
//            $action = 'edit_type';
//        }
	} // Set Category
	elseif ($action == 'set_category' && $user->rights->requestmanager->creer &&
		($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)
	) {
		$object->oldcopy = clone $object;
		$object->fk_category = GETPOST('category', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_category';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Label
	elseif ($action == 'set_label' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->label = GETPOST('label', 'alpha');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_label';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set ThirdParty Origin
	elseif ($action == 'set_thirdparty' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->socid = GETPOST('socid', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_thirdparty';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Source
	elseif ($action == 'set_source' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->fk_source = GETPOST('source', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_source';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Urgency
	elseif ($action == 'set_urgency' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->fk_urgency = GETPOST('urgency', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_urgency';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Impact
	elseif ($action == 'set_impact' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->fk_impact = GETPOST('impact', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_impact';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Priority
	elseif ($action == 'set_priority' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->fk_priority = GETPOST('priority', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_priority';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Date Creation
	elseif ($action == 'set_date_creation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->date_creation = dol_mktime(GETPOST('date_creation_hour', 'int'), GETPOST('date_creation_min', 'int'), 0, GETPOST('date_creation_month', 'int'), GETPOST('date_creation_day', 'int'), GETPOST('date_creation_year', 'int'));
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_date_creation';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Date Operation
	elseif ($action == 'set_date_operation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->date_operation = dol_mktime(GETPOST('operation_hour', 'int'), GETPOST('operation_min', 'int'), 0, GETPOST('operation_month', 'int'), GETPOST('operation_day', 'int'), GETPOST('operation_year', 'int'));

		// calculate deadline date with operation date or now and the offset deadline time in minutes
		if (GETPOST('recalculate_date_deadline', 'int') == 1) {
			if ($requestManagerStatusDictionaryLine->id > 0) {
				$deadline_offset = $requestManagerStatusDictionaryLine->fields['deadline'];
				$now = dol_now();
				if (isset($deadline_offset) && $deadline_offset > 0) {
					$object->date_deadline = ($object->date_operation > 0 ? $object->date_operation : $now) + ($deadline_offset * 60);
//                } elseif (intval($conf->global->REQUESTMANAGER_DEADLINE_TIME_DEFAULT) > 0) {
//                    $object->date_deadline = ($object->date_operation > 0 ? $object->date_operation : $now) + (intval($conf->global->REQUESTMANAGER_DEADLINE_TIME_DEFAULT) * 60);
				}
			} else {
				setEventMessages($requestManagerStatusDictionaryLine->error, $requestManagerStatusDictionaryLine->errors, 'errors');
				$action = 'edit_date_operation';
				$error++;
			}
		}

		if (!$error) {
			$result = $object->update($user);
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
				$action = 'edit_date_operation';
			} else {
				header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
				exit();
			}
		}
	} // Set Date Deadline
	elseif ($action == 'set_date_deadline' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->date_deadline = dol_mktime(GETPOST('deadline_hour', 'int'), GETPOST('deadline_min', 'int'), 0, GETPOST('deadline_month', 'int'), GETPOST('deadline_day', 'int'), GETPOST('deadline_year', 'int'));
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_date_deadline';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Assigned usergroups
	elseif ($action == 'set_assigned_usergroups' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->assigned_usergroup_ids = GETPOST('assigned_usergroups', 'array');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_assigned_usergroups';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Assigned users
	elseif ($action == 'set_assigned_users' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->assigned_user_ids = GETPOST('assigned_users', 'array');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_assigned_users';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set myself in Assigned users
	elseif (($action == 'set_myself_assigned_user' || ($action == 'confirm_set_myself_assigned_user' && $confirm == 'yes')) && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		if (count($object->assigned_user_ids) == 0 || $action == 'confirm_set_myself_assigned_user') {
			$object->assigned_user_ids[] = $user->id;
			$result = $object->update($user);
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
			} else {
				header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
				exit();
			}
		}
	} // Set Assigned Notification
	elseif (!empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL) && $action == 'set_assigned_notification' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->notify_assigned_by_email = GETPOST('assigned_notification', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_assigned_notification';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Description
	elseif ($action == 'set_description' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		$object->description = GETPOST('description', 'restricthtml');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_description';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set categories
	elseif ($action == 'set_categories' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		// Category association
		$object->fetch_tags(1);
		$object->oldcopy = clone $object;
		$categories = GETPOST('categories');
		$result = $object->setCategories($categories);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_categories';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Requesters
	elseif ($action == 'set_requesters' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		//$object->requester_ids = GETPOST('requester_contacts', 'array');
		$object->notify_requester_by_email = GETPOST('requester_notification', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_requesters';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Watchers
	elseif ($action == 'set_watchers' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		//$object->watcher_ids = GETPOST('watcher_contacts', 'array');
		$object->notify_watcher_by_email = GETPOST('watcher_notification', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_watchers';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Status
	elseif ($action == 'set_status' && $user->rights->requestmanager->creer &&
		($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)
	) {
		$status = GETPOST('status', 'int');
		$result = $object->set_status($status, -1, $user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Resolve
	elseif ($action == 'confirm_resolve' && $confirm == 'yes' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$status = GETPOST('status', 'int');
		$reason_resolution = GETPOST('reason_resolution', 'int');
		$reason_resolution_details = GETPOST('reason_resolution_details', 'restricthtml');
		$reason_resolution_details = urldecode($reason_resolution_details);
		$result = $object->set_status($status, RequestManager::STATUS_TYPE_RESOLVED, $user, $reason_resolution, $reason_resolution_details);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Close
	elseif ($action == 'confirm_close' && $confirm == 'yes' && $user->rights->requestmanager->creer && /*$object->statut_type == RequestManager::STATUS_TYPE_RESOLVED &&*/
		empty($conf->global->REQUESTMANAGER_AUTO_CLOSE_REQUEST)
	) {
		$status = GETPOST('status', 'int');
		$result = $object->set_status($status, RequestManager::STATUS_TYPE_CLOSED, $user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set Not Resolved
	elseif ($action == 'confirm_notresolved' && $confirm == 'yes' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_RESOLVED) {
		$selected_status = GETPOST('new_status', 'int');
		$result = $object->set_status($selected_status, -1, $user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Re Open
	elseif ($action == 'confirm_reopen' && $confirm == 'yes' && $user->rights->requestmanager->cloturer && $object->statut_type == RequestManager::STATUS_TYPE_CLOSED) {
		$selected_status = GETPOST('new_status', 'int');
		$result = $object->set_status($selected_status, -1, $user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Create survey
	elseif (isModEnabled('questionnaire') && $action == 'confirm_rm_create_survey' && $confirm == 'yes') {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));
		$result = $object->createSurvey($user, $questionnaire_id, $date_answer_deadline);
		if ($result < 0 || !empty($object->errors)) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Send survey
	elseif (isModEnabled('questionnaire') && $action == 'confirm_rm_send_survey' && $confirm == 'yes') {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$only_new = GETPOST('only_new', 'aZ09');
		$force_date_answer_deadline = GETPOST('force_date_answer_deadline', 'aZ09');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));
		$subject = GETPOST('subject', 'none');
		$message = GETPOST('message', 'none');
		$result = $object->sendSurvey($user, $questionnaire_id, $date_answer_deadline, $subject, $message, !empty($force_date_answer_deadline), !empty($only_new));
		if ($result < 0 || !empty($object->errors)) {
			setEventMessages($object->error, $object->errors, 'errors');
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Extrafields
	elseif ($action == 'update_extras' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		$object->oldcopy = clone $object;
		// Fill array 'array_options' with data from update form
		$ret = $extrafields->setOptionalsFromPost($extralabels, $object, GETPOST('attribute'));
		if ($ret < 0) {
			$error++;
		}
		if (!$error) {
			$result = $object->insertExtraFields();
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
				$error++;
			} else {
				// Call trigger
				$object->context = array('extrafieldaddupdate' => 1);
				$result = $object->call_trigger('REQUESTMANAGER_EXTRAFIELDS_ADD_UPDATE', $user);
				if ($result < 0) $error++;
				// End call trigger
			}
		}
		if ($error) $action = 'edit_extras';
	} // Add a new line
	elseif ($action == 'addline' && $object->socid > 0 && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		require_once DOL_DOCUMENT_ROOT . '/product/class/product.class.php';
		if (isModEnabled('variants')) {
			require_once DOL_DOCUMENT_ROOT . '/variants/class/ProductCombination.class.php';
		}

		$langs->load('errors');
		$error = 0;

		// Set if we used free entry or predefined product
		$predef = '';
		$product_desc = (GETPOST('dp_desc') ? GETPOST('dp_desc') : '');
		if (!$product_desc) {
			$product_desc = GETPOST('product_desc');
		}
		$price_ht = GETPOST('price_ht');
		$price_ht_devise = GETPOST('multicurrency_price_ht');
		$prod_entry_mode = GETPOST('prod_entry_mode');
		if ($prod_entry_mode == 'free') {
			$idprod = 0;
			$tva_tx = (GETPOST('tva_tx') ? GETPOST('tva_tx') : 0);
		} else {
			$idprod = GETPOST('idprod', 'int');
			$tva_tx = '';
		}

		$qty = GETPOST('qty' . $predef);
		$remise_percent = GETPOST('remise_percent' . $predef);

		// Extrafields
		$extrafieldsline = new ExtraFields($db);
		$extralabelsline = $extrafieldsline->fetch_name_optionals_label($object->table_element_line);
		$array_options = $extrafieldsline->getOptionalsFromPost($extralabelsline, $predef);
		// Unset extrafield
		if (is_array($extralabelsline)) {
			// Get extra fields
			foreach ($extralabelsline as $key => $value) {
				unset($_POST["options_" . $key]);
			}
		}

		if (empty($idprod) && ($price_ht < 0) && ($qty < 0)) {
			setEventMessages($langs->trans('ErrorBothFieldCantBeNegative', $langs->transnoentitiesnoconv('UnitPriceHT'), $langs->transnoentitiesnoconv('Qty')), null, 'errors');
			$error++;
		}
		if ($prod_entry_mode == 'free' && empty($idprod) && GETPOST('type') < 0) {
			setEventMessages($langs->trans('ErrorFieldRequired', $langs->transnoentitiesnoconv('Type')), null, 'errors');
			$error++;
		}
		if ($prod_entry_mode == 'free' && empty($idprod) && (!($price_ht >= 0) || $price_ht == '') && (!($price_ht_devise >= 0) || $price_ht_devise == ''))    // Unit price can be 0 but not ''
		{
			setEventMessages($langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("UnitPriceHT")), null, 'errors');
			$error++;
		}
		if ($qty == '') {
			setEventMessages($langs->trans('ErrorFieldRequired', $langs->transnoentitiesnoconv('Qty')), null, 'errors');
			$error++;
		}
		if ($prod_entry_mode == 'free' && empty($idprod) && empty($product_desc)) {
			setEventMessages($langs->trans('ErrorFieldRequired', $langs->transnoentitiesnoconv('Description')), null, 'errors');
			$error++;
		}

		if (!$error && isModEnabled('variants') && $prod_entry_mode != 'free') {
			if ($combinations = GETPOST('combinations', 'array')) {
				//Check if there is a product with the given combination
				$prodcomb = new ProductCombination($db);

				if ($res = $prodcomb->fetchByProductCombination2ValuePairs($idprod, $combinations)) {
					$idprod = $res->fk_product_child;
				} else {
					setEventMessage($langs->trans('ErrorProductCombinationNotFound'), 'errors');
					$error++;
				}
			}
		}

		if (!$error && ($qty >= 0) && (!empty($product_desc) || !empty($idprod))) {
			// Clean parameters
			$date_start = dol_mktime(GETPOST('date_start' . $predef . 'hour'), GETPOST('date_start' . $predef . 'min'), GETPOST('date_start' . $predef . 'sec'), GETPOST('date_start' . $predef . 'month'), GETPOST('date_start' . $predef . 'day'), GETPOST('date_start' . $predef . 'year'));
			$date_end = dol_mktime(GETPOST('date_end' . $predef . 'hour'), GETPOST('date_end' . $predef . 'min'), GETPOST('date_end' . $predef . 'sec'), GETPOST('date_end' . $predef . 'month'), GETPOST('date_end' . $predef . 'day'), GETPOST('date_end' . $predef . 'year'));
			$price_base_type = (GETPOST('price_base_type', 'alpha') ? GETPOST('price_base_type', 'alpha') : 'HT');

			// Ecrase $pu par celui du produit
			// Ecrase $desc par celui du produit
			// Ecrase $tva_tx par celui du produit
			// Ecrase $base_price_type par celui du produit
			if (!empty($idprod)) {
				$prod = new Product($db);
				$prod->fetch($idprod);

				$label = ((GETPOST('product_label') && GETPOST('product_label') != $prod->label) ? GETPOST('product_label') : '');

				// Update if prices fields are defined
				$tva_tx = get_default_tva($mysoc, $object->thirdparty, $prod->id);
				$tva_npr = get_default_npr($mysoc, $object->thirdparty, $prod->id);
				if (empty($tva_tx)) $tva_npr = 0;

				$pu_ht = $prod->price;
				$pu_ttc = $prod->price_ttc;
				$price_min = $prod->price_min;
				$price_base_type = $prod->price_base_type;

				// multiprix
				if (!empty($conf->global->PRODUIT_MULTIPRICES) && !empty($object->thirdparty->price_level)) {
					$pu_ht = $prod->multiprices[$object->thirdparty->price_level];
					$pu_ttc = $prod->multiprices_ttc[$object->thirdparty->price_level];
					$price_min = $prod->multiprices_min[$object->thirdparty->price_level];
					$price_base_type = $prod->multiprices_base_type[$object->thirdparty->price_level];
					if (!empty($conf->global->PRODUIT_MULTIPRICES_USE_VAT_PER_LEVEL))  // using this option is a bug. kept for backward compatibility
					{
						if (isset($prod->multiprices_tva_tx[$object->thirdparty->price_level])) $tva_tx = $prod->multiprices_tva_tx[$object->thirdparty->price_level];
						if (isset($prod->multiprices_recuperableonly[$object->thirdparty->price_level])) $tva_npr = $prod->multiprices_recuperableonly[$object->thirdparty->price_level];
					}
				} elseif (!empty($conf->global->PRODUIT_CUSTOMER_PRICES)) {
					require_once DOL_DOCUMENT_ROOT . '/product/class/productcustomerprice.class.php';

					$prodcustprice = new Productcustomerprice($db);

					$filter = array('t.fk_product' => $prod->id, 't.fk_soc' => $object->thirdparty->id);

					$result = $prodcustprice->fetch_all('', '', 0, 0, $filter);
					if ($result >= 0) {
						if (count($prodcustprice->lines) > 0) {
							$pu_ht = price($prodcustprice->lines[0]->price);
							$pu_ttc = price($prodcustprice->lines[0]->price_ttc);
							$price_base_type = $prodcustprice->lines[0]->price_base_type;
							$tva_tx = $prodcustprice->lines[0]->tva_tx;
							if ($prodcustprice->lines[0]->default_vat_code && !preg_match('/\(.*\)/', $tva_tx)) $tva_tx .= ' (' . $prodcustprice->lines[0]->default_vat_code . ')';
							$tva_npr = $prodcustprice->lines[0]->recuperableonly;
							if (empty($tva_tx)) $tva_npr = 0;
						}
					} else {
						setEventMessages($prodcustprice->error, $prodcustprice->errors, 'errors');
					}
				}

				$tmpvat = price2num(preg_replace('/\s*\(.*\)/', '', $tva_tx));
				$tmpprodvat = price2num(preg_replace('/\s*\(.*\)/', '', $prod->tva_tx));

				// if price ht is forced (ie: calculated by margin rate and cost price)
				if (!empty($price_ht)) {
					$pu_ht = price2num($price_ht, 'MU');
					$pu_ttc = price2num($pu_ht * (1 + ($tmpvat / 100)), 'MU');
				}
				// On reevalue prix selon taux tva car taux tva transaction peut etre different
				// de ceux du produit par defaut (par exemple si pays different entre vendeur et acheteur).
				elseif ($tmpvat != $tmpprodvat) {
					if ($price_base_type != 'HT') {
						$pu_ht = price2num($pu_ttc / (1 + ($tmpvat / 100)), 'MU');
					} else {
						$pu_ttc = price2num($pu_ht * (1 + ($tmpvat / 100)), 'MU');
					}
				}

				$desc = '';

				// Define output language
				if (!empty($conf->global->MAIN_MULTILANGS) && !empty($conf->global->PRODUIT_TEXTS_IN_THIRDPARTY_LANGUAGE)) {
					$outputlangs = $langs;
					$newlang = '';
					if (empty($newlang) && GETPOST('lang_id', 'aZ09'))
						$newlang = GETPOST('lang_id', 'aZ09');
					if (empty($newlang))
						$newlang = $object->thirdparty->default_lang;
					if (!empty($newlang)) {
						$outputlangs = new Translate("", $conf);
						$outputlangs->setDefaultLang($newlang);
					}

					$desc = (!empty($prod->multilangs [$outputlangs->defaultlang] ["description"])) ? $prod->multilangs [$outputlangs->defaultlang] ["description"] : $prod->description;
				} else {
					$desc = $prod->description;
				}

				$desc = dol_concatdesc($desc, $product_desc);

				// Add custom code and origin country into description
				if (empty($conf->global->MAIN_PRODUCT_DISABLE_CUSTOMCOUNTRYCODE) && (!empty($prod->customcode) || !empty($prod->country_code))) {
					$tmptxt = '(';
					// Define output language
					if (!empty($conf->global->MAIN_MULTILANGS) && !empty($conf->global->PRODUIT_TEXTS_IN_THIRDPARTY_LANGUAGE)) {
						$outputlangs = $langs;
						$newlang = '';
						if (empty($newlang) && GETPOST('lang_id', 'alpha'))
							$newlang = GETPOST('lang_id', 'alpha');
						if (empty($newlang))
							$newlang = $object->thirdparty->default_lang;
						if (!empty($newlang)) {
							$outputlangs = new Translate("", $conf);
							$outputlangs->setDefaultLang($newlang);
							$outputlangs->load('products');
						}
						if (!empty($prod->customcode))
							$tmptxt .= $outputlangs->transnoentitiesnoconv("CustomCode") . ': ' . $prod->customcode;
						if (!empty($prod->customcode) && !empty($prod->country_code))
							$tmptxt .= ' - ';
						if (!empty($prod->country_code))
							$tmptxt .= $outputlangs->transnoentitiesnoconv("CountryOrigin") . ': ' . getCountry($prod->country_code, 0, $db, $outputlangs, 0);
					} else {
						if (!empty($prod->customcode))
							$tmptxt .= $langs->transnoentitiesnoconv("CustomCode") . ': ' . $prod->customcode;
						if (!empty($prod->customcode) && !empty($prod->country_code))
							$tmptxt .= ' - ';
						if (!empty($prod->country_code))
							$tmptxt .= $langs->transnoentitiesnoconv("CountryOrigin") . ': ' . getCountry($prod->country_code, 0, $db, $langs, 0);
					}
					$tmptxt .= ')';
					$desc = dol_concatdesc($desc, $tmptxt);
				}

				$type = $prod->type;
				$fk_unit = $prod->fk_unit;
			} else {
				$pu_ht = price2num($price_ht, 'MU');
				$pu_ttc = price2num(GETPOST('price_ttc'), 'MU');
				$tva_npr = (preg_match('/\*/', $tva_tx) ? 1 : 0);
				$tva_tx = str_replace('*', '', $tva_tx);
				$label = (GETPOST('product_label') ? GETPOST('product_label') : '');
				$desc = $product_desc;
				$type = GETPOST('type');
				$fk_unit = GETPOST('units', 'alpha');
				$pu_ht_devise = price2num($price_ht_devise, 'MU');
			}

			// Margin
			$fournprice = price2num(GETPOST('fournprice' . $predef) ? GETPOST('fournprice' . $predef) : '');
			$buyingprice = price2num(GETPOST('buying_price' . $predef) != '' ? GETPOST('buying_price' . $predef) : '');    // If buying_price is '0', we muste keep this value

			// Local Taxes
			$localtax1_tx = get_localtax($tva_tx, 1, $object->thirdparty);
			$localtax2_tx = get_localtax($tva_tx, 2, $object->thirdparty);

			$desc = dol_htmlcleanlastbr($desc);

			$info_bits = 0;
			if ($tva_npr)
				$info_bits |= 0x01;

			if (!empty($price_min) && (price2num($pu_ht) * (1 - price2num($remise_percent) / 100) < price2num($price_min))) {
				$mesg = $langs->trans("CantBeLessThanMinPrice", price(price2num($price_min, 'MU'), 0, $langs, 0, 0, -1, $conf->currency));
				setEventMessages($mesg, null, 'errors');
			} else {
				// Insert line
				$result = $object->addline($desc, $pu_ht, $qty, $tva_tx, $localtax1_tx, $localtax2_tx, $idprod, $remise_percent, $info_bits, 0, $price_base_type, $pu_ttc, $date_start, $date_end, $type, -1, 0, GETPOST('fk_parent_line'), $fournprice, $buyingprice, $label, $array_options, $fk_unit, '', 0, $pu_ht_devise);

				if ($result > 0) {
					$ret = $object->fetch($object->id); // Reload to get new records

					if (empty($conf->global->MAIN_DISABLE_PDF_AUTOUPDATE)) {
						// Define output language
						$outputlangs = $langs;
						$newlang = GETPOST('lang_id', 'alpha');
						if (!empty($conf->global->MAIN_MULTILANGS) && empty($newlang))
							$newlang = $object->thirdparty->default_lang;
						if (!empty($newlang)) {
							$outputlangs = new Translate("", $conf);
							$outputlangs->setDefaultLang($newlang);
						}

						// TOODO : generate document
						//$object->generateDocument($object->modelpdf, $outputlangs, $hidedetails, $hidedesc, $hideref);
					}

					unset($_POST['prod_entry_mode']);

					unset($_POST['qty']);
					unset($_POST['type']);
					unset($_POST['remise_percent']);
					unset($_POST['price_ht']);
					unset($_POST['multicurrency_price_ht']);
					unset($_POST['price_ttc']);
					unset($_POST['tva_tx']);
					unset($_POST['product_ref']);
					unset($_POST['product_label']);
					unset($_POST['product_desc']);
					unset($_POST['fournprice']);
					unset($_POST['buying_price']);
					unset($_POST['np_marginRate']);
					unset($_POST['np_markRate']);
					unset($_POST['dp_desc']);
					unset($_POST['idprod']);
					unset($_POST['units']);

					unset($_POST['date_starthour']);
					unset($_POST['date_startmin']);
					unset($_POST['date_startsec']);
					unset($_POST['date_startday']);
					unset($_POST['date_startmonth']);
					unset($_POST['date_startyear']);
					unset($_POST['date_endhour']);
					unset($_POST['date_endmin']);
					unset($_POST['date_endsec']);
					unset($_POST['date_endday']);
					unset($_POST['date_endmonth']);
					unset($_POST['date_endyear']);
				} else {
					setEventMessages($object->error, $object->errors, 'errors');
				}
			}
		}
		$action = '';
	} // Update a line
	elseif ($action == 'updateline' && $object->socid > 0 && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && GETPOST('save')) {
		// Clean parameters
		$date_start = '';
		$date_end = '';
		$date_start = dol_mktime(GETPOST('date_starthour'), GETPOST('date_startmin'), GETPOST('date_startsec'), GETPOST('date_startmonth'), GETPOST('date_startday'), GETPOST('date_startyear'));
		$date_end = dol_mktime(GETPOST('date_endhour'), GETPOST('date_endmin'), GETPOST('date_endsec'), GETPOST('date_endmonth'), GETPOST('date_endday'), GETPOST('date_endyear'));
		$description = dol_htmlcleanlastbr(GETPOST('product_desc'));
		$pu_ht = GETPOST('price_ht');
		$vat_rate = (GETPOST('tva_tx') ? GETPOST('tva_tx') : 0);
		$pu_ht_devise = GETPOST('multicurrency_subprice');

		// Define info_bits
		$info_bits = 0;
		if (preg_match('/\*/', $vat_rate))
			$info_bits |= 0x01;

		// Define vat_rate
		$vat_rate = str_replace('*', '', $vat_rate);
		$localtax1_rate = get_localtax($vat_rate, 1, $object->thirdparty, $mysoc);
		$localtax2_rate = get_localtax($vat_rate, 2, $object->thirdparty, $mysoc);

		// Add buying price
		$fournprice = price2num(GETPOST('fournprice') ? GETPOST('fournprice') : '');
		$buyingprice = price2num(GETPOST('buying_price') != '' ? GETPOST('buying_price') : '');    // If buying_price is '0', we muste keep this value

		// Extrafields Lines
		$extrafieldsline = new ExtraFields($db);
		$extralabelsline = $extrafieldsline->fetch_name_optionals_label($object->table_element_line);
		$array_options = $extrafieldsline->getOptionalsFromPost($extralabelsline);
		// Unset extrafield POST Data
		if (is_array($extralabelsline)) {
			foreach ($extralabelsline as $key => $value) {
				unset($_POST["options_" . $key]);
			}
		}

		// Define special_code for special lines
		$special_code = GETPOST('special_code');
		if (!GETPOST('qty')) $special_code = 3;

		// Check minimum price
		$productid = GETPOST('productid', 'int');
		if (!empty($productid)) {
			$product = new Product($db);
			$product->fetch($productid);

			$type = $product->type;

			$price_min = $product->price_min;
			if (!empty($conf->global->PRODUIT_MULTIPRICES) && !empty($object->thirdparty->price_level))
				$price_min = $product->multiprices_min [$object->thirdparty->price_level];

			$label = ((GETPOST('update_label') && GETPOST('product_label')) ? GETPOST('product_label') : '');

			if ($price_min && (price2num($pu_ht) * (1 - price2num(GETPOST('remise_percent')) / 100) < price2num($price_min))) {
				setEventMessages($langs->trans("CantBeLessThanMinPrice", price(price2num($price_min, 'MU'), 0, $langs, 0, 0, -1, $conf->currency)), null, 'errors');
				$error++;
			}
		} else {
			$type = GETPOST('type');
			$label = (GETPOST('product_label') ? GETPOST('product_label') : '');

			// Check parameters
			if (GETPOST('type') < 0) {
				setEventMessages($langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("Type")), null, 'errors');
				$error++;
			}
		}

		if (!$error) {

			if (empty($user->rights->margins->creer)) {
				foreach ($object->lines as &$line) {
					if ($line->id == GETPOST('lineid')) {
						$fournprice = $line->fk_fournprice;
						$buyingprice = $line->pa_ht;
						break;
					}
				}
			}
			$result = $object->updateline(GETPOST('lineid'), $description, $pu_ht, GETPOST('qty'), GETPOST('remise_percent'), $vat_rate, $localtax1_rate, $localtax2_rate, 'HT', $info_bits, $date_start, $date_end, $type, GETPOST('fk_parent_line'), 0, $fournprice, $buyingprice, $label, $special_code, $array_options, GETPOST('units'), $pu_ht_devise);

			if ($result >= 0) {
				if (empty($conf->global->MAIN_DISABLE_PDF_AUTOUPDATE)) {
					// Define output language
					$outputlangs = $langs;
					$newlang = '';
					if ($conf->global->MAIN_MULTILANGS && empty($newlang) && GETPOST('lang_id', 'aZ09'))
						$newlang = GETPOST('lang_id', 'aZ09');
					if ($conf->global->MAIN_MULTILANGS && empty($newlang))
						$newlang = $object->thirdparty->default_lang;
					if (!empty($newlang)) {
						$outputlangs = new Translate("", $conf);
						$outputlangs->setDefaultLang($newlang);
					}

					$ret = $object->fetch($object->id); // Reload to get new records
				}

				unset($_POST['qty']);
				unset($_POST['type']);
				unset($_POST['productid']);
				unset($_POST['remise_percent']);
				unset($_POST['price_ht']);
				unset($_POST['multicurrency_price_ht']);
				unset($_POST['price_ttc']);
				unset($_POST['tva_tx']);
				unset($_POST['product_ref']);
				unset($_POST['product_label']);
				unset($_POST['product_desc']);
				unset($_POST['fournprice']);
				unset($_POST['buying_price']);

				unset($_POST['date_starthour']);
				unset($_POST['date_startmin']);
				unset($_POST['date_startsec']);
				unset($_POST['date_startday']);
				unset($_POST['date_startmonth']);
				unset($_POST['date_startyear']);
				unset($_POST['date_endhour']);
				unset($_POST['date_endmin']);
				unset($_POST['date_endsec']);
				unset($_POST['date_endday']);
				unset($_POST['date_endmonth']);
				unset($_POST['date_endyear']);
			} else {
				setEventMessages($object->error, $object->errors, 'errors');
			}
		}
	} elseif ($action == 'updateline' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && GETPOST('cancel') == $langs->trans('Cancel')) {
		header('Location: ' . $_SERVER['PHP_SELF'] . '?id=' . $object->id);
		exit();
	} // Remove a product line
	elseif ($action == 'confirm_deleteline' && $object->socid > 0 && $confirm == 'yes' && $user->rights->requestmanager->creer) {
		$result = $object->deleteline($lineid);
		if ($result > 0) {
			// Define output language
			$outputlangs = $langs;
			$newlang = '';
			if ($conf->global->MAIN_MULTILANGS && empty($newlang) && GETPOST('lang_id', 'aZ09'))
				$newlang = GETPOST('lang_id', 'aZ09');
			if ($conf->global->MAIN_MULTILANGS && empty($newlang))
				$newlang = $object->thirdparty->default_lang;
			if (!empty($newlang)) {
				$outputlangs = new Translate("", $conf);
				$outputlangs->setDefaultLang($newlang);
			}
			if (empty($conf->global->MAIN_DISABLE_PDF_AUTOUPDATE)) {
				$ret = $object->fetch($object->id); // Reload to get new records
			}

			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		} else {
			setEventMessages($object->error, $object->errors, 'errors');
		}
	}
	/*********************************************************
	 * Times spent management
	 *********************************************************/
	// Set estimated duration
	elseif ($action == 'set_estimated_duration' && $permissionedittimespent) {
		$object->oldcopy = clone $object;
		$object->estimated_duration = GETPOST('estimated_durationhour', 'int') * 3600 + GETPOST('estimated_durationmin', 'int') * 60;    // We store duration in seconds
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_estimated_duration';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set estimated duration
	elseif ($action == 'set_effective_duration' && $permissionedittimespent) {
		$object->oldcopy = clone $object;
		$object->effective_duration = GETPOST('effective_durationhour', 'int') * 3600 + GETPOST('effective_durationmin', 'int') * 60;    // We store duration in seconds
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_effective_duration';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Set reported progress
	elseif ($action == 'set_reported_progress' && $permissionedittimespent) {
		$object->oldcopy = clone $object;
		$object->reported_progress = GETPOST('reported_progress', 'int');
		$result = $object->update($user);
		if ($result < 0) {
			setEventMessages($object->error, $object->errors, 'errors');
			$action = 'edit_reported_progress';
		} else {
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Add time spent
	elseif ($action == 'add_time_spent' && $permissionedittimespent) {
		$date = dol_mktime(GETPOST('time_spent_datehour'), GETPOST('time_spent_datemin'), 0, GETPOST('time_spent_datemonth'), GETPOST('time_spent_dateday'), GETPOST('time_spent_dateyear'));
		$user_id = GETPOST('time_spent_user', 'int');
		$note = GETPOST('time_spent_note', 'alpha');
		$duration_hour = GETPOST('time_spent_durationhour', 'int');
		$duration_hour = $duration_hour > 0 ? $duration_hour : 0;
		$duration_minute = GETPOST('time_spent_durationmin', 'int');
		$duration_minute = $duration_minute > 0 ? $duration_minute : 0;
		$duration = $duration_hour * 3600 + $duration_minute * 60;    // We store duration in seconds

		if ($user->rights->requestmanager->timesspent->manage || $user->id == $user_id) {
			$result = $object->add_time_spent($user, $date, $user_id, $note, $duration);
			if ($result < 0) {
				setEventMessages($object->error, $object->errors, 'errors');
			} else {
				header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
				exit();
			}
		}
	}
	/*********************************************************
	 * Contacts management
	 *********************************************************/
	// Add contact
	elseif ($action == 'add_contact' && $user->rights->requestmanager->creer && $object->statut_type != RequestManager::STATUS_TYPE_CLOSED && $object->statut_type != RequestManager::STATUS_TYPE_RESOLVED) {
		$object->oldcopy = clone $object;
		$object->add_contact_action(intval(GETPOST('add_contact_type_id')));
	} // Delete contact
	elseif ($action == 'del_contact' && $user->rights->requestmanager->creer && $object->statut_type != RequestManager::STATUS_TYPE_CLOSED && $object->statut_type != RequestManager::STATUS_TYPE_RESOLVED) {
		$object->oldcopy = clone $object;
		$object->del_contact_action(intval(GETPOST('del_contact_type_id')));
	}
	/*********************************************************
	 * Message form
	 *********************************************************/
	// Add file
	elseif (GETPOST('addfile', 'alpha')) {
		dol_include_once('/requestmanager/lib/requestmanagermessage.lib.php');

		// Set tmp user directory
		$vardir = $conf->user->dir_output . "/" . $user->id;
		$upload_dir_tmp = $vardir . '/temp/rm-' . $object->id . '/';             // TODO Add $keytoavoidconflict in upload_dir path

		requestmanagermessage_add_file_process($object, $upload_dir_tmp, 0, 0, 'addedfile', '', null);
		$action = 'premessage';
	} // Remove file
	elseif (!empty($_POST['removedfile']) && empty($_POST['removAll'])) {
		dol_include_once('/requestmanager/lib/requestmanagermessage.lib.php');

		// Set tmp user directory
		$vardir = $conf->user->dir_output . "/" . $user->id;
		$upload_dir_tmp = $vardir . '/temp/rm-' . $object->id . '/';             // TODO Add $keytoavoidconflict in upload_dir path

		// TODO Delete only files that was uploaded from email form. This can be addressed by adding the trackid into the temp path then changing donotdeletefile to 2 instead of 1 to say "delete only if into temp dir"
		// GETPOST('removedfile','alpha') is position of file into $_SESSION["listofpaths"...] array.
		requestmanagermessage_remove_file_process($object, GETPOST('removedfile', 'alpha'), 0, 0);   // We do not delete because if file is the official PDF of doc, we don't want to remove it physically
		$action = 'premessage';
	} // Remove all files
	elseif (GETPOST('removAll', 'alpha')) {
		dol_include_once('/requestmanager/class/html.formrequestmanagermessage.class.php');
		$formrequestmanagermessage = new FormRequestManagerMessage($db, $requestmanager);
		$formrequestmanagermessage->remove_all_attached_files();
	} // Add message
	elseif ($action == 'premessage' && GETPOST('addmessage', 'alpha') && !$_POST['addfile'] && !$_POST['removAll'] && !$_POST['removedfile'] && !$_POST['modelselected'] && !$_POST['addknowledgebasedescription'] &&
		$user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)
	) {
		dol_include_once('/requestmanager/class/html.formrequestmanagermessage.class.php');
		$formrequestmanagermessage = new FormRequestManagerMessage($db, $object);

		$requestmanagermessage = new RequestManagerMessage($db);

		$requestmanagermessage->message_type = GETPOST('message_type', 'int');
		$requestmanagermessage->notify_assigned = GETPOST('notify_assigned', 'int');
		$requestmanagermessage->notify_requesters = GETPOST('notify_requesters', 'int');
		$requestmanagermessage->notify_watchers = GETPOST('notify_watchers', 'int');
		$requestmanagermessage->attached_files = $formrequestmanagermessage->get_attached_files();
		$requestmanagermessage->knowledge_base_ids = GETPOST('knowledgebaseselected', 'array');
		$requestmanagermessage->label = GETPOST('subject', 'alpha');
		$requestmanagermessage->note = GETPOST('message', 'restricthtml');
		$requestmanagermessage->requestmanager = $object;

		// Get extra fields of the message
		$message_extrafields = new ExtraFields($db);
		$message_extralabels = $message_extrafields->fetch_name_optionals_label($requestmanagermessage->table_element);
		$ret = $message_extrafields->setOptionalsFromPost($message_extralabels, $requestmanagermessage);

		// create
		$result = $requestmanagermessage->create($user);
		if ($result < 0) {
			setEventMessages($requestmanagermessage->error, $requestmanagermessage->errors, 'errors');
			$action = 'premessage';
		} else {
			$formrequestmanagermessage->clear_datas_in_session();
			$formrequestmanagermessage->remove_all_attached_files();
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id);
			exit();
		}
	} // Clear message
	elseif ($action == 'rm_reset_data_in_session' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)
	) {
		dol_include_once('/requestmanager/class/html.formrequestmanagermessage.class.php');
		$formrequestmanagermessage = new FormRequestManagerMessage($db, $object);
		$formrequestmanagermessage->clear_datas_in_session();

		header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=premessage&messagemode=init#formmessagebeforetitle');
		exit();
	}
	/*********************************************************
	 * Create a event
	 *********************************************************/
	elseif ($action == 'confirm_create_event' && (!empty($user->rights->agenda->myactions->create) || !empty($user->rights->agenda->allactions->create)) &&
		$object->isRequestTypeOption('display_events') && (count($authorizedButtons) == 0 || in_array('create_event', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)
	) {
		$assigned_to_user = array();
		foreach ($object->assigned_user_ids as $id) {
			$assigned_to_user[$id] = array(
				'id' => $id,
				'transparency' => 0,
				'answer_status' => 0,
				'mandatory' => 0
			);
		}

		if (!isset($assigned_to_user[$user->id])) {
			$assigned_to_user[$user->id] = array(
				'id' => $user->id,
				'transparency' => 0,
				'answer_status' => 0,
				'mandatory' => 0
			);
		}

		$_SESSION['assignedtouser'] = json_encode($assigned_to_user);

		$create_params = array(
			'action' => 'create',
			'originid' => $object->id,
			'origin' => $object->element,
			'socid' => $object->socid,
			'donotclearsession' => 1,
			'backtopage' => dol_buildpath('/requestmanager/card.php', 1) . '?id=' . $object->id,
		);

		if ($object->isRequestTypeOption('display_events_create_same_for_each_requester')) {
			$create_same_for_each_requester = GETPOST('create_same_for_each_requester', 'aZ09');
			$create_params['create_same_for_each_requester'] = $create_same_for_each_requester;
			if (empty($create_same_for_each_requester)) {
				$create_params['socpeopleassigned'] = array_values($object->requester_ids);
			}
		} else {
			$create_params['socpeopleassigned'] = array_values($object->requester_ids);
		}

		if ($object->isRequestTypeOption('display_events_create_same_for_each_request_children')) {
			$create_same_for_each_request_children = GETPOST('create_same_for_each_request_children', 'aZ09');
			$create_params['create_same_for_each_request_children'] = $create_same_for_each_request_children;
		}

		header('Location: ' . DOL_URL_ROOT . '/comm/action/card.php?' . http_build_query($create_params));
		exit();
	}

	include DOL_DOCUMENT_ROOT . '/core/actions_printing.inc.php';

	// Actions to build doc
	$upload_dir = !empty($conf->requestmanager->multidir_output[$object->entity]) ? $conf->requestmanager->multidir_output[$object->entity] : $conf->requestmanager->dir_output;
	$permissioncreate = $user->rights->requestmanager->creer;
	include DOL_DOCUMENT_ROOT . '/core/actions_builddoc.inc.php';
}


/*
 * View
 */

llxHeader('', $langs->trans('RequestManagerRequest'), 'EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es');

$form = new Form($db);
$formother = new FormOther($db);
$formfile = new FormFile($db);
$formrequestmanager = new FormRequestManager($db);

$user_static = new User($db);
$usergroup_static = new UserGroup($db);
$contact_static = new Contact($db);

$now = dol_now();

// Add new request
if ($action == 'create') {
	print load_fiche_titre($langs->trans("RequestManagerNewRequest"));

	$origin = GETPOST('origin', 'alpha');
	$originid = GETPOST('originid', 'int');

	$other_linked_objects = GETPOST('other_linked_objects', 'array');
	$children_request = GETPOST('children_request', 'array');

	$object->fk_type = GETPOSTISSET('fixed_type') ? GETPOST('fixed_type', 'int') : GETPOST('type', 'int');
	$object->fk_category = GETPOST('category', 'int');
	$object->socid = GETPOST('socid', 'int');
	$object->requester_ids = GETPOST('requester_ids', 'array');
	$object->fk_source = GETPOST('source', 'int');
	$object->fk_urgency = GETPOST('urgency', 'int');
	$object->fk_impact = GETPOST('impact', 'int');
	$object->fk_priority = GETPOST('priority', 'int');
	$object->estimated_duration = (int)GETPOST('estimated_durationhour', 'int') * 3600 + (int)GETPOST('estimated_durationmin', 'int') * 60;	// We store duration in seconds
	$object->date_operation = dol_mktime(GETPOST('operation_hour', 'int'), GETPOST('operation_min', 'int'), 0, GETPOST('operation_month', 'int'), GETPOST('operation_day', 'int'), GETPOST('operation_year', 'int'));
	$object->date_deadline = dol_mktime(GETPOST('deadline_hour', 'int'), GETPOST('deadline_min', 'int'), 0, GETPOST('deadline_month', 'int'), GETPOST('deadline_day', 'int'), GETPOST('deadline_year', 'int'));
	$object->assigned_usergroup_ids = GETPOST('assigned_usergroups', 'array');
	$object->assigned_user_ids = GETPOST('assigned_users', 'array');
	$object->watcher_ids = GETPOST('contact_watcher_ids', 'array');
	$object->tag_ids = GETPOST('categories', 'array');
	$object->notify_requester_by_email = GETPOST('notify_requester_by_email', 'int');
	$object->notify_assigned_by_email = GETPOST('notify_assigned_by_email', 'int');
	$object->notify_watcher_by_email = GETPOST('notify_watcher_by_email', 'int');
	if (GETPOST('action', 'alphanohtml') == 'create') {
		if ($object->notify_requester_by_email === "") $object->notify_requester_by_email = 1;
		if ($object->notify_assigned_by_email === "") $object->notify_assigned_by_email = 1;
		if ($object->notify_watcher_by_email === "") $object->notify_watcher_by_email = 1;
	}
	$object->label = GETPOST('label', 'alpha');
	if (empty($object->label)) $object->label = $object->getLibType();
	$object->description = GETPOST('description', 'restricthtml');
	// Fill array 'array_options' with data from add form
	//$ret = $extrafields->setOptionalsFromPost($extralabels, $object);

	$init_form = empty($_POST['add_button']) || !empty($_POST['reload_type']);
	$initial_assigned_usergroup_ids = GETPOST('initial_assigned_usergroup_ids', 'custom', 0, FILTER_CALLBACK, array('options' => 'rm_sanitize_id_list'));
	$initial_assigned_usergroup_ids = !empty($initial_assigned_usergroup_ids) ? explode(',', $initial_assigned_usergroup_ids) : (empty($_POST['add_button']) ? $object->assigned_usergroup_ids : array());
	$initial_assigned_user_ids = GETPOST('initial_assigned_user_ids', 'custom', 0, FILTER_CALLBACK, array('options' => 'rm_sanitize_id_list'));
	$initial_assigned_user_ids = !empty($initial_assigned_user_ids) ? explode(',', $initial_assigned_user_ids) : (empty($_POST['add_button']) ? $object->assigned_user_ids : array());
	$initial_date_operation = GETPOST('initial_date_operation', 'int');
	$initial_date_deadline = GETPOST('initial_date_deadline', 'int');

	$type_readonly = GETPOST('fixed_type', 'int') > 0 || GETPOST('type_readonly', 'int') > 0;

	// Load objectsrc
	if (!empty($origin) && !empty($originid)) {
		$objectsrc = fetchObjectByElement($originid, $origin);
		if (is_object($objectsrc)) {
			if (empty($objectsrc->lines) && method_exists($objectsrc, 'fetch_lines')) {
				$objectsrc->fetch_lines();
			}
			$objectsrc->fetch_thirdparty();

			// Init values from origin
			if ($init_form) {
				$projectid = (!empty($objectsrc->fk_project) ? $objectsrc->fk_project : 0);
				$ref_client = (!empty($objectsrc->ref_client) ? $objectsrc->ref_client : '');
				$ref_int = (!empty($objectsrc->ref_int) ? $objectsrc->ref_int : '');

				$object->socid = $objectsrc->thirdparty->id;
				$object->thirdparty = $objectsrc->thirdparty;

				$dateinvoice = (empty($dateinvoice) ? (empty($conf->global->MAIN_AUTOFILL_DATE) ? -1 : '') : $dateinvoice);

				// Replicate extrafields
				$objectsrc->fetch_optionals($originid);
				$object->array_options = array_merge($object->array_options, $objectsrc->array_options);
			}
		}
	}

	// Init values from initial status options
	$authorized_usergroup = $authorized_user = array();
	$assigned_usergroup_not_replaced = $assigned_user_not_replaced = true;
	if ($object->fk_type > 0) {
		dol_include_once('/advancedictionaries/class/dictionary.class.php');
		$dictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
		$result = $dictionary->fetch_lines(1, array('type' => array(RequestManager::STATUS_TYPE_INITIAL), 'request_type' => array($object->fk_type)));
		if ($result < 0) {
			setEventMessages($dictionary->error, $dictionary->errors, 'errors');
		} elseif (count($dictionary->lines) == 1) {
			$initial_status = array_values($dictionary->lines);
			$initial_status = $initial_status[0];

			$authorized_usergroup = !empty($initial_status->fields['authorized_usergroup']) ? explode(',', $initial_status->fields['authorized_usergroup']) : array();
			$authorized_user = !empty($initial_status->fields['authorized_user']) ? explode(',', $initial_status->fields['authorized_user']) : array();
			$assigned_usergroup_not_replaced = empty($initial_status->fields['assigned_usergroup_replaced']);
			$assigned_user_not_replaced = empty($initial_status->fields['assigned_user_replaced']);
			if ($init_form) {
				// Init assigned
				if ($assigned_usergroup_not_replaced) {
					$assigned_usergroups = !empty($initial_status->fields['assigned_usergroup']) ? explode(',', $initial_status->fields['assigned_usergroup']) : array();
					$object->assigned_usergroup_ids = array_merge($initial_assigned_usergroup_ids, $assigned_usergroups);
					$object->assigned_usergroup_ids = array_flip(array_flip($object->assigned_usergroup_ids));
				}
				if ($assigned_user_not_replaced) {
					$assigned_users = !empty($initial_status->fields['assigned_user']) ? explode(',', $initial_status->fields['assigned_user']) : array();
					$assigned_users = !empty($initial_status->fields['assigned_user_current']) ? array_merge($assigned_users, array($user->id)) : $assigned_users;
					$object->assigned_user_ids = array_merge($initial_assigned_user_ids, $assigned_users);
					$object->assigned_user_ids = array_flip(array_flip($object->assigned_user_ids));
				}

				// Init dates
				$object->date_operation = null;
				if (!empty($initial_status->fields['operation'])) {
					if ($initial_status->fields['operation'] > 0) {
						$object->date_operation = $now + ($initial_status->fields['operation'] * 60);
					} elseif ($initial_status->fields['operation'] == -1 && $initial_date_operation !== '') {
						$object->date_operation = $initial_date_operation;
					} elseif ($initial_status->fields['operation'] == -2) {
						$object->date_operation = $now;
					}
				}
				$object->date_deadline = null;
				if (!empty($initial_status->fields['deadline'])) {
					if ($initial_status->fields['deadline'] > 0) {
						$object->date_deadline = (!empty($object->date_operation) ? $object->date_operation : $now) + ($initial_status->fields['deadline'] * 60);
					} elseif ($initial_status->fields['deadline'] == -1 && $initial_date_deadline !== '') {
						$object->date_deadline = $initial_date_deadline;
					} elseif ($initial_status->fields['deadline'] == -2) {
						$object->date_deadline = $now;
					}
				}
			}
		}
	}

	$mandatory_thirdparty = $object->getRequestTypeOption('mandatory_thirdparty');
	$mandatory_contact = $object->getRequestTypeOption('mandatory_contact');
	$origin_type = $object->getRequestTypeOption('origin_type');
	$user_groups = $usergroup_static->listGroupsForUser($user->id);
	$display_category = $object->isRequestTypeOption('display_category');
	$display_source = $object->isRequestTypeOption('display_source');
	$display_urgency = $object->isRequestTypeOption('display_urgency');
	$display_impact = $object->isRequestTypeOption('display_impact');
	$display_priority = $object->isRequestTypeOption('display_priority');
	$display_watcher = $object->isRequestTypeOption('display_watcher');
	$display_products = $object->isRequestTypeOption('display_products');
	$manage_times = $object->isRequestTypeOption('manage_times');

	print '<form id="addrequest" name="addrequest" action="' . $_SERVER["PHP_SELF"] . '" method="POST">';
	print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
	print '<input type="hidden" id="action" name="action" value="add">';
	print '<input type="hidden" id="subaction" name="subaction" value="">';
	if (!empty($originid)) {
		print '<input type="hidden" name="origin" value="' . $origin . '">';
		print '<input type="hidden" name="originid" value="' . $originid . '">';
	}
	print '<input type="hidden" name="initial_assigned_user_ids" value="' . implode(',', $initial_assigned_user_ids) .'">';
	print '<input type="hidden" name="initial_assigned_usergroup_ids" value="' . implode(',', $initial_assigned_usergroup_ids) .'">';
	print '<input type="hidden" name="initial_date_operation" value="' . $initial_date_operation .'">';
	print '<input type="hidden" name="initial_date_deadline" value="' . $initial_date_deadline .'">';
	print '<input type="hidden" name="type_readonly" value="' . ($type_readonly ? 1 : 0) .'">';

	print dol_get_fiche_head();

	print '<table class="border" width="100%">';

	print <<<SCRIPT
	<script type="text/javascript">
		jQuery(document).ready(function() {
			jQuery("#type").change(function() {
				jQuery("#addrequest #subaction").val("reload_type");
				requestmanager_refresh_create_page();
			});
			jQuery("#socid").change(function() {
				requestmanager_refresh_create_page();
			});
			jQuery("#requester_ids").change(function() {
				var third_party_id = parseInt($("#socid").val());
				if (third_party_id <= 0) {
					requestmanager_refresh_create_page();
				}
			});
		});

		function requestmanager_refresh_create_page() {
			jQuery("#addrequest #action").val("create");
			jQuery("#addrequest").submit();
		}
	</script>
SCRIPT;

	// Type / Category
	print '<tr><td class="titlefieldcreate fieldrequired">' . $langs->trans('RequestManagerType') . '</td><td' . ($display_category ? '' : ' colspan="3"') . '>';
	if ($type_readonly) {
		print $object->getLibType();
		print '<input type="hidden" name="type" value="' . $object->fk_type .'">';
	} else {
		print $formrequestmanager->select_type(array_keys($user_groups), $object->fk_type, 'type', 1, 0, null, 0, 0, 'minwidth300');
	}
	if ($display_category) {
		print '</td><td class="titlefieldcreate">' . $langs->trans('RequestManagerCategory') . '</td><td>';
		print $formrequestmanager->select_category($object->fk_type, $object->fk_category, 'category', 1, 0, array(), 0, 0, 'minwidth300');
	}
	print '</td></tr>';

	// Third party origin / Contact origin
	print '<tr>';
	if (empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_THIRDPARTY) {
		print '<td class="titlefieldcreate'. ($mandatory_thirdparty ? ' fieldrequired' : '') .'">' . $langs->trans('RequestManagerThirdPartyOrigin') . '</td><td' . (empty($origin_type) ? '' : ' colspan="3"') . '>';
		if ($v18p) {
			print $form->select_company($object->socid, 'socid', '(s.client:=:1 OR s.client:=:2 OR s.client:=:3) AND (status:=:1)', 'SelectThirdParty', 0, 0, null, 0, 'minwidth300');
		} else {
			print $form->select_company($object->socid, 'socid', '(s.client = 1 OR s.client = 2 OR s.client = 3) AND status=1', 'SelectThirdParty', 0, 0, null, 0, 'minwidth300');
		}
		print '</td>';
	}
	if (empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_CONTACT) {
		print '<td class="titlefieldcreate'. ($mandatory_contact ? ' fieldrequired' : '') .'">' . $langs->trans('RequestManagerRequesterContacts') . '</td><td' . (empty($origin_type) ? '' : ' colspan="3"') . '>';
		print $formrequestmanager->multiselect_contacts($object->socid > 0 ? $object->socid : 0, $object->requester_ids, 'requester_ids', '', '', 0, 'minwidth300');
		print '</td>';
	}
	print '</tr>';

	// Source / Urgency
	if ($display_source || $display_urgency) {
		print '<tr>';
		if ($display_source) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerSource') . '</td><td' . ($display_urgency ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->select_source($object->fk_source, 'source', 1, 0, array(), 0, 0, 'minwidth300');
			print '</td>';
		}
		if ($display_urgency) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerUrgency') . '</td><td' . ($display_source ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->select_urgency($object->fk_urgency, 'urgency', 1, 0, array(), 0, 0, 'minwidth300');
			print '</td>';
		}
		print '</tr>';
	}

	// Impact / Priority
	if ($display_impact || $display_priority) {
		print '<tr>';
		if ($display_impact) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerImpact') . '</td><td' . ($display_priority ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->select_impact($object->fk_impact, 'impact', 1, 0, array(), 0, 0, 'minwidth300');
			print '</td>';
		}
		if ($display_priority) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerPriority') . '</td><td' . ($display_impact ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->select_priority($object->fk_priority, 'priority', 1, 0, array(), 0, 0, 'minwidth300');
			print '</td>';
		}
		print '</tr>';
	}

	// Times spent - Estimated duration
	if ($manage_times) {
		print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerEstimatedDuration') . '</td><td colspan="3">';
		print $form->select_duration('estimated_duration', $object->estimated_duration, 0, 'text', 0, 1);
		print '</td></tr>';
	}

	// Date Operation / Date Deadline
	print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerOperation') . '</td><td>';
	$form->select_date($object->date_operation, 'operation_', 1, 1, 1, '', 1);
	print '</td><td class="titlefieldcreate">' . $langs->trans('RequestManagerDeadline') . '</td><td>';
	$form->select_date($object->date_deadline, 'deadline_', 1, 1, 1, '', 1);
	print '</td></tr>';

	// Assigned usergroups / Assigned users
	if ($assigned_user_not_replaced || $assigned_usergroup_not_replaced) {
		print '<tr>';
		if ($assigned_usergroup_not_replaced) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerAssignedUserGroups') . '</td><td' . ($assigned_user_not_replaced ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->multiselect_dolgroups($object->assigned_usergroup_ids, 'assigned_usergroups', '', 0, $authorized_usergroup);
			print '</td>';
		}
		if ($assigned_user_not_replaced) {
			print '<td class="titlefieldcreate">' . $langs->trans('RequestManagerAssignedUsers') . '</td><td' . ($assigned_usergroup_not_replaced ? '' : ' colspan="3"') . '>';
			print $formrequestmanager->multiselect_dolusers($object->assigned_user_ids, 'assigned_users', null, 0, $authorized_user, '', 0, 0, 0, '', 0, '', '', 1, 0);
			print '</td>';
		}
		print '</tr>';
	}

	// Watchers
	if ($display_watcher) {
		print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerWatcherContacts') . '</td><td colspan="3">';
		print $formrequestmanager->multiselect_contacts($object->socid, $object->watcher_ids, 'contact_watcher_ids', '', '', 0, 'minwidth300');
		print '</td></tr>';
	}

	// Categories/Tags products
	if (isModEnabled('categorie') && $display_products) {
		print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerTags') . '</td><td colspan="3">';
		print $formrequestmanager->multiselect_categories($object->tag_ids, 'categories',  '', 0, '', 0, '100%');
		print '</td></tr>';
	}

	// Origin (element)
	if (!empty($origin) && !empty($originid) && is_object($objectsrc)) {
		print '<tr><td class="titlefieldcreate">' . $langs->trans('Origin') . '</td><td colspan="3">';
		print $objectsrc->getNomUrl(1);
		print '</td></tr>';
	}

	// Related objects (element)
	if (!empty($other_linked_objects)) {
		$linked_objects_to_print = array();
		require_once DOL_DOCUMENT_ROOT . '/core/lib/functions2.lib.php';
		foreach ($other_linked_objects as $element_type => $element_ids) {
			$element_prop = getElementProperties($element_type);
			if (is_array($element_prop) && $conf->{$element_prop['module']}->enabled) {
				dol_include_once('/' . $element_prop['classpath'] . '/' . $element_prop['classfile'] . '.class.php');
				foreach ($element_ids as $element_id) {
					$objecttmp = new $element_prop['classname']($db);
					$ret = $objecttmp->fetch($element_id, $element_ref);
					if ($ret > 0) {
						$linked_objects_to_print[] = $objecttmp->getNomUrl(1) . '<input type="hidden" name="other_linked_objects[' . dol_escape_htmltag($element_type) . '][]" value="' . dol_escape_htmltag($element_id) . '">';
					} elseif ($ret == 0) {
						$linked_objects_to_print[] = "Not found (Element type: $element_type; ID: $element_id)";
					} else {
						$linked_objects_to_print[] = "Error load (Element type: $element_type; ID: $element_id): " . $objecttmp->error . (is_array($objecttmp->errors) ? (($objecttmp->error != '' ? ', ' : '') . join(', ', $objecttmp->errors)) : '');
					}
				}
			}
		}

		if (!empty($linked_objects_to_print)) {
			print '<tr><td class="titlefieldcreate">' . $langs->trans('RelatedObjects') . '</td><td colspan="3">';
			print implode(', ', $linked_objects_to_print);
			print '</td></tr>';
		}
	}

	// Children request
	if (!empty($children_request)) {
		$children_request_to_print = array();
		foreach ($children_request as $request_id) {
			if ($request_id > 0) {
				$request_tmp = new RequestManager($db);
				$ret = $request_tmp->fetch($request_id);
				if ($ret > 0) {
					$children_request_to_print[] = $request_tmp->getNomUrl(1) . '<input type="hidden" name="children_request[]" value="' . dol_escape_htmltag($request_id) . '">';
				} elseif ($ret == 0) {
					$children_request_to_print[] = "Request not found (ID: $request_id)";
				} else {
					$children_request_to_print[] = "Error load request (ID: $request_id): " . $request_tmp->errorsToString();
				}
			}
		}

		if (!empty($children_request_to_print)) {
			print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerChildrenRequest') . '</td><td colspan="3">';
			print implode(', ', $children_request_to_print);
			print '</td></tr>';
		}
	}

	// Notifications by email
	print '<tr><td class="titlefieldcreate">' . $langs->trans('Notifications') . '</td><td colspan="3">';
	$to_print = array();
	if (empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_CONTACT) {
		$to_print[] = '<input type="checkbox" id="notify_requester_by_email" name="notify_requester_by_email"' . ($object->notify_requester_by_email ? ' checked="checked"' : '') . ' value="1"><label for="notify_requester_by_email">' . $langs->trans('RequestManagerRequesterContacts') . '</label>';
	}
	$to_print[] = '<input type="checkbox" id="notify_assigned_by_email" name="notify_assigned_by_email"' . ($object->notify_assigned_by_email ? ' checked="checked"' : '') . ' value="1"><label for="notify_assigned_by_email">' . $langs->trans('RequestManagerAssigned') . '</label>';
	if ($display_watcher) {
		$to_print[] = '<input type="checkbox" id="notify_watcher_by_email" name="notify_watcher_by_email"' . ($object->notify_watcher_by_email ? ' checked="checked"' : '') . ' value="1"><label for="notify_watcher_by_email">' . $langs->trans('RequestManagerWatcherContacts') . '</label>';
	}
	print implode('&nbsp;&nbsp;', $to_print);
	print '</td></tr>';

	// Label
	print '<tr><td class="titlefieldcreate fieldrequired">' . $langs->trans('RequestManagerLabel') . '</td><td colspan="3">';
	print '<input type="text" class="centpercent" id="label" name="label" value="' . dol_escape_htmltag($object->label) . '">';
	print '</td></tr>';

	// Description
	print '<tr><td class="titlefieldcreate">' . $langs->trans('RequestManagerDescription') . '</td><td colspan="3">';
	$doleditor = new DolEditor('description', $object->description, '', 200, 'dolibarr_notes', 'In', 0, false, true, ROWS_3, '90%');
	print $doleditor->Create(1);
	print '</td></tr>';

	// Other attributes
	include DOL_DOCUMENT_ROOT . '/core/tpl/extrafields_add.tpl.php';

	print "</table>\n";

	print dol_get_fiche_end();

	print '<div class="center">';
	print '<input type="submit" name="add_button" class="button" value="' . $langs->trans("RequestManagerCreateRequest") . '">';
	print '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;';
	print '<input type="button" class="button" value="' . $langs->trans("Cancel") . '" onClick="javascript:history.go(-1)">';
	print '</div>';

	print "</form>";

	// Show origin lines
	if ($display_products && !empty($origin) && !empty($originid) && !empty($objectsrc->lines) && is_object($objectsrc)) {
		print '<br>';

		$title = $langs->trans('ProductsAndServices');
		print load_fiche_titre($title);

		print '<table class="noborder" width="100%">';

		$objectsrc->printOriginLinesList();

		print '</table>';
	}

	$parameters = array();
	$reshook = $hookmanager->executeHooks('insertExtraFooter', $parameters, $object, $action);    // Note that $action and $object may have been modified by some hooks
	if ($reshook < 0) setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');
	if ($reshook == 0) {
		if ($object->fk_type > 0 && ($object->socid > 0 || !empty($object->requester_ids))) {
			$already_requested = $object->getAlreadyRequested();
			if (!is_array($already_requested)) {
				setEventMessages($object->error, $object->errors, 'errors');
			} elseif (!empty($already_requested)) {
				print '<br>';
				$request_static = new RequestManager($db);
				$title = $langs->trans('RequestManagerListOfRequests') .
					' ( ' . $request_static->LibStatusType(RequestManager::STATUS_TYPE_INITIAL) .
					', ' . $request_static->LibStatusType(RequestManager::STATUS_TYPE_IN_PROGRESS). ' )';
				print load_fiche_titre($title, '', '');
				print '<div class="div-table-responsive-no-min">';
				print '<table class="noborder allwidth">';
				print '<tr class="liste_titre">';
				print '<td>' . $langs->trans("Ref") . '</td>';
				print '<td>' . $langs->trans("RequestManagerLabel") . '</td>';
				print '<td class="center">' . $langs->trans("DateCreation") . '</td>';
				print '<td class="right">' . $langs->trans("Status") . '</td>';
				print '</tr>';
				foreach ($already_requested as $request_info) {
					$request_static->id = $request_info['id'];
					$request_static->fk_parent = $request_info['fk_parent'];
					$request_static->ref = $request_info['ref'];
					$request_static->ref_ext = $request_info['ref_ext'];
					$request_static->fk_type = $request_info['fk_type'];
					$request_static->socid = $request_info['socid'];
					$request_static->label = $request_info['label'];
					$request_static->date_creation = $request_info['date_creation'];
					$request_static->statut = $request_info['status'];
					print '<tr class="liste">';
					print '<td>' . $request_static->getNomUrl(1, 'parent_path') . '</td>';
					print '<td>' . $request_static->label . '</td>';
					print '<td class="center">' . dol_print_date($request_static->date_creation, 'dayhour') . '</td>';
					print '<td class="right">' . $request_static->getLibStatut(5) . '</td>';
					print '</tr>';
				}
				print '</table>';
			}
		}
	}
} elseif ($object->id > 0) {
	/*
	 * Show object in view mode
	 */

	$head = requestmanager_prepare_head($object);
	print dol_get_fiche_head($head, 'card', $langs->trans('RequestManagerCard'), 0, 'requestmanager@requestmanager');

	$formconfirm = '';

    // Confirm resolve
   	if ($action == 'resolve') {
		$status = GETPOST('status', 'int');
        $reason_resolution = GETPOST('reason_resolution', 'int');
        $reason_resolution_details = GETPOST('reason_resolution_details');

        $formquestion = array(
            array('type' => 'other', 'name' => 'reason_resolution', 'label' => $langs->trans('RequestManagerReasonResolution'),
                'value' => $formrequestmanager->select_reason_resolution($object->fk_type, $reason_resolution, 'reason_resolution', 1, 0, array(), 0, 0, 'minwidth300')),
        );

        $doleditor = new DolEditor('reason_resolution_details', $reason_resolution_details, '', 200, 'dolibarr_notes', 'In', 0, false, true, ROWS_3, '90%');
        $formquestion = array_merge($formquestion, array(
            array('type' => 'other', 'name' => 'reason_resolution_details', 'label' => $langs->trans('RequestManagerReasonResolutionDetails'), 'value' => $doleditor->Create(1, isModEnabled('fckeditor') ? ".on('change', function(e) { $('textarea#reason_resolution_details').val(encodeURIComponent(this.getData())); });" : '')),
        ));

   		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id . '&status=' . $status, $langs->trans('RequestManagerResolveRequest'), $langs->trans('RequestManagerConfirmResolveRequest', $object->ref), 'confirm_resolve', $formquestion, 0, 1, 520, 800);
   	}

    // Confirm close
	elseif ($action == 'close') {
		$status = GETPOST('status', 'int');
   		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id . '&status=' . $status, $langs->trans('RequestManagerCloseRequest'), $langs->trans('RequestManagerConfirmCloseRequest', $object->ref), 'confirm_close', '', 0, 1);
   	}

    // Confirm not resolved
	elseif ($action == 'notresolved') {
        // Get group of user
        $user_groups = $usergroup_static->listGroupsForUser($user->id);
        $user_groups = is_array($user_groups) ? array_keys($user_groups) : array();

        // Get all status of the request type
        $requestManagerStatusDictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
        $requestManagerStatusDictionary->fetch_lines(1, array('request_type' => array($object->fk_type)));

        $status_list = array();
        foreach ($requestManagerStatusDictionary->lines as $s) {
            $not_authorized_user = !empty($s->fields['authorized_user']) ? !in_array($user->id, explode(',', $s->fields['authorized_user'])) : false;
            $not_authorized_usergroup = false;
            if (!empty($s->fields['authorized_usergroup'])) {
                $not_authorized_usergroup = true;
                $authorized_usergroup = explode(',', $s->fields['authorized_usergroup']);
                foreach ($authorized_usergroup as $group_id) {
                    if (in_array($group_id, $user_groups)) {
                        $not_authorized_usergroup = false;
                        break;
                    }
                }
            }
            if ($not_authorized_user || $not_authorized_usergroup) continue;
            if ($s->fields['type'] == RequestManager::STATUS_TYPE_RESOLVED || $s->fields['type'] == RequestManager::STATUS_TYPE_CLOSED) continue;

            $sort_key = $s->fields['type'] . '_' . $s->id;
            $status_list[$sort_key] = array($s->id, $object->LibStatut($s->id, 13));
        }
        ksort($status_list);

        $status_array = array();
        foreach ($status_list as $s) {
            $status_array[$s[0]] = $s[1];
        }

        $selected_status = GETPOST('new_status', 'int');

        $formquestion = array(
            array('type' => 'other', 'name' => 'new_status', 'label' => $langs->trans('RequestManagerToNewStatus'), 'value' => $form->selectarray('new_status', $status_array, $selected_status)),
        );

   		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerNotResolvedRequest'), $langs->trans('RequestManagerConfirmNotResolvedRequest', $object->ref), 'confirm_notresolved', $formquestion, 0, 1);
   	}

    // Confirm re open
	elseif ($action == 'reopen') {
        // Get group of user
        $user_groups = $usergroup_static->listGroupsForUser($user->id);
        $user_groups = is_array($user_groups) ? array_keys($user_groups) : array();

        // Get all status of the request type
        $requestManagerStatusDictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');
        $requestManagerStatusDictionary->fetch_lines(1, array('request_type' => array($object->fk_type)));

        $status_list = array();
        foreach ($requestManagerStatusDictionary->lines as $s) {
            $not_authorized_user = !empty($s->fields['authorized_user']) ? !in_array($user->id, explode(',', $s->fields['authorized_user'])) : false;
            $not_authorized_usergroup = false;
            if (!empty($s->fields['authorized_usergroup'])) {
                $not_authorized_usergroup = true;
                $authorized_usergroup = explode(',', $s->fields['authorized_usergroup']);
                foreach ($authorized_usergroup as $group_id) {
                    if (in_array($group_id, $user_groups)) {
                        $not_authorized_usergroup = false;
                        break;
                    }
                }
            }
            if ($not_authorized_user || $not_authorized_usergroup) continue;
            if ($s->fields['type'] == RequestManager::STATUS_TYPE_RESOLVED || $s->fields['type'] == RequestManager::STATUS_TYPE_CLOSED) continue;

            $sort_key = $s->fields['type'] . '_' . $s->id;
            $status_list[$sort_key] = array($s->id, $object->LibStatut($s->id, 13));
        }
        ksort($status_list);

        $status_array = array();
        foreach ($status_list as $s) {
            $status_array[$s[0]] = $s[1];
        }

        $selected_status = GETPOST('new_status', 'int');

        $formquestion = array(
            array('type' => 'other', 'name' => 'new_status', 'label' => $langs->trans('RequestManagerToNewStatus'), 'value' => $form->selectarray('new_status', $status_array, $selected_status)),
        );

        $formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerReOpenRequest'), $langs->trans('RequestManagerConfirmReOpenRequest', $object->ref), 'confirm_reopen', $formquestion, 0, 1);
    }

    // Confirm delete
	elseif ($action == 'delete') {
		$delete_event_auto = GETPOST('delete_event_auto', 'aZ09');
		$delete_event_manual = GETPOST('delete_event_manual', 'aZ09');

		$formquestion = array(
			array('type' => 'checkbox', 'name' => 'delete_event_auto', 'label' => $langs->trans('RequestManagerDeleteAlsoLinkedAutoEvent'), 'value' => !empty($delete_event_auto)),
			array('type' => 'checkbox', 'name' => 'delete_event_manual', 'label' => $langs->trans('RequestManagerDeleteAlsoLinkedManualEvent'), 'value' => !empty($delete_event_manual)),
		);

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerDeleteRequest'), $langs->trans('RequestManagerConfirmDeleteRequest', $object->ref), 'confirm_delete', $formquestion, 0, 1);
   	}

    // Confirmation to delete line
	elseif ($action == 'ask_deleteline')
    {
        $formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id . '&lineid=' . $lineid, $langs->trans('DeleteProductLine'), $langs->trans('ConfirmDeleteProductLine'), 'confirm_deleteline', '', 0, 1);
    }

    // Confirmation to assign myself
    elseif ($action == 'set_myself_assigned_user')
    {
        $formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerAssignMySelf'), $langs->trans('RequestManagerConfirmAssignMySelf', $object->ref), 'confirm_set_myself_assigned_user', '', 0, 1);
    }

	// Confirm create survey
	elseif ($action == 'rm_create_survey' && isModEnabled('questionnaire')) {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));

		$questionnaires = $object->getRequestQuestionnaire();
		$questionnaire = null;
		if (is_array($questionnaires) && count($questionnaires) == 1) {
			$questionnaire = array_values($questionnaires)[0];
		}
		if ($date_answer_deadline === '') $date_answer_deadline = dol_now() + (is_object($questionnaire) && $questionnaire->answer_deadline > 0 ? $questionnaire->answer_deadline : $conf->global->QUESTIONNAIRE_DEFAULT_ANSWER_DEADLINE) * 24 * 60 * 60;

		$questionnaire_arr = array();
		if (is_array($questionnaires)) {
			foreach ($questionnaires as $questionnaire) {
				if ($questionnaire_id == '') $questionnaire_id = $questionnaire->id;
				$questionnaire_arr[$questionnaire->id] = $questionnaire->ref . ' - ' . $questionnaire->title;
			}
		}

		$formquestion = array(
			array('type' => 'select', 'tdclass' => 'titlefieldcreate', 'name' => 'questionnaire_id', 'label' => $langs->trans('RequestManagerSurvey'), 'values' => $questionnaire_arr, 'default' => $questionnaire_id),
			array('type' => 'date', 'tdclass' => 'titlefieldcreate', 'name' => 'date_answer_deadline', 'label' => $langs->trans('RequestManagerSetDateAnswerDeadline'), 'value' => $date_answer_deadline),
		);

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerCreateSurvey'), $langs->trans('RequestManagerConfirmCreateSurvey'), 'confirm_rm_create_survey', $formquestion, '', 1, 230, 800);
	}
	// Confirm send survey
	elseif ($action == 'rm_send_survey' && isModEnabled('questionnaire')) {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$only_new = GETPOST('only_new', 'aZ09');
		$force_date_answer_deadline = GETPOST('force_date_answer_deadline', 'aZ09');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));

		$questionnaires = $object->getRequestQuestionnaire();
		$questionnaire = null;
		if (is_array($questionnaires) && count($questionnaires) == 1) {
			$questionnaire = array_values($questionnaires)[0];
		}
		if ($date_answer_deadline === '') $date_answer_deadline = dol_now() + (is_object($questionnaire) && $questionnaire->answer_deadline > 0 ? $questionnaire->answer_deadline : $conf->global->QUESTIONNAIRE_DEFAULT_ANSWER_DEADLINE) * 24 * 60 * 60;

		// Cree l'objet formulaire mail
		include_once DOL_DOCUMENT_ROOT . '/core/class/html.formmail.class.php';
		$formmail = new FormMail($db);
		$formmail->withform = 0;
		$formmail->withfrom = false;
		$formmail->withto = false;
		$formmail->withtouser = false;
		$formmail->withoptiononeemailperrecipient = false;
		$formmail->withtocc = false;
		$formmail->withtoccuser = false;
		$formmail->withtoccc = false;
		$formmail->withreplyto = false;
		$formmail->witherrorsto = false;
		$formmail->withdeliveryreceipt = false;
		$formmail->withfile = false;

		dol_include_once('/questionnaire/class/invitation.class.php');
		$invitation_user_static = new InvitationUser($db);
		// Make substitution in email content
		$substitutionarray = getCommonSubstitutionArray($langs, 1, null, $invitation_user_static);

		$substitutionarray['__CHECK_READ__'] = '__CHECK_READ__';
		$substitutionarray['__PERSONALIZED__'] = '__PERSONALIZED__';    // deprecated
		$substitutionarray['__CONTACTCIVNAME__'] = '__CONTACTCIVNAME__';

		$parameters = array(
			'mode' => 'formemail',
			'onlykey' => true,
		);
		complete_substitutions_array($substitutionarray, $langs, $invitation_user_static, $parameters);

		// Tableau des substitutions
		$formmail->substit = $substitutionarray;

		// Tableau des parametres complementaires du post
		$formmail->param['models'] = 'questionnaire';
		$formmail->param['models_id'] = GETPOST('modelmailselected', 'int');
		$questionnaire_arr = array();
		if (is_array($questionnaires)) {
			foreach ($questionnaires as $questionnaire) {
				$questionnaire_arr[$questionnaire->id] = $questionnaire->ref . ' - ' . $questionnaire->title;
			}
		}

		$formquestion = array();

		if ($v13p) {
			$more_name = ',subject,message';
		} else {
			$more_name = '';
			$formquestion[] = array('name' => 'subject');
			$formquestion[] = array('name' => 'message');
		}
		$formquestion[] = array('type' => 'onecolumn', 'name' => 'modelselected' . $more_name, 'value' => $formmail->get_form());

		$formquestion[] = array('type' => 'other', 'tdclass' => 'titlefieldcreate', 'name' => 'questionnaire_id', 'label' => $langs->trans('RequestManagerSurvey'), 'value' => $form->selectarray('questionnaire_id', $questionnaire_arr, $questionnaire_id));

		if ($v13p) {
			$more_name = ',date_answer_deadline,date_answer_deadlineday,date_answer_deadlinemonth,date_answer_deadlineyear,date_answer_deadlinehour,date_answer_deadlinemin';
		} else {
			$more_name = '';
			$formquestion[] = array('name' => 'date_answer_deadline');
			$formquestion[] = array('name' => 'date_answer_deadlineday');
			$formquestion[] = array('name' => 'date_answer_deadlinemonth');
			$formquestion[] = array('name' => 'date_answer_deadlineyear');
			$formquestion[] = array('name' => 'date_answer_deadlinehour');
			$formquestion[] = array('name' => 'date_answer_deadlinemin');
		}
		$formquestion[] = array('type' => 'other', 'tdclass' => 'titlefieldcreate', 'name' => 'force_date_answer_deadline', 'label' => $form->textwithpicto($langs->trans('RequestManagerSetDateAnswerDeadline'), $langs->trans('RequestManagerSetDateAnswerDeadlineHelp')), 'value' => '<input type="checkbox" id="force_date_answer_deadline" name="force_date_answer_deadline"' . ($force_date_answer_deadline ? ' selected="selected"' : '') . '> ' . $form->select_date($date_answer_deadline, 'date_answer_deadline', 0, 0, 0, '', 1, 0, 1));

		$formquestion[] = array('type' => 'checkbox', 'tdclass' => 'titlefieldcreate', 'name' => 'only_new', 'label' => $form->textwithpicto($langs->trans('RequestManagerSendOnlyToNewInvitation'), $langs->trans('RequestManagerSendOnlyToNewInvitationHelp')), 'value' => !empty($only_new));

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerSendSurvey'), $langs->trans('RequestManagerConfirmSendSurvey'), 'confirm_rm_send_survey', $formquestion, '', 1, 600, 1000);
		$formconfirm .= <<<SCRIPT
	<script type="text/javascript">
		jQuery(document).ready(function() {
			jQuery("#subject").closest('tr').find('td:first').addClass('titlefieldcreate');
		});
	</script>
SCRIPT;
	}

	// Confirm create sub request
	elseif ($action == 'create_request_child') {
		$new_request_type = GETPOST('new_request_type', 'int');
		$create_for_each_requester = GETPOST('create_for_each_requester', 'aZ09');
		if (!isset($_GET['create_for_each_requester']) && !isset($_POST['create_for_each_requester']))
			$create_for_each_requester = !empty($requestManagerStatusDictionaryLine->fields['new_request_for_each_requester']);

		$formquestion = array(
			array('type' => 'hidden', 'name' => 'new_request_type', 'value' => $new_request_type),
			array('type' => 'checkbox', 'name' => 'create_for_each_requester', 'label' => $langs->trans('RequestManagerCreateForEachRequester'), 'value' => !empty($create_for_each_requester)),
		);

		$new_request_type_label = $object->LibType($new_request_type);
		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('RequestManagerCreateRequestChild', $new_request_type_label), $langs->trans('RequestManagerConfirmCreateRequestChild', $new_request_type_label), 'confirm_create_request_child', $formquestion, 0, 1, 200, 700);
	}

	// Confirm create event
	elseif ($action == 'create_event') {
		$formquestion = array();
		if ($object->isRequestTypeOption('display_events_create_same_for_each_requester')) {
			$create_same_for_each_requester = GETPOST('create_same_for_each_requester', 'aZ09');
			$formquestion[] = array('type' => 'checkbox', 'name' => 'create_same_for_each_requester', 'label' => $langs->trans('RequestManagerCreateSameForEachRequester'), 'value' => !empty($create_same_for_each_requester));
		}
		if ($object->isRequestTypeOption('display_events_create_same_for_each_request_children')) {
			$create_same_for_each_request_children = GETPOST('create_same_for_each_request_children', 'aZ09');
			$formquestion[] = array('type' => 'checkbox', 'name' => 'create_same_for_each_request_children', 'label' => $langs->trans('RequestManagerCreateSameForEachRequestChildren'), 'value' => !empty($create_same_for_each_request_children));
		}

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $object->id, $langs->trans('AddAction'), $langs->trans('RequestManagerConfirmAddAction'), 'confirm_create_event', $formquestion, 0, 1, 200, 700);
	}

	// Hook
    $parameters = array();
    $reshook = $hookmanager->executeHooks('formConfirm', $parameters, $object, $action); // Note that $action and $object may have been modified by hook
    if (empty($reshook)) $formconfirm.=$hookmanager->resPrint;
    elseif ($reshook > 0) $formconfirm=$hookmanager->resPrint;

	// Print form confirm
	print $formconfirm;

	// Request card
	$linkback = '<a href="' . dol_buildpath('/requestmanager/list.php', 1) . '?restore_lastsearch_values=1">' . $langs->trans("BackToList") . '</a>';

	$morehtmlref = '';
	// Ref
	if ($action != 'edit_ref' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)
		$morehtmlref .= ' <a href="' . $_SERVER["PHP_SELF"] . '?action=edit_ref&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetRef')) . '</a> ';
	$morehtmlref.='<div class="refidno">';
	if ($action == 'edit_ref' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL) {
		$morehtmlref .= $langs->trans('RequestManagerNewRef') . ' : ';
		$morehtmlref .= '<form name="editref" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
		$morehtmlref .= '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
		$morehtmlref .= '<input type="hidden" name="action" value="set_ref">';
		$morehtmlref .= '<input type="text" size="100" name="ref" value="'.dol_escape_htmltag($object->ref).'">';
		$morehtmlref .= '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
		$morehtmlref .= '</form><br>';
	}
	// External Reference
    $morehtmlref.=$langs->trans('RequestManagerExternalReference') . ' : ' . $object->ref_ext;
    // Type
  	$morehtmlref.='<br>'.$langs->trans('RequestManagerType') . ' ';
//    if ($action != 'edit_type' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)
//        $morehtmlref .= '<a href="' . $_SERVER["PHP_SELF"] . '?action=edit_type&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetType')) . '</a> ';
    $morehtmlref .= ': ';
//    if ($action == 'edit_type' && $user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL) {
//        $morehtmlref .= '<form name="edittype" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
//        $morehtmlref .= '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
//        $morehtmlref .= '<input type="hidden" name="action" value="set_type">';
//        $groupslist = $usergroup_static->listGroupsForUser($user->id);
//        $morehtmlref .= $formrequestmanager->select_type(array_keys($groupslist), $object->fk_type, 'type', 1, 0, array(), 0, 0, 'minwidth300');
//        $morehtmlref .= '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
//        $morehtmlref .= '</form>';
//    } else {
        $morehtmlref .= $object->getLibType();
//    }
    // Label
  	$morehtmlref.='<br>'.$langs->trans('RequestManagerLabel') . ' ';
    if ($action != 'edit_label' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
        $morehtmlref .= '<a href="' . $_SERVER["PHP_SELF"] . '?action=edit_label&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetLabel')) . '</a> ';
    $morehtmlref .= ': ';
    if ($action == 'edit_label' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
        $morehtmlref .= '<form name="editlabel" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
        $morehtmlref .= '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
        $morehtmlref .= '<input type="hidden" name="action" value="set_label">';
        $morehtmlref .= '<input type="text" size="100" name="label" value="'.dol_escape_htmltag($object->label).'">';
        $morehtmlref .= '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
        $morehtmlref .= '</form>';
    } else {
        $morehtmlref .= $object->label;
    }
    // Parent request
    if ($object->fk_parent > 0) {
        $object->fetch_parent();
        $morehtmlref.='<br>'.$langs->trans('RequestManagerParentRequest') . ' : ' . $object->parent->getNomUrl(1, 'parent_path');
    }
    $morehtmlref.='</div>';

	dol_banner_tab($object, 'ref', $linkback, 1, 'ref', 'ref', $morehtmlref);

	if (!empty($conf->global->REQUESTMANAGER_DISPLAY_STATUS_THREAD) && !empty($object->status_thread)) {
		$status_thread_tmp = explode('_', $object->status_thread);
		$to_print = array();
		foreach ($status_thread_tmp as $s_id) {
			if ($s_id > 0) $to_print[] = $object->LibStatut($s_id, 5);
		}
		if (!empty($to_print)) {
			print '<div class="status_thread">' . implode(' <i class="fa fa-chevron-double-right"></i> ', $to_print) . '</div>';
		}
	}

    // Hook
    $parameters = array();
    $reshook = $hookmanager->executeHooks('addNextBannerTab', $parameters, $object, $action); // Note that $action and $object may have been modified by hook

    print '<div class="fichecenter">';
    print '<div class="fichehalfleft">';
    print '<div class="underbanner clearboth"></div>';

	print '<table class="border tableforfield" width="100%">';

    // Description
	print '<tr><td class="titlefield">';
	print '<table class="nobordernopadding" width="100%"><tr><td>';
	print $langs->trans('RequestManagerDescription');
	print '</td>';
	if ($action != 'edit_description' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_description&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetDescription'), 1) . '</a></td>';
	}
	print '</tr></table>';
	print '</td><td>';
	if ($action == 'edit_description' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<form name="editdescription" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
		print '<input type="hidden" name="action" value="set_description">';
		$doleditor = new DolEditor('description', $object->description, '', 200, 'dolibarr_notes', 'In', 0, false, true, ROWS_3, '90%');
		print $doleditor->Create(1);
		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
		print '</form>';
	} else {
		print $object->description;
	}
	print '</td></tr>';

    // Category
    if ($object->isRequestTypeOption('display_category')) {
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerCategory');
        print '</td>';
        if ($action != 'edit_category' && $user->rights->requestmanager->creer &&
            ($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)
        )
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_category&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetCategory'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_category' && $user->rights->requestmanager->creer &&
            ($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)
        ) {
            print '<form name="editcategory" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_category">';
            print $formrequestmanager->select_category($object->fk_type, $object->fk_category, 'category', 1, 0, array(), 0, 0, 'minwidth300');
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            print $object->getLibCategory();
        }
        print '</td></tr>';
    }

	$origin_type = $object->getRequestTypeOption('origin_type');
    // ThirdParty Origin
	if (empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_THIRDPARTY) {
		print '<tr><td class="titlefield">';
		print '<table class="nobordernopadding" width="100%"><tr><td>';
		print $langs->trans('RequestManagerThirdPartyOrigin');
		print '</td>';
		if ($action != 'edit_thirdparty' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
			print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_thirdparty&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetThirdPartyOrigin'), 1) . '</a></td>';
		print '</tr></table>';
		print '</td><td>';
		if ($action == 'edit_thirdparty' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
			print '<form name="editthirdparty" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
			print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
			print '<input type="hidden" name="action" value="set_thirdparty">';
			if ($v18p) {
				print $form->select_company($object->socid, 'socid', '(s.client:=:1 OR s.client:=:2 OR s.client:=:3) AND (status:=:1)', 'SelectThirdParty', 0, 0, null, 0, 'minwidth300');
			} else {
				print $form->select_company($object->socid, 'socid', '(s.client = 1 OR s.client = 2 OR s.client = 3) AND status=1', 'SelectThirdParty', 0, 0, null, 0, 'minwidth300');
			}
			print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
			print '</form>';
		} else {
			if (is_object($object->thirdparty) && $object->thirdparty->id > 0) print $object->thirdparty->getNomUrl(1);
		}
		print '</td></tr>';
	}

    // Source
    if ($object->isRequestTypeOption('display_source')) {
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerSource');
        print '</td>';
        if ($action != 'edit_source' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_source&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetSource'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_source' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            print '<form name="editsource" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_source">';
            print $formrequestmanager->select_source($object->fk_source, 'source', 1, 0, array(), 0, 0, 'minwidth300');
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            print $object->getLibSource();
        }
        print '</td></tr>';
    }

    // Urgency
    if ($object->isRequestTypeOption('display_urgency')) {
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerUrgency');
        print '</td>';
        if ($action != 'edit_urgency' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_urgency&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetUrgency'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td>';
        if ($action == 'edit_urgency' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            print '<td>';
            print '<form name="editurgency" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_urgency">';
            print $formrequestmanager->select_urgency($object->fk_urgency, 'urgency', 1, 0, array(), 0, 0, 'minwidth300');
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
            print '</td>';
        } else {
            $color = $object->getColorUrgency();
            print '<td' . (empty($color) ? '' : ' style="background-color: ' . $color . ';"') . '>';
            print $object->getLibUrgency();
            print '</td>';
        }
        print '</tr>';
    }

    // Impact
    if ($object->isRequestTypeOption('display_impact')) {
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerImpact');
        print '</td>';
        if ($action != 'edit_impact' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_impact&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetImpact'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_impact' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            print '<form name="editimpact" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_impact">';
            print $formrequestmanager->select_impact($object->fk_impact, 'impact', 1, 0, array(), 0, 0, 'minwidth300');
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            print $object->getLibImpact();
        }
        print '</td></tr>';
    }

    // Priority
    if ($object->isRequestTypeOption('display_priority')) {
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerPriority');
        print '</td>';
        if ($action != 'edit_priority' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_priority&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetPriority'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_priority' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            print '<form name="editpriority" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_priority">';
            print $formrequestmanager->select_priority($object->fk_priority, 'priority', 1, 0, array(), 0, 0, 'minwidth300');
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            print $object->getLibPriority();
        }
        print '</td></tr>';
    }

    // Date Creation
    print '<tr><td class="titlefield">';
   	print '<table class="nobordernopadding" width="100%"><tr><td>';
   	print $langs->trans('DateCreation');
   	print '</td>';
   	if ($action != 'edit_date_creation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
   		print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_date_creation&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetDateCreation'), 1) . '</a></td>';
   	print '</tr></table>';
   	print '</td><td>';
   	if ($action == 'edit_date_creation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
   		print '<form name="editdatecreation" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
   		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
   		print '<input type="hidden" name="action" value="set_date_creation">';
        $form->select_date($object->date_creation, 'date_creation_', 1, 1, 1, '', 1);
   		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
   		print '</form>';
   	} else {
        print $object->date_creation > 0 ? dol_print_date($object->date_creation, 'dayhour') : '';
   	}
    print '</td></tr>';

    // Times spent
    if ($object->isRequestTypeOption('manage_times')) {
        $can_set_times = $object->isRequestTypeOption('display_set_times');
        $children_total_time_spent = $object->get_children_total_time_spent();

        // Estimated duration
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerEstimatedDuration');
        print '</td>';
        if ($action != 'edit_estimated_duration' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times)
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_estimated_duration&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetEstimatedDuration'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_estimated_duration' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times) {
            print '<form name="editestimatedduration" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_estimated_duration">';
            print $form->select_duration('estimated_duration', $object->estimated_duration, 0, 'text', 0, 1);
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            $estimated_duration = $object->estimated_duration + $children_total_time_spent['total_estimated_duration'];
            if ($estimated_duration > 0) print convertSecondToTime($estimated_duration, 'allhourmin');
        }
        print '</td></tr>';

        // Effective duration
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerEffectiveDuration');
        print '</td>';
        if ($action != 'edit_effective_duration' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times)
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_effective_duration&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetEffectiveDuration'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_effective_duration' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times) {
            print '<form name="editeffectiveduration" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_effective_duration">';
            print $form->select_duration('effective_duration', $object->effective_duration, 0, 'text', 0, 1);
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            $effective_duration = $object->effective_duration + $object->total_effective_duration + $children_total_time_spent['total_effective_duration'];
            if ($effective_duration > 0) print convertSecondToTime($effective_duration, 'allhourmin');
        }
        print '</td></tr>';

        // Reported progress
        print '<tr><td class="titlefield">';
        print '<table class="nobordernopadding" width="100%"><tr><td>';
        print $langs->trans('RequestManagerReportedProgress');
        print '</td>';
        if ($action != 'edit_reported_progress' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times)
            print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_reported_progress&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetReportedProgress'), 1) . '</a></td>';
        print '</tr></table>';
        print '</td><td>';
        if ($action == 'edit_reported_progress' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) && $can_set_times) {
            print '<form name="editreportedprogress" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
            print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
            print '<input type="hidden" name="action" value="set_reported_progress">';
            print $formother->select_percent($object->reported_progress, 'reported_progress', 0, 5, 0, 100, 1);
            print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
            print '</form>';
        } else {
            $reported_progress = $children_total_time_spent['total_reported_progress'] > 0 ? ($can_set_times && isset($object->reported_progress) ? ($object->reported_progress + $children_total_time_spent['total_reported_progress']) / 2 : $children_total_time_spent['total_reported_progress']) : $object->reported_progress;
            if ($reported_progress > 0) print $reported_progress . ' %';
        }
        print '</td></tr>';

        // Calculated progression
        print '<tr><td class="titlefield">';
        print $langs->trans('RequestManagerCalculatedProgression');
        print '</td><td>';
        $estimated_duration = $object->estimated_duration + $children_total_time_spent['total_estimated_duration'];
        $effective_duration = $object->effective_duration + $object->total_effective_duration + $children_total_time_spent['total_effective_duration'];
        if ($estimated_duration > 0) {
            print ($effective_duration > 0 ? round($effective_duration / $estimated_duration * 100, 2) : 0) . ' %';
        } else {
            print '<span class="opacitymedium">' . $langs->trans('RequestManagerEffectiveDurationNotDefined') . '</span>';
        }
        print '</td></tr>';

        // Effective cost
        print '<tr><td class="titlefield">';
        print $langs->trans('RequestManagerEffectiveCost');
        print '</td><td>';
        $effective_cost = $object->effective_cost + $children_total_time_spent['total_effective_cost'];
        print !empty($effective_cost) ? price($effective_cost, '', $langs, 1, 0, 0, $conf->currency) : '';
        print '</td></tr>';
    }

    // Date Operation
    print '<tr><td class="titlefield">';
   	print '<table class="nobordernopadding" width="100%"><tr><td>';
   	print $langs->trans('RequestManagerOperation');
   	print '</td>';
   	if ($action != 'edit_date_operation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
   		print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_date_operation&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetOperation'), 1) . '</a></td>';
   	print '</tr></table>';
   	print '</td><td>';
   	if ($action == 'edit_date_operation' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
   		print '<form name="editdateoperation" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
   		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
   		print '<input type="hidden" name="action" value="set_date_operation">';
        $form->select_date($object->date_operation, 'operation_', 1, 1, 1, '', 1);
        print ' <input type="checkbox" name="recalculate_date_deadline" value="1" checked="checked"> '.$langs->trans('RequestManagerRecalculateDeadLineDate');
   		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
   		print '</form>';
   	} else {
        print $object->date_operation > 0 ? dol_print_date($object->date_operation, 'dayhour') : '';
   	}
    print '</td></tr>';

    // Date Deadline
    print '<tr><td class="titlefield">';
   	print '<table class="nobordernopadding" width="100%"><tr><td>';
   	print $langs->trans('RequestManagerDeadline');
   	print '</td>';
   	if ($action != 'edit_date_deadline' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS))
   		print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_date_deadline&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetDeadline'), 1) . '</a></td>';
   	print '</tr></table>';
   	print '</td><td>';
   	if ($action == 'edit_date_deadline' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
   		print '<form name="editdatedeadline" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
   		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
   		print '<input type="hidden" name="action" value="set_date_deadline">';
        $form->select_date($object->date_deadline, 'deadline_', 1, 1, 1, '', 1);
   		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
   		print '</form>';
   	} else {
        if ($object->date_deadline > 0) {
            print dol_print_date($object->date_deadline, 'dayhour');
            if ($object->date_deadline < $now) {
                // picto warning for deadline / alert time is up
                print ' ' . img_warning($langs->trans("Late"));
            }
        }
   	}
    print '</td></tr>';

    // Reason for resolution
    if ($object->fk_reason_resolution > 0) {
        print '<tr><td class="titlefield">'.$langs->trans('RequestManagerReasonResolution').'</td><td>';
        print $object->getLibReasonResolution();
        print '</td></tr>';
    }

    // Details reason for resolution
    if (!empty($object->reason_resolution_details)) {
        print '<tr><td class="titlefield">'.$langs->trans('RequestManagerReasonResolutionDetails').'</td><td>';
        print $object->reason_resolution_details;
        print '</td></tr>';
    }

	print '</table>';

    print '</div>';
    print '<div class="fichehalfright"><div class="ficheaddleft">';
    print '<div class="underbanner clearboth"></div>';

    print '<table class="border centpercent">';

	// Categories
	if (isModEnabled('categorie') && $object->isRequestTypeOption('display_products')) {
		print '<tr><td class="titlefield">';
		print '<table class="nobordernopadding" width="100%"><tr><td>';
		print $langs->trans('RequestManagerTags');
		print '</td>';
		if ($action != 'edit_categories' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_INITIAL))
			print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_categories&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetCategories'), 1) . '</a></td>';
		print '</tr></table>';
		print '</td><td>';
		if ($action == 'edit_categories' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS || $object->statut_type == RequestManager::STATUS_TYPE_INITIAL)) {
			print '<form name="editcategories" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
			print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
			print '<input type="hidden" name="action" value="set_categories">';
			print $formrequestmanager->showCategories($object->id, CategorieRequestManager::TYPE_REQUESTMANAGER, 0, TRUE);
			print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
			print '</form>';
		} else {
			print $formrequestmanager->showCategories($object->id, CategorieRequestManager::TYPE_REQUESTMANAGER, 1);
		}
		print '</td></tr>';
	}

	$notificationUrl = "#";
	if (!empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL) && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		if ($object->notify_assigned_by_email) {
			$notificationChangeTo = '0';
		} else {
			$notificationChangeTo = '1';
		}
		$notificationUrl = $_SERVER['PHP_SELF'] . '?id=' . $object->id . '&token=' . newToken() . '&action=set_assigned_notification&assigned_notification=' . $notificationChangeTo;
	}

	// Assigned usergroups
	print '<tr><td class="titlefield">';
	print '<table class="nobordernopadding" width="100%"><tr><td>';
	print $langs->trans('RequestManagerAssignedUserGroups');
	print '</td>';
	if ($action != 'edit_assigned_usergroups' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<td class="right minwidth100">';
		if (!empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL)) {
			print '<a href="' . $notificationUrl . '">';
			print img_object($langs->trans($object->notify_assigned_by_email ? "RequestManagerAssignedNotification" : "RequestManagerNoNotifications"), $object->notify_assigned_by_email ? 'email' : 'no_email@requestmanager');
			print '</a>';
		}
		print '<a href="' . $_SERVER["PHP_SELF"] . '?action=edit_assigned_usergroups&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetAssignedUserGroups'), 1) . '</a>';
		print '</td>';
	}
	print '</tr></table>';
	print '</td><td>';
	if ($action == 'edit_assigned_usergroups' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<form name="editassignedusergroups" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
		print '<input type="hidden" name="action" value="set_assigned_usergroups">';
		print $formrequestmanager->multiselect_dolgroups($object->assigned_usergroup_ids, 'assigned_usergroups');
		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
		print '</form>';
	} else {
		$toprint = array();
		foreach ($object->assigned_usergroup_ids as $assigned_usergroup_id) {
			$usergroup_static->fetch($assigned_usergroup_id);
			$toprint[] = $usergroup_static->getFullName($langs);
		}
		print implode(', ', $toprint);
	}
	print '</td></tr>';

	// Assigned users
	print '<tr><td class="titlefield">';
	print '<table class="nobordernopadding" width="100%"><tr><td>';
	print $langs->trans('RequestManagerAssignedUsers');
	print '</td>';
	if ($action != 'edit_assigned_users' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<td class="right minwidth100">';
		if (!empty($conf->global->REQUESTMANAGER_NOTIFICATION_ASSIGNED_BY_EMAIL)) {
			print '<a href="' . $notificationUrl . '">';
			print img_object($langs->trans($object->notify_assigned_by_email ? "RequestManagerAssignedNotification" : "RequestManagerNoNotifications"), $object->notify_assigned_by_email ? 'email' : 'no_email@requestmanager');
			print '</a>';
		}
		print '<a href="' . $_SERVER["PHP_SELF"] . '?action=edit_assigned_users&id=' . $object->id . '&token=' . newToken() . '">' . img_edit($langs->trans('RequestManagerSetAssignedUsers'), 1) . '</a>';
		print '</td>';
	}
	print '</tr></table>';
	print '</td><td>';
	if ($action == 'edit_assigned_users' && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
		print '<form name="editassignedusers" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '" method="post">';
		print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
		print '<input type="hidden" name="action" value="set_assigned_users">';
		print $formrequestmanager->multiselect_dolusers($object->assigned_user_ids,'assigned_users', null, 0, '', '', 0, 0, 0, '', 0, '', '', 1, 0);
		print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
		print '</form>';
	} else {
		$toprint = array();
		foreach ($object->assigned_user_ids as $assigned_user_id) {
			$user_static->fetch($assigned_user_id);
			$toprint[] = $user_static->getNomUrl(1);
		}
		print implode(', ', $toprint);
		if (!in_array($user->id, $object->assigned_user_ids) && $user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
			print '&nbsp;&nbsp;<a href="' . $_SERVER['PHP_SELF'] . '?id=' . $object->id . '&action=set_myself_assigned_user&token='.newToken().'" class="button" style="color: #3c3c3c;" title="' . $langs->trans("RequestManagerAssignToMe") . '">' . $langs->trans("RequestManagerAssignToMe") .'</a>';
		}
	}
	print '</td></tr>';

	// Other attributes
	$object->save_status = $object->statut;
	$object->statut = $object->statut_type;
	include DOL_DOCUMENT_ROOT . '/core/tpl/extrafields_view.tpl.php';
	$object->statut = $object->save_status;

	// Requesters
	if (empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_CONTACT) {
		print '<tr><td class="center" colspan="2">';
		print $langs->trans('RequestManagerRequesterContacts');
		$notificationUrl = "#";
		if ($user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
			if ($object->notify_requester_by_email) {
				$notificationChangeTo = '0';
			} else {
				$notificationChangeTo = '1';
			}
			$notificationUrl = $_SERVER['PHP_SELF'] . '?id=' . $object->id . '&token=' . newToken() . '&action=set_requesters&requester_notification=' . $notificationChangeTo;
		}
		print '&nbsp;';
		print '<a href="' . $notificationUrl . '">';
		print img_object($langs->trans($object->notify_requester_by_email ? "Notifications" : "RequestManagerNoNotifications"), $object->notify_requester_by_email ? 'email' : 'no_email@requestmanager');
		print '</a>';
		print '</td></tr>';
		print '<tr><td colspan="2">';
		if ($user->rights->requestmanager->creer && $object->statut_type != RequestManager::STATUS_TYPE_CLOSED && $object->statut_type != RequestManager::STATUS_TYPE_RESOLVED) {
			// form to add requester contact
			$formrequestmanager->form_add_contact($object, RequestManager::CONTACT_TYPE_ID_REQUEST);
		}
		$object->show_contact_list(RequestManager::CONTACT_TYPE_ID_REQUEST);
		print '</td></tr>';
	}

    // Watchers
    if ($object->isRequestTypeOption('display_watcher')) {
        print '<tr><td class="center" colspan="2">';
        print $langs->trans('RequestManagerWatcherContacts');
        $notificationUrl = "#";
        if ($user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            if ($object->notify_watcher_by_email) {
                $notificationChangeTo = '0';
            } else {
                $notificationChangeTo = '1';
            }
            $notificationUrl = $_SERVER['PHP_SELF'] . '?id=' . $object->id . '&token=' . newToken() . '&action=set_watchers&watcher_notification=' . $notificationChangeTo;
        }
        print '&nbsp;';
        print '<a href="' . $notificationUrl . '">';
        print img_object($langs->trans($object->notify_watcher_by_email ? "Notifications" : "RequestManagerNoNotifications"), $object->notify_watcher_by_email ? 'email' : 'no_email@requestmanager');
        print '</a>';
        print '</td></tr>';
        print '<tr><td colspan="2">';
        if ($user->rights->requestmanager->creer && $object->statut_type != RequestManager::STATUS_TYPE_CLOSED && $object->statut_type != RequestManager::STATUS_TYPE_RESOLVED) {
            // form to add requester contact
            $formrequestmanager->form_add_contact($object, RequestManager::CONTACT_TYPE_ID_WATCHER);
        }
        $object->show_contact_list(RequestManager::CONTACT_TYPE_ID_WATCHER);
        print '</td></tr>';
    }

	print '</table>';

    if ($object->isRequestTypeOption('manage_times') && $object->isRequestTypeOption('display_set_times')) {
        // Times spent
        $formrequestmanager->showTimesSpentBlock($object);
    }

    print '</div>';
    print '</div></div>';
    print '<div class="clearboth"></div>';

	// Specifics Information
    $parameters = array();
    $reshook = $hookmanager->executeHooks('addMoreSpecificsInformation', $parameters, $object, $action); // Note that $action and $object may have been
    if (!empty($hookmanager->resArray)) {
        ksort($hookmanager->resArray);
        print implode("\n", $hookmanager->resArray);
    }

    /*
    * Lines
    */
    if ($object->isRequestTypeOption('display_products') && $object->socid > 0) {
        $langs->load('bills');
        $result = $object->getLinesArray();
        print '<br />';
        print '<form name="addproduct" id="addproduct" action="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . (($action != 'editline') ? '#addline' : '#line_' . GETPOST('lineid')) . '" method="POST">';
        print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
        print '<input type="hidden" name="action" value="' . (($action != 'editline') ? 'addline' : 'updateline') . '">';
        print '<input type="hidden" name="mode" value="">';
        print '<input type="hidden" name="id" value="' . $object->id . '">';

        if (!empty($conf->use_javascript_ajax) && $object->socid > 0 && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            include DOL_DOCUMENT_ROOT . '/core/tpl/ajaxrow.tpl.php';
        }

        $numlines = count($object->lines);

        print '<div class="fichecenter">';
        print '<div class="underbanner clearboth"></div><br>';

        print load_fiche_titre('<span class="rm_lines_block_title_label" style="font-weight: bolder !important; font-size: medium !important;">' . $langs->trans("RequestManagerProductLines") . '&nbsp;<img class="rm_lines_block_title_icon" src=""></img></span>', '', '', 0, 'rm_lines_block_title');
        print '<div id="rm_lines_block_content"' . (empty($numlines) ? ' style="display: none;"' : '') . '>';

        print '<div class="div-table-responsive">';
        print '<table id="tablelines" class="noborder noshadow" width="100%">';
        // Show object lines
        if (!empty($object->lines)) {
            // probleme de statut initial qui vaut 1 et non 0 (et ne permet pas de modifier ou supprimer une ligne)
            $object->save_status = $object->statut;
            if ($user->rights->requestmanager->creer && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
                $object->statut = 0;
            }
            if (!($object->socid > 0)) $object->statut = 1;
            $object->printObjectLines($action, $mysoc, $object->thirdparty, $lineid, 1);
            $object->statut = $object->save_status;
        }
        // form to add new line
        if ($user->rights->requestmanager->creer && $object->socid > 0 && ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS)) {
            if ($action != 'editline') {
                $var = true;

                // Add free products/services
                $object->formAddObjectLine(1, $mysoc, $object->thirdparty);

                $parameters = array();
                $reshook = $hookmanager->executeHooks('formAddObjectLine', $parameters, $object, $action); // Note that $action and $object may have been modified by hook
            }
        }
        print '</table>';
        print '</div>';

        print '</div>';

        $show_label = json_encode($langs->trans('Show'));
        $hide_label = json_encode($langs->trans('Hide'));
        $arrow_up = json_encode(img_picto('', 'sort_asc', '', false, 1));
        $arrow_down = json_encode(img_picto('', 'sort_desc', '', false, 1));
        print <<<SCRIPT
    <script type="text/javascript" language="javascript">
        $(document).ready(function () {
            var rm_count_block_title = $("#rm_lines_block_title");
            var rm_count_block_title_div = $('#rm_lines_block_title div.titre');
            var rm_count_block_title_icon = $(".rm_lines_block_title_icon");
            var rm_count_block_content = $("#rm_lines_block_content");
            
            rm_count_block_title_div.css('cursor', 'pointer');
            rm_update_title_icon();
            rm_count_block_title.on('click', function() {
                rm_count_block_content.toggle();
                rm_update_title_icon();
            });
            
            function rm_update_title_icon() {
                if (rm_count_block_content.is(':visible')) {
                    rm_count_block_title_div.attr('title', $hide_label)
                    rm_count_block_title_icon.attr('src', $arrow_down);
                } else {
                    rm_count_block_title_div.attr('title', $show_label)
                    rm_count_block_title_icon.attr('src', $arrow_up);
                }          
            }
        });
    </script>
SCRIPT;

        print '</div>';
        print '<div class="clearboth"></div>';

        print "</form>\n";
    }

    // Get current status infos
    $requestManagerStatusDictionary = Dictionary::getDictionary($db, 'requestmanager', 'requestmanagerstatus');

    /*
	 * Boutons status
	 */
    if ($user->rights->requestmanager->creer) {
        print '<div class="tabsStatusAction">';

        // Get group of user
        $user_groups = $usergroup_static->listGroupsForUser($user->id);
        $user_groups = is_array($user_groups) ? array_keys($user_groups) : array();

        // Get count children request by status type
        $children_count = $object->getCountChildrenRequestByStatusType();

        // Show status (hide if has new request type in the setting of this status and has none child or all children is not closed)
        if (empty($requestManagerStatusDictionaryLine->fields['new_request_type']) ||
            ($children_count[RequestManager::STATUS_TYPE_INITIAL] + $children_count[RequestManager::STATUS_TYPE_IN_PROGRESS] + $children_count[RequestManager::STATUS_TYPE_RESOLVED] == 0 &&
                $children_count[RequestManager::STATUS_TYPE_CLOSED] >= 0)
        ) {
            // Get all status of the request type
            $requestManagerStatusDictionary->fetch_lines(1, array('request_type' => array($object->fk_type)));

            // Parse all status and add status in previous or next groups
            $previousStatusButton = array();
            $nextStatusButton = array();
            $next_status = explode(',', $requestManagerStatusDictionaryLine->fields['next_status']);
            foreach ($requestManagerStatusDictionary->lines as $s) {
                $not_authorized_user = !empty($s->fields['authorized_user']) ? !in_array($user->id, explode(',', $s->fields['authorized_user'])) : false;
                $not_authorized_usergroup = false;
                if (!empty($s->fields['authorized_usergroup'])) {
                    $not_authorized_usergroup = true;
                    $authorized_usergroup = explode(',', $s->fields['authorized_usergroup']);
                    foreach ($authorized_usergroup as $group_id) {
                        if (in_array($group_id, $user_groups)) {
                            $not_authorized_usergroup = false;
                            break;
                        }
                    }
                }
                if ($not_authorized_user || $not_authorized_usergroup) continue;

                $out = '<div class="inline-block divButAction noMarginBottom">';
                $options_url = '&token='.newToken().'&status=' . $s->id;
                if ($s->fields['type'] == RequestManager::STATUS_TYPE_INITIAL || $s->fields['type'] == RequestManager::STATUS_TYPE_IN_PROGRESS) {
                    $options_url .= '&action=set_status';
                } elseif ($s->fields['type'] == RequestManager::STATUS_TYPE_RESOLVED) {
                    $options_url .= '&action=resolve';
                } elseif ($s->fields['type'] == RequestManager::STATUS_TYPE_CLOSED) {
                    $options_url .= '&action=close';
                } else {
					$options_url = '';
				}
                $out .= '<a class="butAction" href="' . $_SERVER['PHP_SELF'] . '?id=' . $object->id . $options_url . '">';
                $out .= $object->LibStatut($s->id, 10);
                $out .= '</a>';
                $out .= '</div>';

                $sort_key = $s->fields['type'] . '_' . $s->id;

                // Add in previous group
                $thisNextStatus = explode(',', $s->fields['next_status']);
                if ($object->statut_type != RequestManager::STATUS_TYPE_RESOLVED && $object->statut_type != RequestManager::STATUS_TYPE_CLOSED &&
                    in_array($object->statut, $thisNextStatus) && ($s->fields['type'] == RequestManager::STATUS_TYPE_IN_PROGRESS || $s->fields['type'] == RequestManager::STATUS_TYPE_INITIAL)) {
                    $previousStatusButton[$sort_key] = $out;
                }

                // Add in next group
                if (in_array($s->id, $next_status) && (empty($conf->global->REQUESTMANAGER_AUTO_CLOSE_REQUEST) || $s->fields['type'] != RequestManager::STATUS_TYPE_CLOSED)) {
                    $nextStatusButton[$sort_key] = $out;
                }
            }

            // Show previous status
            print '<div class="tabsStatusActionPrevious">';
            if ($user->rights->requestmanager->last_status) {
                ksort($previousStatusButton);
                print implode('', $previousStatusButton);
            }
            print '</div>';

            // Show next status
            print '<div class="tabsStatusActionNext">';
            ksort($nextStatusButton);
            print implode('', $nextStatusButton);
            print '</div>';
        }

        print '</div>';
    }

    print dol_get_fiche_end();

    //Select mail models is same action as premessage
   	if (GETPOST('modelmessageselected', 'int') > 0) $action = 'premessage';

   	if ($action != 'premessage') {
        /*
    	 * Boutons Actions
    	 */
        print '<div class="tabsAction">';

        $parameters = array('status_infos'=>$requestManagerStatusDictionaryLine);
        $reshook = $hookmanager->executeHooks('addMoreActionsButtons', $parameters, $object, $action); // Note that $action and $object may have been
        // modified by hook
        if (empty($reshook)) {
            if ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL || $object->statut_type == RequestManager::STATUS_TYPE_IN_PROGRESS) {
				$backtopage = dol_buildpath('/requestmanager/card.php', 1) . '?id=' . $object->id;
				$commun_params = '&originid=' . $object->id . '&origin=' . $object->element . '&socid=' . $object->socid . '&backtopage=' . urlencode($backtopage);
				if (!empty($conf->global->REQUESTMANAGER_TITLE_TO_REF_CUSTOMER_WHEN_CREATE_OTHER_ELEMENT)) {
					$ref_client = '&ref_client=' . urlencode($object->label);
				} else {
					$ref_client = '';
				}

				// Add child request
                if ($user->rights->requestmanager->creer && $object->isRequestTypeOption('display_children_request')) {
					if (!empty($requestManagerStatusDictionaryLine->fields['new_request_type'])) {
						foreach (explode(',', $requestManagerStatusDictionaryLine->fields['new_request_type']) as $request_type_id) {
							print '<div class="inline-block divButAction">' .
								'<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=create_request_child&new_request_type=' . $request_type_id .
								'" title="' . $langs->trans('RequestManagerCreateRequestChild', $object->LibType($request_type_id)) . '">'
								. '<i class="fa fa-arrow-turn-down-right"></i> ' . $object->LibType($request_type_id). '</a></div>';
						}
					}
					if (!empty($requestManagerStatusDictionaryLine->fields['new_request_type_linked'])) {
						foreach (explode(',', $requestManagerStatusDictionaryLine->fields['new_request_type_linked']) as $request_type_id) {
							print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/requestmanager/card.php', 2) . '?action=create&fixed_type='.$request_type_id . '&token=' . newToken().'&socid=' . $object->socid . $commun_params .
								'" title="' . $langs->trans('RequestManagerCreateRequestLinked', $object->LibType($request_type_id)) . '">'
								. '<i class="fa fa-link"></i> ' . $object->LibType($request_type_id). '</a></div>';
						}
					}
				}

                // Add proposal
                if (isModEnabled('propal') && (count($authorizedButtons) == 0 || in_array('create_propal', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("propal");
                    if ($user->rights->propal->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/comm/propal/card.php?action=create&token=' . newToken() . $commun_params . $ref_client . '">' . $langs->trans("AddProp") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddProp") . '</a></div>';
                    }
                }

                // Add order
                if (isModEnabled('commande') && (count($authorizedButtons) == 0 || in_array('create_order', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("orders");
                    if ($user->rights->commande->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/commande/card.php?action=create&token=' . newToken() . $commun_params . $ref_client . '">' . $langs->trans("AddOrder") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddOrder") . '</a></div>';
                    }
                }

                // Add invoice
                if ($user->socid == 0 && isModEnabled('facture') && (count($authorizedButtons) == 0 || in_array('create_invoice', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("bills");
                    $langs->load("compta");
                    if ($user->rights->facture->creer) {
                        $object->fetch_thirdparty();
                        if ($object->thirdparty->client != 0 && $object->thirdparty->client != 2) {
                            print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/compta/facture/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddBill") . '</a></div>';
                        } else {
                            print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("ThirdPartyMustBeEditAsCustomer")) . '" href="#">' . $langs->trans("AddBill") . '</a></div>';
                        }
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddBill") . '</a></div>';
                    }
                }

                // Add supplier proposal
                if (isModEnabled('supplier_proposal') && (count($authorizedButtons) == 0 || in_array('create_supplier_proposal', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("supplier_proposal");
                    if ($user->rights->supplier_proposal->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/supplier_proposal/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddSupplierProposal") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddSupplierProposal") . '</a></div>';
                    }
                }

                // Add supplier order
                if (isModEnabled('fournisseur') && (count($authorizedButtons) == 0 || in_array('create_supplier_order', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("suppliers");
                    if ($user->rights->fournisseur->commande->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/fourn/commande/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddSupplierOrder") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddSupplierOrder") . '</a></div>';
                    }
                }

                // Add supplier invoice
                if (isModEnabled('fournisseur') && (count($authorizedButtons) == 0 || in_array('create_supplier_invoice', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("suppliers");
                    if ($user->rights->fournisseur->facture->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/fourn/facture/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddSupplierInvoice") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddSupplierInvoice") . '</a></div>';
                    }
                }

                // Add contract
                if (isModEnabled('contrat') && (count($authorizedButtons) == 0 || in_array('create_contract', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("contracts");
                    if ($user->rights->contrat->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/contrat/card.php?action=create&token=' . newToken() . $commun_params . $ref_client . '">' . $langs->trans("AddContract") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddContract") . '</a></div>';
                    }
                }

                // Add intervention
                if (isModEnabled('ficheinter') && (count($authorizedButtons) == 0 || in_array('create_inter', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("interventions");
                    if ($user->rights->ficheinter->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/fichinter/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddIntervention") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddIntervention") . '</a></div>';
                    }
                }

                // Add project
                if (isModEnabled('projet') && (count($authorizedButtons) == 0 || in_array('create_project', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("projects");
                    if ($user->rights->projet->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/projet/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddProject") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddProject") . '</a></div>';
                    }
                }

                // Add trip
                if (isModEnabled('deplacement') && (count($authorizedButtons) == 0 || in_array('create_trip', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    $langs->load("trips");
                    if ($user->rights->deplacement->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . DOL_URL_ROOT . '/compta/deplacement/card.php?action=create&token=' . newToken() . $commun_params . '">' . $langs->trans("AddTrip") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddTrip") . '</a></div>';
                    }
                }

                // Add request
                if ((count($authorizedButtons) == 0 || in_array('create_request_manager', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                    if ($user->rights->requestmanager->creer) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/requestmanager/card.php', 2) . '?action=create&socid=' . $object->socid . '&token=' . newToken() . $commun_params . '">' . $langs->trans("RequestManagerAddRequest") . '</a></div>';
                    } else {
                        print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("RequestManagerAddRequest") . '</a></div>';
                    }
                }

                if (isModEnabled('agenda')) {
                    // Add message
                    if ($user->rights->requestmanager->creer && $object->isRequestTypeOption('display_messages')) {
                        print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=premessage&messagemode=init#formmessagebeforetitle">'
                            . $langs->trans('RequestManagerAddMessage') . '</a></div>';
                    }

                    // Add event
                    if ($object->isRequestTypeOption('display_events') && (count($authorizedButtons) == 0 || in_array('create_event', $authorizedButtons)) && !in_array('no_buttons', $authorizedButtons)) {
                        $langs->load("commercial");
                        if (!empty($user->rights->agenda->myactions->create) || !empty($user->rights->agenda->allactions->create)) {
							if ($object->isRequestTypeOption('display_events_create_same_for_each_requester') || $object->isRequestTypeOption('display_events_create_same_for_each_request_children')) {
								print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=create_event">' . $langs->trans("AddAction") . '</a></div>';
							} else {
								print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=confirm_create_event&confirm=yes">' . $langs->trans("AddAction") . '</a></div>';
							}
                        } else {
                            print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">' . $langs->trans("AddAction") . '</a></div>';
                        }
                    }
                }

				// Create/send survey
				if (isModEnabled('questionnaire') && !empty($requestManagerStatusDictionaryLine->fields['survey'])) {
					print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=rm_create_survey">' . $langs->trans("RequestManagerCreateSurvey") . '</a></div>';
					print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=rm_send_survey">' . $langs->trans("RequestManagerSendSurvey") . '</a></div>';
				}
            }

            // Not Resolved
            if ($object->statut_type == RequestManager::STATUS_TYPE_RESOLVED) {
                if ($user->rights->requestmanager->creer) {
                    print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=notresolved">'
                        . $langs->trans('ReOpen') . '</a></div>';
                } else {
                    print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">'
                        . $langs->trans("ReOpen") . '</a></div>';
                }
            }

            // ReOpen
            if ($object->statut_type == RequestManager::STATUS_TYPE_CLOSED) {
                if ($user->rights->requestmanager->cloturer) {
                    print '<div class="inline-block divButAction"><a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&token=' . newToken() . '&action=reopen">'
                        . $langs->trans('ReOpen') . '</a></div>';
                } else {
                    print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">'
                        . $langs->trans("ReOpen") . '</a></div>';
                }
            }

            // Delete
            if ($object->statut_type == RequestManager::STATUS_TYPE_INITIAL) {
                if ($user->rights->requestmanager->supprimer) {
                    print '<div class="inline-block divButAction"><a class="butActionDelete" href="' . $_SERVER["PHP_SELF"] . '?id=' . $object->id . '&action=delete&token='.newToken().'">'
                        . $langs->trans('Delete') . '</a></div>';
                } else {
                    print '<div class="inline-block divButAction"><a class="butActionRefused" title="' . dol_escape_js($langs->trans("NotAllowed")) . '" href="#">'
                        . $langs->trans("Delete") . '</a></div>';
                }
            }
        }

        print '</div>';

        $parameters = array();
        $reshook = $hookmanager->executeHooks('addMoreInfoBlocs', $parameters, $object, $action); // Note that $action and $object may have been
        print $hookmanager->resPrint;

        /*
       	 * Linked Objects
       	 */

        print '<div class="fichecenter"><div class="fichehalfleft">';

		/*
		 * Documents generes
		 */
		$filename = dol_sanitizeFileName($object->ref);
		$filedir = $conf->requestmanager->multidir_output[$object->entity] . "/" . dol_sanitizeFileName($object->ref);
		$urlsource = $_SERVER["PHP_SELF"] . "?id=" . $object->id;
		$genallowed = $user->rights->requestmanager->creer;
		$delallowed = $user->rights->requestmanager->creer;
		print '<a name="builddoc"></a>'; // ancre
		//print $formfile->showdocuments('requestmanager', $filename, $filedir, $urlsource, $genallowed, $delallowed, $object->modelpdf, 1, 0, 0, 28, 0, '', 0, '', $soc->default_lang, '', $object);
		print $formfile->showdocuments('requestmanager', $filename, $filedir, $urlsource, $genallowed, $delallowed, $object->modelpdf, 1, 0, 0, 28, 0, '', 0, '', '', '', $object);

		// Show links to link elements
		print '<div id="linked_objects_list">';
		// Show links to link elements
		$linktoelem = $form->showLinkToObjectBlock($object);

		$compatibleImportElementsList = false;
		if ($user->rights->requestmanager->creer && $object->statut_type == RequestManager::STATUS_TYPE_INITIAL) {
			$compatibleImportElementsList = array('requestmanager'); // import from linked elements
		}
		$somethingshown = $form->showLinkedObjectBlock($object, $linktoelem, $compatibleImportElementsList);
		print '</div>';

		// Children Request
		if ($object->isRequestTypeOption('display_children_request')) {
			print '<div id="children_request_linked_objects_list">';
			$formrequestmanager->showChildrenRequestBlock($object);
			print '</div>';
		}

		// Children Request linked objects
		if ($object->isRequestTypeOption('display_children_request')) {
			print '<div id="children_request_list">';
			$formrequestmanager->showChildrenRequestLinkedObjectsBlock($object);
			print '</div>';
		}

        print '</div><div class="fichehalfright"><div class="ficheaddleft">';

		// List of actions on element
		include_once DOL_DOCUMENT_ROOT.'/core/class/html.formactions.class.php';
		$formactions = new FormActions($db);
		$save_right = $user->rights->agenda->myactions->create;
		$user->rights->agenda->myactions->create = 0;
		//$somethingshown = $formactions->showactions($object, 'requestmanager', $socid, 1);
		$somethingshown = $formactions->showactions($object, 'requestmanager', 0, 1);
		$user->rights->agenda->myactions->create = $save_right;

		print '</div></div></div>';
    } elseif (isModEnabled('agenda') && $object->isRequestTypeOption('display_messages')) {
        /*
         * Affiche formulaire message
         */

        print '<div id="formmessagebeforetitle" name="formmessagebeforetitle"></div>';
        print '<div class="clearboth"></div>';
        print '<br>';
        print load_fiche_titre($langs->trans('RequestManagerAddMessage') . '&nbsp;<span id="rm-saving-status"></span>');

        print dol_get_fiche_head();

        // Cree l'objet formulaire message
        $formrequestmanagermessage = new FormRequestManagerMessage($db, $object);

        $loaded = $formrequestmanagermessage->load_datas_in_session();

        // Tableau des parametres complementaires du post
        $formrequestmanagermessage->param['action'] = $action;
        $formrequestmanagermessage->param['models_id'] = GETPOST('modelmessageselected', 'int');
        $formrequestmanagermessage->param['knowledgebase_ids'] = GETPOST('knowledgebaseselected', 'array');
        $formrequestmanagermessage->param['returnurl'] = $_SERVER["PHP_SELF"] . '?id=' . $object->id;

        // Init list of files
        if (!$loaded && GETPOST("messagemode") == 'init') {
            $formrequestmanagermessage->clear_attached_files();
//            $formrequestmanagermessage->add_attached_files($file, basename($file), dol_mimetype($file));
        }

        // Show form
        print $formrequestmanagermessage->get_message_form();

        print dol_get_fiche_end();
    }
}

// End of page
llxFooter();
$db->close();
