<?php
/* Copyright (C) 2024      Easya-Solutions       <support@easya.solutions>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/**
 * \file    htdocs/requestmanager/class/actions_requestmanager.class.php
 * \ingroup requestmanager
 * \brief   Example hook overload.
 *
 * Put detailed description here.
 */

/**
 * Class ActionsRequestManager
 */
class ActionsRequestManager
{
    /**
     * @var DoliDB Database handler.
     */
    public $db;
    /**
     * @var string Error
     */
    public $error = '';
    /**
     * @var array Errors
     */
    public $errors = array();

    /**
     * @var array Hook results. Propagated to $hookmanager->resArray for later reuse
     */
    public $results = array();

    /**
     * @var string String displayed by executeHook() immediately after return
     */
    public $resprints;

	/**
	 * @var int		Priority of hook (50 is used if value is not defined)
	 */
	public $priority = 90;

    /**
     * Constructor
     *
     * @param        DoliDB $db Database handler
     */
    public function __construct($db)
    {
        $this->db = $db;
    }

    /**
     * Overloading the updateSession function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function updateSession($parameters, &$object, &$action, $hookmanager)
    {
        return $this->_managerModuleHistory($parameters, $object, $action, $hookmanager);
    }

    /**
     * Overloading the afterLogin function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function afterLogin($parameters, &$object, &$action, $hookmanager)
    {
        return $this->_managerModuleHistory($parameters, $object, $action, $hookmanager);
    }

	/**
	 * Overloading the getTabInfo function : replacing the parent's function with the one below
	 *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
	 */
	public function getTabInfo($parameters, &$object, &$action, $hookmanager)
	{
		global $langs;

		if ($object->element == 'requestmanager') {
			$langs->load('requestmanager@requestmanager');

			$morehtmlref = '<div class="refidno">';
			// External Reference
			$morehtmlref .= $langs->trans('RequestManagerExternalReference') . ' : ' . $object->ref_ext;
			// Type
			$morehtmlref .= '<br>' . $langs->trans('RequestManagerType') . ' : ' . $object->getLibType();
			// Label
			$morehtmlref .= '<br>' . $langs->trans('RequestManagerLabel') . ' : ' . $object->label;
			// Parent request
			if ($object->fk_parent > 0) {
				$object->fetch_parent();
				$morehtmlref .= '<br>' . $langs->trans('RequestManagerParentRequest') . ' : ' . $object->parent->getNomUrl(1, 'parent_path');
			}
			$morehtmlref .= '</div>';

			$parameters['tab_info']['banner_tab']['more_html_ref'] = $morehtmlref;
		}

		return 0;
	}

    /**
     * Manage for the module History
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
    protected function _managerModuleHistory($parameters, &$object, &$action, $hookmanager)
    {
        if (isModEnabled('history')) {
            if (preg_match('/\/history\/history\.php/i', $_SERVER["PHP_SELF"]) && GETPOST('type_object') == 'requestmanager') {
                dol_include_once('/requestmanager/lib/requestmanager.lib.php');
                dol_include_once('/requestmanager/class/requestmanager.class.php');
            }
        }

        return 0;
    }

    /**
     * Overloading the addSearchEntry function : replacing the parent's function with the one below
     *
     * @param   array			$parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          &$action        Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function addSearchEntry($parameters, &$object, &$action, $hookmanager)
    {
        global $langs;
        $langs->load('requestmanager@requestmanager');

        $search_boxvalue = $parameters['search_boxvalue'];
        //$arrayresult = $parameters['arrayresult'];

        $arrayresult['searchintorequestmanager']=array(
            'position'=>16,
            'shortcut'=>'R',
            'img'=>'object_requestmanager@requestmanager',
            'label'=>$langs->trans("RequestManagerSearchIntoRequests", $search_boxvalue),
            'text'=>img_picto('','object_requestmanager@requestmanager').' '.$langs->trans("RequestManagerSearchIntoRequests", $search_boxvalue),
            'url'=>dol_buildpath('/requestmanager/list.php', 1).($search_boxvalue?'?sall='.urlencode($search_boxvalue):'')
        );

        $this->results = $arrayresult;

        return 0; // or return 1 to replace standard code
    }

    /**
     * Overloading the addMoreActionsButtons function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function addMoreActionsButtons($parameters, &$object, &$action, $hookmanager)
    {
        global $user, $langs;

        $contexts = explode(':', $parameters['context']);

        if (in_array('thirdpartycard', $contexts) ||
			in_array('commcard', $contexts) ||
			in_array('contactcard', $contexts) ||
            in_array('suppliercard', $contexts) ||
            in_array('propalcard', $contexts) ||
	        in_array('supplier_proposalcard', $contexts) ||
            in_array('ordercard', $contexts) ||
            in_array('invoicecard', $contexts) ||
	        in_array('ordersuppliercard', $contexts) ||
	        in_array('invoicesuppliercard', $contexts) ||
            in_array('interventioncard', $contexts) ||
            in_array('contractcard', $contexts) ||
	        in_array('projectcard', $contexts) ||
            in_array('expensereportcard', $contexts)
        ) {
            $langs->load('requestmanager@requestmanager');

			$socid = $object->element == 'societe' ? $object->id : $object->socid;
			$contact_id = $object->element == 'contact' ? $object->id : 0;
            $origin = $object->element != 'societe' && $object->element != 'contact' ? '&origin=' . urlencode($object->element) . '&originid=' . $object->id : '';

            if ($user->rights->requestmanager->creer)
                print '<div class="inline-block divButAction"><a class="butAction" href="' . dol_buildpath('/requestmanager/card.php', 2) . '?action=create&token=' . newToken(). ($socid > 0 ? '&socid=' . $socid : '') . ($contact_id > 0 ? '&requester_ids[]=' . $contact_id : '') . $origin . '">' . $langs->trans("RequestManagerAddRequest") . '</a></div>';
            else
                print '<div class="inline-block divButAction"><a class="butActionRefused" href="#" title="' . $langs->trans("NotEnoughPermissions") . '">' . $langs->trans("RequestManagerAddRequest") . '</a></div>';
        }

        return 0;
    }

    /**
     * Overloading the doActions function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function doActions($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $user, $langs;

        $contexts = explode(':', $parameters['context']);

        if (in_array('actioncard', $contexts)) {
            if ($action == 'create' || $action == 'add') {
                $actioncode = GETPOST('actioncode', 'alpha');

                if ($actioncode == 'AC_RM_OUT' || $actioncode == 'AC_RM_PRIV' || $actioncode == 'AC_RM_IN') {
                    $langs->load('requestmanager@requestmanager');

                    $this->errors[] = $langs->trans('RequestManagerErrorCanOnlyCreateMessageFromRequest');
                    $action = '';
                    return -1;
                }
            } else {
                $id = GETPOST('id', 'int');
                $object->fetch($id);

                if ($object->type_code == 'AC_RM_OUT' || $object->type_code == 'AC_RM_PRIV' || $object->type_code == 'AC_RM_IN') {
                    $langs->load('requestmanager@requestmanager');

                    if ($action == 'edit' || $action == 'update') {
                        /*$this->errors[] = $langs->trans('RequestManagerErrorMessageCanNotBeModified');
                        $action = '';
                        return -1;
                        /*$error = 0;
                        $cancel = GETPOST('cancel', 'alpha');
                        if (empty($cancel) && $user->rights->requestmanager->creer) {
                            dol_include_once('/requestmanager/class/requestmanagermessage.class.php');
                            $requestmanagermessage = new RequestManagerMessage($this->db);

                            require_once DOL_DOCUMENT_ROOT . '/comm/action/class/cactioncomm.class.php';
                            $cactioncomm = new CActionComm($this->db);

                            $backtopage = GETPOST('backtopage', 'alpha');

                            $fulldayevent = GETPOST('fullday');
                            $aphour = GETPOST('aphour');
                            $apmin = GETPOST('apmin');
                            $p2hour = GETPOST('p2hour');
                            $p2min = GETPOST('p2min');
                            $percentage = in_array(GETPOST('status'), array(-1, 100)) ? GETPOST('status') : (in_array(GETPOST('complete'), array(-1, 100)) ? GETPOST('complete') : GETPOST("percentage"));    // If status is -1 or 100, percentage is not defined and we must use status

                            // Clean parameters
                            if ($aphour == -1) $aphour = '0';
                            if ($apmin == -1) $apmin = '0';
                            if ($p2hour == -1) $p2hour = '0';
                            if ($p2min == -1) $p2min = '0';

                            $requestmanagermessage->fetch($id);
                            $requestmanagermessage->fetch_userassigned();

                            $datep = dol_mktime($fulldayevent ? '00' : $aphour, $fulldayevent ? '00' : $apmin, 0, $_POST["apmonth"], $_POST["apday"], $_POST["apyear"]);
                            $datef = dol_mktime($fulldayevent ? '23' : $p2hour, $fulldayevent ? '59' : $p2min, $fulldayevent ? '59' : '0', $_POST["p2month"], $_POST["p2day"], $_POST["p2year"]);

                            $requestmanagermessage->fk_action = dol_getIdFromCode($this->db, GETPOST("actioncode"), 'c_actioncomm');
                            $requestmanagermessage->label = GETPOST("label");
                            $requestmanagermessage->datep = $datep;
                            $requestmanagermessage->datef = $datef;
                            $requestmanagermessage->percentage = $percentage;
                            $requestmanagermessage->priority = GETPOST("priority");
                            $requestmanagermessage->fulldayevent = GETPOST("fullday") ? 1 : 0;
                            $requestmanagermessage->location = GETPOST('location');
                            $requestmanagermessage->socid = GETPOST("socid");
                            $requestmanagermessage->contactid = GETPOST("contactid", 'int');
                            //$requestmanagermessage->societe->id = $_POST["socid"];			// deprecated
                            //$requestmanagermessage->contact->id = $_POST["contactid"];		// deprecated
                            $requestmanagermessage->fk_project = GETPOST("projectid", 'int');
                            $requestmanagermessage->note = GETPOST("note");
                            $requestmanagermessage->pnote = GETPOST("note");
                            $requestmanagermessage->fk_element = GETPOST("fk_element");
                            $requestmanagermessage->elementtype = GETPOST("elementtype");

                            if (!$datef && $percentage == 100) {
                                $error++;
                                $donotclearsession = 1;
                                setEventMessages($langs->transnoentitiesnoconv("ErrorFieldRequired", $langs->transnoentitiesnoconv("DateEnd")), $requestmanagermessage->errors, 'errors');
                                $action = 'edit';
                            }

                            $transparency = (GETPOST("transparency") == 'on' ? 1 : 0);

                            // Users
                            $listofuserid = array();
                            if (!empty($_SESSION['assignedtouser']))    // Now concat assigned users
                            {
                                // Restore array with key with same value than param 'id'
                                $tmplist1 = json_decode($_SESSION['assignedtouser'], true);
                                $tmplist2 = array();
                                foreach ($tmplist1 as $key => $val) {
                                    if ($val['id'] > 0 && $val['id'] != $assignedtouser) $listofuserid[$val['id']] = $val;
                                }
                            } else {
                                $assignedtouser = (!empty($requestmanagermessage->userownerid) && $requestmanagermessage->userownerid > 0 ? $requestmanagermessage->userownerid : 0);
                                if ($assignedtouser) $listofuserid[$assignedtouser] = array('id' => $assignedtouser, 'mandatory' => 0, 'transparency' => ($user->id == $assignedtouser ? $transparency : ''));    // Owner first
                            }

                            $requestmanagermessage->userassigned = array();
                            $requestmanagermessage->userownerid = 0; // Clear old content
                            $i = 0;
                            foreach ($listofuserid as $key => $val) {
                                if ($i == 0) $requestmanagermessage->userownerid = $val['id'];
                                $requestmanagermessage->userassigned[$val['id']] = array('id' => $val['id'], 'mandatory' => 0, 'transparency' => ($user->id == $val['id'] ? $transparency : ''));
                                $i++;
                            }

                            $requestmanagermessage->transparency = $transparency;        // We set transparency on event (even if we can also store it on each user, standard says this property is for event)

                            if (!empty($conf->global->AGENDA_ENABLE_DONEBY)) {
                                if (GETPOST("doneby")) $requestmanagermessage->userdoneid = GETPOST("doneby", "int");
                            }

                            // Check parameters
                            if (!GETPOST('actioncode') > 0) {
                                $error++;
                                $donotclearsession = 1;
                                $action = 'edit';
                                setEventMessages($langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("Type")), null, 'errors');
                            } else {
                                $result = $cactioncomm->fetch(GETPOST('actioncode'));
                            }
                            if (empty($requestmanagermessage->userownerid)) {
                                $error++;
                                $donotclearsession = 1;
                                $action = 'edit';
                                setEventMessages($langs->trans("ErrorFieldRequired", $langs->transnoentitiesnoconv("ActionsOwnedBy")), null, 'errors');
                            }

                            // Fill array 'array_options' with data from add form
                            $extrafields = new ExtraFields($this->db);
                            $extralabels = $extrafields->fetch_name_optionals_label($requestmanagermessage->table_element);
                            $ret = $extrafields->setOptionalsFromPost($extralabels, $requestmanagermessage);
                            if ($ret < 0) $error++;

                            if (!$error) {
                                $this->db->begin();

                                $result = $requestmanagermessage->update($user);

                                if ($result > 0) {
                                    unset($_SESSION['assignedtouser']);

                                    $this->db->commit();
                                } else {
                                    setEventMessages($requestmanagermessage->error, $requestmanagermessage->errors, 'errors');
                                    $this->db->rollback();
                                }
                            }
                        }

                        if (!$error) {
                            if (!empty($backtopage)) {
                                unset($_SESSION['assignedtouser']);
                                header("Location: " . $backtopage);
                                exit;
                            }
                        }

                        return 1;*/
                    } elseif ($action == 'confirm_delete' && GETPOST("confirm") == 'yes' && $user->rights->requestmanager->creer) {
                        dol_include_once('/requestmanager/class/requestmanagermessage.class.php');
                        $requestmanagermessage = new RequestManagerMessage($this->db);

                        $id = GETPOST('id', 'int');
                        $requestmanagermessage->fetch($id);

                        $result = $requestmanagermessage->delete();

                        if ($result >= 0) {
                            header("Location: index.php");
                            exit;
                        } else {
                            setEventMessages($requestmanagermessage->error, $requestmanagermessage->errors, 'errors');
                        }
                        return 1;
                    }
                }
            }
        } elseif (!empty($conf->global->REQUESTMANAGER_TIMESLOTS_ACTIVATE) && in_array('contractcard', $contexts)) {
            if ($action == 'update_extras' && GETPOST('attribute') == 'rm_timeslots_periods') {
                // fetch optionals attributes and labels
                $ef = new ExtraFields($this->db);
                $extralabels = $ef->fetch_name_optionals_label($object->table_element);
                $ret = $ef->setOptionalsFromPost($extralabels, $object, GETPOST('attribute'));

                dol_include_once('/requestmanager/lib/requestmanagertimeslots.lib.php');
                $res = requestmanagertimeslots_get_periods($object->array_options['options_rm_timeslots_periods']);
                if (!is_array($res)) {
                    $action = 'edit_extras';
                    $this->errors[] = $langs->trans('RequestManagerTimeSlotsPeriodsName') . ': ' . $res;
                    return 1;
                }
            }
        }

        // Management of the user group(s) in charge for the planning
        //----------------------------------------------------------------------
        if (!empty($conf->global->REQUESTMANAGER_PLANNING_ACTIVATE) && (in_array('thirdpartycard', $contexts) || in_array('commcard', $contexts))) {
            if ($action == 'set_edit_usergroups_in_charge' && $user->rights->requestmanager->usergroup_in_charge->manage) {
                if (!($object->id > 0)) {
                    $id = (GETPOST('socid','int') ? GETPOST('socid','int') : GETPOST('id','int'));
                    $object->fetch($id);
                }
                $request_types_planned = !empty($conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) ? explode(',', $conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) : array();
                dol_include_once('/advancedictionaries/class/dictionary.class.php');
                $requestmanagerrequesttype = Dictionary::getDictionary($this->db, 'requestmanager', 'requestmanagerrequesttype');
                $requestmanagerrequesttype->fetch_lines(1);

                dol_include_once('/requestmanager/class/requestmanagerplanning.class.php');
                $requestmanagerplanning = new RequestManagerPlanning($this->db);

                foreach ($requestmanagerrequesttype->lines as $request_type) {
                    if (!in_array($request_type->id, $request_types_planned) || !isset($_POST['usergroups_in_charge_' . $request_type->id])) continue;

                    $usergroups_in_charge = GETPOST('usergroups_in_charge_' . $request_type->id, 'array');

                    // Save users in charge for the request type
                    if ($requestmanagerplanning->setUserGroupsInChargeForCompany($object->id, $request_type->id, $usergroups_in_charge) > 0) {
                        header('Location: ' . $_SERVER["PHP_SELF"] . '?socid=' . $object->id);
                        exit;
                    } else {
                        setEventMessages($requestmanagerplanning->error, $requestmanagerplanning->errors, 'errors');
                    }
                }

                return 1;
            }
        }

        return 0;
    }

    /**
     * Overloading the formObjectOptions function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function formObjectOptions($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $langs, $form, $user;

        $contexts = explode(':', $parameters['context']);

		// Management of the creation of the same event for x requester and/or each request children
		//---------------------------------------------------------------------------------------------
		if (in_array('actioncard', $contexts)) {
			$create_same_for_each_requester = GETPOST('create_same_for_each_requester', 'aZ09');
			$create_same_for_each_request_children = GETPOST('create_same_for_each_request_children', 'aZ09');

			if (!empty($create_same_for_each_requester) || !empty($create_same_for_each_request_children)) {
				$create_same_for_each_requester = !empty($create_same_for_each_requester) ? 'true' : 'false';
				$create_same_for_each_request_children = !empty($create_same_for_each_request_children) ? 'true' : 'false';

				print '<tr><td id="requestmanager_options">' . $langs->trans("RequestManagerMessageNotify") . '</td><td>';
				print <<<SCRIPT
            <script type="text/javascript">
                jQuery(document).ready(function () {
                	var _td_requestmanager_options = jQuery('#requestmanager_options');
                	var _form = _td_requestmanager_options.closest('form');

                	if ($create_same_for_each_requester) {
						_form.append('<input type="hidden" id="create_same_for_each_requester" name="create_same_for_each_requester" value="1" />');
                	}
                	if ($create_same_for_each_request_children) {
						_form.append('<input type="hidden" id="create_same_for_each_request_children" name="create_same_for_each_request_children" value="1" />');
						$('#socid').closest('tr').remove();
						$('[name="fk_element"]').prev('tr').hide();
                	}
					$('#contactid').closest('tr').remove();

                    _td_requestmanager_options.remove();
                });
            </script>
SCRIPT;
				print "</td></tr>";
			}
		}

		if (in_array('actioncard', $contexts) &&
            ($object->type_code == 'AC_RM_OUT' || $object->type_code == 'AC_RM_PRIV' || $object->type_code == 'AC_RM_IN') &&
            $action != "create" && empty($object->context['requestmanager_hook'])
        ) {
            $langs->load('requestmanager@requestmanager');

            dol_include_once('/requestmanager/class/requestmanagermessage.class.php');
            $requestManagerMessage = new RequestManagerMessage($this->db);
            $requestManagerMessage->fetch($object->id);
            $requestManagerMessage->fetch_optionals();

            require_once DOL_DOCUMENT_ROOT . '/core/class/extrafields.class.php';
            $ef = new ExtraFields($this->db);
            $extralabels = $ef->fetch_name_optionals_label($requestManagerMessage->table_element);

            $requestManagerMessage->fetch_knowledge_base(1);
            print '<tr><td class="nowrap" class="titlefield">' . $langs->trans("RequestManagerMessageKnowledgeBase") . '</td>';
            print '<td colspan="3">';
            $toprint = array();
            foreach ($requestManagerMessage->knowledge_base_list as $knowledge_base) {
                $toprint[] = '<li class="select2-search-choice-dolibarr noborderoncategories style="background: #aaa">' . $knowledge_base->fields['code'] . ' - ' . $knowledge_base->fields['title'] . '</li>';
            }
            print '<div class="select2-container-multi-dolibarr" style="width: 90%;"><ul class="select2-choices-dolibarr">' . implode(' ', $toprint) . '</ul></div>';
            print '</td>';
            print '</tr>';

            print '<tr><td class="nowrap" class="titlefield">' . $langs->trans("RequestManagerMessageNotify") . '</td>';
            print ($action == "edit" ? '<td>' : '<td colspan="3">');
            print '<input type="checkbox" id="rm_notify_assigned" name="rm_notify_assigned" value="1"' . (!empty($requestManagerMessage->notify_assigned) ? ' checked="checked"' : '') . ' disabled="disabled" />';
            print '&nbsp;' . $langs->trans("RequestManagerAssigned");
            print ' &nbsp; ';
            print '<input type="checkbox" id="rm_notify_requesters" name="rm_notify_requesters" value="1"' . (!empty($requestManagerMessage->notify_requesters) ? ' checked="checked"' : '') . ' disabled="disabled" />';
            print '&nbsp;' . $langs->trans("RequestManagerRequesterContacts");
            print ' &nbsp; ';
            print '<input type="checkbox" id="rm_notify_watchers" name="rm_notify_watchers" value="1"' . (!empty($requestManagerMessage->notify_watcher) ? ' checked="checked"' : '') . ' disabled="disabled" />';
            print '&nbsp;' . $langs->trans("RequestManagerWatcherContacts");
            print "</td></tr>";

            // Other attributes
            print $requestManagerMessage->showOptionals($ef, 'view', array('colspan' => 3));

            return 1;
        }
        elseif (in_array('contractcard', $contexts)) {
            global $extrafields;

            if (empty($conf->global->REQUESTMANAGER_TIMESLOTS_ACTIVATE)) {
                unset($extrafields->attributes[$object->table_element]['label']['rm_timeslots_separator']);
                unset($extrafields->attributes[$object->table_element]['label']['rm_timeslots_periods']);
                unset($extrafields->attribute_label['rm_timeslots_separator']);
                unset($extrafields->attribute_label['rm_timeslots_periods']);
            } else {
                global $form;

                if (!isset($form)) {
                    require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';
                    $form = new Form($this->db);
                }
                $langs->load('requestmanager@requestmanager');
                $help = str_replace("'", "\\'", $form->textwithpicto('', $langs->trans("RequestManagerTimeSlotsPeriodsDesc"), 1, 'help', '', 0, 2, 'help_timeslots_periods'));

                $this->resprints = <<<SCRIPT
            <script type="text/javascript">
                $(document).ready(function () {
                    // Help button
                    var rm_timeslots_periods = $("td a[href*='&action=edit_extras&attribute=rm_timeslots_periods']").closest('tr').find('td:first-child');
                    if (!rm_timeslots_periods.length) {
                        rm_timeslots_periods = $("textarea#options_rm_timeslots_periods").closest('tr').find('table td:first-child');
                    }
                    rm_timeslots_periods.append(' $help');

                    // Disabled CKEDITOR
                    if (typeof CKEDITOR == "object") {
                        setTimeout(function () {
                            if (typeof CKEDITOR.instances != "undefined" && "options_rm_timeslots_periods" in CKEDITOR.instances) {
                                CKEDITOR.instances["options_rm_timeslots_periods"].destroy();
                            }
                        }, 500);
                    }
                });
            </script>
SCRIPT;
            }
        } elseif (in_array('projectcard', $contexts) && $action == 'create') {
            $origin = GETPOST('origin', 'alpha');
            $originid = GETPOST('originid', 'int');

            print '<input type="hidden" name="origin" value="'.htmlspecialchars($origin).'">';
            print '<input type="hidden" name="originid" value="'.htmlspecialchars($originid).'">';
        }

        // Management of the user group(s) in charge for the planning
        //----------------------------------------------------------------------
        if (!empty($conf->global->REQUESTMANAGER_PLANNING_ACTIVATE) && $user->rights->requestmanager->usergroup_in_charge->lire && (in_array('thirdpartycard', $contexts) || in_array('commcard', $contexts))) {
            $langs->load('requestmanager@requestmanager');
            dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
            $formrequestmanager = new FormRequestManager($this->db);

            $request_types_planned = !empty($conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) ? explode(',', $conf->global->REQUESTMANAGER_PLANNING_REQUEST_TYPE) : array();
            dol_include_once('/advancedictionaries/class/dictionary.class.php');
            $requestmanagerrequesttype = Dictionary::getDictionary($this->db, 'requestmanager', 'requestmanagerrequesttype');
            $requestmanagerrequesttype->fetch_lines(1);

            dol_include_once('/requestmanager/class/requestmanagerplanning.class.php');
            $requestmanagerplanning = new RequestManagerPlanning($this->db);
            $usergroups_in_charge = $action != 'create' ? $requestmanagerplanning->getUserGroupsInChargeForCompany($object->id) : array();

            // User groups in charge
            if ($action == 'create' || $action == 'edit') {
                foreach ($requestmanagerrequesttype->lines as $request_type) {
                    if (!in_array($request_type->id, $request_types_planned)) continue;
                    $LabelTagName = 'RequestManagerPlanningUserGroupsInChargeLabel_' . $request_type->fields['code'];
                    print '<tr><td>' . $langs->trans($langs->trans($LabelTagName) != $LabelTagName ? $LabelTagName : 'RequestManagerPlanningUserGroupsInChargeLabel', $request_type->fields['label']) . '</td>';
                    print '<td colspan="3">';
                    $usergroups_in_charge_for_request_type = isset($Post['usergroups_in_charge_' . $request_type->id]) ? $Post['usergroups_in_charge_' . $request_type->id] : (isset($usergroups_in_charge[$request_type->id]) ? $usergroups_in_charge[$request_type->id] : array());
                    print $formrequestmanager->multiselect_dolgroups($usergroups_in_charge_for_request_type, 'usergroups_in_charge_' . $request_type->id);
                    print '</td></tr>';
                }
            } else {
                foreach ($requestmanagerrequesttype->lines as $request_type) {
                    if (!in_array($request_type->id, $request_types_planned)) continue;
                    $LabelTagName = 'RequestManagerPlanningUserGroupsInChargeLabel_' . $request_type->fields['code'];
                    print '<tr><td>';
                    print '<table class="nobordernopadding" width="100%"><tr><td>';
                    print $langs->trans($langs->trans($LabelTagName) != $LabelTagName ? $LabelTagName : 'RequestManagerPlanningUserGroupsInChargeLabel', $request_type->fields['label']);
                    print '</td>';
                    if ($action != 'edit_usergroups_in_charge_' . $request_type->id && $user->rights->requestmanager->usergroup_in_charge->manage)
                        print '<td align="right"><a href="' . $_SERVER["PHP_SELF"] . '?action=edit_usergroups_in_charge_' . $request_type->id . '&token=' . newToken() . '&socid=' . $object->id . '">' . img_edit('', 1) . '</a></td>';
                    print '</tr></table>';
                    print '</td><td>';
                    if ($action == 'edit_usergroups_in_charge_' . $request_type->id && $user->rights->requestmanager->usergroup_in_charge->manage) {
                        print '<form name="edit_usergroups_in_charge" action="' . $_SERVER["PHP_SELF"] . '?socid=' . $object->id . '" method="post">';
                        print '<input type="hidden" name="token" value="' . $_SESSION ['newtoken'] . '">';
                        print '<input type="hidden" name="action" value="set_edit_usergroups_in_charge">';
                        $usergroups_in_charge_for_request_type = isset($Post['usergroups_in_charge_' . $request_type->id]) ? $Post['usergroups_in_charge_' . $request_type->id] : (isset($usergroups_in_charge[$request_type->id]) ? $usergroups_in_charge[$request_type->id] : array());
                        print $formrequestmanager->multiselect_dolgroups($usergroups_in_charge_for_request_type, 'usergroups_in_charge_' . $request_type->id);
                        print '<input type="submit" class="button" value="' . $langs->trans('Modify') . '">';
                        print '</form>';
                    } else {
                        require_once DOL_DOCUMENT_ROOT . '/user/class/usergroup.class.php';
                        $usergroup_static = new UserGroup($this->db);
                        $toprint = array();
                        $usergroups_in_charge_for_request_type = isset($usergroups_in_charge[$request_type->id]) ? $usergroups_in_charge[$request_type->id] : array();
                        foreach ($usergroups_in_charge_for_request_type as $usergroup_id) {
                            $usergroup_static->fetch($usergroup_id);
                            $toprint[] = $usergroup_static->name;
                        }
                        print implode(', ', $toprint);
                    }
                    print '</td></tr>';
                }
            }
        }

        // Management the edition of the extrafields by status of the request
        //----------------------------------------------------------------------
        if (!empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ACTIVATE)) {
            $langs->load('requestmanager@requestmanager');
            dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
            $formrequestmanager = new FormRequestManager($this->db);

            $element_types = $formrequestmanager->element_type_array();
            $element_types = array_keys($element_types);

            if (in_array($object->table_element, $element_types)) {
				if ($object->element == 'requestmanager') {
					/**
					 * @var RequestManager $object
					 */
					if (!empty($conf->global->REQUESTMANAGER_DISPLAY_EXTRAFIELDS_ON_REQUEST_CHILDREN) && $object->fk_parent > 0) {
						$object->fetch_parent();
						$linkedObjects['requestmanager'] = [$object->parent];
						$linkedObjectsIds['requestmanager'] = [$object->parent->id];
					} else {
						$linkedObjects['requestmanager'] = [$object];
						$linkedObjectsIds['requestmanager'] = [$object->id];
					}
				} else {
					$linkedObjectsIds = array();
					$linkedObjects = array();
					if ($object->element == 'project_task') {
						if ($action == 'create') {
							$project_id = GETPOST('id', 'int');
						} else {
							$project_id = $object->fk_project;
						}
						if ($project_id > 0) {
							require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
							$proj = new Project($this->db);
							$proj->fetch($project_id);
							$proj->fetchObjectLinked();
							$linkedObjectsIds = $proj->linkedObjectsIds;
							$linkedObjects = $proj->linkedObjects;
						}
					} elseif ($object->id > 0) {
						$object->fetchObjectLinked();
						$linkedObjectsIds = $object->linkedObjectsIds;
						$linkedObjects = $object->linkedObjects;
					}

					$origin = GETPOST('origin', 'alpha');
					$originid = GETPOST('originid', 'int');
					if ($origin == 'requestmanager' && $originid > 0 &&
						(empty($linkedObjectsIds['requestmanager']) || !in_array($originid, $linkedObjectsIds['requestmanager']))
					) {
						dol_include_once('/requestmanager/class/requestmanager.class.php');
						$requestmanager_tmp = new RequestManager($this->db);
						$requestmanager_tmp->fetch($originid);
						$linkedObjects['requestmanager'] = [$requestmanager_tmp];
						$linkedObjectsIds['requestmanager'][] = $originid;
					}
				}

				global $extrafields, $extrafields_task;
				$ef = &$extrafields;
				if ($object->table_element == 'projet_task' && is_object($extrafields_task)) {
					$ef = &$extrafields_task;
				}

				$element_table = $object->table_element;
				$extra_field_management = array();
				if (!empty($linkedObjects['requestmanager'])) {
					foreach ($linkedObjects['requestmanager'] as $request) {
						/**
						 * @var RequestManager $request
						 */
						$tmp = $request->getExtraFieldManagement();
						foreach ($tmp as $element_type => $element_type_list) {
							if ($element_table != $element_type) continue;

							foreach ($element_type_list as $extra_field_key => $extra_field_info) {
								if (!isset($extra_field_management[$extra_field_key])) {
									$extra_field_management[$extra_field_key] = array(
										'show' => false,
										'mandatory' => false,
										'readonly' => false,
									);
								}
								$extra_field_management[$extra_field_key]['show'] |= !empty($extra_field_info['show']);
								$extra_field_management[$extra_field_key]['mandatory'] |= !empty($extra_field_info['mandatory']);
								$extra_field_management[$extra_field_key]['readonly'] |= !empty($extra_field_info['readonly']);
							}
						}
					}
				}

				if (!empty($extra_field_management)) {
					$last_separator = null;
					$last_separator_has_extra_fields = false;
					$edition = $action == 'create' || $action == 'edit';
					foreach ($ef->attributes[$element_table]['label'] as $k => $l) {
						// Manage separator
						if ($ef->attributes[$element_table]['type'][$k] == 'separate') {
							if (isset($last_separate) && !$last_separator_has_extra_fields) $ef->attributes[$element_table]['list'][$last_separate] = 0;
							$last_separate = $k;
							$last_separator_has_extra_fields = false;
							continue;
						}

						$ef->attributes[$element_table]['required'][$k] |= !empty($extra_field_management[$k]['mandatory']);
						$ef->attributes[$element_table]['list'][$k] = !empty($extra_field_management[$k]['show']) && (!$edition || empty($extra_field_management[$k]['readonly'])) ? $ef->attributes[$element_table]['list'][$k] : 0;

						$perms = isset($ef->attributes[$element_table]['editperms'][$k]) ? dol_eval($ef->attributes[$element_table]['editperms'][$k], 1) : 1;
						$ef->attributes[$element_table]['editperms'][$k] = $perms && empty($extra_field_management[$k]['readonly']) ? 1 : 0;

						if (!empty($ef->attributes[$element_table]['list'][$k])) {
							$last_separator_has_extra_fields = true;
						}
					}
					// Manage separator
					if (isset($last_separate) && !$last_separator_has_extra_fields) $ef->attributes[$element_table]['list'][$last_separate] = 0;
				}
            }
        }

        return 0;
    }

    /**
     * Overloading the printTopRightMenu function : replacing the parent's function with the one below
     *
	 * @param   array			$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function printTopRightMenu($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $langs, $user;

        if (in_array('toprightmenu', explode(':', $parameters['context']))) {
            $out = '';

            if ($user->rights->requestmanager->lire) {
                $langs->load('requestmanager@requestmanager');

                // Load nb request and if is updated
                //----------------------------------------------------------------------
                // Last view date
                $user->fetch_optionals();
                $lastViewDate = isset($user->array_options['options_rm_last_check_follow_list_date']) ? $user->array_options['options_rm_last_check_follow_list_date'] : 0;
                if (is_string($lastViewDate)) $lastViewDate = strtotime($lastViewDate);

                dol_include_once('/requestmanager/class/requestmanager.class.php');
                $requestManager = new RequestManager($this->db);

                $isListsFollowModified = $requestManager->isListsFollowModified($lastViewDate) ? 1 : 0;
                $nbRequests = $requestManager->countMyAssignedRequests(array(RequestManager::STATUS_TYPE_INITIAL, RequestManager::STATUS_TYPE_IN_PROGRESS));

                // Oblyon compatibility
                //----------------------------------------------------------------------
                $text_selector = $conf->theme == "oblyon" ? 'a' : '.mainmenuaspan';

                // Add replace create request button class and request button text and blink
                //-------------------------------------------------------------------------------
                $out .= <<<SCRIPT
            <script type="text/javascript">
            	$(document).ready(function () {
            	    var rm_request_updated = $isListsFollowModified;
            	    var rm_top_menu_button_requestmanager = null;
            		
            		$.map($('li#mainmenutd_requestmanager'), function(item) {
                        var _this = $(item);
                        if (_this.find('a[href*="/requestmanager/lists_follow.php"]').length > 0) {
                            rm_top_menu_button_requestmanager = _this;
                        }
            		});

            		if (rm_top_menu_button_requestmanager) {
                        // Add count in the text of the request button
                        var rm_top_menu_button_requestmanager_text = rm_top_menu_button_requestmanager.find('$text_selector');
                        rm_top_menu_button_requestmanager_text.append(' (' + $nbRequests + ')');

                        // Blink managment if new status of my request
            		    rm_top_menu_button_requestmanager.addClass('rm_my_request_updated_blink');
                        var requestmanager_my_request_updated_blink = null;
                        requestmanager_update_my_request_updated();
             		} else {
            		    console.error('Request Manager: Request button not found in the top menu')
            		}

            		function requestmanager_update_my_request_updated() {
                        if (rm_request_updated && !requestmanager_my_request_updated_blink) {
                            // Start blink
                            requestmanager_my_request_updated_blink = setInterval(function() { rm_top_menu_button_requestmanager.toggleClass("rm_my_request_updated_blink_color"); }, 1000);
                        } else if (!rm_request_updated && requestmanager_my_request_updated_blink) {
                            // Stop blink
                            clearInterval(requestmanager_my_request_updated_blink);
                            requestmanager_my_request_updated_blink = null;
                        }
            		}
                });
            </script>
SCRIPT;
            }

			$hide_new_request_button = true;
            if ($user->rights->requestmanager->creer) {
                if (!empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_MENU) && empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU)) {
					$hide_new_request_button = false;
					$out .= <<<SCRIPT
            <script type="text/javascript">
            	$(document).ready(function () {
            	    var rm_top_menu_button_requestmanager_new = null;
            		
            		$.map($('li#mainmenutd_requestmanager'), function(item) {
                        var _this = $(item);
                        if (_this.find('a[href*="/requestmanager/card.php?action=create"]').length > 0) {
                            rm_top_menu_button_requestmanager_new = _this;
                        }
            		});

            		if (rm_top_menu_button_requestmanager_new) {
            		    // Replace class for create button
            		    rm_top_menu_button_requestmanager_new.find('.mainmenu.requestmanager').removeClass('requestmanager').addClass('requestmanager_new');
            		    // Oblyon theme
            		    rm_top_menu_button_requestmanager_new.find('.icon').removeClass('icon--requestmanager').addClass('icon--requestmanager_new');
            		} else {
            		    console.error('Request Manager: New request button not found in the top menu')
            		}
                });
            </script>
SCRIPT;
                } elseif (!empty($conf->global->REQUESTMANAGER_DISPLAY_BUTTON_CREATE_REQ_IN_TOP_RIGHT_MENU)) {
					$hide_new_request_button = false;
					$text = '<a href="' . dol_buildpath('/requestmanager/card.php', 1) . '?action=create&token=' . newToken() . '">';
                    $text.= img_picto($langs->trans("RequestManagerMenuTopCreateFast"), 'requestmanager_new.png@requestmanager', 'class="tr_menu_requestmanager_new"');
                    $text .= '</a>';
                    $out .= @Form::textwithtooltip('', $langs->trans("RequestManagerMenuTopCreateFast"), 2, 1, $text, 'login_block_elem', 2);
                }
            }
            if ($hide_new_request_button) {
            	// if test on menu dont work
				$out .= <<<SCRIPT
            <script type="text/javascript">
            	$(document).ready(function () {
            		$.map($('li#mainmenutd_requestmanager'), function(item) {
                        var _this = $(item);
                        if (_this.find('a[href*="/requestmanager/card.php?action=create"]').length > 0) {
                            _this.remove();
                        }
            		});
                });
            </script>
SCRIPT;
			}

            $this->resprints = $out;
        }

        return 0;
    }

    /**
     * Overloading the addEventConfidentialityElementOrigin function : replacing the parent's function with the one below
     *
     * @param   array			$parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          &$action        Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function addEventConfidentialityElementOrigin($parameters, &$object, &$action, $hookmanager)
    {
        global $langs;

        if (in_array('eventconfidentialitydao', explode(':', $parameters['context']))) {
            $this->results = array(
                'requestmanager' => $langs->trans('RequestManagerRequest'),
            );

            return 1;
        }

        return 0;
    }

    /**
     * Overloading the showLinkToObjectBlock function : replacing the parent's function with the one below
     *
     * @param   array         	$parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          &$action        Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
   	public function showLinkToObjectBlock($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $langs;

        $contexts = explode(':', $parameters['context']);

		$possiblelinks=array();
		if (is_object($object->thirdparty) && ! empty($object->thirdparty->id) && $object->thirdparty->id > 0) {
			$listofidcompanytoscan = $object->thirdparty->id;
			if (($object->thirdparty->parent > 0) && !empty($conf->global->THIRDPARTY_INCLUDE_PARENT_IN_LINKTO)) $listofidcompanytoscan .= ',' . $object->thirdparty->parent;
			if (($object->fk_project > 0) && !empty($conf->global->THIRDPARTY_INCLUDE_PROJECT_THIRDPARY_IN_LINKTO)) {
				include_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
				$tmpproject = new Project($this->db);
				$tmpproject->fetch($object->fk_project);
				if ($tmpproject->socid > 0 && ($tmpproject->socid != $object->thirdparty->id)) $listofidcompanytoscan .= ',' . $tmpproject->socid;
				unset($tmpproject);
			}

			$possiblelinks['requestmanager'] = array(
				'enabled' => isModEnabled('requestmanager'),
				'perms' => 1,
				'label' => 'RequestManagerLinkToRequestManager',
				'sql' => "SELECT s.rowid AS socid, GROUP_CONCAT(s.nom SEPARATOR ', ') AS `name`, s.client, t.rowid, t.ref, CONCAT(crmrt.label, ' - ', t.label) AS ref_client FROM " . MAIN_DB_PREFIX . "societe as s" .
					" INNER JOIN  " . MAIN_DB_PREFIX . "requestmanager as t ON t.fk_soc = s.rowid" .
					" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
					"   ON (ee.sourcetype = 'requestmanager' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
					"   OR (ee.targettype = 'requestmanager' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
					" LEFT JOIN  " . MAIN_DB_PREFIX . "c_requestmanager_request_type as crmrt ON crmrt.rowid = t.fk_type" .
					' WHERE t.fk_soc IN (' . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('requestmanager') . ')' .
					' AND ee.rowid IS NULL' .
					' AND t.rowid != ' . $object->id .
					' GROUP BY t.rowid',
			);
			// Todo modifier dans le standard des projets lier
			$possiblelinks['project'] = array(
				'enabled' => isModEnabled('projet'),
				'perms' => 1,
				'label' => 'RequestManagerLinkToProject',
				'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.title AS ref_client FROM " . MAIN_DB_PREFIX . "societe as s" .
					" INNER JOIN  " . MAIN_DB_PREFIX . "projet as t ON t.fk_soc = s.rowid" .
					" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
					"   ON (ee.sourcetype = 'projet' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
					"   OR (ee.targettype = 'projet' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
					" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('project') . ')' .
					' AND ee.rowid IS NULL' .
					' AND t.rowid != ' . $object->id .
					' GROUP BY t.rowid, s.rowid',
			);

			if (in_array('requestmanagercard', $contexts) && empty($conf->global->REQUESTMANAGER_DISABLE_SHOW_LINK_TO_OBJECT_BLOCK)) {
				$possiblelinks['propal'] = array(
					'enabled' => isModEnabled('propal'),
					'perms' => 1,
					'label' => 'LinkToProposal',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_client, t.total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "propal as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'propal' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'propal' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('propal') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$possiblelinks['order'] = array(
					'enabled' => isModEnabled('commande'),
					'perms' => 1,
					'label' => 'LinkToOrder',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_client, t.total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "commande as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'commande' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'commande' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('commande') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$v14p = version_compare(DOL_VERSION, '14.0.0', '>=');
				$possiblelinks['invoice'] = array(
					'enabled' => isModEnabled('facture'),
					'perms' => 1,
					'label' => 'LinkToInvoice',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, " . (version_compare(DOL_VERSION, "10.0.0") >= 0 ? "t.ref" : "t.facnumber") . " AS ref, t.ref_client, " . ($v14p ? "t.total_ht" : "t.total") . " as total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "facture as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'facture' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'facture' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('facture') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				if ((float)DOL_VERSION >= 8.0) {
					$possiblelinks['invoice_template'] = array(
						'enabled' => isModEnabled('facture'),
						'perms' => 1,
						'label' => 'LinkToTemplateInvoice',
						'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.titre as ref, " . ($v14p ? "t.total_ht" : "t.total") . " as total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
							" INNER JOIN  " . MAIN_DB_PREFIX . "facture_rec as t ON t.fk_soc = s.rowid" .
							" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
							"   ON (ee.sourcetype = 'facture' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
							"   OR (ee.targettype = 'facture' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
							" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('facture') . ')' .
							' AND ee.rowid IS NULL' .
							' AND t.rowid != ' . $object->id .
							' GROUP BY t.rowid, s.rowid',
					);
				}
				$possiblelinks['contrat'] = array(
					'enabled' => isModEnabled('contrat'),
					'perms' => 1,
					'label' => 'LinkToContract',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_supplier, '' as total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "contrat as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'contrat' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'contrat' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('contract') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$possiblelinks['fichinter'] = array(
					'enabled' => isModEnabled('ficheinter'),
					'perms' => 1,
					'label' => 'LinkToIntervention',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "fichinter as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'fichinter' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'fichinter' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('intervention') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$possiblelinks['supplier_proposal'] = array(
					'enabled' => isModEnabled('supplier_proposal'),
					'perms' => 1,
					'label' => 'LinkToSupplierProposal',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, '' as ref_supplier, t.total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "supplier_proposal as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'supplier_proposal' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'supplier_proposal' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('supplier_proposal') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$possiblelinks['order_supplier'] = array(
					'enabled' => isModEnabled('supplier_order'),
					'perms' => 1,
					'label' => 'LinkToSupplierOrder',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_supplier, t.total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "commande_fournisseur as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'order_supplier' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'order_supplier' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('commande_fournisseur') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);
				$possiblelinks['invoice_supplier'] = array(
					'enabled' => isModEnabled('supplier_invoice'),
					'perms' => 1,
					'label' => 'LinkToSupplierInvoice',
					'sql' => "SELECT s.rowid as socid, s.nom as name, s.client, t.rowid, t.ref, t.ref_supplier, t.total_ht FROM " . MAIN_DB_PREFIX . "societe as s" .
						" INNER JOIN  " . MAIN_DB_PREFIX . "facture_fourn as t ON t.fk_soc = s.rowid" .
						" LEFT JOIN  " . MAIN_DB_PREFIX . "element_element as ee" .
						"   ON (ee.sourcetype = 'invoice_supplier' AND ee.fk_source = t.rowid AND ee.targettype = '" . $object->element . "' AND ee.fk_target = " . $object->id . ")" .
						"   OR (ee.targettype = 'invoice_supplier' AND ee.fk_target = t.rowid AND ee.sourcetype = '" . $object->element . "' AND ee.fk_source = " . $object->id . ")" .
						" WHERE t.fk_soc IN (" . $listofidcompanytoscan . ') AND t.entity IN (' . getEntity('facture_fourn') . ')' .
						' AND ee.rowid IS NULL' .
						' AND t.rowid != ' . $object->id .
						' GROUP BY t.rowid, s.rowid',
				);

				$this->results = $possiblelinks;
				return 1;
			}
		}

        $this->results = $possiblelinks;
        return 0;
    }

    /**
     * Overloading the addObjectDataIntoList function : replacing the parent's function with the one below
     *
     * @param   array         	$parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          &$action        Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function addObjectDataIntoList($parameters, &$object, &$action, $hookmanager)
    {
        if (in_array('infrassearchdao', explode(':', $parameters['context']))) {
            $this->results = array(
                'requestmanager' => array(
                    'interface_label' => 'RequestManagerInfraSSearchLib',
                    'tables' => array(MAIN_DB_PREFIX . 'requestmanager', MAIN_DB_PREFIX . 'requestmanager_extrafields', MAIN_DB_PREFIX . 'requestmanagerdet'),
                    'objname' => 'RequestManager',
                    'objlibs' => array('/requestmanager/class/requestmanager.class.php'),
                    'langs' => array('requestmanager@requestmanager'),
                    'complete_label' => 'label',
                    'sql_join' => 'LEFT JOIN ' . MAIN_DB_PREFIX . 'requestmanager_extrafields ON (' . MAIN_DB_PREFIX . 'requestmanager.rowid = ' . MAIN_DB_PREFIX . 'requestmanager_extrafields.fk_object)' .
                        ' LEFT JOIN ' . MAIN_DB_PREFIX . 'requestmanagerdet ON (' . MAIN_DB_PREFIX . 'requestmanager.rowid = ' . MAIN_DB_PREFIX . 'requestmanagerdet.fk_requestmanager)',
                    'id_field' => MAIN_DB_PREFIX . 'requestmanager.rowid',
                )
            );
        }

        return 0;
    }

    /**
     * Overloading the mainCardTabAddMore function : replacing the parent's function with the one below
     *
     * @param   array         	$parameters     Hook metadatas (context, etc...)
     * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param   string          &$action        Current action (if set). Generally create or edit or null
     * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
     * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function mainCardTabAddMore($parameters, &$object, &$action, $hookmanager)
    {
        $contexts = explode(':', $parameters['context']);

        if (in_array('projectcard', $contexts) && $action != 'presend') {
            global $form;
            print '<div class="fichecenter"><div class="fichehalfleft">';

            // Show links to link elements
            $linktoelem = $form->showLinkToObjectBlock($object, null, array('projet'));
            $somethingshown = $form->showLinkedObjectBlock($object, $linktoelem);

            print '</div><div class="fichehalfright"><div class="ficheaddleft">';

            print '</div></div></div>';
        }

        return 0;
    }

    /**
     * Overloading the showLinkedObjectBlock function : replacing the parent's function with the one below
     *
	 * @param   array         	$parameters     Hook metadatas (context, etc...)
	 * @param   CommonObject    &$object        The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
	 * @param   string          &$action        Current action (if set). Generally create or edit or null
	 * @param   HookManager     $hookmanager    Hook manager propagated to allow calling another hook
	 * @return  int                             Return integer < 0 on error, 0 on success, 1 to replace standard code
     */
	public function showLinkedObjectBlock($parameters, &$object, &$action, $hookmanager)
    {
        global $conf, $langs;
        $morehtmlright = $parameters['morehtmlright'];
        $compatibleImportElementsList = $parameters['compatibleImportElementsList'];

        if (isset($object->linkedObjectsIds['project'])) {
            require_once DOL_DOCUMENT_ROOT . '/projet/class/project.class.php';
            $object->linkedObjects['project'] = array();
            foreach ($object->linkedObjectsIds['project'] as $i => $objectid) {
                $proj = new Project($this->db);
                $proj->fetch($objectid);
                $object->linkedObjects['project'][$i] = $proj;
            }
        }

        $nbofdifferenttypes = count($object->linkedObjects);

        print '<!-- showLinkedObjectBlock -->';
        print load_fiche_titre($langs->trans('RelatedObjects'), $morehtmlright, '', 0, 0, 'showlinkedobjectblock');


        print '<div class="div-table-responsive-no-min">';
        print '<table class="noborder allwidth" data-block="showLinkedObject" data-element="' . $object->element . '"  data-elementid="' . $object->id . '"   >';

        print '<tr class="liste_titre">';
        print '<td>' . $langs->trans("Type") . '</td>';
        print '<td>' . $langs->trans("Ref") . '</td>';
        print '<td align="center"></td>';
        print '<td align="center">' . $langs->trans("Date") . '</td>';
        print '<td align="right">' . $langs->trans("AmountHTShort") . '</td>';
        print '<td align="right">' . $langs->trans("Status") . '</td>';
        print '<td></td>';
        print '</tr>';

        $nboftypesoutput = 0;

        foreach ($object->linkedObjects as $objecttype => $objects) {
            $tplpath = $element = $subelement = $objecttype;

            // to display inport button on tpl
            $showImportButton = false;
            if (!empty($compatibleImportElementsList) && in_array($element, $compatibleImportElementsList)) {
                $showImportButton = true;
            }

            if ($objecttype != 'supplier_proposal' && preg_match('/^([^_]+)_([^_]+)/i', $objecttype, $regs)) {
                $element = $regs[1];
                $subelement = $regs[2];
                $tplpath = $element . '/' . $subelement;
            }
            $tplname = 'linkedobjectblock';

            // To work with non standard path
            if ($objecttype == 'facture') {
                $tplpath = 'compta/' . $element;
                if (!isModEnabled('facture')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'facturerec') {
                $tplpath = 'compta/facture';
                $tplname = 'linkedobjectblockForRec';
                if (!isModEnabled('facture')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'propal') {
                $tplpath = 'comm/' . $element;
                if (!isModEnabled('propal')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'supplier_proposal') {
                if (!isModEnabled('supplier_proposal')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'shipping' || $objecttype == 'shipment') {
                $tplpath = 'expedition';
                if (!isModEnabled('expedition')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'delivery') {
                $tplpath = 'livraison';
                if (!isModEnabled('expedition')) continue;    // Do not show if module disabled
            } else if ($objecttype == 'invoice_supplier') {
                $tplpath = 'fourn/facture';
            } else if ($objecttype == 'order_supplier') {
                $tplpath = 'fourn/commande';
            } else if ($objecttype == 'expensereport') {
                $tplpath = 'expensereport';
            } else if ($objecttype == 'subscription') {
                $tplpath = 'adherents';
            } else if ($objecttype == 'project') {
                $tplpath = 'requestmanager';
                $tplname = 'linkedobjectblockForProject';
                if (!isModEnabled('projet')) continue;    // Do not show if module disabled
            }

            global $linkedObjectBlock;
            $linkedObjectBlock = $objects;

            // Output template part (modules that overwrite templates must declare this into descriptor)
            $dirtpls = array_merge($conf->modules_parts['tpl'], array('/' . $tplpath . '/tpl'));
            foreach ($dirtpls as $reldir) {
                if ($nboftypesoutput == ($nbofdifferenttypes - 1))    // No more type to show after
                {
                    global $noMoreLinkedObjectBlockAfter;
                    $noMoreLinkedObjectBlockAfter = 1;
                }

                $res = @include dol_buildpath($reldir . '/' . $tplname . '.tpl.php');
                if ($res) {
                    $nboftypesoutput++;
                    break;
                }
            }
        }

        if (!$nboftypesoutput) {
            print '<tr><td class="impair opacitymedium" colspan="7">' . $langs->trans("None") . '</td></tr>';
        }

        print '</table>';

        if (!empty($compatibleImportElementsList)) {
            $res = @include dol_buildpath('core/tpl/ajax/objectlinked_lineimport.tpl.php');
        }

        print '</div>';

        return 1;
    }
}
