<?php
/* Copyright (C) 2019  Open-Dsi <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file        core/dictionaries/requestmanagerextrafields.dictionary.php
 * \ingroup     requestmanager
 * \brief       Class of the dictionary ExtraFields
 */

dol_include_once('/advancedictionaries/class/dictionary.class.php');

/**
 * Class for RequestManagerExtraFieldsDictionary
 */
class RequestManagerExtraFieldsDictionary extends Dictionary
{
	/**
	 * @var int         Version of this dictionary
	 */
	public $version = 1;

	/**
	 * @var array       List of languages to load
	 */
	public $langs = array('requestmanager@requestmanager');

	/**
	 * @var string      Family name of which this dictionary belongs
	 */
	public $family = 'requestmanager';

	/**
	 * @var string      Family label for show in the list, translated if key found
	 */
	public $familyLabel = 'Module163018Name';

	/**
	 * @var int         Position of the dictionary into the family
	 */
	public $familyPosition = 12;

	/**
	 * @var string      Module name of which this dictionary belongs
	 */
	public $module = 'requestmanager';

	/**
	 * @var string      Module name for show in the list, translated if key found
	 */
	public $moduleLabel = 'Module163018Name';

	/**
	 * @var string      Name of this dictionary for show in the list, translated if key found
	 */
	public $nameLabel = 'RequestManagerExtraFieldsDictionaryLabel';

	/**
	 * @var string      Name of the dictionary table without prefix (ex: c_country)
	 */
	public $table_name = 'c_requestmanager_extrafields';

    /**
     * @var array  Fields of the dictionary table
     * 'name' => array(
     *   'name'       => string,         // Name of the field
     *   'label'      => string,         // Label of the field, translated if key found
     *   'type'       => string,         // Type of the field (varchar, text, int, double, date, datetime, boolean, price, phone, mail, url,
     *                                                         password, select, sellist, radio, checkbox, chkbxlst, link, custom)
     *   'database' => array(            // Description of the field in the database always rewrite default value if set
     *     'type'      => string,        // Data type
     *     'length'    => string,        // Length of the data type (require)
     *     'default'   => string,        // Default value in the database
     *   ),
     *   'is_require' => bool,           // Set at true if this field is required
     *   'options'    => array()|string, // Parameters same as extrafields (ex: 'table:label:rowid::active=1' or array(1=>'value1', 2=>'value2') )
     *                                      string: sellist, chkbxlst, link | array: select, radio, checkbox
     *                                      The key of the value must be not contains the character ',' and for chkbxlst it's a rowid
     *   'is_not_show'       => bool,    // Set at true if this field is not show must be set at true if you want to search or edit
     *   'td_title'          => array (
     *      'moreClasses'    => string,  // Add more classes in the title balise td
     *      'moreAttributes' => string,  // Add more attributes in the title balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'td_output'         => array (
     *      'moreClasses'    => string,  // Add more classes in the output balise td
     *      'moreAttributes' => string,  // Add more attributes in the output balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_output'       => array (
     *      'moreAttributes' => string,  // Add more attributes in when show output field
     *   ),
     *   'is_not_searchable' => bool,    // Set at true if this field is not searchable
     *   'td_search'         => array (
     *      'moreClasses'    => string,  // Add more classes in the search input balise td
     *      'moreAttributes' => string,  // Add more attributes in the search input balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_search_input' => array (
     *      'size'           => int,     // Size attribute of the search input field (input text)
     *      'moreClasses'    => string,  // Add more classes in the search input field
     *      'moreAttributes' => string,  // Add more attributes in the search input field
     *   ),
     *   'is_not_addable'    => bool,    // Set at true if this field is not addable
     *   'is_not_editable'   => bool,    // Set at true if this field is not editable
     *   'td_input'         => array (
     *      'moreClasses'    => string,  // Add more classes in the input balise td
     *      'moreAttributes' => string,  // Add more attributes in the input balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_input'        => array (
     *      'moreClasses'    => string,  // Add more classes in the input field
     *      'moreAttributes' => string,  // Add more attributes in the input field
     *   ),
     *   'help' => '',                   // Help text for this field or url, translated if key found
     *   'is_not_sortable'   => bool,    // Set at true if this field is not sortable
     *   'min'               => int,     // Value minimum (include) if type is int, double or price
     *   'max'               => int,     // Value maximum (include) if type is int, double or price
     * )
     */
	public $fields = array(
		'request_type' => array(),
		'request_status' => array(),
		'element_type' => array(),
		'element_extrafields' => array(),
		'element_extrafields_showed' => array(),
		'mandatory' => array(
			'name' => 'mandatory',
			'label' => 'RequestManagerExtraFieldsDictionaryMandatory',
			'help' => 'RequestManagerExtraFieldsDictionaryMandatoryHelp',
			'type' => 'boolean',
		),
	);

	/**
	 * @var array  List of index for the database
	 *   array(
	 *   'fields'    => array( ... ), // List of field name who constitute this index
	 *   'is_unique' => bool,         // Set at true if this index is unique
	 *   )
	 */
	public $indexes = array();

	/**
	 * @var array  List of fields/indexes added, updated or deleted for a version
	 *     array(
	 *     'version' => array(
	 *     'fields' => array('field_name'=>'a', 'field_name'=>'u', 'field_name'=>'d', ...), // List of field name who is added(a) or updated(u) or deleted(d) for a version
	 *     'indexes' => array('idx_number'=>'a', 'idx_number'=>'u', 'idx_number'=>'d', ...), // List of indexes number who is added(a) or updated(u) or deleted(d) for a version
	 *     ),
	 *     )
	 */
	public $updates = array(
		1 => array(
			'fields' => array(
				'element_extrafields_showed' => 'a',
			)
		),
	);

	/**
	 * @var bool    Is multi entity (false = partaged, true = by entity)
	 */
	public $is_multi_entity = true;

	/**
	 * @var bool    Edit in the add form
	 */
	public $edit_in_add_form = false;

	/**
	 * @var Dictionary    Cache of the list of the request type for next_status show output
	 */
	public $request_type_cache = null;
	/**
	 * @var Dictionary    Cache of the list of the status for next_status show output
	 */
	public $status_cache = null;
	/**
	 * @var FormRequestManager    Form Request Manager
	 */
	public $formrequestmanager = null;
	/**
	 * @var array    Cache of the list of the element type for next_status show output
	 */
	public $element_type_cache = null;
	/**
	 * @var array    Cache of the list of the extrafields for next_status show output
	 */
	public $extrafields_cache = null;

	/**
	 * Load the cache of the list of the request type for next_status show output
	 *
	 * @return  void
	 */
	public function load_request_type()
	{
		if (!isset($this->request_type_cache)) {
			$this->request_type_cache = Dictionary::getDictionary($this->db, 'requestmanager', 'requestmanagerrequesttype');
			$this->request_type_cache->fetch_lines();
		}
	}

	/**
	 * Load the cache of the list of the status for next_status show output
	 *
	 * @return  void
	 */
	public function load_status()
	{
		if (!isset($this->status_cache)) {
			$this->status_cache = new RequestManagerStatusDictionary($this->db);
			$this->status_cache->fetch_lines();
		}
	}

	/**
	 * Load the cache of the list of the status for next_status show output
	 *
	 * @return  void
	 */
	public function load_form_requestmanager()
	{
		if (!isset($this->formrequestmanager)) {
			dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
			$this->formrequestmanager = new FormRequestManager($this->db);
		}
	}

	/**
	 * Load the cache of the list of the status for next_status show output
	 *
	 * @return  void
	 */
	public function load_element_types()
	{
		if (!isset($this->element_type_cache)) {
			$this->load_form_requestmanager();
			$this->element_type_cache = $this->formrequestmanager->element_type_array();
			if (!is_array($this->element_type_cache)) {
				$this->element_type_cache = null;
			}
		}
	}

	/**
	 * Load the cache of the list of the extrafields for extrafields show output
	 *
	 * @return  void
	 */
	public function load_extrafields()
	{
		if (!isset($this->extrafields_cache)) {
			$this->load_form_requestmanager();
			$this->load_element_types();
			$tmp = $this->formrequestmanager->multiselect_extrafields(array_keys($this->element_type_cache), '', '', '', '', '', false, 0, array(), '', '', 1);
			if (is_array($tmp)) {
				$this->extrafields_cache = array();
				foreach ($tmp as $v) {
					$this->extrafields_cache[$v['key']] = $v['label'];
				}
			}
		}
	}

	/**
	 * Initialize the dictionary
	 *
	 * @return  void
	 */
	protected function initialize()
	{
		global $conf, $langs, $user;

		$langs->loadLangs(array('requestmanager@requestmanager', 'admin', 'propal', 'orders', 'contracts', 'suppliers', 'supplier_proposal', 'projects', 'trips', 'bills', 'interventions', 'commercial'));

		$this->fields['request_type'] = array(
			'name' => 'request_type',
			'label' => 'RequestManagerRequestType',
			'type' => 'chkbxlst',
			'options' => 'c_requestmanager_request_type:label:rowid::active=1 and entity IN ('.$this->getEntity().')',
			'is_require' => true,
		);

		$this->fields['request_status'] = array(
			'name' => 'request_status',
			'label' => 'RequestManagerExtraFieldsDictionaryRequestStatus',
			'help' => 'RequestManagerExtraFieldsDictionaryRequestStatusHelp',
			'type' => 'chkbxlst',
			'options' => 'c_requestmanager_status:code|label:rowid::active=1 and entity IN ('.$this->getEntity().')',
			'label_separator' => ' - ',
			'show_input' => array(
				'moreAttributes' => ' style="width:100%;"',
				'moreClasses' => '',
			),
		);

		$this->load_element_types();
		$this->fields['element_type'] = array(
			'name' => 'element_type',
			'label' => 'RequestManagerExtraFieldsDictionaryElementType',
			'help' => 'RequestManagerExtraFieldsDictionaryElementTypeHelp',
			'type' => 'checkbox',
			'options' => $this->element_type_cache,
		);

		$this->load_extrafields();
		$this->fields['element_extrafields'] = array(
			'name' => 'element_extrafields',
			'label' => 'RequestManagerExtraFieldsDictionaryExtraFields',
			'help' => 'RequestManagerExtraFieldsDictionaryExtraFieldsHelp',
			'type' => 'checkbox',
			'options' => $this->extrafields_cache,
			'show_input' => array(
				'moreAttributes' => ' style="width:100%;"',
				'moreClasses' => '',
			),
		);

		$this->fields['element_extrafields_showed'] = array(
			'name' => 'element_extrafields_showed',
			'label' => 'RequestManagerExtraFieldsDictionaryExtraFieldsShowed',
			'help' => 'RequestManagerExtraFieldsDictionaryExtraFieldsShowedHelp',
			'type' => 'checkbox',
			'options' => $this->extrafields_cache,
			'show_input' => array(
				'moreAttributes' => ' style="width:100%;"',
				'moreClasses' => '',
			),
		);
	}
}

class RequestManagerExtraFieldsDictionaryLine extends DictionaryLine
{
	/**
	 * Return HTML string to put an input field into a page
	 *
	 * @param  string  $fieldName      Name of the field
	 * @param  string  $value          Preselected value to show (for date type it must be in timestamp format, for amount or price it must be a php numeric value)
	 * @param  string  $keyprefix      Prefix string to add into name and id of field (can be used to avoid duplicate names)
	 * @param  string  $keysuffix      Suffix string to add into name and id of field (can be used to avoid duplicate names)
	 * @param  int     $objectid       Current object id
	 * @param  int     $options_only   1: Return only the html output of the options of the select input
	 * @return string
	 */
	public function showInputFieldAD($fieldName, $value = null, $keyprefix = '', $keysuffix = '', $objectid = 0, $options_only = 0)
	{
		if ($fieldName == 'request_status') {
			if (isset($this->dictionary->fields[$fieldName])) {
				$field = $this->dictionary->fields[$fieldName];

				$type = $field['type'];

				$fieldHtmlName = $keyprefix.$fieldName.$keysuffix;

				$moreClasses = trim($field['show_input']['moreClasses']);
				if (empty($moreClasses)) {
					if ($type == 'date') {
						$moreClasses = ' minwidth100imp';
					} elseif ($type == 'datetime') {
						$moreClasses = ' minwidth200imp';
					} elseif (in_array($type, array('int', 'double', 'price'))) {
						$moreClasses = ' maxwidth75';
					} elseif (in_array($type, array('varchar', 'phone', 'mail', 'url', 'password', 'select', 'sellist', 'radio', 'checkbox', 'link', 'chkbxlst'))) {
						$moreClasses = ' minwidth200';
					} elseif ($type == 'boolean') {
						$moreClasses = '';
					} else {
						$moreClasses = ' minwidth100';
					}
				} else {
					$moreClasses = ' '.$moreClasses;
				}

				$moreAttributes = trim($field['show_input']['moreAttributes']);
				$moreAttributes = !empty($moreAttributes) ? ' '.$moreAttributes : '';

				if ($value === null) {
					$value = $this->fields[$fieldName];
				}
				$value_arr = $value;
				if (!isset($value_arr)) {
					$value_arr = array();
				}
				if (!is_array($value_arr)) {
					$value_arr = explode(',', $value_arr);
				}

				$element_type_selected = $this->fields['request_type'] ?? null;
				if (!isset($element_type_selected)) {
					$element_type_selected = array();
				}
				if (!is_array($element_type_selected)) {
					$element_type_selected = explode(',', $element_type_selected);
				}
				$element_type_selected = array_filter($element_type_selected, 'strlen');

				$this->dictionary->load_form_requestmanager();

				$events = array();
				$events[] = array(
					'action' => 'getRequestStatus',
					'url' => dol_buildpath('/requestmanager/ajax/request_status_list.php', 1),
					'htmlname' => $fieldHtmlName,
					'params' => array('selected' => '{{#'.$fieldHtmlName.'}}')
				);
				$out = $this->dictionary->formrequestmanager->add_select_events($keyprefix.'request_type'.$keysuffix, $events);
				// $out .= parent::showInputFieldAD($fieldName, $value, $keyprefix, $keysuffix, $objectid, $options_only);
				$out .= $this->dictionary->formrequestmanager->multiselect_request_status($element_type_selected, $value_arr, $fieldHtmlName, 0, array(), 0, 0, 0, $moreClasses, $moreAttributes);
				return $out;
			}
		} elseif ($fieldName == 'element_extrafields') {
			if (isset($this->dictionary->fields[$fieldName])) {
				$field = $this->dictionary->fields[$fieldName];

				$type = $field['type'];

				$fieldHtmlName = $keyprefix.$fieldName.$keysuffix;

				$moreClasses = trim($field['show_input']['moreClasses']);
				if (empty($moreClasses)) {
					if ($type == 'date') {
						$moreClasses = ' minwidth100imp';
					} elseif ($type == 'datetime') {
						$moreClasses = ' minwidth200imp';
					} elseif (in_array($type, array('int', 'double', 'price'))) {
						$moreClasses = ' maxwidth75';
					} elseif (in_array($type, array('varchar', 'phone', 'mail', 'url', 'password', 'select', 'sellist', 'radio', 'checkbox', 'link', 'chkbxlst'))) {
						$moreClasses = ' minwidth200';
					} elseif ($type == 'boolean') {
						$moreClasses = '';
					} else {
						$moreClasses = ' minwidth100';
					}
				} else {
					$moreClasses = ' '.$moreClasses;
				}

				$moreAttributes = trim($field['show_input']['moreAttributes']);
				$moreAttributes = !empty($moreAttributes) ? ' '.$moreAttributes : '';

				if ($value === null) {
					$value = $this->fields[$fieldName];
				}
				$value_arr = $value;
				if (!isset($value_arr)) {
					$value_arr = array();
				}
				if (!is_array($value_arr)) {
					$value_arr = explode(',', $value_arr);
				}

				$element_type_selected = $this->fields['element_type'] ?? null;
				if (!isset($element_type_selected)) {
					$element_type_selected = array();
				}
				if (!is_array($element_type_selected)) {
					$element_type_selected = explode(',', $element_type_selected);
				}
				$element_type_selected = array_filter($element_type_selected, 'strlen');

				$this->dictionary->load_form_requestmanager();

				$events = array();
				$events[] = array(
					'action' => 'getExtraFields',
					'url' => dol_buildpath('/requestmanager/ajax/extrafields_list.php', 1),
					'htmlname' => $fieldHtmlName,
					'params' => array('selected' => '{{#'.$fieldHtmlName.'}}')
				);
				$out = $this->dictionary->formrequestmanager->add_select_events($keyprefix.'element_type'.$keysuffix, $events);
				$out .= $this->dictionary->formrequestmanager->multiselect_extrafields($element_type_selected, $value_arr, $fieldHtmlName, '', '', $moreClasses, false, 0, array(), $moreAttributes);
				return $out;
			}
		} elseif ($fieldName == 'element_extrafields_showed') {
			if (isset($this->dictionary->fields[$fieldName])) {
				$field = $this->dictionary->fields[$fieldName];

				$type = $field['type'];

				$fieldHtmlName = $keyprefix.$fieldName.$keysuffix;

				$moreClasses = trim($field['show_input']['moreClasses']);
				if (empty($moreClasses)) {
					if ($type == 'date') {
						$moreClasses = ' minwidth100imp';
					} elseif ($type == 'datetime') {
						$moreClasses = ' minwidth200imp';
					} elseif (in_array($type, array('int', 'double', 'price'))) {
						$moreClasses = ' maxwidth75';
					} elseif (in_array($type, array('varchar', 'phone', 'mail', 'url', 'password', 'select', 'sellist', 'radio', 'checkbox', 'link', 'chkbxlst'))) {
						$moreClasses = ' minwidth200';
					} elseif ($type == 'boolean') {
						$moreClasses = '';
					} else {
						$moreClasses = ' minwidth100';
					}
				} else {
					$moreClasses = ' '.$moreClasses;
				}

				$moreAttributes = trim($field['show_input']['moreAttributes']);
				$moreAttributes = !empty($moreAttributes) ? ' '.$moreAttributes : '';

				if ($value === null) {
					$value = $this->fields[$fieldName];
				}
				$value_arr = $value;
				if (!isset($value_arr)) {
					$value_arr = array();
				}
				if (!is_array($value_arr)) {
					$value_arr = explode(',', $value_arr);
				}

				$element_type_selected = $this->fields['element_type'] ?? null;
				if (!isset($element_type_selected)) {
					$element_type_selected = array();
				}
				if (!is_array($element_type_selected)) {
					$element_type_selected = explode(',', $element_type_selected);
				}
				$element_type_selected = array_filter($element_type_selected, 'strlen');

				$this->dictionary->load_form_requestmanager();

				$events = array();
				$events[] = array(
					'action' => 'getExtraFields',
					'url' => dol_buildpath('/requestmanager/ajax/extrafields_list.php', 1),
					'htmlname' => $fieldHtmlName,
					'params' => array(
						'selected' => '{{#'.$fieldHtmlName.'}}',
						'element_type' => '{{#'.$keyprefix.'element_type'.$keysuffix.'}}',
						'element_extrafields_showed' => 1
					)
				);
				$out = $this->dictionary->formrequestmanager->add_select_events($keyprefix.'element_extrafields'.$keysuffix, $events);
				$out .= $this->dictionary->formrequestmanager->multiselect_extrafields($element_type_selected, $value_arr, $fieldHtmlName, '', '', $moreClasses, false, 0, array(), $moreAttributes, '', 0, array(), 1);
				return $out;
			}
		}

		return parent::showInputFieldAD($fieldName, $value, $keyprefix, $keysuffix, $objectid, $options_only);
	}

	/**
	 * Return HTML string to put an output field into a page
	 *
	 * @param	string	$fieldName	Name of the field
	 * @param	string	$value		Value to show
	 * @return	string				Formatted value
	 */
	public function showOutputFieldAD($fieldName, $value = null)
	{
		if ($fieldName == 'element_extrafields') {
			if (isset($this->dictionary->fields[$fieldName])) {
				if ($value === null) {
					$value = $this->fields[$fieldName];
				}
				if (is_array($value)) {
					$value_arr = $value;
				} else {
					$value_arr = array_filter(explode(',', (string) $value), 'strlen');
				}

				$this->dictionary->load_extrafields();

				$toprint = array();
				foreach ($value_arr as $v) {
					$toprint[] = '<li class="select2-search-choice-dolibarr noborderoncategories" style="background: #aaa">'.$this->dictionary->extrafields_cache[$v].'</li>';
				}
				$value = '<div class="select2-container-multi-dolibarr" style="width: 90%;"><ul class="select2-choices-dolibarr">'.implode(' ', $toprint).'</ul></div>';

				return $value;
			}
		}

		return parent::showOutputFieldAD($fieldName, $value);
	}
}
