<?php
/* Copyright (C) 2018  Open-Dsi <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file        core/dictionaries/requestmanagerrequesttype.dictionary.php
 * \ingroup     requestmanager
 * \brief       Class of the dictionary Urgency
 */

dol_include_once('/advancedictionaries/class/dictionary.class.php');
dol_include_once('/requestmanager/class/requestmanager.class.php');

/**
 * Class for RequestManagerRequestTypeDictionary
 */
class RequestManagerRequestTypeDictionary extends Dictionary
{
	/**
	 * @var int         Version of this dictionary
	 */
	public $version = 5;

	/**
	 * @var array       List of languages to load
	 */
	public $langs = array('requestmanager@requestmanager', 'companies');

	/**
	 * @var string      Family name of which this dictionary belongs
	 */
	public $family = 'requestmanager';

	/**
	 * @var string      Family label for show in the list, translated if key found
	 */
	public $familyLabel = 'Module163018Name';

	/**
	 * @var int         Position of the dictionary into the family
	 */
	public $familyPosition = 1;

	/**
	 * @var string      Module name of which this dictionary belongs
	 */
	public $module = 'requestmanager';

	/**
	 * @var string      Module name for show in the list, translated if key found
	 */
	public $moduleLabel = 'Module163018Name';

	/**
	 * @var string      Name of this dictionary for show in the list, translated if key found
	 */
	public $nameLabel = 'RequestManagerRequestTypeDictionaryLabel';

	/**
	 * @var string      Name of the dictionary table without prefix (ex: c_country)
	 */
	public $table_name = 'c_requestmanager_request_type';

    /**
     * @var array  Fields of the dictionary table
     * 'name' => array(
     *   'name'       => string,         // Name of the field
     *   'label'      => string,         // Label of the field, translated if key found
     *   'type'       => string,         // Type of the field (varchar, text, int, double, date, datetime, boolean, price, phone, mail, url,
     *                                                         password, select, sellist, radio, checkbox, chkbxlst, link, custom)
     *   'database' => array(            // Description of the field in the database always rewrite default value if set
     *     'type'      => string,        // Data type
     *     'length'    => string,        // Length of the data type (require)
     *     'default'   => string,        // Default value in the database
     *   ),
     *   'is_require' => bool,           // Set at true if this field is required
     *   'options'    => array()|string, // Parameters same as extrafields (ex: 'table:label:rowid::active=1' or array(1=>'value1', 2=>'value2') )
     *                                      string: sellist, chkbxlst, link | array: select, radio, checkbox
     *                                      The key of the value must be not contains the character ',' and for chkbxlst it's a rowid
     *   'is_not_show'       => bool,    // Set at true if this field is not show must be set at true if you want to search or edit
     *   'td_title'          => array (
     *      'moreClasses'    => string,  // Add more classes in the title balise td
     *      'moreAttributes' => string,  // Add more attributes in the title balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'td_output'         => array (
     *      'moreClasses'    => string,  // Add more classes in the output balise td
     *      'moreAttributes' => string,  // Add more attributes in the output balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_output'       => array (
     *      'moreAttributes' => string,  // Add more attributes in when show output field
     *   ),
     *   'is_not_searchable' => bool,    // Set at true if this field is not searchable
     *   'td_search'         => array (
     *      'moreClasses'    => string,  // Add more classes in the search input balise td
     *      'moreAttributes' => string,  // Add more attributes in the search input balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_search_input' => array (
     *      'size'           => int,     // Size attribute of the search input field (input text)
     *      'moreClasses'    => string,  // Add more classes in the search input field
     *      'moreAttributes' => string,  // Add more attributes in the search input field
     *   ),
     *   'is_not_addable'    => bool,    // Set at true if this field is not addable
     *   'is_not_editable'   => bool,    // Set at true if this field is not editable
     *   'td_input'         => array (
     *      'moreClasses'    => string,  // Add more classes in the input balise td
     *      'moreAttributes' => string,  // Add more attributes in the input balise td
     *      'align'          => string,  // Overwrirte the align by default
     *   ),
     *   'show_input'        => array (
     *      'moreClasses'    => string,  // Add more classes in the input field
     *      'moreAttributes' => string,  // Add more attributes in the input field
     *   ),
     *   'help' => '',                   // Help text for this field or url, translated if key found
     *   'is_not_sortable'   => bool,    // Set at true if this field is not sortable
     *   'min'               => int,     // Value minimum (include) if type is int, double or price
     *   'max'               => int,     // Value maximum (include) if type is int, double or price
     * )
     */
	public $fields = array(
		'code' => array(
			'name' => 'code',
			'label' => 'Code',
			'type' => 'varchar',
			'database' => array(
				'length' => 32,
			),
			'is_require' => true,
		),
		'label' => array(
			'name' => 'label',
			'label' => 'Label',
			'type' => 'varchar',
			'database' => array(
				'length' => 255,
			),
			'is_require' => true,
			'show_input' => array(
				'moreAttributes' => ' style="width:95%;"',
			),
		),
		'user_group' => array(),
		'category' => array(),
		'origin_type' => array(
			'name' => 'origin_type',
			'label' => 'RequestManagerOriginTypeName',
			'type' => 'select',
			'options' => array(
				RequestManager::ORIGIN_TYPE_THIRDPARTY => 'ThirdParty',
				RequestManager::ORIGIN_TYPE_CONTACT => 'Contact'
			),
			'database' => array(
				'type' => 'tinyint',
				'length' => 4,
			),
			'help' => 'RequestManagerOriginTypeDesc',
		),
		'mandatory_thirdparty' => array(
			'name' => 'mandatory_thirdparty',
			'label' => 'RequestManagerMandatoryThirdPartyName',
			'type' => 'boolean',
			'help' => 'RequestManagerMandatoryThirdPartyDesc',
		),
		'mandatory_contact' => array(
			'name' => 'mandatory_contact',
			'label' => 'RequestManagerMandatoryContactName',
			'type' => 'boolean',
			'help' => 'RequestManagerMandatoryContactDesc',
		),
		'display_events_not_linked' => array(
			'name' => 'display_events_not_linked',
			'label' => 'RequestManagerDisplayEventsNotLinkedName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayEventsNotLinkedDesc',
		),
		'display_category' => array(
			'name' => 'display_category',
			'label' => 'RequestManagerDisplayCategoryName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayCategoryDesc',
		),
		'display_source' => array(
			'name' => 'display_source',
			'label' => 'RequestManagerDisplaySourceName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplaySourceDesc',
		),
		'display_urgency' => array(
			'name' => 'display_urgency',
			'label' => 'RequestManagerDisplayUrgencyName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayUrgencyDesc',
		),
		'display_impact' => array(
			'name' => 'display_impact',
			'label' => 'RequestManagerDisplayImpactName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayImpactDesc',
		),
		'display_priority' => array(
			'name' => 'display_priority',
			'label' => 'RequestManagerDisplayPriorityName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayPriorityDesc',
		),
		'display_products' => array(
			'name' => 'display_products',
			'label' => 'RequestManagerDisplayProductsName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayProductsDesc',
		),
		'display_events' => array(
			'name' => 'display_events',
			'label' => 'RequestManagerDisplayEventsName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayEventsDesc',
		),
		'display_events_create_same_for_each_requester' => array(
			'name' => 'display_events_create_same_for_each_requester',
			'label' => 'RequestManagerDisplayEventsCreateSameForEachRequesterName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayEventsCreateSameForEachRequesterDesc',
		),
		'display_events_create_same_for_each_request_children' => array(
			'name' => 'display_events_create_same_for_each_request_children',
			'label' => 'RequestManagerDisplayEventsCreateSameForEachRequestChildrenName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayEventsCreateSameForEachRequestChildrenDesc',
		),
		'display_messages' => array(
			'name' => 'display_messages',
			'label' => 'RequestManagerDisplayMessagesName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayMessagesDesc',
		),
		'display_watcher' => array(
			'name' => 'display_watcher',
			'label' => 'RequestManagerDisplayWatcherName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayWatcherDesc',
		),
		'display_children_request' => array(
			'name' => 'display_children_request',
			'label' => 'RequestManagerDisplayChildrenRequestName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplayChildrenRequestDesc',
		),
		'manage_times' => array(
			'name' => 'manage_times',
			'label' => 'RequestManagerManageTimesName',
			'type' => 'boolean',
			'help' => 'RequestManagerManageTimesDesc',
		),
		'display_set_times' => array(
			'name' => 'display_set_times',
			'label' => 'RequestManagerDisplaySetTimesName',
			'type' => 'boolean',
			'help' => 'RequestManagerDisplaySetTimesDesc',
		),
		'active_sum_children_request_times' => array(
			'name' => 'active_sum_children_request_times',
			'label' => 'RequestManagerActiveSommeChildrenRequestTimesName',
			'type' => 'boolean',
			'help' => 'RequestManagerActiveSommeChildrenRequestTimesDesc',
		),
	);

	/**
	 * @var array  List of index for the database
	 *   array(
	 *   'fields'    => array( ... ), // List of field name who constitute this index
	 *   'is_unique' => bool,         // Set at true if this index is unique
	 *   )
	 */
	public $indexes = array(
		0 => array(
			'fields' => array('code'),
			'is_unique' => true,
		),
	);

	/**
	 * @var array  List of fields/indexes added, updated or deleted for a version
	 *     array(
	 *     'version' => array(
	 *     'fields' => array('field_name'=>'u', ...), // List of field name who is updated(u) for a version
	 *     'deleted_fields' => array('field_name'=> array('name', 'type', other_custom_data_required_for_delete), ...), // List of field name who is deleted for a version
	 *     'indexes' => array('idx_number'=>'u', 'idx_number'=>'d', ...), // List of indexes number who is updated(u) or deleted(d) for a version
	 *     ),
	 *     )
	 */
	public $updates = array(
		1 => array(
			'fields' => array(
				'display_events_not_linked' => 'a',
				'display_category' => 'a',
				'display_source' => 'a',
				'display_urgency' => 'a',
				'display_impact' => 'a',
				'display_priority' => 'a',
				'display_products' => 'a',
				'display_events' => 'a',
				'display_messages' => 'a',
				'display_watcher' => 'a',
				'display_children_request' => 'a',
				'manage_times' => 'a',
				'display_set_times' => 'a',
				'active_sum_children_request_times' => 'a',
			),
		),
		2 => array(
			'fields' => array(
				'code' => 'u',
			)
		),
		3 => array(
			'fields' => array(
				'origin_type' => 'a',
			)
		),
		4 => array(
			'fields' => array(
				'mandatory_thirdparty' => 'a',
				'mandatory_contact' => 'a',
			)
		),
		5 => array(
			'fields' => array(
				'display_events_create_same_for_each_requester' => 'a',
				'display_events_create_same_for_each_request_children' => 'a',
			)
		),
	);

	/**
	 * @var bool    Is multi entity (false = partaged, true = by entity)
	 */
	public $is_multi_entity = true;

	/**
	 * Initialize the dictionary
	 *
	 * @return  void
	 */
	protected function initialize()
	{
		global $conf, $user;

		require_once DOL_DOCUMENT_ROOT.'/core/lib/admin.lib.php';
		$dol_sup_v6 = versioncompare(explode('.', DOL_VERSION), explode('.', '6.0.0')) >= 0;

		if ($dol_sup_v6) {
			if (isModEnabled('multicompany') && $conf->entity == 1 && ($conf->global->MULTICOMPANY_TRANSVERSE_MODE || ($user->admin && !$user->entity))) {
				$entity_filter = "entity IS NOT NULL";
			} else {
				$entity_filter = "entity IN (0,".$conf->entity.")";
			}
		} else {
			if (isModEnabled('multicompany') && !empty($conf->multicompany->transverse_mode)) {
				$entity_filter = "entity IN (0,1)";
			} else {
				$entity_filter = "entity = ".$conf->entity;
			}
		}
		$this->fields['user_group'] = array(
			'name' => 'user_group',
			'label' => 'RequestManagerUserGroup',
			'type' => 'chkbxlst',
			'options' => 'usergroup:nom:rowid::'.$entity_filter,
			'is_require' => true,
		);

		$this->fields['category'] = array(
			'name' => 'category',
			'label' => 'RequestManagerCategories',
			'type' => 'chkbxlst',
			'options' => 'c_requestmanager_category:label:rowid::active=1 AND entity IN ('.$this->getEntity().')',
		);
	}
}


class RequestManagerRequestTypeDictionaryLine extends DictionaryLine
{
	/**
	 * Return HTML string to put an input field into a page
	 *
	 * @param string $fieldName    Name of the field
	 * @param string $value        Preselected value to show (for date type it must be in timestamp format, for amount or price it must be a php numeric value)
	 * @param string $keyprefix    Prefix string to add into name and id of field (can be used to avoid duplicate names)
	 * @param string $keysuffix    Suffix string to add into name and id of field (can be used to avoid duplicate names)
	 * @param int    $objectid     Current object id
	 * @param int    $options_only 1: Return only the html output of the options of the select input
	 * @return string
	 */
	public function showInputFieldAD($fieldName, $value = null, $keyprefix = '', $keysuffix = '', $objectid = 0, $options_only = 0)
	{
		if ($fieldName == 'manage_times') {
			$displayChildrenRequestFieldHtmlName = $keyprefix.'display_children_request'.$keysuffix;
			$manageTimesFieldHtmlName = $keyprefix.'manage_times'.$keysuffix;
			$displaySetTimesFieldHtmlName = $keyprefix.'display_set_times'.$keysuffix;
			$activeSumChildrenRequestTimesFieldHtmlName = $keyprefix.'active_sum_children_request_times'.$keysuffix;
			$updateDisplaySetTimesFieldHtmlName = 'update_'.$displaySetTimesFieldHtmlName;
			$updateActiveSumChildrenRequestTimesFieldHtmlName = 'update_'.$activeSumChildrenRequestTimesFieldHtmlName;

			print <<<SCRIPT
            <input type="hidden" id="h_$displaySetTimesFieldHtmlName" name="$displaySetTimesFieldHtmlName" value="" disabled="disabled">
            <input type="hidden" id="h_$activeSumChildrenRequestTimesFieldHtmlName" name="$activeSumChildrenRequestTimesFieldHtmlName" value="" disabled="disabled">

<script type="text/javascript">
    $(document).ready(function() {
        $updateDisplaySetTimesFieldHtmlName();
        $updateActiveSumChildrenRequestTimesFieldHtmlName();
        
        $('#$manageTimesFieldHtmlName').on('change', function() {
            $updateDisplaySetTimesFieldHtmlName();
            $updateActiveSumChildrenRequestTimesFieldHtmlName();
        });
        
        $('#$displayChildrenRequestFieldHtmlName').on('change', function() {
            $updateActiveSumChildrenRequestTimesFieldHtmlName();
        });
        
        function $updateDisplaySetTimesFieldHtmlName() {
            var disabled = !$('#$manageTimesFieldHtmlName').is(':checked');

            $('#$displaySetTimesFieldHtmlName').prop('disabled', disabled);
            $('#h_$displaySetTimesFieldHtmlName').prop('disabled', !disabled);
            if (disabled) {
                $('#$displaySetTimesFieldHtmlName').prop('checked', false);
            }
        }
        function $updateActiveSumChildrenRequestTimesFieldHtmlName() {
            var disabled = !$('#$manageTimesFieldHtmlName').is(':checked') || !$('#$displayChildrenRequestFieldHtmlName').is(':checked');

            $('#$activeSumChildrenRequestTimesFieldHtmlName').prop('disabled', disabled);
            $('#h_$activeSumChildrenRequestTimesFieldHtmlName').prop('disabled', !disabled);
            if (disabled) {
                $('#$activeSumChildrenRequestTimesFieldHtmlName').prop('checked', false);
            }
        }
    });
</script>
SCRIPT;
		}

		return parent::showInputFieldAD($fieldName, $value, $keyprefix, $keysuffix, $objectid, $options_only);
	}
}
