<?php
/* Copyright (C) 2004		Rodolphe Quiedeville	<rodolphe@quiedeville.org>
 * Copyright (C) 2004-2006	Laurent Destailleur		<eldy@users.sourceforge.net>
 * Copyright (C) 2005-2012	Regis Houssin			<regis.houssin@inodbox.com>
 * Copyright (C) 2017    	Ferran Marcet       	<fmarcet@2byte.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 */

/**
 *	\file       htdocs/requestmanager/info.php
 *	\ingroup    requestmanager
 *	\brief      Page d'affichage des infos d'un processus
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../main.inc.php")) $res=@include '../main.inc.php';					// to work if your module directory is into dolibarr root htdocs directory
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../dolibarr/htdocs/main.inc.php';     // Used on dev env only
if (! $res && file_exists("../../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../../dolibarr/htdocs/main.inc.php';   // Used on dev env only
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/functions2.lib.php';
if (isModEnabled('projet')) {
	require_once DOL_DOCUMENT_ROOT.'/projet/class/project.class.php';
}
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/requestmanager/lib/requestmanager.lib.php');

// Load translation files required by the page
$langs->loadLangs(array('requestmanager@requestmanager', 'other'));

$id = GETPOST('id', 'int');
$ref = GETPOST('ref', 'alpha');

$object = new RequestManager($db);
// Load object
if ($id > 0 || !empty($ref)) {
	$ret = $object->fetch($id, $ref);
	if ($ret > 0) {
	} elseif ($ret < 0) {
		dol_print_error('', $object->error);
	} else {
		print $langs->trans('NoRecordFound');
		exit();
	}
}

// Security check
$result = restrictedArea($user, 'requestmanager', $id);
if (!$user->rights->requestmanager->read_file)
	accessforbidden();


/*
 *	View
 */

$form = new Form($db);

$title = $langs->trans('RequestManagerRequest')." - ".$langs->trans('Info');
$help_url = 'EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es';
llxHeader('', $title, $help_url);

$head = requestmanager_prepare_head($object);
print dol_get_fiche_head($head, 'info', $langs->trans('RequestManagerRequest'), -1, 'requestmanager@requestmanager');

$object->info($object->id);


// Request manager card

$linkback = '<a href="' . dol_buildpath('/requestmanager/list.php', 1) . '?restore_lastsearch_values=1">' . $langs->trans("BackToList") . '</a>';

$morehtmlref = '<div class="refidno">';
// External Reference
$morehtmlref .= $langs->trans('RequestManagerExternalReference') . ' : ' . $object->ref_ext;
// Type
$morehtmlref .= '<br>' . $langs->trans('RequestManagerType') . ' : ' . $object->getLibType();
// Label
$morehtmlref .= '<br>' . $langs->trans('RequestManagerLabel') . ' : ' . $object->label;
// Parent request
if ($object->fk_parent > 0) {
	$object->fetch_parent();
	$morehtmlref .= '<br>' . $langs->trans('RequestManagerParentRequest') . ' : ' . $object->parent->getNomUrl(1, 'parent_path');
}
// Thirdparty
$origin_type = $object->getRequestTypeOption('origin_type');
if ($object->socid > 0 && empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_THIRDPARTY) {
	$object->fetch_thirdparty();
	if (is_object($object->thirdparty)) {
		$morehtmlref .= '<br>'.$langs->trans('RequestManagerThirdPartyOrigin').' : '.$object->thirdparty->getNomUrl(1);
	}
}
$morehtmlref .= '</div>';

dol_banner_tab($object, 'ref', $linkback, 1, 'ref', 'ref', $morehtmlref);

print '<div class="fichecenter">';
print '<div class="underbanner clearboth"></div>';

print '<br>';

dol_print_object_info($object);

$deltadateforserver = getServerTimeZoneInt('now');
$deltadateforclient = ((int) $_SESSION['dol_tz'] + (int) $_SESSION['dol_dst']);
$deltadateforuser = round($deltadateforclient - $deltadateforserver);

// User resolve (old method using already loaded object and not id is kept for backward compatibility)
if (!empty($object->user_resolved) || !empty($object->user_resolved_id)) {
	print $langs->trans("RequestManagerResolvedBy");
	print ': ';
	if (is_object($object->user_resolved)) {
		if ($object->user_resolved->id) {
			print $object->user_resolved->getNomUrl(-1, '', 0, 0, 0);
		} else {
			print $langs->trans("Unknown");
		}
	} else {
		$userstatic = new User($db);
		$userstatic->fetch($object->user_resolved_id ? $object->user_resolved_id : $object->user_resolved_id);
		if ($userstatic->id) {
			print $userstatic->getNomUrl(-1, '', 0, 0, 0);
		} else {
			print $langs->trans("Unknown");
		}
	}
	print '<br>';
}

// Date resolve
if (!empty($object->date_resolved)) {
	print $langs->trans("RequestManagerDateResolved");
	print ': ';
	print dol_print_date($object->date_resolved, 'dayhour', 'tzserver');
	if ($deltadateforuser) {
		print ' '.$langs->trans("CurrentHour").' &nbsp; / &nbsp; '.dol_print_date($object->date_resolved, "dayhour", 'tzuserrel').' &nbsp;'.$langs->trans("ClientHour");
	}
	print '<br>';
}

print '</div>';

print dol_get_fiche_end();

// End of page
llxFooter();
$db->close();
