<?php
/* Copyright (C) 2020      Open-DSI              <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *       \file       htdocs/requestmanager/survey.php
 *       \ingroup    requestmanager
 *       \brief      Management page of survey invitations attached to a request
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../main.inc.php")) $res=@include '../main.inc.php';					// to work if your module directory is into dolibarr root htdocs directory
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../dolibarr/htdocs/main.inc.php';     // Used on dev env only
if (! $res && file_exists("../../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../../dolibarr/htdocs/main.inc.php';   // Used on dev env only
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT . '/contact/class/contact.class.php';
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/questionnaire/class/questionnaire.class.php');
dol_include_once('/questionnaire/class/invitation.class.php');
dol_include_once('/requestmanager/lib/requestmanager.lib.php');

$v13p = version_compare(DOL_VERSION, '13.0.0', '>=');

$langs->load('other');
$langs->load('requestmanager@requestmanager');
$langs->load('questionnaire@questionnaire');

$action		= GETPOST('action','alpha');
$massaction	= GETPOST('massaction','alpha');
$confirm	= GETPOST('confirm','alpha');
$id			= GETPOST('id','int');
$ref		= GETPOST('ref','alpha');
$toselect 	= GETPOST('toselect', 'array');

// Security check
$result = restrictedArea($user, 'requestmanager', $id);
if (!$user->rights->questionnaire->read || !isModEnabled('questionnaire'))
	accessforbidden();

$search_ref						= GETPOST('search_ref','alpha');
$search_questionnaire			= GETPOST('search_questionnaire','alpha');
$search_request					= GETPOST('search_request','alpha');
$search_contact					= GETPOST('search_contact','alpha');
$search_email					= GETPOST('search_email','alpha');
$search_sent					= GETPOST('search_sent','int');
$search_date_sent_start			= dol_mktime(0, 0, 0, GETPOST('search_date_sent_startmonth', 'int'), GETPOST('search_date_sent_startday', 'int'), GETPOST('search_date_sent_startyear', 'int'));
$search_date_sent_end			= dol_mktime(23, 59, 59, GETPOST('search_date_sent_endmonth', 'int'), GETPOST('search_date_sent_endday', 'int'), GETPOST('search_date_sent_endyear', 'int'));
$search_date_sent_remind_start	= dol_mktime(0, 0, 0, GETPOST('search_date_sent_remind_startmonth', 'int'), GETPOST('search_date_sent_remind_startday', 'int'), GETPOST('search_date_sent_remind_startyear', 'int'));
$search_date_sent_remind_end	= dol_mktime(23, 59, 59, GETPOST('search_date_sent_remind_endmonth', 'int'), GETPOST('search_date_sent_remind_endday', 'int'), GETPOST('search_date_sent_remind_endyear', 'int'));
$search_date_limit_answer_start	= dol_mktime(0, 0, 0, GETPOST('search_date_limit_answer_startmonth', 'int'), GETPOST('search_date_limit_answer_startday', 'int'), GETPOST('search_date_limit_answer_startyear', 'int'));
$search_date_limit_answer_end	= dol_mktime(23, 59, 59, GETPOST('search_date_limit_answer_endmonth', 'int'), GETPOST('search_date_limit_answer_endday', 'int'), GETPOST('search_date_limit_answer_endyear', 'int'));
$search_date_validation_start	= dol_mktime(0, 0, 0, GETPOST('search_date_validation_startmonth', 'int'), GETPOST('search_date_validation_startday', 'int'), GETPOST('search_date_validation_startyear', 'int'));
$search_date_validation_end		= dol_mktime(23, 59, 59, GETPOST('search_date_validation_endmonth', 'int'), GETPOST('search_date_validation_endday', 'int'), GETPOST('search_date_validation_endyear', 'int'));
$search_date_creation_start		= dol_mktime(0, 0, 0, GETPOST('search_date_creation_startmonth', 'int'), GETPOST('search_date_creation_startday', 'int'), GETPOST('search_date_creation_startyear', 'int'));
$search_date_creation_end		= dol_mktime(23, 59, 59, GETPOST('search_date_creation_endmonth', 'int'), GETPOST('search_date_creation_endday', 'int'), GETPOST('search_date_creation_endyear', 'int'));
$search_date_modification_start	= dol_mktime(0, 0, 0, GETPOST('search_date_modification_startmonth', 'int'), GETPOST('search_date_modification_startday', 'int'), GETPOST('search_date_modification_startyear', 'int'));
$search_date_modification_end	= dol_mktime(23, 59, 59, GETPOST('search_date_modification_endmonth', 'int'), GETPOST('search_date_modification_endday', 'int'), GETPOST('search_date_modification_endyear', 'int'));

$view_request_status	= GETPOST('view_request_status','int');
$viewstatut				= GETPOST('viewstatut', 'alpha');
$optioncss				= GETPOST('optioncss', 'alpha');
$object_statut			= GETPOST('search_statut', 'alpha');

// Get parameters
$limit = GETPOST('limit')?GETPOST('limit','int'):$conf->liste_limit;
$sortfield = GETPOST("sortfield",'alpha');
$sortorder = GETPOST("sortorder",'alpha');
$page = GETPOST("page",'int');
if (empty($page) || $page == -1) { $page = 0; }     // If $page is not defined, or '' or -1
$offset = $limit * $page;
$pageprev = $page - 1;
$pagenext = $page + 1;
if (! $sortfield) $sortfield='qiu.ref';
if (! $sortorder) $sortorder='ASC';

// Load request
$request = new RequestManager($db);
$request_id = 0;
if ($id > 0 || !empty($ref)) {
	$ret = $request->fetch($id, $ref);
	if ($ret > 0) {
		$request->loadRequestStatus();
		$request_id = $request->id;
	} elseif ($ret < 0) {
		dol_print_error('', $request->error);
	} else {
		accessforbidden($langs->trans('NoRecordFound') . ' (Request)');
	}
}

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$contextpage='requestmanagerallsurveylist';
if ($request_id > 0) $contextpage='requestmanagersurveylist';

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$hookmanager->initHooks(array($contextpage));
$extrafields = new ExtraFields($db);

// fetch optionals attributes and labels
$extralabels = $extrafields->fetch_name_optionals_label('quest_invitation_user');
$search_array_options=$extrafields->getOptionalsFromPost($extralabels,'','search_');

$arrayfields = array(
	'qiu.ref' => array('label' => $langs->trans("Ref"), 'checked' => 1),
	'qiu.fk_questionnaire' => array('label' => $langs->trans("RequestManagerSurvey"), 'checked' => 1),
	'rmqiu.fk_requestmanager' => array('label' => $langs->trans("RequestManagerRequest"), 'checked' => 1, 'enabled' => empty($request_id)),
	'qiu.fk_element' => array('label' => $langs->trans("Contact"), 'checked' => 1),
	'qiu.email' => array('label' => $langs->trans("Email"), 'checked' => 1),
	'qiu.sent' => array('label' => $langs->trans("RequestManagerSent"), 'checked' => 1),
	'qiu.date_envoi' => array('label' => $langs->trans("RequestManagerDateSent"), 'checked' => 1),
	'qiu.date_sent_remind' => array('label' => $langs->trans("RequestManagerDateSentRemind"), 'checked' => 1),
	'qiu.date_limite_reponse' => array('label' => $langs->trans("questionnaire_date_limite_reponse"), 'checked' => 1),
	'qiu.date_validation' => array('label' => $langs->trans("ValidationDate"), 'checked' => 1),
	'qiu.date_creation' => array('label' => $langs->trans("DateCreation"), 'checked' => 0, 'position' => 500),
	'qiu.tms' => array('label' => $langs->trans("DateModificationShort"), 'checked' => 0, 'position' => 500),
	'qiu.fk_statut' => array('label' => $langs->trans("StatusInvitation"), 'checked' => 1, 'position' => 1000),
	'link' => array('label' => $langs->trans("LinkInvit"), 'checked' => 1, 'position' => 1000),
);
// Extra fields
if (version_compare(DOL_VERSION, "13.0.0") >= 0) {
	include DOL_DOCUMENT_ROOT.'/core/tpl/extrafields_list_array_fields.tpl.php';
} else {
	if (is_array($extrafields->attribute_label) && count($extrafields->attribute_label)) {
		foreach ($extrafields->attribute_label as $key => $val) {
			if (!empty($extrafields->attribute_list[$key])) $arrayfields["ef." . $key] = array('label' => $extrafields->attribute_label[$key], 'checked' => (($extrafields->attribute_list[$key] < 0) ? 0 : 1), 'position' => $extrafields->attribute_pos[$key], 'enabled' => (abs($extrafields->attribute_list[$key]) != 3 && $extrafields->attribute_perms[$key]));
		}
	}
}

$object = new InvitationUser($db);	// To be passed as parameter of executeHooks that need


/*
 * Actions
 */

if (GETPOST('cancel')) { $action='list'; $massaction=''; }
if (! GETPOST('confirmmassaction') &&
	$massaction != 'pre_rm_send_survey' && $massaction != 'confirm_rm_send_survey' &&
	$massaction != 'pre_rm_update_date_limit' && $massaction != 'confirm_rm_update_date_limit' &&
	$massaction != 'pre_rm_reopen' && $massaction != 'confirm_rm_reopen' &&
	$massaction != 'pre_rm_delete' && $massaction != 'confirm_rm_delete') { $massaction=''; }

if (GETPOST('modelselected', 'aZ09')) { $action = 'rm_send_survey'; $massaction = 'pre_rm_send_survey'; }

$parameters=array('request' => &$request);
$reshook=$hookmanager->executeHooks('doActions',$parameters,$object,$action);    // Note that $action and $object may have been modified by some hooks
if ($reshook < 0) setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');

if (empty($reshook)) {
	include DOL_DOCUMENT_ROOT . '/core/actions_changeselectedfields.inc.php';

	// Do we click on purge search criteria ?
	if (GETPOST('button_removefilter_x', 'alpha') || GETPOST('button_removefilter.x', 'alpha') || GETPOST('button_removefilter', 'alpha')) // All tests are required to be compatible with all browsers
	{
		$search_ref = '';
		$search_questionnaire = '';
		$search_request = '';
		$search_contact = '';
		$search_email = '';
		$search_sent = '';
		$search_date_sent_start = '';
		$search_date_sent_end = '';
		$search_date_sent_remind_start = '';
		$search_date_sent_remind_end = '';
		$search_date_limit_answer_start = '';
		$search_date_limit_answer_end = '';
		$search_date_validation_start = '';
		$search_date_validation_end = '';
		$search_date_creation_start = '';
		$search_date_creation_end = '';
		$search_date_modification_start = '';
		$search_date_modification_end = '';
		$viewstatut = '';
		$object_statut = '';
		$toselect = array();
	}
	if ($object_statut != '') $viewstatut = $object_statut;
	if ($request_id > 0 && $view_request_status == "") $view_request_status = $request->statut;

	// Mass actions
	$objectclass = 'InvitationUser';
	$objectlabel = 'InvitationUser';
	$permtoread = $user->rights->questionnaire->read;
	$permtodelete = $user->rights->questionnaire->delete;
	$uploaddir = $conf->questionnaire->dir_output;
	include DOL_DOCUMENT_ROOT . '/core/actions_massactions.inc.php';

	if (!$error) {
		// Create survey
		if ($action == 'confirm_rm_create_survey' && $confirm == 'yes' && $request_id > 0) {
			$questionnaire_id = GETPOST('questionnaire_id', 'int');
			$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));
			$result = $request->createSurvey($user, $questionnaire_id, $date_answer_deadline);
			if ($result < 0 || !empty($request->errors)) {
				setEventMessages($request->error, $request->errors, 'errors');
			} else {
				header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . $request_id);
				exit();
			}
		} elseif ($action == 'confirm_rm_send_survey' && $confirm == 'yes' && is_array($toselect)) {
			$questionnaire_id = GETPOST('questionnaire_id', 'int');
			$only_new = GETPOST('only_new', 'aZ09');
			$force_date_answer_deadline = GETPOST('force_date_answer_deadline', 'aZ09');
			$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));
			$subject = GETPOST('subject', 'none');
			$message = GETPOST('message', 'none');

			if ($request_id > 0) {
				$selected_list = array();
				foreach ($toselect as $ids) {
					$tmp = explode('_', $ids);
					$selected_list[$tmp[1]] = $tmp[1];
				}
				$result = $request->sendSurvey($user, $questionnaire_id, $date_answer_deadline, $subject, $message, !empty($force_date_answer_deadline), !empty($only_new), $selected_list);
				if ($result < 0 || !empty($request->errors)) {
					setEventMessages($request->error, $request->errors, 'errors');
				}
			} else {
				$request_list = array();
				foreach ($toselect as $ids) {
					$tmp = explode('_', $ids);
					$request_list[$tmp[1]][$tmp[2]] = $tmp[2];
				}
				foreach ($request_list as $r_id => $selected_list) {
					$request = new RequestManager($db);
					$result = $request->fetch($r_id);
					if ($result > 0) $result = $request->sendSurvey($user, $questionnaire_id, $date_answer_deadline, $subject, $message, !empty($force_date_answer_deadline), !empty($only_new), $selected_list);
					if ($result < 0 || !empty($request->errors)) {
						setEventMessage($langs->trans('RequestManagerErrorWhenSendSurveyOfRequest', $request->ref, $r_id), 'errors');
						setEventMessages($request->error, $request->errors, 'errors');
					}
				}
			}
		} elseif ($action == 'confirm_rm_update_date_limit' && $confirm == 'yes' && is_array($toselect)) {
			$date_limit = dol_mktime(0, 0, 0, GETPOST('date_limitmonth', 'int'), GETPOST('date_limitday', 'int'), GETPOST('date_limityear', 'int'));

			$invitation_user = new InvitationUser($db);
			foreach ($toselect as $ids) {
				$tmp = explode('_', $ids);
				$inv_selected = $tmp[0];
				$result = $invitation_user->load($inv_selected);
				$invitation_user->date_limite_reponse = $date_limit;
				if ($result > 0) $result = $invitation_user->save();
				if ($result < 0) {
					setEventMessage($langs->trans('RequestManagerErrorWhenUpdateDateLimitInvitationUser', $inv_selected, $invitation_user->ref), 'errors');
					setEventMessages($invitation_user->error, $invitation_user->errors, 'errors');
				}
			}
		} elseif ($action == 'confirm_rm_reopen' && $confirm == 'yes' && is_array($toselect)) {
			$invitation_user = new InvitationUser($db);
			foreach ($toselect as $ids) {
				$tmp = explode('_', $ids);
				$inv_selected = $tmp[0];
				$result = $invitation_user->load($inv_selected);
				if ($result > 0) $result = $invitation_user->reopen();
				if ($result < 0) {
					setEventMessage($langs->trans('RequestManagerErrorWhenReOpenInvitationUser', $inv_selected, $invitation_user->ref), 'errors');
					setEventMessages($invitation_user->error, $invitation_user->errors, 'errors');
				}
			}
		} elseif ($action == 'confirm_rm_cancel' && $confirm == 'yes' && is_array($toselect)) {
			$invitation_user = new InvitationUser($db);
			foreach ($toselect as $ids) {
				$tmp = explode('_', $ids);
				$inv_selected = $tmp[0];
				$result = $invitation_user->load($inv_selected);
				if ($result > 0) $result = $invitation_user->cancel();
				if ($result < 0) {
					setEventMessage($langs->trans('RequestManagerErrorWhenCancelInvitationUser', $inv_selected, $invitation_user->ref), 'errors');
					setEventMessages($invitation_user->error, $invitation_user->errors, 'errors');
				}
			}
		} elseif ($action == 'confirm_rm_delete' && $confirm == 'yes' && $user->rights->questionnaire->delete && is_array($toselect)) {
			$questionnaire_cached = array();

			foreach ($toselect as $ids) {
				$tmp = explode('_', $ids);
				$inv_selected = $tmp[0];
				$error = 0;
				$db->begin();

				$invitation_user = new InvitationUser($db);
				$result = $invitation_user->load($inv_selected);
				if ($result > 0) $result = $invitation_user->delete($user);
				if ($result < 0) {
					setEventMessage($langs->trans('RequestManagerErrorWhenDeleteInvitationUser', $inv_selected, $invitation_user->ref), 'errors');
					setEventMessages($invitation_user->error, $invitation_user->errors, 'errors');
					$error++;
				}
				if ($result > 0 && $invitation_user->statut != InvitationUser::STATUS_VALIDATED) {
					// Load survey
					if (!isset($questionnaire_cached[$invitation_user->fk_questionnaire])) {
						$questionnaire = new Questionnaire($db);
						$result = $questionnaire->load($invitation_user->fk_questionnaire);
						if ($result > 0) $questionnaire_cached[$invitation_user->fk_questionnaire] = $questionnaire;
					} else {
						$questionnaire = $questionnaire_cached[$invitation_user->fk_questionnaire];
					}
					if ($result > 0) $result = $questionnaire->deleteAllAnswersUser($inv_selected);
					if ($result < 0) {
						setEventMessage($langs->trans('RequestManagerErrorWhenDeleteAllAnswersUser', $inv_selected, $invitation_user->ref), 'errors');
						setEventMessages($questionnaire->error, $questionnaire->errors, 'errors');
						$error++;
					}
				}

				if ($error) {
					$db->rollback();
				} else {
					$db->commit();
				}
			}
		}
	}
}


/*
 * View
 */

$form = new Form($db);

llxHeader('', $langs->trans('RequestManagerRequest'), 'EN:Request_Manager_En|FR:Request_Manager_Fr|ES:Request_Manager_Es');

$title = $langs->trans('RequestManagerFollowSurvey');

if ($request_id > 0) {
	$title = $langs->trans('RequestManagerSurveyOnRequest');

	$head = requestmanager_prepare_head($request);
	print dol_get_fiche_head($head, 'request_survey', $langs->trans('RequestManagerCard'), -1, 'requestmanager@requestmanager');

	// Request manager card
	$linkback = '<a href="' . dol_buildpath('/requestmanager/list.php', 1) . '?restore_lastsearch_values=1">' . $langs->trans("BackToList") . '</a>';

	$morehtmlref = '<div class="refidno">';
	// External Reference
	$morehtmlref .= $langs->trans('RequestManagerExternalReference') . ' : ' . $request->ref_ext;
	// Type
	$morehtmlref .= '<br>' . $langs->trans('RequestManagerType') . ' : ' . $request->getLibType();
	// Label
	$morehtmlref .= '<br>' . $langs->trans('RequestManagerLabel') . ' : ' . $request->label;
	// Parent request
	if ($request->fk_parent > 0) {
		$request->fetch_parent();
		$morehtmlref .= '<br>' . $langs->trans('RequestManagerParentRequest') . ' : ' . $request->parent->getNomUrl(1, 'parent_path');
	}
	// Thirdparty
	$origin_type = $request->getRequestTypeOption('origin_type');
	if ($request->socid > 0 && empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_THIRDPARTY) {
		$request->fetch_thirdparty();
		$morehtmlref .= '<br>' . $langs->trans('RequestManagerThirdPartyOrigin') . ' : ' . $request->thirdparty->getNomUrl(1);
	}
	$morehtmlref .= '</div>';

	dol_banner_tab($request, 'ref', $linkback, 1, 'ref', 'ref', $morehtmlref);

	print '<div class="fichecenter">';
	print '<div class="underbanner clearboth"></div>';

	print '<table class="border centpercent">';
	print "</table>\n";

	print '</div>';

	print dol_get_fiche_end();
}

$sql = 'SELECT qiu.rowid, qiu.ref, qiu.fk_element, qiu.type_element';
if (empty($request_id)) $sql .= ', rmqiu.fk_requestmanager, rm.ref AS request_ref, rm.fk_type AS fk_request_type, rm.label AS request_label';
$sql .= ', qiu.fk_questionnaire, qq.ref AS questionnaire_ref, qq.title AS questionnaire_title';
$sql .= ', sp.rowid as contact_id, sp.firstname as contact_firstname, sp.lastname as contact_lastname';
$sql .= ', qiu.token, qiu.email, qiu.sent, qiu.date_envoi, qiu.date_sent_remind, qiu.date_limite_reponse, qiu.date_validation';
$sql .= ', qiu.date_creation, qiu.tms, qiu.fk_statut';
// Add fields from extrafields
if (version_compare(DOL_VERSION, "13.0.0") >= 0) {
	if (!empty($extrafields->attributes[$object->table_element]['label'])) {
		foreach ($extrafields->attributes[$object->table_element]['label'] as $key => $val) {
			$sql .= ($extrafields->attributes[$object->table_element]['type'][$key] != 'separate' ? ", ef.".$key.' as options_'.$key : '');
		}
	}
} else {
	foreach ($extrafields->attribute_label as $key => $val) $sql .= ($extrafields->attribute_type[$key] != 'separate' ? ", ef." . $key . ' as options_' . $key : '');
}
// Add fields from hooks
$parameters = array();
$reshook = $hookmanager->executeHooks('printFieldListSelect', $parameters);    // Note that $action and $object may have been modified by hook
$sql .= $hookmanager->resPrint;
$sql .= ' FROM ' . MAIN_DB_PREFIX . 'requestmanager_quest_invitation_user AS rmqiu';
$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "quest_invitation_user AS qiu ON qiu.rowid = rmqiu.fk_quest_invitation_user";
if (version_compare(DOL_VERSION, "13.0.0") >= 0) {
	if (isset($extrafields->attributes[$object->table_element]['label']) && is_array($extrafields->attributes[$object->table_element]['label']) && count($extrafields->attributes[$object->table_element]['label'])) {
		$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . $object->table_element . "_extrafields as ef on (rm.rowid = ef.fk_object)";
	}
} else {
	if (is_array($extrafields->attribute_label) && count($extrafields->attribute_label)) $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "requestmanager_extrafields as ef on (rm.rowid = ef.fk_object)";
}
$sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "quest_questionnaire AS qq ON qq.rowid = qiu.fk_questionnaire";
if (empty($request_id)) $sql .= " LEFT JOIN " . MAIN_DB_PREFIX . "requestmanager AS rm ON rm.rowid = rmqiu.fk_requestmanager";
$sql .= ' LEFT JOIN ' . MAIN_DB_PREFIX . 'socpeople AS sp ON sp.rowid = qiu.fk_element AND qiu.type_element = \'contact\'';
// Add From from hooks
$parameters = array();
$reshook = $hookmanager->executeHooks('printFieldListFrom', $parameters);    // Note that $action and $object may have been modified by hook
$sql .= $hookmanager->resPrint;
$sql .= ' WHERE qq.entity IN (' . getEntity('questionnaire', 1) . ')';
if(empty($user->rights->questionnaire->readall)) $sql.= ' AND qq.fk_user_author = '.$user->id;
if ($request_id > 0) {
	$sql .= ' AND rmqiu.fk_status = ' . $view_request_status;
	$sql .= ' AND rmqiu.fk_requestmanager = ' . $request_id;
} elseif ($search_request) 						$sql .= natural_search('rm.ref', $search_request);
if ($search_ref)								$sql .= natural_search('qiu.ref', $search_ref);
if ($search_questionnaire)						$sql .= natural_search(array('qq.ref', 'qq.title'), $search_questionnaire);
if ($search_contact)							$sql .= natural_search(array('sp.firstname', 'sp.lastname'), $search_contact);
if ($search_email)								$sql .= natural_search('qiu.email', $search_email);
if ($search_sent != '' && $search_sent != '-1')	$sql .= " AND qiu.sent = " . $db->escape($search_sent);
if ($search_date_sent_start)					$sql .= " AND qiu.date_envoi >= '" . $db->idate($search_date_sent_start) . "'";
if ($search_date_sent_end)						$sql .= " AND qiu.date_envoi <= '" . $db->idate($search_date_sent_end) . "'";
if ($search_date_sent_remind_start)				$sql .= " AND qiu.date_sent_remind >= '" . $db->idate($search_date_sent_remind_start) . "'";
if ($search_date_sent_remind_end)				$sql .= " AND qiu.date_sent_remind <= '" . $db->idate($search_date_sent_remind_end) . "'";
if ($search_date_limit_answer_start)			$sql .= " AND qiu.date_limite_reponse >= '" . $db->idate($search_date_limit_answer_start) . "'";
if ($search_date_limit_answer_end)				$sql .= " AND qiu.date_limite_reponse <= '" . $db->idate($search_date_limit_answer_end) . "'";
if ($search_date_validation_start)				$sql .= " AND qiu.date_validation >= '" . $db->idate($search_date_validation_start) . "'";
if ($search_date_validation_end)				$sql .= " AND qiu.date_validation <= '" . $db->idate($search_date_validation_end) . "'";
if ($search_date_creation_start)				$sql .= " AND qiu.date_creation >= '" . $db->idate($search_date_creation_start) . "'";
if ($search_date_creation_end)					$sql .= " AND qiu.date_creation <= '" . $db->idate($search_date_creation_end) . "'";
if ($search_date_modification_start)			$sql .= " AND qiu.tms >= '" . $db->idate($search_date_modification_start) . "'";
if ($search_date_modification_end)				$sql .= " AND qiu.tms <= '" . $db->idate($search_date_modification_end) . "'";
if ($viewstatut != '' && $viewstatut != '-1')	$sql .= " AND qiu.fk_statut IN (" . $db->escape($viewstatut) . ')';
// Add where from extra fields
include DOL_DOCUMENT_ROOT.'/core/tpl/extrafields_list_search_sql.tpl.php';
// Add where from hooks
$parameters = array();
$reshook = $hookmanager->executeHooks('printFieldListWhere', $parameters);    // Note that $action and $object may have been modified by hook
$sql .= $hookmanager->resPrint;

$sql .= ' GROUP BY qiu.rowid';
$sql .= $db->order($sortfield, $sortorder);

// Count total nb of records
$nbtotalofrecords = '';
if (empty($conf->global->MAIN_DISABLE_FULL_SCANLIST)) {
	$result = $db->query($sql);
	$nbtotalofrecords = $db->num_rows($result);
}

$sql .= $db->plimit($limit + 1, $offset);

$resql = $db->query($sql);
if ($resql) {
	$num = $db->num_rows($resql);

	$arrayofselected = is_array($toselect) ? $toselect : array();

	$param = '&viewstatut=' . urlencode($viewstatut) . '&view_request_status=' . urlencode($view_request_status);
	if (!empty($contextpage) && $contextpage != $_SERVER["PHP_SELF"]) $param .= '&contextpage=' . urlencode($contextpage);
	if ($limit > 0 && $limit != $conf->liste_limit) $param .= '&limit=' . urlencode($limit);
	if ($search_ref) $param .= '&search_ref=' . urlencode($search_ref);
	if ($search_questionnaire) $param .= '&search_questionnaire=' . urlencode($search_questionnaire);
	if ($search_request) $param .= '&search_request=' . urlencode($search_request);
	if ($search_contact) $param .= '&search_contact=' . urlencode($search_contact);
	if ($search_email) $param .= '&search_email=' . urlencode($search_email);
	if ($search_sent != '' && $search_sent != '-1') $param .= '&search_sent=' . urlencode($search_sent);
	if ($search_date_sent_start) $param .= '&search_date_sent_start=' . urlencode($search_date_sent_start);
	if ($search_date_sent_end) $param .= '&search_date_sent_end=' . urlencode($search_date_sent_end);
	if ($search_date_sent_remind_start) $param .= '&search_date_sent_remind_start=' . urlencode($search_date_sent_remind_start);
	if ($search_date_sent_remind_end) $param .= '&search_date_sent_remind_end=' . urlencode($search_date_sent_remind_end);
	if ($search_date_limit_answer_start) $param .= '&search_date_limit_answer_start=' . urlencode($search_date_limit_answer_start);
	if ($search_date_limit_answer_end) $param .= '&search_date_limit_answer_end=' . urlencode($search_date_limit_answer_end);
	if ($search_date_validation_start) $param .= '&search_date_validation_start=' . urlencode($search_date_validation_start);
	if ($search_date_validation_end) $param .= '&search_date_validation_end=' . urlencode($search_date_validation_end);
	if ($search_date_creation_start) $param .= '&search_date_creation_start=' . urlencode($search_date_creation_start);
	if ($search_date_creation_end) $param .= '&search_date_creation_end=' . urlencode($search_date_creation_end);
	if ($search_date_modification_start) $param .= '&search_date_modification_start=' . urlencode($search_date_modification_start);
	if ($search_date_modification_end) $param .= '&search_date_modification_end=' . urlencode($search_date_modification_end);
	if ($optioncss != '') $param .= '&optioncss=' . urlencode($optioncss);
	// Add $param from extra fields
	include DOL_DOCUMENT_ROOT . '/core/tpl/extrafields_list_search_param.tpl.php';

	// List of mass actions available
	$arrayofmassactions = array(
		'pre_rm_send_survey' => $langs->trans("RequestManagerSendSurvey"),
		'pre_rm_update_date_limit' => $langs->trans("RequestManagerUpdateDateLimit"),
		'pre_rm_reopen' => $langs->trans("ReOpen"),
		'pre_rm_cancel' => $langs->trans("Cancel"),
	);
	if ($user->rights->questionnaire->delete) $arrayofmassactions['pre_rm_delete'] = '<span class="fal fa-trash paddingrightonly"></span>' . $langs->trans("Delete");
	if (in_array($massaction, array('pre_rm_send_survey', 'pre_rm_update_date_limit', 'pre_rm_reopen', 'pre_rm_cancel', 'pre_rm_delete'))) $arrayofmassactions = array();
	$massactionbutton = $form->selectMassAction('', $arrayofmassactions);

	// Lignes des champs de filtre
	print '<form method="POST" id="searchFormList" action="' . $_SERVER["PHP_SELF"] . ($request_id > 0 ? '?id=' . $request_id : '') . '">';
	if ($optioncss != '') print '<input type="hidden" name="optioncss" value="' . $optioncss . '">';
	print '<input type="hidden" name="token" value="' . $_SESSION['newtoken'] . '">';
	print '<input type="hidden" name="formfilteraction" id="formfilteraction" value="list">';
	print '<input type="hidden" name="action" value="list">';
	print '<input type="hidden" name="sortfield" value="' . $sortfield . '">';
	print '<input type="hidden" name="sortorder" value="' . $sortorder . '">';
	print '<input type="hidden" name="page" value="' . $page . '">';

	$morehtmlright = '';
	// Send survey
	if ($request_id > 0 && !($massaction == 'pre_rm_send_survey' || $action == 'rm_send_survey' || $action == 'rm_create_survey')) {
		$morehtmlright = '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $request_id . $param . '&token=' . newToken(). '&action=rm_create_survey">' . $langs->trans("RequestManagerCreateSurvey") . '</a>';
		$morehtmlright .= '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?id=' . $request_id . $param . '&token=' . newToken() . '&action=rm_send_survey">' . $langs->trans("RequestManagerSendSurvey") . '</a>';
	}

	print_barre_liste($title, $page, $_SERVER["PHP_SELF"] . ($request_id > 0 ? '?id=' . $request_id : ''), $param, $sortfield, $sortorder, $massactionbutton, $num, $nbtotalofrecords, 'questionnaire@questionnaire', 0, $morehtmlright, '', $limit);

	$formconfirm = '';

	// Confirm create survey
	if ($action == 'rm_create_survey' && $request_id > 0) {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));

		$questionnaires = $request->getRequestQuestionnaire();
		$questionnaire = null;
		if (is_array($questionnaires) && count($questionnaires) == 1) {
			$questionnaire = array_values($questionnaires)[0];
		}
		if ($date_answer_deadline === '') $date_answer_deadline = dol_now() + (is_object($questionnaire) && $questionnaire->answer_deadline > 0 ? $questionnaire->answer_deadline : $conf->global->QUESTIONNAIRE_DEFAULT_ANSWER_DEADLINE) * 24 * 60 * 60;

		$questionnaire_arr = array();
		if (is_array($questionnaires)) {
			foreach ($questionnaires as $questionnaire) {
				if ($questionnaire_id == '') $questionnaire_id = $questionnaire->id;
				$questionnaire_arr[$questionnaire->id] = $questionnaire->ref . ' - ' . $questionnaire->title;
			}
		}

		$formquestion = array(
			array('type' => 'select', 'tdclass' => 'titlefieldcreate', 'name' => 'questionnaire_id', 'label' => $langs->trans('RequestManagerSurvey'), 'values' => $questionnaire_arr, 'default' => $questionnaire_id),
			array('type' => 'date', 'tdclass' => 'titlefieldcreate', 'name' => 'date_answer_deadline', 'label' => $langs->trans('RequestManagerSetDateAnswerDeadline'), 'value' => $date_answer_deadline),
		);

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . $request_id, $langs->trans('RequestManagerCreateSurvey'), $langs->trans('RequestManagerConfirmCreateSurvey'), 'confirm_rm_create_survey', $formquestion, '', 0, 200, 500, 1);
	}
	// Confirm send survey
	elseif ($massaction == 'pre_rm_send_survey' || $action == 'rm_send_survey') {
		$questionnaire_id = GETPOST('questionnaire_id', 'int');
		$only_new = GETPOST('only_new', 'aZ09');
		$force_date_answer_deadline = GETPOST('force_date_answer_deadline', 'aZ09');
		$date_answer_deadline = dol_mktime(0, 0, 0, GETPOST('date_answer_deadlinemonth', 'int'), GETPOST('date_answer_deadlineday', 'int'), GETPOST('date_answer_deadlineyear', 'int'));

		$nb_selected = count($toselect);
		$questionnaires = $request->getRequestQuestionnaire();
		$questionnaire = null;
		if (is_array($questionnaires) && count($questionnaires) == 1) {
			$questionnaire = array_values($questionnaires)[0];
		}
		if ($date_answer_deadline === '') $date_answer_deadline = dol_now() + (is_object($questionnaire) && $questionnaire->answer_deadline > 0 ? $questionnaire->answer_deadline : $conf->global->QUESTIONNAIRE_DEFAULT_ANSWER_DEADLINE) * 24 * 60 * 60;

		// Cree l'objet formulaire mail
		include_once DOL_DOCUMENT_ROOT . '/core/class/html.formmail.class.php';
		$formmail = new FormMail($db);
		$formmail->withform = 0;
		$formmail->withfrom = false;
		$formmail->withto = false;
		$formmail->withtouser = false;
		$formmail->withoptiononeemailperrecipient = false;
		$formmail->withtocc = false;
		$formmail->withtoccuser = false;
		$formmail->withtoccc = false;
		$formmail->withreplyto = false;
		$formmail->witherrorsto = false;
		$formmail->withdeliveryreceipt = false;
		$formmail->withfile = false;

		dol_include_once('/questionnaire/class/invitation.class.php');
		$invitation_user_static = new InvitationUser($db);
		// Make substitution in email content
		$substitutionarray = getCommonSubstitutionArray($langs, 1, null, $invitation_user_static);

		$substitutionarray['__CHECK_READ__'] = '__CHECK_READ__';
		$substitutionarray['__PERSONALIZED__'] = '__PERSONALIZED__';    // deprecated
		$substitutionarray['__CONTACTCIVNAME__'] = '__CONTACTCIVNAME__';

		$parameters = array(
			'mode' => 'formemail',
			'onlykey' => true,
		);
		complete_substitutions_array($substitutionarray, $langs, $invitation_user_static, $parameters);

		// Tableau des substitutions
		$formmail->substit = $substitutionarray;

		// Tableau des parametres complementaires du post
		$formmail->param['models'] = 'questionnaire';
		$formmail->param['models_id'] = GETPOST('modelmailselected', 'int');
		if (!empty($conf->global->MAILING_LIMIT_SENDBYWEB) && $nb_selected > $conf->global->MAILING_LIMIT_SENDBYWEB) {
			$langs->load("errors");
			setEventMessage(img_warning() . ' ' . $langs->trans('WarningNumberOfRecipientIsRestrictedInMassAction', $conf->global->MAILING_LIMIT_SENDBYWEB), 'errors');
		} else {
			$questionnaire_arr = array();
			if (is_array($questionnaires)) {
				foreach ($questionnaires as $questionnaire) {
					$questionnaire_arr[$questionnaire->id] = $questionnaire->ref . ' - ' . $questionnaire->title;
				}
			}

			$formquestion = array();

			if ($v13p) {
				$more_name = ',subject,message';
			} else {
				$more_name = '';
				$formquestion[] = array('name' => 'subject');
				$formquestion[] = array('name' => 'message');
			}
			$formquestion[] = array('type' => 'onecolumn', 'name' => 'modelselected' . $more_name, 'value' => $formmail->get_form());

			$formquestion[] = array('type' => 'other', 'tdclass' => 'titlefieldcreate', 'name' => 'questionnaire_id', 'label' => $langs->trans('RequestManagerSurvey'), 'value' => $form->selectarray('questionnaire_id', $questionnaire_arr, $questionnaire_id));

			if ($v13p) {
				$more_name = ',date_answer_deadline,date_answer_deadlineday,date_answer_deadlinemonth,date_answer_deadlineyear,date_answer_deadlinehour,date_answer_deadlinemin';
			} else {
				$more_name = '';
				$formquestion[] = array('name' => 'date_answer_deadline');
				$formquestion[] = array('name' => 'date_answer_deadlineday');
				$formquestion[] = array('name' => 'date_answer_deadlinemonth');
				$formquestion[] = array('name' => 'date_answer_deadlineyear');
				$formquestion[] = array('name' => 'date_answer_deadlinehour');
				$formquestion[] = array('name' => 'date_answer_deadlinemin');
			}
			$formquestion[] = array('type' => 'other', 'tdclass' => 'titlefieldcreate', 'name' => 'force_date_answer_deadline', 'label' => $form->textwithpicto($langs->trans('RequestManagerSetDateAnswerDeadline'), $langs->trans('RequestManagerSetDateAnswerDeadlineHelp')), 'value' => '<input type="checkbox" id="force_date_answer_deadline" name="force_date_answer_deadline"' . ($force_date_answer_deadline ? ' selected="selected"' : '') . '> ' . $form->select_date($date_answer_deadline, 'date_answer_deadline', 0, 0, 0, '', 1, 0, 1));

			$formquestion[] = array('type' => 'checkbox', 'tdclass' => 'titlefieldcreate', 'name' => 'only_new', 'label' => $form->textwithpicto($langs->trans('RequestManagerSendOnlyToNewInvitation'), $langs->trans('RequestManagerSendOnlyToNewInvitationHelp')), 'value' => !empty($only_new));

			$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans('RequestManagerConfirmMassSendSurvey'), $langs->trans($nb_selected > 0 ? 'RequestManagerConfirmMassSendSurveyQuestion' : 'RequestManagerConfirmMassSendSurveyAllQuestion', $nb_selected), 'confirm_rm_send_survey', $formquestion, '', 0, 200, 500, 1);
			$formconfirm .= <<<SCRIPT
	<script type="text/javascript">
		jQuery(document).ready(function() {
			jQuery("#subject").closest('tr').find('td:first').addClass('titlefieldcreate');
		});
	</script>
SCRIPT;
		}
	} // Confirm modification date limit of the survey
	elseif ($massaction == 'pre_rm_update_date_limit') {
		$date_limit = dol_mktime(0, 0, 0, GETPOST('date_limitmonth', 'int'), GETPOST('date_limitday', 'int'), GETPOST('date_limityear', 'int'));

		$questionnaires = $request->getRequestQuestionnaire();
		$questionnaire = null;
		if (is_array($questionnaires) && count($questionnaires) == 1) {
			$questionnaire = array_values($questionnaires)[0];
		}
		if ($date_limit === '') $date_limit = dol_now() + (is_object($questionnaire) && $questionnaire->answer_deadline > 0 ? $questionnaire->answer_deadline : $conf->global->QUESTIONNAIRE_DEFAULT_ANSWER_DEADLINE) * 24 * 60 * 60;

		$formquestion = array(
			array('type' => 'date', 'name' => 'date_limit', 'label' => $langs->trans('questionnaire_date_limite_reponse'), 'value' => $date_limit),
		);

		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans("RequestManagerConfirmMassUpdateDateLimit"), $langs->trans("RequestManagerConfirmMassUpdateDateLimitQuestion", count($toselect)), "confirm_rm_update_date_limit", $formquestion, '', 0, 200, 500, 1);
	} // Confirm reopen survey
	elseif ($massaction == 'pre_rm_reopen') {
		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans("RequestManagerConfirmMassReOpening"), $langs->trans("RequestManagerConfirmMassReOpeningQuestion", count($toselect)), "confirm_rm_reopen", null, '', 0, 200, 500, 1);
	} // Confirm cancel survey
	elseif ($massaction == 'pre_rm_cancel') {
		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans("RequestManagerConfirmMassCancel"), $langs->trans("RequestManagerConfirmMassCancelQuestion", count($toselect)), "confirm_rm_cancel", null, '', 0, 200, 500, 1);
	} // Confirm delete survey
	elseif ($massaction == 'pre_rm_delete') {
		$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans("ConfirmMassDeletion"), $langs->trans("ConfirmMassDeletionQuestion", count($toselect)), "confirm_rm_delete", null, '', 0, 200, 500, 1);
	}

	// Hook
	$parameters = array();
	$reshook = $hookmanager->executeHooks('formConfirm', $parameters, $request, $action); // Note that $action and $object may have been modified by hook
	if (empty($reshook)) $formconfirm.=$hookmanager->resPrint;
	elseif ($reshook > 0) $formconfirm=$hookmanager->resPrint;

	// Print form confirm
	print $formconfirm;

	$moreforfilter = '';

	// Filter on request status
	if ($request_id > 0) {
		$moreforfilter .= '<div class="divsearchfield">';
		$moreforfilter .= $langs->trans('RequestManagerStatus') . ': ';
		$status_array = array();
		foreach (RequestManager::$status_list as $status) {
			if ($request->statut != $status->id && $request->hasSurvey($status->id) < 1) continue;
			$status_array[$status->id] = $request->LibStatut($status->id, 13);
		}
		$moreforfilter .= $form->selectarray('view_request_status', $status_array, $view_request_status, 0, 0, 0, '', 0, 0, 0, '', ' minwidth300');
		$moreforfilter .= '</div>';
	}

	$parameters = array();
	$reshook = $hookmanager->executeHooks('printFieldPreListTitle', $parameters);    // Note that $action and $object may have been modified by hook
	if (empty($reshook)) $moreforfilter .= $hookmanager->resPrint;
	else $moreforfilter = $hookmanager->resPrint;

	if (!empty($moreforfilter)) {
		print '<div class="liste_titre liste_titre_bydiv centpercent">';
		print $moreforfilter;
		print '</div>';
	}

	$varpage = empty($contextpage) ? $_SERVER["PHP_SELF"] : $contextpage;
	$selectedfields = $form->multiSelectArrayWithCheckbox('selectedfields', $arrayfields, $varpage);    // This also change content of $arrayfields
	if ($massactionbutton) $selectedfields .= $form->showCheckAddButtons('checkforselect', 1);

	print '<div class="div-table-responsive">';
	print '<table class="tagtable nobottomiftotal liste' . ($moreforfilter ? " listwithfilterbefore" : "") . '">' . "\n";

	print '<tr class="liste_titre_filter">';
	// Ref
	if (!empty($arrayfields['qiu.ref']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_ref" value="' . dol_escape_htmltag($search_ref) . '">';
		print '</td>';
	}
	// Survey Ref
	if (!empty($arrayfields['qiu.fk_questionnaire']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_questionnaire" value="' . dol_escape_htmltag($search_questionnaire) . '">';
		print '</td>';
	}
	// Request Ref
	if (!empty($arrayfields['rmqiu.fk_requestmanager']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_request" value="' . dol_escape_htmltag($search_request) . '">';
		print '</td>';
	}
	// Contact
	if (!empty($arrayfields['qiu.fk_element']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_contact" value="' . dol_escape_htmltag($search_contact) . '">';
		print '</td>';
	}
	// Email
	if (!empty($arrayfields['qiu.email']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_email" value="' . dol_escape_htmltag($search_email) . '">';
		print '</td>';
	}
	// Sent mail status
	if (!empty($arrayfields['qiu.sent']['checked'])) {
		print '<td class="liste_titre center">';
		$sent_status_list = array(
			InvitationUser::STATUS_DRAFT => 'questionnaireStatusNotSent',
			InvitationUser::STATUS_VALIDATED => 'questionnaireStatusSent',
		);
		print $form->selectarray('search_sent', $sent_status_list, $search_sent, 1, 0, 0, '', 1);
		print '</td>';
	}
	// Date sent mail
	if (!empty($arrayfields['qiu.date_envoi']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_sent_start ? $search_date_sent_start : -1, 'search_date_sent_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_sent_end ? $search_date_sent_end : -1, 'search_date_sent_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Date sent remind mail
	if (!empty($arrayfields['qiu.date_sent_remind']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_sent_remind_start ? $search_date_sent_remind_start : -1, 'search_date_sent_remind_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_sent_remind_end ? $search_date_sent_remind_end : -1, 'search_date_sent_remind_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Date limit answer
	if (!empty($arrayfields['qiu.date_limite_reponse']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_limit_answer_start ? $search_date_limit_answer_start : -1, 'search_date_limit_answer_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_limit_answer_end ? $search_date_limit_answer_end : -1, 'search_date_limit_answer_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Date validation
	if (!empty($arrayfields['qiu.date_validation']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_validation_start ? $search_date_validation_start : -1, 'search_date_validation_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_validation_end ? $search_date_validation_end : -1, 'search_date_validation_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Extra fields
	include DOL_DOCUMENT_ROOT . '/core/tpl/extrafields_list_search_input.tpl.php';

	// Fields from hook
	$parameters = array('arrayfields' => $arrayfields);
	$reshook = $hookmanager->executeHooks('printFieldListOption', $parameters);    // Note that $action and $object may have been modified by hook
	print $hookmanager->resPrint;

	// Date creation
	if (!empty($arrayfields['qiu.date_creation']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_creation_start ? $search_date_creation_start : -1, 'search_date_creation_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_creation_end ? $search_date_creation_end : -1, 'search_date_creation_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Date modification
	if (!empty($arrayfields['qiu.tms']['checked'])) {
		print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $langs->trans('From') . ' ';
		print $form->selectDate($search_date_modification_start ? $search_date_modification_start : -1, 'search_date_modification_start', 0, 0, 1);
		print '</div>';
		print '<div class="nowrap">';
		print $langs->trans('to') . ' ';
		print $form->selectDate($search_date_modification_end ? $search_date_modification_end : -1, 'search_date_modification_end', 0, 0, 1);
		print '</div>';
		print '</td>';
	}
	// Status
	if (!empty($arrayfields['qiu.fk_statut']['checked'])) {
		print '<td class="liste_titre maxwidthonsmartphone right">';
		$status_list = array(
			InvitationUser::STATUS_DRAFT => 'answerStatusDraft',
			InvitationUser::STATUS_SAVED => 'answerStatusSaved',
			InvitationUser::STATUS_VALIDATED => 'answerStatusValidated',
			InvitationUser::STATUS_PARTIALLY => 'answerStatusPartially',
			InvitationUser::STATUS_CANCEL => 'answerStatusCancel',
		);
		print $form->selectarray('search_statut', $status_list, $viewstatut, 1, 0, 0, '', 1);
		print '</td>';
	}
	// Link
	if (!empty($arrayfields['link']['checked'])) {
		print '<td class="liste_titre">';
		print '</td>';
	}
	// Action column
	print '<td class="liste_titre center">';
	$searchpicto = $form->showFilterButtons();
	print $searchpicto;
	print '</td>';

	print "</tr>\n";

	// Fields title
	if ($request_id > 0) $param .= '&id=' . urlencode($request_id);
	print '<tr class="liste_titre">';
	if (!empty($arrayfields['qiu.ref']['checked'])) print_liste_field_titre($arrayfields['qiu.ref']['label'], $_SERVER["PHP_SELF"], 'qiu.ref', '', $param, '', $sortfield, $sortorder);
	if (!empty($arrayfields['qiu.fk_questionnaire']['checked'])) print_liste_field_titre($arrayfields['qiu.fk_questionnaire']['label'], $_SERVER["PHP_SELF"], 'qq.ref', '', $param, '', $sortfield, $sortorder);
	if (!empty($arrayfields['rmqiu.fk_requestmanager']['checked'])) print_liste_field_titre($arrayfields['rmqiu.fk_requestmanager']['label'], $_SERVER["PHP_SELF"], 'rm.ref', '', $param, '', $sortfield, $sortorder);
	if (!empty($arrayfields['qiu.fk_element']['checked'])) print_liste_field_titre($arrayfields['qiu.fk_element']['label'], $_SERVER["PHP_SELF"], 'sp.lastname', '', $param, '', $sortfield, $sortorder);
	if (!empty($arrayfields['qiu.email']['checked'])) print_liste_field_titre($arrayfields['qiu.email']['label'], $_SERVER["PHP_SELF"], 'qiu.email', '', $param, '', $sortfield, $sortorder);
	if (!empty($arrayfields['qiu.sent']['checked'])) print_liste_field_titre($arrayfields['qiu.sent']['label'], $_SERVER["PHP_SELF"], 'qiu.sent', '', $param, '', $sortfield, $sortorder, 'center ');
	if (!empty($arrayfields['qiu.date_envoi']['checked'])) print_liste_field_titre($arrayfields['qiu.date_envoi']['label'], $_SERVER["PHP_SELF"], 'qiu.date_envoi', '', $param, '', $sortfield, $sortorder, 'center ');
	if (!empty($arrayfields['qiu.date_sent_remind']['checked'])) print_liste_field_titre($arrayfields['qiu.date_sent_remind']['label'], $_SERVER["PHP_SELF"], 'qiu.date_sent_remind', '', $param, '', $sortfield, $sortorder, 'center ');
	if (!empty($arrayfields['qiu.date_limite_reponse']['checked'])) print_liste_field_titre($arrayfields['qiu.date_limite_reponse']['label'], $_SERVER["PHP_SELF"], 'qiu.date_limite_reponse', '', $param, '', $sortfield, $sortorder, 'center ');
	if (!empty($arrayfields['qiu.date_validation']['checked'])) print_liste_field_titre($arrayfields['qiu.date_validation']['label'], $_SERVER["PHP_SELF"], 'qiu.date_validation', '', $param, '', $sortfield, $sortorder, 'center ');
	// Extra fields
	include DOL_DOCUMENT_ROOT.'/core/tpl/extrafields_list_search_title.tpl.php';
	// Hook fields
	$parameters=array('arrayfields'=>$arrayfields,'param'=>$param,'sortfield'=>$sortfield,'sortorder'=>$sortorder);
	$reshook = $hookmanager->executeHooks('printFieldListTitle', $parameters);    // Note that $action and $object may have been modified by hook
	print $hookmanager->resPrint;
	if (!empty($arrayfields['qiu.date_creation']['checked'])) print_liste_field_titre($arrayfields['qiu.date_creation']['label'], $_SERVER["PHP_SELF"], "qiu.date_creation", "", $param, '', $sortfield, $sortorder, 'nowrap center ');
	if (!empty($arrayfields['qiu.tms']['checked'])) print_liste_field_titre($arrayfields['qiu.tms']['label'], $_SERVER["PHP_SELF"], "qiu.tms", "", $param, '', $sortfield, $sortorder, 'nowrap center ');
	if (!empty($arrayfields['qiu.fk_statut']['checked'])) print_liste_field_titre($arrayfields['qiu.fk_statut']['label'], $_SERVER["PHP_SELF"], "qiu.fk_statut", "", $param, '', $sortfield, $sortorder, 'right ');
	print_liste_field_titre($arrayfields['link']['label'], '', '', '', '', '', '', '', 'center ');
	print_liste_field_titre($selectedfields, $_SERVER["PHP_SELF"], "", '', '', '', $sortfield, $sortorder, 'maxwidthsearch center ');
	print '</tr>' . "\n";

	$invitation_user_static = new InvitationUser($db);
	$questionnaire_static = new Questionnaire($db);

	$base_url_answer_card = dol_buildpath('/questionnaire/answer/card.php', 1);
	$base_url_answer_link = empty($conf->global->QUESTIONNAIRE_CUSTOM_DOMAIN) ? dol_buildpath('/questionnaire/public/toAnswer.php', 2) : $conf->global->QUESTIONNAIRE_CUSTOM_DOMAIN . 'toAnswer.php';

	$requests_cached = array();
	$contacts_cached = array();
	$now = dol_now();
	$i = 0;
	while ($i < min($num, $limit)) {
		$obj = $db->fetch_object($resql);

		$invitation_user_static->id = $obj->rowid;
		$invitation_user_static->ref = $obj->ref;
		$invitation_user_static->fk_questionnaire = $obj->fk_questionnaire;
		$invitation_user_static->fk_element = $obj->fk_element;
		$invitation_user_static->type_element = $obj->type_element;
		$invitation_user_static->token = $obj->token;
		$invitation_user_static->email = $obj->email;
		$invitation_user_static->sent = $obj->sent;
		$invitation_user_static->date_envoi = $db->jdate($obj->date_envoi);
		$invitation_user_static->date_sent_remind = $db->jdate($obj->date_sent_remind);
		$invitation_user_static->date_limite_reponse = $db->jdate($obj->date_limite_reponse);
		$invitation_user_static->date_validation = $db->jdate($obj->date_validation);
		$invitation_user_static->date_creation = $db->jdate($obj->date_creation);
		$invitation_user_static->date_modification = $db->jdate($obj->tms);
		$invitation_user_static->fk_statut = $obj->fk_statut;

		$questionnaire_static->id = $obj->fk_questionnaire;
		$questionnaire_static->ref = $obj->questionnaire_ref;
		$questionnaire_static->title = $obj->questionnaire_title;

		print '<tr class="oddeven">';

		// Ref
		if (!empty($arrayfields['qiu.ref']['checked'])) {
			print '<td class="nowrap">';
			print '<a href="' . $base_url_answer_card . '?id=' . $invitation_user_static->id . '">' . $invitation_user_static->ref . '</a>';
			print '</td>';
		}
		// Survey Ref
		if (!empty($arrayfields['qiu.fk_questionnaire']['checked'])) {
			print '<td class="nowrap">';
			print $questionnaire_static->getNomUrl(1);
			if (!empty($questionnaire_static->title)) print ' - ' . $questionnaire_static->title;
			print '</td>';
		}
		// Request Ref
		if (!empty($arrayfields['rmqiu.fk_requestmanager']['checked'])) {
			if (!isset($requests_cached[$obj->fk_requestmanager])) {
				$request_static = new RequestManager($db);
				$request_static->fetch($obj->fk_requestmanager);
				$requests_cached[$obj->fk_requestmanager] = $request_static;
			}
			$request_static = $requests_cached[$obj->fk_requestmanager];

			print '<td class="nowrap">';
			if ($request_static->id > 0) print $request_static->getNomUrl(1, 'questionnaire');
			print '</td>';
		}
		// Contact
		if (!empty($arrayfields['qiu.fk_element']['checked'])) {
			print '<td class="nowrap">';
			if (!isset($contacts_cached[$obj->contact_id])) {
				$contact_static = new Contact($db);
				$contact_static->fetch($obj->contact_id);
				$contacts_cached[$obj->contact_id] = $contact_static;
			}
			$contact_static = $contacts_cached[$obj->contact_id];

			if ($contact_static->id > 0) print $contact_static->getNomUrl(1);
			print '</td>';
		}
		// Email
		if (!empty($arrayfields['qiu.email']['checked'])) {
			print '<td class="nowrap">';
			print $obj->email;
			print '</td>';
		}
		// Sent mail status
		if (!empty($arrayfields['qiu.sent']['checked'])) {
			print '<td class="nowrap center">';
			print InvitationUser::LibStatut($invitation_user_static->sent, 6);
			print '</td>';
		}
		// Date sent mail
		if (!empty($arrayfields['qiu.date_envoi']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_envoi, 'day');
			print '</td>';
		}
		// Date sent remind mail
		if (!empty($arrayfields['qiu.date_sent_remind']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_sent_remind, 'day');
			print '</td>';
		}
		// Date limit answer
		if (!empty($arrayfields['qiu.date_limite_reponse']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_limite_reponse, 'day');
			print '</td>';
		}
		// Date validation
		if (!empty($arrayfields['qiu.date_validation']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_validation, 'day');
			print '</td>';
		}

		// Extra fields
		include DOL_DOCUMENT_ROOT.'/core/tpl/extrafields_list_print_fields.tpl.php';
		// Fields from hook
		$parameters=array('arrayfields'=>$arrayfields, 'obj'=>$obj, 'i'=>$i);
		$reshook = $hookmanager->executeHooks('printFieldListValue', $parameters);    // Note that $action and $object may have been modified by hook
		print $hookmanager->resPrint;

		// Date creation
		if (!empty($arrayfields['qiu.date_creation']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_creation, 'dayhour');
			print '</td>';
		}
		// Date modification
		if (!empty($arrayfields['qiu.tms']['checked'])) {
			print '<td class="nowrap center">';
			print dol_print_date($invitation_user_static->date_modification, 'dayhour');
			print '</td>';
		}
		// Status
		if (!empty($arrayfields['qiu.fk_statut']['checked'])) {
			print '<td class="nowrap right">' . $invitation_user_static->getLibStatut(6) . '</td>';
		}
		// TODO check if token is valid with questionaire ?
		$url_answer_link = $base_url_answer_link . '?id=' . $invitation_user_static->fk_questionnaire . '&action=answer&fk_invitation=' . $invitation_user_static->id . '&token=' . $invitation_user_static->token;
		// Link
		if (!empty($arrayfields['link']['checked'])) {
			print '<td class="nowrap center">';
			if (!empty($invitation_user_static->token)) print '<input style="width:100px;" class="button questionnaire_copy_text" type="text" value="' . $langs->trans('CopyLink') . '" text_to_copy="' . $url_answer_link . '"/>';
			print '</td>';
		}
		// Action column
		print '<td class="nowrap right">';
		if ($invitation_user_static->fk_statut == InvitationUser::STATUS_DRAFT || $invitation_user_static->fk_statut == InvitationUser::STATUS_SAVED) {
			print '<a href="' . $url_answer_link . '" target="_blank"><i class="fa fa-edit" title="' . dol_escape_js($langs->trans('RequestManagerQuestionnaireAnswer'), 2) . '"></i></a> ';
		}
		if ($massactionbutton || $massaction)   // If we are in select mode (massactionbutton defined) or if we have already selected and sent an action ($massaction) defined
		{
			$key = $obj->rowid . '_' . ($request_id > 0 ? '' : $obj->fk_requestmanager . '_') . $obj->contact_id;
			$selected = 0;
			if (in_array($key, $arrayofselected)) $selected = 1;
			print '<input id="cb' . $key . '" class="flat checkforselect" type="checkbox" name="toselect[]" value="' . $key . '"' . ($selected ? ' checked="checked"' : '') . '>';
		}
		print '</td>';

		print "</tr>\n";

		$i++;
	}

	$db->free($resql);

	$parameters = array('arrayfields' => $arrayfields, 'sql' => $sql);
	$reshook = $hookmanager->executeHooks('printFieldListFooter', $parameters);    // Note that $action and $object may have been modified by hook
	print $hookmanager->resPrint;

	print '</table>' . "\n";
	print '</div>' . "\n";

	print '</form>' . "\n";
} else {
	dol_print_error($db);
}


llxFooter();
$db->close();
