<?php
/* Copyright (C) 2020      Open-Dsi	 <support@open-dsi.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 *	\file       	htdocs/requestmanager/time.php
 *	\ingroup    	requestmanager
 *	\brief      	Page of time record list for a request
 */

// Change this following line to use the correct relative path (../, ../../, etc)
$res=0;
if (! $res && file_exists("../main.inc.php")) $res=@include '../main.inc.php';					// to work if your module directory is into dolibarr root htdocs directory
if (! $res && file_exists("../../main.inc.php")) $res=@include '../../main.inc.php';			// to work if your module directory is into a subdir of root htdocs directory
if (! $res && file_exists("../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../dolibarr/htdocs/main.inc.php';     // Used on dev env only
if (! $res && file_exists("../../../../dolibarr/htdocs/main.inc.php")) $res=@include '../../../../dolibarr/htdocs/main.inc.php';   // Used on dev env only
if (! $res) die("Include of main fails");
require_once DOL_DOCUMENT_ROOT.'/core/class/html.formfile.class.php';
require_once DOL_DOCUMENT_ROOT.'/core/lib/date.lib.php';
dol_include_once('/requestmanager/class/requestmanager.class.php');
dol_include_once('/requestmanager/class/requestmanagertimespent.class.php');
dol_include_once('/requestmanager/class/html.formrequestmanager.class.php');
dol_include_once('/requestmanager/lib/opendsi_tools.lib.php');
dol_include_once('/requestmanager/lib/requestmanager.lib.php');

$langs->loadLangs(array('requestmanager@requestmanager', 'projects'));

$id					= GETPOST('id', 'int');
$ref				= GETPOST('ref','alpha');
$line_id			= GETPOST('line_id', 'int');
$action				= GETPOST('action','aZ09');
$confirm			= GETPOST('confirm','aZ09');
$massaction			= GETPOST('massaction', 'aZ09');
$toselect			= GETPOST('toselect', 'array');
$optioncss			= GETPOST('optioncss','alpha');
$confirm_edit_btn 	= GETPOST('button_confirm_edit_time_spent_x') || GETPOST('button_confirm_edit_time_spent.x') || GETPOST('button_confirm_edit_time_spent'); // All tests are required to be compatible with all browsers
if ($confirm_edit_btn) $action = 'update_time_spent';
//$show_files		= GETPOST('show_files','int');

// Load object
//-------------------------
$time_spent = new RequestManagerTimeSpent($db);
if ($line_id > 0) {
	$ret = $time_spent->fetch($line_id);
	if ($ret > 0) {
		$line_id = $time_spent->id;
	} elseif ($ret < 0) {
		dol_print_error('', $time_spent->error);
	} elseif ($ret == 0) {
		$langs->load("errors");
		accessforbidden($langs->trans('ErrorRecordNotFound'));
	}
}
$object = new RequestManager($db);
if ($id > 0 || !empty($ref)) {
    $ret = $object->fetch($id, $ref);
    if ($ret > 0) {
		$id = $object->id;
		if (!$object->isRequestTypeOption('manage_times') || !$object->isRequestTypeOption('display_set_times')) {
			accessforbidden();
		}
    } elseif ($ret < 0) {
        dol_print_error('', $object->error);
    } elseif ($ret == 0) {
        $langs->load("errors");
        accessforbidden($langs->trans('ErrorRecordNotFound'));
    }
}

// Security
if (in_array($action, [ 'delete' ])) $action = '';
if (in_array($massaction, [ 'predelete', 'delete' ])) $massaction = '';

// Set permission
$canread = $user->rights->requestmanager->timesspent->manage || $user->admin || $user->rights->requestmanager->lire;
$cancreate = $user->rights->requestmanager->timesspent->manage || $user->admin || $user->rights->requestmanager->creer;
$candelete = $user->rights->requestmanager->timesspent->manage || $user->admin || $user->rights->requestmanager->creer;

// Security check
if ($user->societe_id > 0) accessforbidden();
$result = restrictedArea($user, 'requestmanager', $id, '', '', '', 'rowid');
if (!$canread) {
	accessForbidden();
}

// Date navigation bar
//-------------------------
$date_nav_mode			= GETPOST('date_nav_mode', 'aZ09');
$date_nav_action		= GETPOST('date_nav_action', 'aZ09');
$date_nav_startday		= GETPOST('date_nav_startday', 'int');
$date_nav_startmonth	= GETPOST('date_nav_startmonth', 'int');
$date_nav_startyear		= GETPOST('date_nav_startyear', 'int');
$date_nav_start			= dol_mktime(0, 0, 0, $date_nav_startmonth, $date_nav_startday, $date_nav_startyear);	// Use tzserver

$date_nav_mode			= in_array($date_nav_action, ['all', 'day', 'week', 'month', 'year']) ? $date_nav_action : $date_nav_mode;
$date_nav_mode			= in_array($date_nav_mode, ['all', 'day', 'week', 'month', 'year']) ? $date_nav_mode : (!empty($user->array_options['options_rm_default_time_date_nav_mode']) ? $user->array_options['options_rm_default_time_date_nav_mode'] : 'month');
if ($date_nav_start == '' || in_array($date_nav_action, ['all', 'day', 'week', 'month', 'year', 'today'])) $date_nav_start = dol_now();
$date_nav_end			= $date_nav_start;

if ($date_nav_mode != 'all') {
	// Get date detail
	$date_nav_start_detail = dol_getdate($date_nav_start);
	$date_nav_startday = $date_nav_start_detail['mday'];
	$date_nav_startmonth = $date_nav_start_detail['mon'];
	$date_nav_startyear = $date_nav_start_detail['year'];

	// Set first date in function of the mode
	if ($date_nav_mode == 'week') {
		$new_date = dol_get_first_day_week($date_nav_startday, $date_nav_startmonth, $date_nav_startyear);
		$date_nav_start = dol_mktime(0, 0, 0, $new_date['first_month'], $new_date['first_day'], $new_date['first_year']);
	} elseif ($date_nav_mode == 'month') {
		$date_nav_start = dol_mktime(0, 0, 0, $date_nav_startmonth, '01', $date_nav_startyear);
	} elseif ($date_nav_mode == 'year') {
		$date_nav_start = dol_mktime(0, 0, 0, '01', '01', $date_nav_startyear);
	}

	// Get date detail
	$date_nav_start_detail = dol_getdate($date_nav_start);
	$date_nav_startday = $date_nav_start_detail['mday'];
	$date_nav_startmonth = $date_nav_start_detail['mon'];
	$date_nav_startyear = $date_nav_start_detail['year'];

	// Set first date in function of the action 'prev' and 'next'
	if ($date_nav_action == 'prev') {
		if ($date_nav_mode == 'day') {
			$date_nav_start = dol_time_plus_duree($date_nav_start, -1, 'd');
		} elseif ($date_nav_mode == 'week') {
			$new_date = dol_get_prev_week($date_nav_startday, 0, $date_nav_startmonth, $date_nav_startyear);
			$date_nav_start = dol_mktime(0, 0, 0, $new_date['month'], $new_date['day'], $new_date['year']);
		} elseif ($date_nav_mode == 'month') {
			$new_date = dol_get_prev_month($date_nav_startmonth, $date_nav_startyear);
			$date_nav_start = dol_mktime(0, 0, 0, $new_date['month'], '01', $new_date['year']);
		} elseif ($date_nav_mode == 'year') {
			$date_nav_start = dol_mktime(0, 0, 0, '01', '01', $date_nav_startyear - 1);
		}
	} elseif ($date_nav_action == 'next') {
		if ($date_nav_mode == 'day') {
			$date_nav_start = dol_time_plus_duree($date_nav_start, 1, 'd');
		} elseif ($date_nav_mode == 'week') {
			$new_date = dol_get_next_week($date_nav_startday, 0, $date_nav_startmonth, $date_nav_startyear);
			$date_nav_start = dol_mktime(0, 0, 0, $new_date['month'], $new_date['day'], $new_date['year']);
		} elseif ($date_nav_mode == 'month') {
			$new_date = dol_get_next_month($date_nav_startmonth, $date_nav_startyear);
			$date_nav_start = dol_mktime(0, 0, 0, $new_date['month'], '01', $new_date['year']);
		} elseif ($date_nav_mode == 'year') {
			$date_nav_start = dol_mktime(0, 0, 0, '01', '01', $date_nav_startyear + 1);
		}
	}

	// Set end date in function of the mode
	if ($date_nav_mode == 'day') {
		$date_nav_end = $date_nav_start;
	} elseif ($date_nav_mode == 'week') {
		$date_nav_end = dol_time_plus_duree($date_nav_start, 6, 'd');
	} elseif ($date_nav_mode == 'month') {
		$date_nav_end = dol_time_plus_duree(dol_time_plus_duree($date_nav_start, 1, 'm'), -1, 'd');
	} elseif ($date_nav_mode == 'year') {
		$date_nav_end = dol_time_plus_duree(dol_time_plus_duree($date_nav_start, 1, 'y'), -1, 'd');
	}

	// Set new start date detail
	$date_nav_start_detail = dol_getdate($date_nav_start);
	$date_nav_startday = $date_nav_start_detail['mday'];
	$date_nav_startmonth = $date_nav_start_detail['mon'];
	$date_nav_startyear = $date_nav_start_detail['year'];
}

// Search
//-------------------------
$search_user                			= GETPOSTISSET('search_user') && $user->admin ? GETPOST("search_user","id") : $user->id;
$search_request             			= GETPOST("search_request","alpha");
$search_request_label					= GETPOST('search_request_label','alpha');
$search_request_thirdparty				= GETPOST('search_request_thirdparty','alpha');
$search_note                			= GETPOST("search_note","alpha");
$search_duration            			= GETPOST("search_duration","alpha");
$search_cost                			= GETPOST("search_cost","alpha");
$search_date_creation_startday			= GETPOST('search_date_creation_startday', 'int');
$search_date_creation_startmonth		= GETPOST('search_date_creation_startmonth', 'int');
$search_date_creation_startyear			= GETPOST('search_date_creation_startyear', 'int');
$search_date_creation_endday			= GETPOST('search_date_creation_endday', 'int');
$search_date_creation_endmonth			= GETPOST('search_date_creation_endmonth', 'int');
$search_date_creation_endyear			= GETPOST('search_date_creation_endyear', 'int');
$search_date_creation_start				= dol_mktime(0, 0, 0, $search_date_creation_startmonth, $search_date_creation_startday, $search_date_creation_startyear);	// Use tzserver
$search_date_creation_end				= dol_mktime(23, 59, 59, $search_date_creation_endmonth, $search_date_creation_endday, $search_date_creation_endyear);
$search_date_modification_startday		= GETPOST('search_date_modification_startday', 'int');
$search_date_modification_startmonth	= GETPOST('search_date_modification_startmonth', 'int');
$search_date_modification_startyear		= GETPOST('search_date_modification_startyear', 'int');
$search_date_modification_endday		= GETPOST('search_date_modification_endday', 'int');
$search_date_modification_endmonth		= GETPOST('search_date_modification_endmonth', 'int');
$search_date_modification_endyear		= GETPOST('search_date_modification_endyear', 'int');
$search_date_modification_start			= dol_mktime(0, 0, 0, $search_date_modification_startmonth, $search_date_modification_startday, $search_date_modification_startyear);	// Use tzserver
$search_date_modification_end			= dol_mktime(23, 59, 59, $search_date_modification_endmonth, $search_date_modification_endday, $search_date_modification_endyear);
$search_user_author						= GETPOST("search_user_author","alpha");
$search_user_modification				= GETPOST("search_user_modification","alpha");
$search_btn								= GETPOST('button_search_x') || GETPOST('button_search.x') || GETPOST('button_search'); // All tests are required to be compatible with all browsers
$search_remove_btn						= GETPOST('button_removefilter_x') || GETPOST('button_removefilter.x') || GETPOST('button_removefilter'); // All tests are required to be compatible with all browsers

// Pagination list
//-------------------------
$limit = GETPOST('limit')?GETPOST('limit','int'):$conf->liste_limit;
$sortfield = GETPOST("sortfield",'alpha');
$sortorder = GETPOST("sortorder",'alpha');
$page = GETPOST("page",'int');
if (empty($page) || $page == -1 || !empty($search_btn) || !empty($search_remove_btn) || (empty($toselect) && $massaction === '0')) { $page = 0; }     // If $page is not defined, or '' or -1
$offset = $limit * $page;
$pageprev = $page - 1;
$pagenext = $page + 1;
if (! $sortfield) $sortfield='ts.time_date';
if (! $sortorder) $sortorder='DESC';

// Initialize technical object to manage context to save list fields
$contextpage=GETPOST('contextpage','aZ')?GETPOST('contextpage','aZ'):'requestmanagertimespentlist';

//$diroutputmassaction=$conf->requestmanager->dir_output . '/temp/massgeneration/'.$user->id;

// Initialize technical object to manage hooks of page. Note that conf->hooks_modules contains array of hook context
$hookmanager->initHooks(array('requestmanagertimespentlist'));

$arrayfields=array(
    'day'=>array('label'=>$langs->trans("Day"), 'checked'=>1),
    'ts.time_date'=>array('label'=>$langs->trans("Date"), 'checked'=>1),
	'ts.fk_user'=>array('label'=>$langs->trans("By"), 'checked'=>1, 'enabled'=>($user->admin ? 1 : 0)),
    'ts.fk_requestmanager'=>array('label'=>$langs->trans("RequestManagerRequest"), 'checked'=>1, 'enabled'=>($id > 0 ? 0 : 1)),
	'rm.label' => array('label' => $langs->trans("RequestManagerLabel"), 'checked' => 1, 'enabled'=>($id > 0 ? 0 : 1)),
	'rm.fk_soc' => array('label' => $langs->trans("RequestManagerThirdPartyOrigin"), 'checked' => 1, 'enabled'=>($id > 0 ? 0 : 1)),
    'ts.note'=>array('label'=>$langs->trans("Note"), 'checked'=>1),
    'ts.duration'=>array('label'=>$langs->trans("Duration"), 'checked'=>1),
    'cost'=>array('label'=>$langs->trans("RequestManagerEffectiveCost"), 'checked'=>1, 'enabled'=>(!isModEnabled('salaries') ? 0 : 1)),
    'ts.fk_user_author' => array('label' => $langs->trans("Author"), 'checked' => 0, 'position' => 10),
    'ts.fk_user_modif' => array('label' => $langs->trans("ModifiedBy"), 'checked' => 0, 'position' => 10),
    'ts.datec' => array('label' => $langs->trans("DateCreation"), 'checked' => 0, 'position' => 500),
    'ts.tms' => array('label' => $langs->trans("DateModificationShort"), 'checked' => 0, 'position' => 500),
);


/*
 * Actions
 */

if (GETPOST('cancel','alpha')) { $action='list'; $massaction=''; }
if (! GETPOST('confirmmassaction','alpha') && $massaction != 'mass_delete_time_spent' && $massaction != 'confirm_mass_delete_time_spent') { $massaction=''; }

$parameters=array();
$reshook=$hookmanager->executeHooks('doActions',$parameters,$object,$action);    // Note that $action and $object may have been modified by some hooks
if ($reshook < 0) setEventMessages($hookmanager->error, $hookmanager->errors, 'errors');

include DOL_DOCUMENT_ROOT.'/core/actions_changeselectedfields.inc.php';

// Do we click on purge search criteria ?
if ($search_remove_btn) {
	$search_user                			= $user->id;
	$search_request             			= '';
	$search_request_label					= '';
	$search_request_thirdparty				= '';
	$search_note                			= '';
	$search_duration            			= '';
	$search_cost                			= '';
	$search_date_creation_startday			= '';
	$search_date_creation_startmonth		= '';
	$search_date_creation_startyear			= '';
	$search_date_creation_endday			= '';
	$search_date_creation_endmonth			= '';
	$search_date_creation_endyear			= '';
	$search_date_modification_startday		= '';
	$search_date_modification_startmonth	= '';
	$search_date_modification_startyear		= '';
	$search_date_modification_endday		= '';
	$search_date_modification_endmonth		= '';
	$search_date_modification_endyear		= '';
	$search_user_author						= '';
	$search_user_modification				= '';
    $toselect								= '';
}

$param = '';
if (!empty($contextpage) && $contextpage != $_SERVER["PHP_SELF"]) $param .= '&contextpage=' . urlencode($contextpage);
if ($limit > 0 && $limit != $conf->liste_limit) $param .= '&limit=' . urlencode($limit);
if ($optioncss != '')						$param .= '&optioncss=' . urlencode($optioncss);
if ($search_user)							$param .= '&search_user=' . urlencode($search_user);
if ($search_request)						$param .= '&search_request=' . urlencode($search_request);
if ($search_request_label)					$param .= '&search_request_label=' . urlencode($search_request_label);
if ($search_request_thirdparty)				$param .= '&search_request_thirdparty=' . urlencode($search_request_thirdparty);
if ($search_note)							$param .= '&search_note=' . urlencode($search_note);
if ($search_duration)						$param .= '&search_duration=' . urlencode($search_duration);
if ($search_cost)							$param .= '&search_cost=' . urlencode($search_cost);
if ($search_date_creation_startday)			$param .= '&search_date_creation_startday=' . urlencode($search_date_creation_startday);
if ($search_date_creation_startmonth)		$param .= '&search_date_creation_startmonth=' . urlencode($search_date_creation_startmonth);
if ($search_date_creation_startyear)		$param .= '&search_date_creation_startyear=' . urlencode($search_date_creation_startyear);
if ($search_date_creation_endday)			$param .= '&search_date_creation_endday=' . urlencode($search_date_creation_endday);
if ($search_date_creation_endmonth)			$param .= '&search_date_creation_endmonth=' . urlencode($search_date_creation_endmonth);
if ($search_date_creation_endyear)			$param .= '&search_date_creation_endyear=' . urlencode($search_date_creation_endyear);
if ($search_date_modification_startday)		$param .= '&search_date_modification_startday=' . urlencode($search_date_modification_startday);
if ($search_date_modification_startmonth)	$param .= '&search_date_modification_startmonth=' . urlencode($search_date_modification_startmonth);
if ($search_date_modification_startyear)	$param .= '&search_date_modification_startyear=' . urlencode($search_date_modification_startyear);
if ($search_date_modification_endday)		$param .= '&search_date_modification_endday=' . urlencode($search_date_modification_endday);
if ($search_date_modification_endmonth)		$param .= '&search_date_modification_endmonth=' . urlencode($search_date_modification_endmonth);
if ($search_date_modification_endyear)		$param .= '&search_date_modification_endyear=' . urlencode($search_date_modification_endyear);
if ($search_user_author)					$param .= '&search_user_author=' . urlencode($search_user_author);
if ($search_user_modification)				$param .= '&search_user_modification=' . urlencode($search_user_modification);

$param_list_nav = '';
if ($sortfield)		$param_list_nav .= '&sortfield=' . urlencode($sortfield);
if ($sortorder)		$param_list_nav .= '&sortorder=' . urlencode($sortorder);
if ($page)			$param_list_nav .= '&page=' . urlencode($page);
if ($contextpage)	$param_list_nav .= '&contextpage=' . urlencode($contextpage);

$param_date_nav_mode = '';
if ($date_nav_mode)		$param_date_nav_mode .= '&date_nav_mode=' . urlencode($date_nav_mode);

$param_date_nav_start = '';
if ($date_nav_startday)		$param_date_nav_start .= '&date_nav_startday=' . urlencode($date_nav_startday);
if ($date_nav_startmonth)	$param_date_nav_start .= '&date_nav_startmonth=' . urlencode($date_nav_startmonth);
if ($date_nav_startyear)	$param_date_nav_start .= '&date_nav_startyear=' . urlencode($date_nav_startyear);

if (empty($reshook)) {
    // Add time spent
    if ($action == 'add_time_spent' && $cancreate) {
		$time_spent_date_minute 	= GETPOST('time_spent_datemin', 'int');
		$time_spent_date_hour 		= GETPOST('time_spent_datehour', 'int');
		$time_spent_date_day 		= GETPOST('time_spent_dateday', 'int');
		$time_spent_date_month 		= GETPOST('time_spent_datemonth', 'int');
		$time_spent_date_year 		= GETPOST('time_spent_dateyear', 'int');
		$time_spent_date 			= dol_mktime($time_spent_date_hour, $time_spent_date_minute, 0, $time_spent_date_month, $time_spent_date_day, $time_spent_date_year);
		$time_spent_user 			= ($user->rights->requestmanager->timesspent->manage || $user->admin) ? GETPOST('time_spent_user', 'int') : $user->id;
		$time_spent_request 		= empty($id) ? GETPOST('time_spent_request', 'int') : $object->id;
		$time_spent_note			= GETPOST('time_spent_note', 'alphanohtml');
		$time_spent_duration_hour	= GETPOST('time_spent_durationhour', 'int');
		$time_spent_duration_min	= GETPOST('time_spent_durationmin', 'int');
		$time_spent_duration_hour	= $time_spent_duration_hour > 0 ? $time_spent_duration_hour : 0;
		$time_spent_duration_min	= $time_spent_duration_min > 0 ? $time_spent_duration_min : 0;
		$time_spent_duration        = $time_spent_duration_hour * 3600 + $time_spent_duration_min * 60;    // We store duration in seconds

		$time_spent = new RequestManagerTimeSpent($db);
		$time_spent->fk_requestmanager = $time_spent_request;
		$time_spent->time_date = $time_spent_date;
		$time_spent->fk_user = $time_spent_user;
		$time_spent->note = $time_spent_note;
		$time_spent->duration = $time_spent_duration;
		$time_spent->thm = 0;

		$result = $time_spent->create($user);
		if ($result < 0) {
			setEventMessages($time_spent->error, $time_spent->errors, 'errors');
		} else {
			setEventMessage($langs->trans("RecordSaved"));
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode .
				'&date_nav_startday=' . urlencode($time_spent_date_day) . '&date_nav_startmonth=' . urlencode($time_spent_date_month) . '&date_nav_startyear=' . urlencode($time_spent_date_year) .
				'&time_spent_user=' . urldecode($time_spent_user) . '&time_spent_request=' . urldecode($time_spent_request) . '&time_spent_datemin=' . urldecode($time_spent_date_minute) .
				'&time_spent_datehour=' . urldecode($time_spent_date_hour) . '&time_spent_dateday=' . urldecode($time_spent_date_day) .
				'&time_spent_datemonth=' . urldecode($time_spent_date_month) . '&time_spent_dateyear=' . urldecode($time_spent_date_year));
			exit();
		}
	} // Edit time spent
    elseif ($action == 'update_time_spent' && $cancreate && $time_spent->canWriteLine() > 0) {
		$time_spent_date 			= dol_mktime(GETPOST('edit_time_spent_datehour', 'int'), GETPOST('edit_time_spent_datemin', 'int'), 0, GETPOST('edit_time_spent_datemonth', 'int'), GETPOST('edit_time_spent_dateday', 'int'), GETPOST('edit_time_spent_dateyear', 'int'));
		$time_spent_user 			= ($user->rights->requestmanager->timesspent->manage || $user->admin) ? GETPOST('edit_time_spent_user', 'int') : $user->id;
		$time_spent_request 		= empty($id) ? GETPOST('edit_time_spent_request', 'int') : $object->id;
		$time_spent_note			= GETPOST('edit_time_spent_note', 'alphanohtml');
		$time_spent_duration_hour	= GETPOST('edit_time_spent_durationhour', 'int');
		$time_spent_duration_min	= GETPOST('edit_time_spent_durationmin', 'int');
		$time_spent_duration_hour	= $time_spent_duration_hour > 0 ? $time_spent_duration_hour : 0;
		$time_spent_duration_min	= $time_spent_duration_min > 0 ? $time_spent_duration_min : 0;
		$time_spent_duration        = $time_spent_duration_hour * 3600 + $time_spent_duration_min * 60;    // We store duration in seconds

		$time_spent->oldcopy = clone $time_spent;
		$time_spent->time_date = $time_spent_date;
		$time_spent->fk_user = $time_spent_user;
		$time_spent->fk_requestmanager = $time_spent_request;
		$time_spent->note = $time_spent_note;
		$time_spent->duration = $time_spent_duration;
		$time_spent->thm = 0;

		$result = $time_spent->update($user);
		if ($result < 0) {
			setEventMessages($time_spent->error, $time_spent->errors, 'errors');
			$action = 'edit_time_spent';
		} else {
			setEventMessage($langs->trans("RecordSaved"));
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start);
			exit();
		}
	} // Delete time spent
    elseif ($action == 'confirm_delete_time_spent' && $confirm == 'yes' && $candelete && $time_spent->canWriteLine() > 0) {
		$result = $time_spent->delete($user);
		if ($result < 0) {
			setEventMessages($time_spent->error, $time_spent->errors, 'errors');
		} else {
			setEventMessage($langs->trans("RecordDeleted"));
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start);
			exit();
		}
	}

    $objectclass = 'RequestManagerTimeSpent';
    $objectlabel = 'TimesSpent';
    $permtoread = $canread;
    $permtodelete = $candelete;
    $uploaddir = $conf->requestmanager->dir_output;
    include DOL_DOCUMENT_ROOT . '/core/actions_massactions.inc.php';
	// Mass delete time spent
	if (!$error && $action == 'confirm_mass_delete_time_spent' && $confirm == 'yes' && $candelete) {
		$db->begin();

		$objecttmp = new RequestManagerTimeSpent($db);
		$nbok = 0;
		foreach ($toselect as $toselectid) {
			$result = $objecttmp->fetch($toselectid);
			if ($result > 0) {
				if ($objecttmp->canWriteLine() > 0) {
					$result = $objecttmp->delete($user);
					if ($result <= 0) {
						setEventMessages($objecttmp->error, $objecttmp->errors, 'errors');
						$error++;
						break;
					} else {
						$nbok++;
					}
				}
			} else {
				setEventMessages($objecttmp->error, $objecttmp->errors, 'errors');
				$error++;
				break;
			}
		}

		if (!$error) {
			$db->commit();
			if ($nbok > 1) {
				setEventMessages($langs->trans("RecordsDeleted", $nbok), null, 'mesgs');
			} elseif ($nbok > 0) {
				setEventMessages($langs->trans("RecordDeleted", $nbok), null, 'mesgs');
			} else {
				setEventMessages($langs->trans("NoRecordDeleted"), null, 'mesgs');
			}
			header('Location: ' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start);
			exit();
		} else {
			$db->rollback();
		}
	}
}


/*
 * View
 */

$now=dol_now();

$form = new Form($db);
$formrequestmanager = new FormRequestManager($db);
$formfile = new FormFile($db);

$title = $langs->trans('RequestManagerRequest') . ' - '. $langs->trans('TimesSpent');
$help_url='';
llxHeader('', $title, $help_url);

if ($id > 0) {
    $head = requestmanager_prepare_head($object);
    print dol_get_fiche_head($head, 'request_time', $langs->trans('RequestManagerCard'), -1, 'requestmanager@requestmanager');

    // Request manager card
    $linkback = '<a href="' . dol_buildpath('/requestmanager/list.php', 1) . '?restore_lastsearch_values=1">' . $langs->trans("BackToList") . '</a>';

    $morehtmlref = '<div class="refidno">';
    // External Reference
    $morehtmlref .= $langs->trans('RequestManagerExternalReference') . ' : ' . $object->ref_ext;
    // Type
    $morehtmlref .= '<br>' . $langs->trans('RequestManagerType') . ' : ' . $object->getLibType();
    // Label
    $morehtmlref .= '<br>' . $langs->trans('RequestManagerLabel') . ' : ' . $object->label;
    // Parent request
    if ($object->fk_parent > 0) {
        $object->fetch_parent();
        $morehtmlref .= '<br>' . $langs->trans('RequestManagerParentRequest') . ' : ' . $object->parent->getNomUrl(1, 'parent_path');
    }
	// Thirdparty
	$origin_type = $object->getRequestTypeOption('origin_type');
	if ($object->socid > 0 && empty($origin_type) || $origin_type == RequestManager::ORIGIN_TYPE_THIRDPARTY) {
		$object->fetch_thirdparty();
		$morehtmlref .= '<br>' . $langs->trans('RequestManagerThirdPartyOrigin') . ' : ' . $object->thirdparty->getNomUrl(1);
	}
    $morehtmlref .= '</div>';

    // Get only request type with options 'manage_times' and 'display_set_times' activated
	$request_type_list = array();
	$object->loadRequestTypes();
	if (is_array(RequestManager::$type_list)) {
		foreach (RequestManager::$type_list as $line) {
			if (!empty($line->fields['manage_times']) && !empty($line->fields['display_set_times'])) {
				$request_type_list[$line->id] = $line->id;
			}
		}
	}
	if (!in_array($object->fk_type, $request_type_list)) $request_type_list[$object->fk_type] = $object->fk_type;
    $object->next_prev_filter = 'te.fk_type IN (' . implode(',', $request_type_list) . ')';

    dol_banner_tab($object, 'ref', $linkback, 1, 'ref', 'ref', $morehtmlref, $param . $param_date_nav_mode . $param_date_nav_start);

    print dol_get_fiche_end();
}


// Confirm delete time spent
if ($action == 'delete_time_spent' && $candelete && $time_spent->canWriteLine() > 0) {
	$formconfirm = $form->formconfirm($_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . '&line_id=' . $line_id . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start, $langs->trans('RequestManagerDeleteTimeSpent'), $langs->trans('RequestManagerConfirmDeleteTimeSpent'), 'confirm_delete_time_spent', '', 0, 1);
}

// Hook
$parameters = array();
$reshook = $hookmanager->executeHooks('formConfirm', $parameters, $object, $action); // Note that $action and $object may have been modified by hook
if (empty($reshook)) $formconfirm.=$hookmanager->resPrint;
elseif ($reshook > 0) $formconfirm=$hookmanager->resPrint;

// Print form confirm
print $formconfirm;

if (!($id > 0)) print load_fiche_titre($title, '', 'requestmanager@requestmanager');

// Show date navigation bar
print '<div class="time_spent_date_nav rm_no_decoration">';
print '<a class="butAction' . ($date_nav_mode == 'all' ? ' rm_selected_date_nav_mode' : '') . '" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=all&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_start . '">' . $langs->trans('All') . '</a>';
print '<a class="butAction' . ($date_nav_mode == 'day' ? ' rm_selected_date_nav_mode' : '') . '" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=day&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_start . '">' . $langs->trans('Day') . '</a>';
print '<a class="butAction' . ($date_nav_mode == 'week' ? ' rm_selected_date_nav_mode' : '') . '" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=week&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_start . '">' . $langs->trans('Week') . '</a>';
print '<a class="butAction' . ($date_nav_mode == 'month' ? ' rm_selected_date_nav_mode' : '') . '" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=month&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_start . '">' . $langs->trans('Month') . '</a>';
print '<a class="butAction' . ($date_nav_mode == 'year' ? ' rm_selected_date_nav_mode' : '') . '" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=year&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_start . '">' . $langs->trans('Year') . '</a>';
if ($date_nav_mode != 'all') {
	print '<form method="POST" id="dateNavForm" action="' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . '">';
	print '<input type="hidden" name="token" value="' . newToken() . '">';
	print ' ' . $langs->trans('From') . ' : ' . $form->select_date($date_nav_start, 'date_nav_start', 0, 0, 2, "addtime", 1, 0, 1);
	print ' ' . $langs->trans('to') . ' ' . $form->select_date($date_nav_end, 'date_nav_end', 0, 0, 2, "addtime", 1, 0, 1, 1);
	print ' <input type="submit" class="button" name="button_date_nav_refresh" value="' . $langs->trans("Modify") . '">';
	print '</form>' . "\n";

	print '<div class="inline-block navigation">';
	$now_label = "Today";
	if ($date_nav_mode == 'week') $now_label = "RequestManagerThisWeek";
	elseif ($date_nav_mode == 'month') $now_label = "RequestManagerThisMonth";
	elseif ($date_nav_mode == 'year') $now_label = "RequestManagerThisYear";
	print '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=today&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '">' . $langs->trans($now_label) . '</a>';
	print '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=prev&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '"><<</a>';
	print '<a class="butAction" href="' . $_SERVER["PHP_SELF"] . '?date_nav_action=next&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '">>></a>';
	print '</div>';
}
print '</div>';

/*
 * Form to add time spent
 */
if ($cancreate) {
	$now = dol_now();
	$tmp = dol_getdate($now);
	$now = dol_mktime(0, 0, 0, $tmp['mon'], $tmp['mday'], $tmp['year']);

	$time_spent_date = GETPOSTISSET('time_spent_datemonth') ? dol_mktime(GETPOST('time_spent_datehour', 'int'), GETPOST('time_spent_datemin', 'int'), 0, GETPOST('time_spent_datemonth', 'int'), GETPOST('time_spent_dateday', 'int'), GETPOST('time_spent_dateyear', 'int')) : $date_nav_start;
	$time_spent_user = GETPOSTISSET('time_spent_user') && ($user->rights->requestmanager->timesspent->manage || $user->admin) ? GETPOST('time_spent_user', 'int') : $user->id;
	$time_spent_request = !empty($time_spent_param = GETPOST('time_spent_request', 'int')) ? $time_spent_param : -1;
	$time_spent_note	= GETPOST('time_spent_note', 'alphanohtml');
	$time_spent_duration_hour	= GETPOST('time_spent_durationhour', 'int');
	$time_spent_duration_min	= GETPOST('time_spent_durationmin', 'int');
	$time_spent_duration_hour	= $time_spent_duration_hour > 0 ? $time_spent_duration_hour : 0;
	$time_spent_duration_min	= $time_spent_duration_min > 0 ? $time_spent_duration_min : 0;
	$time_spent_duration        = $time_spent_duration_hour * 3600 + $time_spent_duration_min * 60;    // We store duration in seconds

	// Lignes des champs de filtre
	print '<form method="POST" id="addForm" action="' . $_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '">';
	print '<input type="hidden" name="token" value="' . newToken() . '">';
	print '<input type="hidden" name="action" value="add_time_spent">';

	print '<div class="div-table-responsive-no-min">';
	print '<table class="noborder nohover centpercent">';

	print '<tr class="liste_titre">';
	print '<td>' . $langs->trans("Date") . '</td>';
	print '<td>' . $langs->trans("By") . '</td>';
	if (empty($id)) print '<td>' . $langs->trans("RequestManagerRequest") . '</td>';
	print '<td>' . $langs->trans("Note") . '</td>';
	print '<td>' . $langs->trans("NewTimeSpent") . '</td>';
	print '<td></td>';
	print "</tr>\n";

	print '<tr class="oddeven">';

	// Date
	print '<td class="maxwidthonsmartphone">';
	print $form->select_date($time_spent_date, 'time_spent_date', ($conf->browser->layout == 'phone' ? 2 : 1), 1, 2, "time_spent_date", 1, 0, 1);
	print '</td>';

	// Contributor
	print '<td class="maxwidthonsmartphone">';
	if ($user->rights->requestmanager->timesspent->manage || $user->admin) {
		print img_object('', 'user', 'class="hideonsmartphone"');
		$events = array();
		$events[] = array('action' => 'getRequests', 'url' => dol_buildpath('/requestmanager/ajax/requests_list.php', 1), 'htmlname' => 'time_spent_request', 'params' => array('selected' => '{{#time_spent_request}}'));
		print $formrequestmanager->add_select_events('time_spent_user', $events);
		print $form->select_dolusers($time_spent_user, 'time_spent_user', 0, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp', 1);
	} else {
		print $user->getNomUrl(1);
	}
	print '</td>';

	// Request
	if (empty($id)) {
		print '<td class="maxwidthonsmartphone">';
		print $formrequestmanager->select_request($time_spent_request, 'time_spent_request', $time_spent_user, 1, '', '', 'minwidth500');
		print '</td>';
	}

	// Note
	print '<td>';
	print '<textarea name="time_spent_note" class="maxwidth100onsmartphone" rows="' . ROWS_2 . '">' . $time_spent_note . '</textarea>';
	print '</td>';

	// Duration - Time spent
	print '<td>';
	print $form->select_duration('time_spent_duration', $time_spent_duration, 0, 'text');
	print '</td>';

	print '<td align="center rm_no_decoration">';
	print '<input type="submit" class="button" value="' . $langs->trans("Add") . '">';
	print '</td></tr>';

	print '</table>';
	print '</div>';
	print '</form>' . "\n";
}

$sql = 'SELECT';
$sql.= ' ts.rowid, ts.time_date, IF(TIME(ts.time_date) = \'00:00:00\', 1, 0) AS is_date, ts.note, ts.duration, ts.thm, IFNULL(ts.duration, 0) * IFNULL(ts.thm, 0) / 3600 AS cost, ts.datec, ts.tms';
$sql.= ', ts.fk_requestmanager, rm.ref AS request_ref, rm.ref_ext AS request_ref_ext, rm.label AS request_label, rm.fk_type AS request_type, crmst.type AS request_status_type, rm.fk_soc AS request_socid';
$sql.= ', rm.fk_soc, s.nom as soc_name_origin, s.name_alias as soc_name_alias_origin, s.client as soc_client_origin, s.fournisseur as soc_fournisseur_origin, s.code_client as soc_code_client_origin, s.code_fournisseur as soc_code_fournisseur_origin';
$sql.= ', ts.fk_user, ts.fk_user_author, ts.fk_user_modif';
// Add fields from hooks
$parameters=array();
$reshook=$hookmanager->executeHooks('printFieldListSelect',$parameters);    // Note that $action and $object may have been modified by hook
$sql.=$hookmanager->resPrint;
$sql.= ' FROM '.MAIN_DB_PREFIX.'requestmanager_time_spent as ts';
$sql.= ' LEFT JOIN '.MAIN_DB_PREFIX.'requestmanager as rm ON rm.rowid = ts.fk_requestmanager';
$sql.= " LEFT JOIN " . MAIN_DB_PREFIX . "societe as s ON s.rowid = rm.fk_soc";
$sql.= " LEFT JOIN ".MAIN_DB_PREFIX."c_requestmanager_status as crmst ON crmst.rowid = rm.fk_status";
$sql.= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as u ON u.rowid = ts.fk_user';
$sql.= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as ua ON ua.rowid = ts.fk_user_author';
$sql.= ' LEFT JOIN '.MAIN_DB_PREFIX.'user as um ON um.rowid = ts.fk_user_modif';
// Add fields from hooks
$parameters=array();
$reshook=$hookmanager->executeHooks('printFieldListFrom',$parameters);    // Note that $action and $object may have been modified by hook
$sql.=$hookmanager->resPrint;
$sql.= ' WHERE rm.entity IN ('.getEntity('requestmanager').')';
if ($search_user > 0)					$sql.= ' AND ts.fk_user = ' . $search_user;
if ($date_nav_mode != 'all')    		$sql.= " AND DATE(ts.time_date) BETWEEN DATE('" . $db->idate($date_nav_start) . "') AND DATE('" . $db->idate($date_nav_end) . "')";
if ($id > 0)                    		$sql.= ' AND ts.fk_requestmanager = ' . $id;
if ($search_request)            		$sql.= natural_search(array('rm.ref', 'rm.ref_ext'), $search_request);
if ($search_request_label)      		$sql.= natural_search('rm.label', $search_request_label);
if ($search_request_thirdparty) 		$sql.= natural_search(array('s.nom', 's.name_alias'), $search_request_thirdparty);
if ($search_note)              			$sql.= natural_search('ts.note', $search_note);
if ($search_duration)          			$sql.= opendsi_natural_search('ts.duration', $search_duration, 5);
if ($search_date_creation_start)		$sql.= " AND ts.datec >= '".$db->idate($search_date_creation_start)."'";
if ($search_date_creation_end)			$sql.= " AND ts.datec <= '".$db->idate($search_date_creation_end)."'";
if ($search_date_modification_start)	$sql.= " AND ts.tms >= '".$db->idate($search_date_modification_start)."'";
if ($search_date_modification_end)		$sql.= " AND ts.tms <= '".$db->idate($search_date_modification_end)."'";
if ($search_user_author > 0)        	$sql.= ' AND ts.fk_user_author = ' . $search_user_author;
if ($search_user_modification > 0)  	$sql.= ' AND ts.fk_user_modif = ' . $search_user_modification;
// Add where from hooks
$parameters=array();
$reshook=$hookmanager->executeHooks('printFieldListWhere',$parameters);    // Note that $action and $object may have been modified by hook
$sql.=$hookmanager->resPrint;
$sql.= ' HAVING 1=1';
if ($search_cost)   $sql .= natural_search('cost', $search_cost, 1);
// Add where from hooks
$parameters=array();
$reshook=$hookmanager->executeHooks('printFieldListHaving',$parameters);    // Note that $action and $object may have been modified by hook
$sql.=$hookmanager->resPrint;

$sql.= $db->order($sortfield,$sortorder);

// Count total nb of records
$nbtotalofrecords = '';
if (empty($conf->global->MAIN_DISABLE_FULL_SCANLIST))
{
    $result = $db->query($sql);
    $nbtotalofrecords = $db->num_rows($result);
}

$sql.= $db->plimit($limit+1, $offset);

$resql=$db->query($sql);
if ($resql)
{
    $num = $db->num_rows($resql);

    $arrayofselected=is_array($toselect)?$toselect:array();

    // List of mass actions available
    $arrayofmassactions =  array();
    if ($candelete) $arrayofmassactions['mass_delete_time_spent']=$langs->trans("Delete");
    if (in_array($massaction, array('mass_delete_time_spent'))) $arrayofmassactions=array();
    $massactionbutton=$form->selectMassAction('', $arrayofmassactions);

    $total = $object->get_sum_consumed_time($user->id, $date_nav_start, $date_nav_end);
    if ($total > 0) {
        print '<div class="info">' . $langs->trans('RequestManagerMyDeclaredTimeOnPeriod'). ' : ' . requestmanager_print_duration($total, 0, 1) . '</div>';
    }

    // Lignes des champs de filtre
    print '<form method="POST" id="searchFormList" action="'.$_SERVER["PHP_SELF"] . '?id=' . urlencode($id) . $param_date_nav_mode . $param_date_nav_start .'">';
    if ($optioncss != '') print '<input type="hidden" name="optioncss" value="'.$optioncss.'">';
    print '<input type="hidden" name="token" value="'.$_SESSION['newtoken'].'">';
    print '<input type="hidden" name="formfilteraction" id="formfilteraction" value="list">';
    print '<input type="hidden" id="form_action" name="action" value="list">';
    print '<input type="hidden" name="sortfield" value="'.$sortfield.'">';
    print '<input type="hidden" name="sortorder" value="'.$sortorder.'">';
    print '<input type="hidden" name="page" value="'.$page.'">';
    print '<input type="hidden" name="contextpage" value="'.$contextpage.'">';

    print_barre_liste($langs->trans('RequestManagerListOfTimeSpent'), $page, $_SERVER["PHP_SELF"], '&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start, $sortfield, $sortorder, $massactionbutton, $num, $nbtotalofrecords, '', 0, '', '', $limit);

//	$topicmail="SendPropalRef";
//	$modelmail="proposal_send";
    $objecttmp=new RequestManager($db);
    $trackid='rm'.$object->id;
    include DOL_DOCUMENT_ROOT.'/core/tpl/massactions_pre.tpl.php';
	// Confirm delete time spent
	if ($massaction == 'mass_delete_time_spent' && $candelete) {
		print $form->formconfirm($_SERVER["PHP_SELF"], $langs->trans("ConfirmMassDeletion"), $langs->trans("ConfirmMassDeletionQuestion", count($toselect)), "confirm_mass_delete_time_spent", null, '', 0, 200, 500, 1);
	}

    $i = 0;

    $moreforfilter='';

    $parameters=array();
    $reshook=$hookmanager->executeHooks('printFieldPreListTitle',$parameters);    // Note that $action and $object may have been modified by hook
    if (empty($reshook)) $moreforfilter .= $hookmanager->resPrint;
    else $moreforfilter = $hookmanager->resPrint;

    if (! empty($moreforfilter)) {
        print '<div class="liste_titre liste_titre_bydiv centpercent">';
        print $moreforfilter;
        print '</div>';
    }

    $varpage=empty($contextpage)?$_SERVER["PHP_SELF"]:$contextpage;
    $selectedfields=$form->multiSelectArrayWithCheckbox('selectedfields', $arrayfields, $varpage);	// This also change content of $arrayfields
    if ($massactionbutton) $selectedfields.=$form->showCheckAddButtons('checkforselect', 1);

    print '<div class="div-table-responsive">';
    print '<table class="tagtable nobottomiftotal liste'.($moreforfilter?" listwithfilterbefore":"").'">'."\n";

	$totalarray = array('totalduration' => 0, 'totalcost' => 0);
    print '<tr class="liste_titre_filter">';
    if (! empty($arrayfields['ts.time_date']['checked'])) {
        print '<td class="liste_titre">';
        print '</td>';
		$totalarray['nbfield']++;
	}
	if (! empty($arrayfields['ts.fk_user']['checked'])) {
		print '<td class="liste_titre">';
		print $form->select_dolusers($search_user, 'search_user', 1, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp');
		print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['ts.fk_requestmanager']['checked'])) {
        print '<td class="liste_titre">';
        print '<input class="flat" size="6" type="text" name="search_request" value="' . dol_escape_htmltag($search_request) . '">';
        print '</td>';
		$totalarray['nbfield']++;
	}
	// Label
	if (!empty($arrayfields['rm.label']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_request_label" value="' . dol_escape_htmltag($search_request_label) . '">';
		print '</td>';
		$totalarray['nbfield']++;
	}
	// Thridparty Origin
	if (!empty($arrayfields['rm.fk_soc']['checked'])) {
		print '<td class="liste_titre">';
		print '<input class="flat" size="6" type="text" name="search_request_thirdparty" value="' . dol_escape_htmltag($search_request_thirdparty) . '">';
		print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['ts.note']['checked'])) {
        print '<td class="liste_titre">';
        print '<input class="flat" size="6" type="text" name="search_note" value="' . dol_escape_htmltag($search_note) . '">';
        print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['ts.duration']['checked'])) {
        print '<td class="liste_titre right">';
        print '<input class="flat" size="6" type="text" name="search_duration" value="' . dol_escape_htmltag($search_duration) . '">';
        print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['cost']['checked'])) {
        print '<td class="liste_titre right">';
        print '<input class="flat" size="6" type="text" name="search_cost" value="' . dol_escape_htmltag($search_cost) . '">';
        print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['ts.fk_user_author']['checked'])) {
        print '<td class="liste_titre center">';
		print $form->select_dolusers($search_user_author, 'search_user_author', 1, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp');
        print '</td>';
		$totalarray['nbfield']++;
	}
    if (! empty($arrayfields['ts.fk_user_modif']['checked'])) {
        print '<td class="liste_titre center">';
		print $form->select_dolusers($search_user_modification, 'search_user_modification', 1, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp');
        print '</td>';
		$totalarray['nbfield']++;
	}
    // Fields from hook
    $parameters=array('arrayfields'=>$arrayfields);
    $reshook=$hookmanager->executeHooks('printFieldListOption',$parameters);    // Note that $action and $object may have been modified by hook
    print $hookmanager->resPrint;
    // Date creation
    if (! empty($arrayfields['ts.datec']['checked'])) {
        print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $form->selectDate($search_date_creation_start ? $search_date_creation_start : -1, 'search_date_creation_start', 0, 0, 1, '', 1, 0, 0, '', '', '', '', 1, '', $langs->trans('From'));
		print '</div>';
		print '<div class="nowrap">';
		print $form->selectDate($search_date_creation_end ? $search_date_creation_end : -1, 'search_date_creation_end', 0, 0, 1, '', 1, 0, 0, '', '', '', '', 1, '', $langs->trans('to'));
		print '</div>';
        print '</td>';
		$totalarray['nbfield']++;
	}
    // Date modification
    if (! empty($arrayfields['ts.tms']['checked'])) {
        print '<td class="liste_titre center">';
		print '<div class="nowrap">';
		print $form->selectDate($search_date_modification_start ? $search_date_modification_start : -1, 'search_date_modification_start', 0, 0, 1, '', 1, 0, 0, '', '', '', '', 1, '', $langs->trans('From'));
		print '</div>';
		print '<div class="nowrap">';
		print $form->selectDate($search_date_modification_end ? $search_date_modification_end : -1, 'search_date_modification_end', 0, 0, 1, '', 1, 0, 0, '', '', '', '', 1, '', $langs->trans('to'));
		print '</div>';
        print '</td>';
		$totalarray['nbfield']++;
	}
    // Action column
    print '<td class="liste_titre" align="middle">';
    $searchpicto=$form->showFilterButtons();
    print $searchpicto;
    print '</td>';
	$totalarray['nbfield']++;
    print "</tr>\n";

    // Fields title
    print '<tr class="liste_titre">';
    if (! empty($arrayfields['ts.time_date']['checked']))           print_liste_field_titre($arrayfields['ts.time_date']['label'],$_SERVER["PHP_SELF"],'ts.time_date','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder);
	if (! empty($arrayfields['ts.fk_user']['checked']))             print_liste_field_titre($arrayfields['ts.fk_user']['label'],$_SERVER["PHP_SELF"],'u.lastname, u.firstname','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder);
    if (! empty($arrayfields['ts.fk_requestmanager']['checked']))   print_liste_field_titre($arrayfields['ts.fk_requestmanager']['label'],$_SERVER["PHP_SELF"],'rm.ref','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder);
	if (! empty($arrayfields['rm.label']['checked'])) 				print_liste_field_titre($arrayfields['rm.label']['label'], $_SERVER["PHP_SELF"], 'rm.label', '', '&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start, '', $sortfield, $sortorder);
	if (! empty($arrayfields['rm.fk_soc']['checked'])) 				print_liste_field_titre($arrayfields['rm.fk_soc']['label'], $_SERVER["PHP_SELF"], 's.nom', '', '&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start, '', $sortfield, $sortorder);
    if (! empty($arrayfields['ts.note']['checked']))                print_liste_field_titre($arrayfields['ts.note']['label'],$_SERVER["PHP_SELF"],'ts.note','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder);
    if (! empty($arrayfields['ts.duration']['checked']))            print_liste_field_titre($arrayfields['ts.duration']['label'],$_SERVER["PHP_SELF"],'ts.duration','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'right ');
    if (! empty($arrayfields['cost']['checked']))                   print_liste_field_titre($arrayfields['cost']['label'],$_SERVER["PHP_SELF"],'cost','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'right ');
    if (! empty($arrayfields['ts.fk_user_author']['checked']))      print_liste_field_titre($arrayfields['ts.fk_user_author']['label'],$_SERVER["PHP_SELF"],'ua.lastname, ua.firstname','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'center ');
    if (! empty($arrayfields['ts.fk_user_modif']['checked']))       print_liste_field_titre($arrayfields['ts.fk_user_modif']['label'],$_SERVER["PHP_SELF"],'um.lastname, um.firstname','','&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'center ');
    // Hook fields
    $parameters=array('arrayfields'=>$arrayfields,'param'=>'&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'sortfield'=>$sortfield,'sortorder'=>$sortorder);
    $reshook=$hookmanager->executeHooks('printFieldListTitle',$parameters);    // Note that $action and $object may have been modified by hook
    print $hookmanager->resPrint;
    if (! empty($arrayfields['ts.datec']['checked']))     			print_liste_field_titre($arrayfields['ts.datec']['label'],$_SERVER["PHP_SELF"],"ts.datec","",'&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'center nowrap ');
    if (! empty($arrayfields['ts.tms']['checked']))       			print_liste_field_titre($arrayfields['ts.tms']['label'],$_SERVER["PHP_SELF"],"ts.tms","",'&id=' . urlencode($id) . $param . $param_date_nav_mode . $param_date_nav_start,'',$sortfield,$sortorder, 'center nowrap ');
    print_liste_field_titre($selectedfields, $_SERVER["PHP_SELF"],"",'','','',$sortfield,$sortorder,'maxwidthsearch center ');
    print '</tr>'."\n";

	$requestmanager_static = new RequestManager($db);
	$time_spent_static = new RequestManagerTimeSpent($db);
	$societestatic_origin = new Societe($db);
    $user_cached = array();

    $now = dol_now();
    $i=0;
    while ($i < min($num,$limit)) {
        $obj = $db->fetch_object($resql);

		$time_spent_static->id = $obj->rowid;
		$time_spent_static->fk_requestmanager = $obj->fk_requestmanager;
		$time_spent_static->fk_user = $obj->fk_user;

		$requestmanager_static->id = $obj->fk_requestmanager;
		$requestmanager_static->fk_type = $obj->request_type;
		$requestmanager_static->statut_type = $obj->request_status_type;
		$requestmanager_static->ref = $obj->request_ref;
		$requestmanager_static->ref_ext = $obj->request_ref_ext;
		$requestmanager_static->label = $obj->request_label;
		$requestmanager_static->socid = $obj->request_socid;

		$societestatic_origin->id = $obj->fk_soc;
		$societestatic_origin->name = $obj->soc_name_origin;
		$societestatic_origin->name_alias = $obj->soc_name_alias_origin;
		$societestatic_origin->client = $obj->soc_client_origin;
		$societestatic_origin->fournisseur = $obj->soc_fournisseur_origin;
		$societestatic_origin->code_client = $obj->soc_code_client_origin;
		$societestatic_origin->code_fournisseur = $obj->soc_code_fournisseur_origin;

		$can_write_line = $time_spent_static->canWriteLine();
		$is_edit_line = $cancreate && $can_write_line > 0 && $action == 'edit_time_spent' && $line_id == $obj->rowid;

		print '<tr class="oddeven" id="' . $obj->rowid . '">';
		if ($is_edit_line) {
			$time_spent_date = GETPOSTISSET('edit_time_spent_datemonth') ? dol_mktime(GETPOST('edit_time_spent_datehour', 'int'), GETPOST('edit_time_spent_datemin', 'int'), 0, GETPOST('edit_time_spent_datemonth', 'int'), GETPOST('edit_time_spent_dateday', 'int'), GETPOST('edit_time_spent_dateyear', 'int')) : $db->jdate($obj->time_date);
			$time_spent_user = GETPOSTISSET('edit_time_spent_user') && $user->admin ? GETPOST('edit_time_spent_user', 'int') : $obj->fk_user;
			$time_spent_request = GETPOSTISSET('edit_time_spent_request') ? GETPOST('edit_time_spent_request', 'int') : $obj->fk_requestmanager;
			$time_spent_note	= GETPOSTISSET('edit_time_spent_note') ? GETPOST('edit_time_spent_note', 'alphanohtml') : $obj->note;
			$time_spent_duration_hour	= GETPOST('edit_time_spent_durationhour', 'int');
			$time_spent_duration_min	= GETPOST('edit_time_spent_durationmin', 'int');
			$time_spent_duration_hour	= $time_spent_duration_hour > 0 ? $time_spent_duration_hour : 0;
			$time_spent_duration_min	= $time_spent_duration_min > 0 ? $time_spent_duration_min : 0;
			$time_spent_duration        = GETPOST('edit_time_spent_durationhour', 'int') ? $time_spent_duration_hour * 3600 + $time_spent_duration_min * 60 : $obj->duration;    // We store duration in seconds

			print '<td colspan="' . $totalarray['nbfield'] . '">';
			print '<input type="hidden" name="line_id" value="' . $obj->rowid . '">';

			print '<div class="div-table-responsive-no-min">';
			print '<table class="noborder nohover centpercent">';

			print '<tr class="liste_titre">';
			print '<td>' . $langs->trans("Date") . '</td>';
			print '<td>' . $langs->trans("By") . '</td>';
			if (empty($id)) print '<td>' . $langs->trans("RequestManagerRequest") . '</td>';
			print '<td>' . $langs->trans("Note") . '</td>';
			print '<td>' . $langs->trans("NewTimeSpent") . '</td>';
			print '<td></td>';
			print "</tr>\n";

			print '<tr class="oddeven">';

			// Date
			print '<td class="maxwidthonsmartphone">';
			print $form->select_date($time_spent_date, 'edit_time_spent_date', ($conf->browser->layout == 'phone' ? 2 : 1), 1, 2, "edit_time_spent_date", 1, 0, 1);
			print '</td>';

			// Contributor
			print '<td class="maxwidthonsmartphone">';
			if ($user->rights->requestmanager->timesspent->manage || $user->admin) {
				print img_object('', 'user', 'class="hideonsmartphone"');
				$events = array();
				$events[] = array('action' => 'getRequests', 'url' => dol_buildpath('/requestmanager/ajax/requests_list.php', 1), 'htmlname' => 'edit_time_spent_request', 'params' => array('selected' => '{{#time_spent_request}}'));
				print $formrequestmanager->add_select_events('edit_time_spent_user', $events);
				print $form->select_dolusers($time_spent_user, 'edit_time_spent_user', 0, null, 0, null, null, 0, 56, '', 0, '', '', 'minwidth200imp', 1);
			} else {
				print $user->getNomUrl(1);
			}
			print '</td>';

			// Request
			if (empty($id)) {
				print '<td class="maxwidthonsmartphone">';
				print $formrequestmanager->select_request($time_spent_request, 'edit_time_spent_request', $time_spent_user, 1, '', '', 'minwidth200imp centpercent');
				print '</td>';
			}

			// Note
			print '<td>';
			print '<textarea name="edit_time_spent_note" class="maxwidth100onsmartphone" rows="' . ROWS_2 . '">' . $time_spent_note . '</textarea>';
			print '</td>';

			// Duration - Time spent
			print '<td>';
			print $form->select_duration('edit_time_spent_duration', $time_spent_duration, 0, 'text');
			print '</td>';

			print '<td class="center">';
			print '<input type="submit" class="button" name="button_confirm_edit_time_spent" value="' . $langs->trans("Save") . '">';
			print '&nbsp;';
			print '<input type="submit" class="button" name="cancel" value="' . $langs->trans("Cancel") . '">';
			print '</td></tr>';

			print '</table>';
			print '</div>';
			print '</td>';
		} else {
			if (!empty($arrayfields['ts.time_date']['checked'])) {
				print '<td class="nowrap">';
				if (!empty($arrayfields['day']['checked'])) {
					print dol_print_date($db->jdate($obj->time_date), '%a', 'tzuserrel') . ' ';
				}
				print dol_print_date($db->jdate($obj->time_date), $obj->is_date ? 'day' : 'dayhour', 'tzuserrel');
				print "</td>\n";
			}
			if (!empty($arrayfields['ts.fk_user']['checked'])) {
				print '<td class="nocellnopadd nowrap">';
				if (!isset($user_cached[$obj->fk_user])) {
					$user_static = new User($db);
					$user_static->fetch($obj->fk_user);
					$user_cached[$obj->fk_user] = $user_static;
				}
				if (isset($user_cached[$obj->fk_user])) print $user_cached[$obj->fk_user]->getNomUrl(1);
				print '</td>';
			}
			if (!empty($arrayfields['ts.fk_requestmanager']['checked'])) {
				print '<td class="nocellnopadd nowrap">';
				print $requestmanager_static->getNomUrl(1);
				print '</td>';
			}
			// Label
			if (!empty($arrayfields['rm.label']['checked'])) {
				print '<td>';
				print $obj->request_label;
				print '</td>';
			}
			// Thridparty Origin
			if (!empty($arrayfields['rm.fk_soc']['checked'])) {
				print '<td>';
				print $societestatic_origin->getNomUrl(1);
				print '</td>';
			}
			if (!empty($arrayfields['ts.note']['checked'])) {
				print '<td class="nocellnopadd">';
				print $obj->note;
				print '</td>';
			}
			if (!empty($arrayfields['ts.duration']['checked'])) {
				print '<td class="nocellnopadd nowrap right">';
				if ($obj->duration > 0) print convertSecondToTime($obj->duration, 'allhourmin');
				$totalarray['totalduration'] += $obj->duration;
				print '</td>';
				if (!$i) $totalarray['durationfield'] = $totalarray['nbfield'];
			}
			if (!empty($arrayfields['cost']['checked'])) {
				print '<td class="nocellnopadd nowrap right">';
				if (!empty($obj->cost)) print price($obj->cost, '', $langs, 1, 0, 0, $conf->currency);
				$totalarray['totalcost'] += $obj->cost;
				print '</td>';
				if (!$i) $totalarray['costfield'] = $totalarray['nbfield'];
			}
			if (!empty($arrayfields['ts.fk_user_author']['checked'])) {
				print '<td class="nocellnopadd nowrap center">';
				if (!isset($user_cached[$obj->fk_user_author])) {
					$user_static = new User($db);
					$user_static->fetch($obj->fk_user_author);
					$user_cached[$obj->fk_user_author] = $user_static;
				}
				if (isset($user_cached[$obj->fk_user_author])) print $user_cached[$obj->fk_user_author]->getNomUrl(1);
				print '</td>';
			}
			if (!empty($arrayfields['ts.fk_user_modif']['checked'])) {
				print '<td class="nocellnopadd nowrap center">';
				if ($obj->fk_user_modif > 0) {
					if (!isset($user_cached[$obj->fk_user_modif])) {
						$user_static = new User($db);
						$user_static->fetch($obj->fk_user_modif);
						$user_cached[$obj->fk_user_modif] = $user_static;
					}
					if (isset($user_cached[$obj->fk_user_modif])) print $user_cached[$obj->fk_user_modif]->getNomUrl(1);
				}
				print '</td>';
			}
			// Fields from hook
			$parameters = array('arrayfields' => $arrayfields, 'obj' => $obj);
			$reshook = $hookmanager->executeHooks('printFieldListValue', $parameters);    // Note that $action and $object may have been modified by hook
			print $hookmanager->resPrint;
			// Date creation
			if (!empty($arrayfields['ts.datec']['checked'])) {
				print '<td class="center nowrap">';
				print dol_print_date($db->jdate($obj->datec), 'dayhour', 'tzuserrel');
				print '</td>';
			}
			// Date modification
			if (!empty($arrayfields['ts.tms']['checked'])) {
				print '<td class="center nowrap">';
				print dol_print_date($db->jdate($obj->tms), 'dayhour', 'tzuserrel');
				print '</td>';
			}
			// Action column
			print '<td class="nowrap center">';
			if ($cancreate && $can_write_line) {
				$param.= '&time_spent_request='.(int) $time_spent_request;
				print '<a class="reposition" href="' . $_SERVER["PHP_SELF"] . '?action=edit_time_spent&line_id=' . $obj->rowid . '&token=' . newToken() . '&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '">' . img_edit() . '</a>';
			}
			if ($candelete && $can_write_line) {
				print '&nbsp;';
				print '<a class="reposition" href="' . $_SERVER["PHP_SELF"] . '?action=delete_time_spent&line_id=' . $obj->rowid . '&token=' . newToken() . '&id=' . urlencode($id) . $param . $param_list_nav . $param_date_nav_mode . $param_date_nav_start . '">' . img_delete() . '</a>';
			}
			if ($massactionbutton || $massaction) {  // If we are in select mode (massactionbutton defined) or if we have already selected and sent an action ($massaction) defined
				$selected = 0;
				if (in_array($obj->rowid, $arrayofselected)) $selected = 1;
				print '&nbsp;';
				print '<input id="cb' . $obj->rowid . '" class="flat checkforselect" type="checkbox" name="toselect[]" value="' . $obj->rowid . '"' . ($selected ? ' checked="checked"' : '') . '>';
			}
			print '</td>';
		}
		print "</tr>\n";

        $i++;
    }

    // Show total line
    if ($totalarray['totalduration'] != 0 || $totalarray['totalcost'] != 0) {
        print '<tr class="liste_total">';
        $i = 0;
        while ($i < $totalarray['nbfield']) {
            $i++;
            if ($i == 1) {
                if ($num < $limit && empty($offset)) print '<td class="left">' . $langs->trans("Total") . '</td>';
                else print '<td class="left">' . $langs->trans("Totalforthispage") . '</td>';
            } elseif ($totalarray['durationfield'] == $i) print '<td class="right">' . convertSecondToTime($totalarray['totalduration'], 'allhourmin') . '</td>';
            elseif ($totalarray['costfield'] == $i) print '<td class="right">' . price($totalarray['totalcost'], '', $langs, 1, 0, 0, $conf->currency) . '</td>';
            else print '<td></td>';
        }
        print '</tr>';
    }

    $db->free($resql);

    $parameters=array('arrayfields'=>$arrayfields, 'sql'=>$sql);
    $reshook=$hookmanager->executeHooks('printFieldListFooter',$parameters);    // Note that $action and $object may have been modified by hook
    print $hookmanager->resPrint;

    print '</table>'."\n";
    print '</div>'."\n";

    print '</form>'."\n";

//	if ($massaction == 'builddoc' || $action == 'remove_file' || $show_files)
//	{
//		/*
//	     * Show list of available documents
//	     */
//		$urlsource=$_SERVER['PHP_SELF'].'?sortfield='.$sortfield.'&sortorder='.$sortorder;
//		$urlsource.=str_replace('&amp;','&',$param);
//
//		$filedir=$diroutputmassaction;
//		$genallowed=$user->rights->requestmanager->lire;
//		$delallowed=$user->rights->requestmanager->creer;
//
//		print $formfile->showdocuments('massfilesarea_requestmanager','',$filedir,$urlsource,0,$delallowed,'',1,1,0,48,1,$param,'','');
//	}
//	else
//	{
//		print '<br><a name="show_files"></a><a href="'.$_SERVER["PHP_SELF"].'?show_files=1'.$param.'#show_files">'.$langs->trans("ShowTempMassFilesArea").'</a>';
//	}
}
else
{
    dol_print_error($db);
}

// End of page
llxFooter();
$db->close();
